@@ -10,6 +10,7 @@ import stat
 import sys
 
 from buildscripts.resmokelib.multiversionconstants import LAST_STABLE_MONGOD_BINARY
+from buildscripts.resmokelib.multiversionconstants import LAST_STABLE_MONGOS_BINARY
 from . import jasper_process
 from . import process
 from .. import config
@@ -106,6 +107,13 @@ def get_default_log_component_verbosity_for_mongod(executable):
     return default_mongod_log_component_verbosity()
 
 
+def _add_testing_set_parameters(suite_set_parameters):
+    # Certain behaviors should only be enabled for resmoke usage. These are traditionally new
+    # commands, insecure access, and increased diagnostics.
+    suite_set_parameters.setdefault("testingDiagnosticsEnabled", True)
+    suite_set_parameters.setdefault("enableTestCommands", True)
+
+
 def mongod_program(  # pylint: disable=too-many-branches,too-many-statements
         logger, executable=None, process_kwargs=None, **kwargs):
     """Return a Process instance that starts mongod arguments constructed from 'kwargs'."""
@@ -187,6 +195,11 @@ def mongod_program(  # pylint: disable=too-many-branches,too-many-statements
             executable != LAST_STABLE_MONGOD_BINARY:
         suite_set_parameters["assertStableTimestampEqualsAppliedThroughOnRecovery"] = True
 
+    if executable == LAST_STABLE_MONGOD_BINARY:
+        suite_set_parameters.setdefault("enableTestCommands", True)
+    else:
+        _add_testing_set_parameters(suite_set_parameters)
+
     _apply_set_parameters(args, suite_set_parameters)
 
     shortcut_opts = {
@@ -264,6 +277,11 @@ def mongos_program(logger, executable=None, process_kwargs=None, **kwargs):
     if "logComponentVerbosity" not in suite_set_parameters:
         suite_set_parameters["logComponentVerbosity"] = default_mongos_log_component_verbosity()
 
+    if executable == LAST_STABLE_MONGOS_BINARY:
+        suite_set_parameters.setdefault("enableTestCommands", True)
+    else:
+        _add_testing_set_parameters(suite_set_parameters)
+
     _apply_set_parameters(args, suite_set_parameters)
 
     # Apply the rest of the command line arguments.
