@@ -2,6 +2,7 @@
 
 import os
 import sys
+import shlex
 
 import argparse
 
@@ -47,7 +48,17 @@ def _add_run(subparsers):  # pylint: disable=too-many-statements
               " positional arguments, they will be run using the suites'"
               " configurations."))
 
-    parser.set_defaults(logger_file="console")
+    parser.add_argument("--configDir", dest="config_dir", metavar="CONFIG_DIR",
+                        help="Directory to search for resmoke configuration files")
+
+    parser.add_argument("--installDir", dest="install_dir", metavar="INSTALL_DIR",
+                        help="Directory to search for MongoDB binaries")
+
+    parser.add_argument(
+        "--alwaysUseLogFiles", dest="always_use_log_files", action="store_true",
+        help=("Logs server output to a file located in the db path and prevents the"
+              " cleaning of dbpaths after testing. Note that conflicting options"
+              " passed in from test files may cause an error."))
 
     parser.add_argument(
         "--basePort", dest="base_port", metavar="PORT",
@@ -100,6 +111,8 @@ def _add_run(subparsers):  # pylint: disable=too-many-statements
         help=("The number of Job instances to use. Each instance will receive its"
               " own MongoDB deployment to dispatch tests to."))
 
+    parser.set_defaults(logger_file="console")
+
     parser.add_argument("--mongo", dest="mongo_executable", metavar="PATH",
                         help="The path to the mongo shell executable for resmoke.py to use.")
 
@@ -222,6 +235,15 @@ def _add_run(subparsers):  # pylint: disable=too-many-statements
         help="Sets the storage engine cache size configuration"
         " setting for all mongod's.")
 
+    parser.add_argument(
+        "--numReplSetNodes", type=int, dest="num_replset_nodes", metavar="N",
+        help="The number of nodes to initialize per ReplicaSetFixture. This is also "
+        "used to indicate the number of replica set members per shard in a "
+        "ShardedClusterFixture.")
+
+    parser.add_argument("--numShards", type=int, dest="num_shards", metavar="N",
+                        help="The number of shards to use in a ShardedClusterFixture.")
+
     parser.add_argument(
         "--wiredTigerCollectionConfigString", dest="wt_coll_config", metavar="CONFIG",
         help="Sets the WiredTiger collection configuration setting for all mongod's.")
@@ -237,6 +259,18 @@ def _add_run(subparsers):  # pylint: disable=too-many-statements
         help="OBSOLETE: Superceded by --suites; specify --suites=SUITE path/to/test"
         " to run a particular test under a particular suite configuration.")
 
+    parser.add_argument(
+        "--mixedBinVersions", type=str, dest="mixed_bin_versions",
+        metavar="version1-version2-..-versionN", help="Runs the test with the provided replica set"
+        " binary version configuration. Specify 'old-new' to configure a replica set with a"
+        " 'last-stable' version primary and 'latest' version secondary. For a sharded cluster"
+        " with two shards and two replica set nodes each, specify 'old-new-old-new'.")
+
+    parser.add_argument(
+        "--linearChain", action="store", dest="linear_chain", choices=("on", "off"),
+        metavar="ON|OFF", help="Enable or disable linear chaining for tests using "
+        "ReplicaSetFixture.")
+
     internal_options = parser.add_argument_group(
         title=_INTERNAL_OPTIONS_TITLE,
         description=("Internal options for advanced users and resmoke developers."
@@ -248,6 +282,10 @@ def _add_run(subparsers):  # pylint: disable=too-many-statements
               " located in the resmokeconfig/suites/ directory, then the basename"
               " without the .yml extension can be specified, e.g. 'console'."))
 
+    # Used for testing resmoke. Do not set this.
+    internal_options.add_argument("--internalParam", action="append", dest="internal_params",
+                                  help=argparse.SUPPRESS)
+
     internal_options.add_argument("--perfReportFile", dest="perf_report_file",
                                   metavar="PERF_REPORT",
                                   help="Writes a JSON file with performance test results.")
@@ -571,3 +609,9 @@ def parse_command_line(sys_args, **kwargs):
             f"Resmoke configuration has invalid subcommand: {subcommand}. Try '--help'")
 
     return subcommand_obj
+
+
+def set_run_options(argstr=''):
+    """Populate the config module variables for the 'run' subcommand with the default options."""
+    parser, parsed_args = _parse(['run'] + shlex.split(argstr))
+    configure_resmoke.validate_and_update_config(parser, parsed_args)
