@@ -4,6 +4,7 @@ from __future__ import absolute_import
 
 import collections
 import datetime
+import json
 import os
 import sys
 import subprocess
@@ -14,20 +15,22 @@ from mock import Mock, patch, MagicMock
 
 import requests
 
-from shrub.config import Configuration
+from shrub.v2 import ShrubProject, BuildVariant
 
 import buildscripts.burn_in_tests as under_test
 from buildscripts.ciconfig.evergreen import parse_evergreen_file
 import buildscripts.util.teststats as teststats_utils
+import buildscripts.resmokelib.parser as _parser
 import buildscripts.resmokelib.config as _config
+_parser.set_run_options()
 
 # pylint: disable=missing-docstring,protected-access,too-many-lines,no-self-use
 
 
 def create_tests_by_task_mock(n_tasks, n_tests):
     return {
-        f"task_{i}": {
-            "resmoke_args": f"--suites=suite_{i}",
+        f"task_{i}_gen": {
+            "display_task_name": f"task_{i}", "resmoke_args": f"--suites=suite_{i}",
             "tests": [f"jstests/tests_{j}" for j in range(n_tests)]
         }
         for i in range(n_tasks)
@@ -36,18 +39,6 @@ def create_tests_by_task_mock(n_tasks, n_tests):
 
 MV_MOCK_SUITES = ["replica_sets_jscore_passthrough", "sharding_jscore_passthrough"]
 
-
-def create_multiversion_tests_by_task_mock(n_tasks, n_tests):
-    assert n_tasks <= len(MV_MOCK_SUITES)
-    return {
-        f"{MV_MOCK_SUITES[i % len(MV_MOCK_SUITES)]}": {
-            "resmoke_args": f"--suites=suite_{i}",
-            "tests": [f"jstests/tests_{j}" for j in range(n_tests)]
-        }
-        for i in range(n_tasks)
-    }
-
-
 _DATE = datetime.datetime(2018, 7, 15)
 RESMOKELIB = "buildscripts.resmokelib"
 
@@ -73,6 +64,7 @@ def mock_git_diff(change_list):
 def mock_changed_git_files(add_files):
     repo = MagicMock()
     repo.index.diff.return_value = mock_git_diff([mock_a_file(f) for f in add_files])
+    repo.working_dir = "."
     return repo
 
 
@@ -84,7 +76,10 @@ def get_evergreen_config(config_file_path):
 
 
 class TestAcceptance(unittest.TestCase):
-    @patch(ns("_write_json_file"))
+    def tearDown(self):
+        _parser.set_run_options()
+
+    @patch(ns("write_file"))
     def test_no_tests_run_if_none_changed(self, write_json_mock):
         """
         Given a git repository with no changes,
@@ -92,28 +87,30 @@ class TestAcceptance(unittest.TestCase):
         Then no tests are discovered to run.
         """
         variant = "build_variant"
-        repo = mock_changed_git_files([])
+        repos = [mock_changed_git_files([])]
         repeat_config = under_test.RepeatConfig()
         gen_config = under_test.GenerateConfig(
             variant,
             "project",
         )  # yapf: disable
+        evg_conf_mock = MagicMock()
+        evg_conf_mock.get_task_names_by_tag.return_value = set()
 
-        under_test.burn_in(repeat_config, gen_config, "", "testfile.json", False, None, repo, None)
+        under_test.burn_in(repeat_config, gen_config, "", "testfile.json", False, None, repos, None)
 
         write_json_mock.assert_called_once()
-        written_config = write_json_mock.call_args[0][0]
+        written_config = json.loads(write_json_mock.call_args[0][1])
         display_task = written_config["buildvariants"][0]["display_tasks"][0]
         self.assertEqual(1, len(display_task["execution_tasks"]))
         self.assertEqual(under_test.BURN_IN_TESTS_GEN_TASK, display_task["execution_tasks"][0])
 
     @unittest.skipIf(sys.platform.startswith("win"), "not supported on windows")
-    @patch(ns("_write_json_file"))
+    @patch(ns("write_file"))
     def test_tests_generated_if_a_file_changed(self, write_json_mock):
         """
-        Given a git repository with no changes,
+        Given a git repository with changes,
         When burn_in_tests is run,
-        Then no tests are discovered to run.
+        Then tests are discovered to run.
         """
         # Note: this test is using actual tests and suites. So changes to those suites could
         # introduce failures and require this test to be updated.
@@ -121,7 +118,7 @@ class TestAcceptance(unittest.TestCase):
         # 'auth_audit' test suites. It needs to be in at least one of those for the test to pass.
         _config.NAMED_SUITES = None
         variant = "enterprise-rhel-62-64-bit"
-        repo = mock_changed_git_files(["jstests/auth/auth1.js"])
+        repos = [mock_changed_git_files(["jstests/auth/auth1.js"])]
         repeat_config = under_test.RepeatConfig()
         gen_config = under_test.GenerateConfig(
             variant,
@@ -129,11 +126,11 @@ class TestAcceptance(unittest.TestCase):
         )  # yapf: disable
         evg_config = get_evergreen_config("etc/evergreen.yml")
 
-        under_test.burn_in(repeat_config, gen_config, "", "testfile.json", False, evg_config, repo,
+        under_test.burn_in(repeat_config, gen_config, "", "testfile.json", False, evg_config, repos,
                            None)
 
         write_json_mock.assert_called_once()
-        written_config = write_json_mock.call_args[0][0]
+        written_config = json.loads(write_json_mock.call_args[0][1])
         n_tasks = len(written_config["tasks"])
         # Ensure we are generating at least one task for the test.
         self.assertGreaterEqual(n_tasks, 1)
@@ -265,12 +262,6 @@ class TestGenerateConfig(unittest.TestCase):
         gen_config = under_test.GenerateConfig("build_variant", "project")
         gen_config.validate(evg_conf_mock)
 
-    def test_validate_use_multiversion(self):
-        evg_conf_mock = MagicMock()
-
-        gen_config = under_test.GenerateConfig("build_variant", "project")
-        gen_config.validate(evg_conf_mock)
-
 
 class TestParseAvgTestRuntime(unittest.TestCase):
     def test__parse_avg_test_runtime(self):
@@ -471,39 +462,37 @@ TESTS_BY_TASK = {
         "tests": ["jstests/test4.js", "jstests/test5.js"]},
     "task4": {
         "resmoke_args": "--suites=suite4", "tests": []},
-    "taskmulti": {
-        "resmoke_args": "--suites=suite4",
-        "tests": ["jstests/multi1.js"],
-        "use_multiversion": "/data/multi"},
 }  # yapf: disable
 
 
 class TestCreateGenerateTasksConfig(unittest.TestCase):
+    @unittest.skipIf(sys.platform.startswith("win"), "not supported on windows")
     def test_no_tasks_given(self):
-        evg_config = Configuration()
+        build_variant = BuildVariant("build variant")
         gen_config = MagicMock(run_build_variant="variant")
         repeat_config = MagicMock()
 
-        evg_config = under_test.create_generate_tasks_config(evg_config, {}, gen_config,
-                                                             repeat_config, None)
+        under_test.create_generate_tasks_config(build_variant, {}, gen_config, repeat_config, None)
 
-        evg_config_dict = evg_config.to_map()
-        self.assertNotIn("tasks", evg_config_dict)
+        evg_config_dict = build_variant.as_dict()
+        self.assertEqual(0, len(evg_config_dict["tasks"]))
 
+    @unittest.skipIf(sys.platform.startswith("win"), "not supported on windows")
     def test_one_task_one_test(self):
         n_tasks = 1
         n_tests = 1
         resmoke_options = "options for resmoke"
-        evg_config = Configuration()
+        build_variant = BuildVariant("build variant")
         gen_config = MagicMock(run_build_variant="variant", distro=None)
         repeat_config = MagicMock()
         repeat_config.generate_resmoke_options.return_value = resmoke_options
         tests_by_task = create_tests_by_task_mock(n_tasks, n_tests)
 
-        evg_config = under_test.create_generate_tasks_config(evg_config, tests_by_task, gen_config,
-                                                             repeat_config, None)
+        under_test.create_generate_tasks_config(build_variant, tests_by_task, gen_config,
+                                                repeat_config, None)
 
-        evg_config_dict = evg_config.to_map()
+        shrub_config = ShrubProject.empty().add_build_variant(build_variant)
+        evg_config_dict = shrub_config.as_dict()
         tasks = evg_config_dict["tasks"]
         self.assertEqual(n_tasks * n_tests, len(tasks))
         cmd = tasks[0]["commands"]
@@ -511,110 +500,34 @@ class TestCreateGenerateTasksConfig(unittest.TestCase):
         self.assertIn("--suites=suite_0", cmd[1]["vars"]["resmoke_args"])
         self.assertIn("tests_0", cmd[1]["vars"]["resmoke_args"])
 
+    @unittest.skipIf(sys.platform.startswith("win"), "not supported on windows")
     def test_n_task_m_test(self):
         n_tasks = 3
         n_tests = 5
-        evg_config = Configuration()
+        build_variant = BuildVariant("build variant")
         gen_config = MagicMock(run_build_variant="variant", distro=None)
         repeat_config = MagicMock()
         tests_by_task = create_tests_by_task_mock(n_tasks, n_tests)
 
-        evg_config = under_test.create_generate_tasks_config(evg_config, tests_by_task, gen_config,
-                                                             repeat_config, None)
+        under_test.create_generate_tasks_config(build_variant, tests_by_task, gen_config,
+                                                repeat_config, None)
 
-        evg_config_dict = evg_config.to_map()
+        evg_config_dict = build_variant.as_dict()
         self.assertEqual(n_tasks * n_tests, len(evg_config_dict["tasks"]))
 
-    def test_multiversion_path_is_used(self):
-        n_tasks = 1
-        n_tests = 1
-        evg_config = Configuration()
-        gen_config = MagicMock(run_build_variant="variant", distro=None)
-        repeat_config = MagicMock()
-        tests_by_task = create_tests_by_task_mock(n_tasks, n_tests)
-        first_task = "task_0"
-        multiversion_path = "multiversion_path"
-        tests_by_task[first_task]["use_multiversion"] = multiversion_path
-
-        evg_config = under_test.create_generate_tasks_config(evg_config, tests_by_task, gen_config,
-                                                             repeat_config, None)
-
-        evg_config_dict = evg_config.to_map()
-        tasks = evg_config_dict["tasks"]
-        self.assertEqual(n_tasks * n_tests, len(tasks))
-        self.assertEqual(multiversion_path, tasks[0]["commands"][2]["vars"]["task_path_suffix"])
-
 
 class TestCreateGenerateTasksFile(unittest.TestCase):
-    @patch("buildscripts.burn_in_tests.create_generate_tasks_config")
-    def test_gen_tasks_configuration_is_returned(self, gen_tasks_config_mock):
-        evg_api = MagicMock()
-        gen_config = MagicMock(use_multiversion=False)
-        repeat_config = MagicMock()
-        tests_by_task = MagicMock()
-
-        task_list = [f"task_{i}" for i in range(10)]
-
-        evg_config = MagicMock()
-        evg_config.to_map.return_value = {
-            "tasks": task_list,
-        }
-
-        gen_tasks_config_mock.return_value = evg_config
-
-        config = under_test.create_generate_tasks_file(tests_by_task, gen_config, repeat_config,
-                                                       evg_api)
-
-        self.assertEqual(config, evg_config.to_map.return_value)
-
+    @unittest.skipIf(sys.platform.startswith("win"), "not supported on windows")
+    @patch(ns("sys.exit"))
     @patch(ns("create_generate_tasks_config"))
-    def test_gen_tasks_multiversion_configuration_is_returned(self, gen_tasks_config_mock):  # pylint: disable=invalid-name
-        evg_api = MagicMock()
-        gen_config = MagicMock(run_build_variant="variant", project="project",
-                               build_variant="build_variant", task_id="task_id",
-                               use_multiversion=True)
-        repeat_config = MagicMock()
-        tests_by_task = MagicMock()
-
-        evg_config = MagicMock()
-        evg_config.to_map.return_value = {
-            'buildvariants': [
-                {
-                    'name': 'build_variant',
-                    'display_tasks': [
-                        {
-                            'name': 'burn_in_tests_multiversion',
-                            'execution_tasks': [
-                                'burn_in_tests_multiversion_gen'
-                            ]
-                        }
-                    ]
-                }
-            ]
-        }  # yapf: disable
-
-        gen_tasks_config_mock.return_value = evg_config
-
-        config = under_test.create_generate_tasks_file(tests_by_task, gen_config, repeat_config,
-                                                       evg_api)
-        self.assertEqual(config, evg_config.to_map.return_value)
-
-    @patch("buildscripts.burn_in_tests.sys.exit")
-    @patch("buildscripts.burn_in_tests.create_generate_tasks_config")
-    def test_cap_on_task_generate(self, gen_tasks_config_mock, exit_mock):
+    @patch(ns("validate_task_generation_limit"))
+    def test_cap_on_task_generate(self, validate_mock, _, exit_mock):
         evg_api = MagicMock()
         gen_config = MagicMock(use_multiversion=False)
         repeat_config = MagicMock()
         tests_by_task = MagicMock()
 
-        task_list = [f"task_{i}" for i in range(1005)]
-
-        evg_config = MagicMock()
-        evg_config.to_map.return_value = {
-            "tasks": task_list,
-        }
-
-        gen_tasks_config_mock.return_value = evg_config
+        validate_mock.return_value = False
 
         exit_mock.side_effect = ValueError("exiting")
         with self.assertRaises(ValueError):
@@ -752,29 +665,6 @@ class TestGatherTaskInfo(unittest.TestCase):
         self.assertIsNone(task_info["use_multiversion"])
         self.assertEqual(distro_name, task_info["distro"])
 
-    def test_multiversion_task(self):
-        suite_name = "suite_1"
-        distro_name = "distro_1"
-        variant = "build_variant"
-        evg_conf_mock = MagicMock()
-        evg_conf_mock.get_task.return_value.is_generate_resmoke_task = False
-        task_mock = create_variant_task_mock("task 1", suite_name, distro_name)
-        task_mock.multiversion_path = "/path/to/multiversion"
-        test_list = [f"test{i}.js" for i in range(3)]
-        tests_by_suite = {
-            suite_name: test_list,
-            "suite 2": [f"test{i}.js" for i in range(1)],
-            "suite 3": [f"test{i}.js" for i in range(2)],
-        }
-
-        task_info = under_test._gather_task_info(task_mock, tests_by_suite, evg_conf_mock, variant)
-
-        self.assertIn(suite_name, task_info["resmoke_args"])
-        for test in test_list:
-            self.assertIn(test, task_info["tests"])
-        self.assertEqual(task_mock.multiversion_path, task_info["use_multiversion"])
-        self.assertEqual(distro_name, task_info["distro"])
-
     def test_generated_task_no_large_on_task(self):
         suite_name = "suite_1"
         distro_name = "distro_1"
@@ -944,14 +834,7 @@ class TestCreateTaskList(unittest.TestCase):
 
 
 class TestFindChangedTests(unittest.TestCase):
-    @patch(ns("find_changed_files"))
-    def test_nothing_found(self, changed_files_mock):
-        repo_mock = MagicMock()
-        changed_files_mock.return_value = set()
-
-        self.assertEqual(0, len(under_test.find_changed_tests(repo_mock)))
-
-    @patch(ns("find_changed_files"))
+    @patch(ns("find_changed_files_in_repos"))
     @patch(ns("os.path.isfile"))
     def test_non_js_files_filtered(self, is_file_mock, changed_files_mock):
         repo_mock = MagicMock()
@@ -963,13 +846,13 @@ class TestFindChangedTests(unittest.TestCase):
         changed_files_mock.return_value = set(file_list)
         is_file_mock.return_value = True
 
-        found_tests = under_test.find_changed_tests(repo_mock)
+        found_tests = under_test.find_changed_tests([repo_mock])
 
         self.assertIn(file_list[0], found_tests)
         self.assertIn(file_list[2], found_tests)
         self.assertNotIn(file_list[1], found_tests)
 
-    @patch(ns("find_changed_files"))
+    @patch(ns("find_changed_files_in_repos"))
     @patch(ns("os.path.isfile"))
     def test_missing_files_filtered(self, is_file_mock, changed_files_mock):
         repo_mock = MagicMock()
@@ -981,11 +864,11 @@ class TestFindChangedTests(unittest.TestCase):
         changed_files_mock.return_value = set(file_list)
         is_file_mock.return_value = False
 
-        found_tests = under_test.find_changed_tests(repo_mock)
+        found_tests = under_test.find_changed_tests([repo_mock])
 
         self.assertEqual(0, len(found_tests))
 
-    @patch(ns("find_changed_files"))
+    @patch(ns("find_changed_files_in_repos"))
     @patch(ns("os.path.isfile"))
     def test_non_jstests_files_filtered(self, is_file_mock, changed_files_mock):
         repo_mock = MagicMock()
@@ -997,7 +880,7 @@ class TestFindChangedTests(unittest.TestCase):
         changed_files_mock.return_value = set(file_list)
         is_file_mock.return_value = True
 
-        found_tests = under_test.find_changed_tests(repo_mock)
+        found_tests = under_test.find_changed_tests([repo_mock])
 
         self.assertIn(file_list[0], found_tests)
         self.assertIn(file_list[2], found_tests)
