@@ -127,6 +127,13 @@ function documentEq(dl, dr, verbose = false, valueComparator) {
     return true;
 }
 
+/**
+ * Returns true if both 'al' and 'ar' are arrays of the same length with the same elements according
+ * to valueComparator.  Order of the elements within the arrays is not significant.
+ *
+ * This is a predicate, not an assertion; use assert.sameMembers() in assert.js for the
+ * equivalent assertion.
+ */
 function arrayEq(al, ar, verbose = false, valueComparator) {
     const debug = msg => verbose ? print(msg) : null;  // Helper to log 'msg' iff 'verbose' is true.
 
@@ -269,7 +276,7 @@ function assertErrorCode(coll, pipe, code, errmsg, options = {}) {
  * Assert that an aggregation fails with a specific code and the error message contains the given
  * string.
  */
-function assertErrMsgContains(coll, pipe, code, expectedMessage) {
+function assertErrCodeAndErrMsgContains(coll, pipe, code, expectedMessage) {
     const response = assert.commandFailedWithCode(
         coll.getDB().runCommand({aggregate: coll.getName(), pipeline: pipe, cursor: {}}), code);
     assert.neq(
@@ -278,6 +285,31 @@ function assertErrMsgContains(coll, pipe, code, expectedMessage) {
         "Error message did not contain '" + expectedMessage + "', found:\n" + tojson(response));
 }
 
+/**
+ * Assert that an aggregation fails with any code and the error message contains the given
+ * string.
+ */
+function assertErrMsgContains(coll, pipe, expectedMessage) {
+    const response = assert.commandFailed(
+        coll.getDB().runCommand({aggregate: coll.getName(), pipeline: pipe, cursor: {}}));
+    assert.neq(
+        -1,
+        response.errmsg.indexOf(expectedMessage),
+        "Error message did not contain '" + expectedMessage + "', found:\n" + tojson(response));
+}
+
+/**
+ * Assert that an aggregation fails with any code and the error message does not contain the given
+ * string.
+ */
+function assertErrMsgDoesNotContain(coll, pipe, expectedMessage) {
+    const response = assert.commandFailed(
+        coll.getDB().runCommand({aggregate: coll.getName(), pipeline: pipe, cursor: {}}));
+    assert.eq(-1,
+              response.errmsg.indexOf(expectedMessage),
+              "Error message contained '" + expectedMessage + "'");
+}
+
 /**
  * Asserts that two arrays are equal - that is, if their sizes are equal and each element in
  * the 'actual' array has a matching element in the 'expected' array, without honoring elements
@@ -332,3 +364,10 @@ function generateCollection({
     assert.eq(numDocs, res.nInserted);
     assert.eq(numDocs, coll.find().itcount());
 }
+
+/**
+ * Returns true if 'coll' exists or false otherwise.
+ */
+function collectionExists(coll) {
+    return Array.contains(coll.getDB().getCollectionNames(), coll.getName());
+}
