@@ -5,14 +5,20 @@
  *
  * Creates multiple unique background indexes in parallel.
  *
- * @tags: [creates_background_indexes]
+ * Marked as 'requires_persistence' to prevent the test from running on 'inMemory' variant, because
+ * the test generates a large oplog and 'inMemory' instances have limited resources to accommodate
+ * all nodes in the replica set (which all run in the same instance), so it may fail with the OOM
+ * error.
+ * @tags: [
+ *     creates_background_indexes,
+ *     requires_persistence,
+ * ]
  */
-
+load("jstests/concurrency/fsm_workload_helpers/assert_handle_fail_in_transaction.js");
 var $config = (function() {
     var data = {
         prefix: "create_index_background_unique_",
         numDocsToLoad: 5000,
-        shouldBuild: true,  // used to ensure ordering of buildIndex/dropIndex calls
         iterationCount: 0,
         getCollectionNameForThread: function(threadId) {
             return this.prefix + threadId.toString();
@@ -28,52 +34,43 @@ var $config = (function() {
         },
     };
 
-    let buildIndexHelper = function(db, collName, data) {
-        data.shouldBuild = false;
-        const res = db.runCommand({
-            createIndexes: data.getCollectionNameForThread(data.tid),
-            indexes: [{key: {x: 1}, name: "x_1", unique: true, background: true}]
-        });
-        assertAlways.commandWorked(res);
-    };
-
-    let dropIndexHelper = function(db, collName, data) {
-        data.shouldBuild = true;
-        // In the case that we have an even # of iterations, we skip the final drop so that
-        // validation can be performed on the indexes created.
-        if (data.iterationCount === data.iterations) {
-            return;
-        }
-
-        assertAlways.commandWorked(
-            db.runCommand({dropIndexes: data.getCollectionNameForThread(data.tid), index: "x_1"}));
-    };
-
     var states = (function() {
         function buildIndex(db, collName) {
             this.iterationCount++;
-            // Make sure successive calls to buildIndex do not occur without a dropIndex in between.
-            // This is to ensure that workloads extending from this one can add additional states
-            // and still maintain the base workload's guarantee that buildIndex and dropIndex are
-            // called in succession.
-            if (!this.shouldBuild) {
-                dropIndexHelper(db, collName, this);
-                return;
-            }
-            buildIndexHelper(db, collName, this);
+
+            const res = db.runCommand({
+                createIndexes: this.getCollectionNameForThread(this.tid),
+                indexes: [{key: {x: 1}, name: "x_1", unique: true, background: true}]
+            });
+            // Multi-statement Transactions can fail with SnapshotUnavailable if there are
+            // pending catalog changes as of the transaction start (see SERVER-43018).
+            assertWorkedOrFailedHandleTxnErrors(res,
+                                                [
+                                                    ErrorCodes.IndexBuildAborted,
+                                                    ErrorCodes.IndexBuildAlreadyInProgress,
+                                                    ErrorCodes.SnapshotUnavailable,
+                                                    ErrorCodes.SnapshotTooOld,
+                                                    ErrorCodes.NoMatchingDocument,
+                                                    ErrorCodes.NotMaster,
+                                                ],
+                                                [
+                                                    ErrorCodes.IndexBuildAborted,
+                                                    ErrorCodes.NoMatchingDocument,
+                                                    ErrorCodes.NotMaster,
+                                                ]);
         }
 
         function dropIndex(db, collName) {
             this.iterationCount++;
-            // Make sure successive calls to dropIndex do not occur without a buildIndex in between.
-            // This is to ensure that workloads extending from this one can add additional states
-            // and still maintain the base workload's guarantee that buildIndex and dropIndex are
-            // called in succession.
-            if (this.shouldBuild) {
-                buildIndexHelper(db, collName, this);
+
+            // In the case that we have an even # of iterations, we skip the final drop so that
+            // validation can be performed on the indexes created.
+            if (this.iterationCount === this.iterations) {
                 return;
             }
-            dropIndexHelper(db, collName, this);
+
+            assertAlways.commandWorked(db.runCommand(
+                {dropIndexes: this.getCollectionNameForThread(this.tid), index: "x_1"}));
         }
 
         return {
@@ -100,7 +97,7 @@ var $config = (function() {
                 const uniqueValuePrefix = i.toString() + "_";
                 bulk.insert(this.buildvariableSizedDoc(uniqueValuePrefix));
             }
-            assertAlways.writeOK(bulk.execute());
+            assertAlways.commandWorked(bulk.execute());
             assertAlways.eq(this.numDocsToLoad, db[collectionName].find({}).itcount());
         }
     }
