@@ -1,6 +1,15 @@
-// Cannot implicitly shard accessed collections because the explain output from a mongod when run
-// against a sharded collection is wrapped in a "shards" object with keys for each shard.
-// @tags: [assumes_unsharded_collection, does_not_support_stepdowns, requires_fastcount]
+/**
+ * Cannot implicitly shard accessed collections because the explain output from a mongod when run
+ * against a sharded collection is wrapped in a "shards" object with keys for each shard.
+ *
+ * @tags: [assumes_unsharded_collection,
+ *         does_not_support_stepdowns,
+ *         requires_fastcount,
+ *         # Projection push down works differently between 4.2 and 4.4. explain() provides a
+ *         # different output between the two versions. This test expects only the 4.4 version
+ *         # output.
+ *         requires_fcv_44]
+ */
 
 // Tests for the .explain() shell helper, which provides syntactic sugar for the explain command.
 
@@ -152,6 +161,12 @@ explain = t.find().max({a: 1}).hint({a: 1}).explain();
 assert.commandWorked(explain);
 assert(isIxscan(db, explain.queryPlanner.winningPlan));
 
+// .allowDiskUse()
+explain = t.explain().find().allowDiskUse().finish();
+assert.commandWorked(explain);
+explain = t.find().allowDiskUse().explain();
+assert.commandWorked(explain);
+
 // .showDiskLoc()
 explain = t.explain().find().showDiskLoc().finish();
 assert.commandWorked(explain);
@@ -212,7 +227,7 @@ assert("queryPlanner" in explain.stages[0].$cursor);
 
 explain = t.explain().aggregate({$project: {a: 3}}, {$group: {_id: null}});
 assert.commandWorked(explain);
-assert.eq(3, explain.stages.length);
+assert.eq(2, explain.stages.length);
 assert("queryPlanner" in explain.stages[0].$cursor);
 
 // Options already provided.
