@@ -213,11 +213,15 @@ if (typeof _threadInject != "undefined") {
             // This test causes collMod commands to hang, which interferes with other tests running
             // collMod.
             "crud_ops_do_not_throw_locktimeout.js",
+
+            // Can fail if isMaster takes too long on a loaded machine.
+            "dbadmin.js",
         ]);
 
         // The following tests cannot run when shell readMode is legacy.
         if (db.getMongo().readMode() === "legacy") {
             var requires_find_command = [
+                "explode_for_sort_collation.js",
                 "update_pipeline_shell_helpers.js",
                 "update_with_pipeline.js",
                 "views/dbref_projection.js",
