@@ -413,6 +413,36 @@ function RollbackTest(name = "RollbackTest", replSet) {
         return curPrimary;
     };
 
+    /**
+     * Insert on primary until its lastApplied >= the rollback node's. Useful for testing rollback
+     * via refetch, which completes rollback recovery when new lastApplied >= old top of oplog.
+     */
+    const _awaitPrimaryAppliedSurpassesRollbackApplied = function() {
+        log(`Waiting for lastApplied on sync source ${curPrimary.host} to surpass lastApplied` +
+            ` on rollback node ${curSecondary.host}`);
+
+        function lastApplied(node) {
+            const reply = assert.commandWorked(node.adminCommand({replSetGetStatus: 1}));
+            return reply.optimes.appliedOpTime.ts;
+        }
+
+        const rollbackApplied = lastApplied(curSecondary);
+        assert.soon(() => {
+            const primaryApplied = lastApplied(curPrimary);
+            jsTestLog(
+                `lastApplied on sync source ${curPrimary.host}:` +
+                ` ${tojson(primaryApplied)}, lastApplied on rollback node ${curSecondary.host}:` +
+                ` ${tojson(rollbackApplied)}`);
+
+            if (timestampCmp(primaryApplied, rollbackApplied) >= 0) {
+                return true;
+            }
+
+            let crudColl = curPrimary.getDB("test")["awaitPrimaryAppliedSurpassesRollbackApplied"];
+            assert.commandWorked(crudColl.insertOne({}));
+        }, "primary's lastApplied never surpassed rollback node's");
+    };
+
     /**
      * Transition to the second stage of rollback testing, where we isolate the old primary and
      * elect the old secondary as the new primary. Then, operations can be performed on the new
@@ -440,7 +470,10 @@ function RollbackTest(name = "RollbackTest", replSet) {
         try {
             // The stepdown freeze period is short because the node is disconnected from
             // the rest of the replica set, so it physically can't become the primary.
-            curPrimary.adminCommand({replSetStepDown: 1, force: true});
+            assert.soon(() => {
+                const res = curPrimary.adminCommand({replSetStepDown: 1, force: true});
+                return (res.ok || res.code === ErrorCodes.NotMaster);
+            });
         } catch (e) {
             // Stepdown may fail if the node has already started stepping down.
             print('Caught exception from replSetStepDown: ' + e);
@@ -476,6 +509,13 @@ function RollbackTest(name = "RollbackTest", replSet) {
 
         lastRBID = assert.commandWorked(curSecondary.adminCommand("replSetGetRBID")).rbid;
 
+        const isMajorityReadConcernEnabledOnRollbackNode =
+            assert.commandWorked(curSecondary.adminCommand({serverStatus: 1}))
+                .storageEngine.supportsCommittedReads;
+        if (!isMajorityReadConcernEnabledOnRollbackNode) {
+            _awaitPrimaryAppliedSurpassesRollbackApplied();
+        }
+
         // The current primary, which is the old secondary, will later become the sync source.
         return curPrimary;
     };
@@ -521,6 +561,10 @@ function RollbackTest(name = "RollbackTest", replSet) {
         return curSecondary;
     };
 
+    this.getTieBreaker = function() {
+        return tiebreakerNode;
+    };
+
     this.restartNode = function(nodeId, signal, startOptions, allowedExitCode) {
         assert(signal === SIGKILL || signal === SIGTERM, `Received unknown signal: ${signal}`);
         assert.gte(nodeId, 0, "Invalid argument to RollbackTest.restartNode()");
