@@ -14,6 +14,7 @@
 "use strict";
 
 load("jstests/replsets/rslib.js");
+load("jstests/libs/fail_point_util.js");
 
 var replSet = new ReplSetTest({name: 'testSet', nodes: 3});
 var nodes = replSet.nodeList();
@@ -53,7 +54,7 @@ function disableFailPoint(node) {
 // Do an initial insert to prevent the secondary from going into recovery
 var numDocuments = 20;
 var coll = primary.getDB("foo").foo;
-assert.writeOK(coll.insert({x: 0}, {writeConcern: {w: 3}}));
+assert.commandWorked(coll.insert({x: 0}, {writeConcern: {w: 3}}));
 replSet.awaitReplication();
 
 // Enable fail point to stop replication.
@@ -62,7 +63,7 @@ secondaries.forEach(enableFailPoint);
 
 var bufferCountBefore = secondary.getDB('foo').serverStatus().metrics.repl.buffer.count;
 for (var i = 1; i < numDocuments; ++i) {
-    assert.writeOK(coll.insert({x: i}));
+    assert.commandWorked(coll.insert({x: i}));
 }
 jsTestLog('Number of documents inserted into collection on primary: ' + numDocuments);
 assert.eq(numDocuments, primary.getDB("foo").foo.find().itcount());
@@ -97,15 +98,14 @@ assert.commandFailedWithCode(
 
 // Prevent the current primary from stepping down
 jsTest.log("disallowing heartbeat stepdown " + secondary.host);
-assert.commandWorked(
-    secondary.adminCommand({configureFailPoint: "blockHeartbeatStepdown", mode: 'alwaysOn'}));
+var blockHeartbeatStepdownFailPoint = configureFailPoint(secondary, "blockHeartbeatStepdown");
 jsTestLog("Shut down the rest of the set so the primary-elect has to step down");
 replSet.stop(primary);
 disableFailPoint(replSet.nodes[2]);  // Fail point needs to be off when node is shut down.
 replSet.stop(2);
 
 jsTestLog("Waiting for secondary to begin stepping down while in drain mode");
-checkLog.contains(secondary, "stepDown - blockHeartbeatStepdown fail point enabled");
+blockHeartbeatStepdownFailPoint.wait();
 
 // Disable fail point to allow replication and allow secondary to finish drain mode while in the
 // process of stepping down.
@@ -134,8 +134,7 @@ assert.eq(ReplSetTest.State.PRIMARY, secondary.adminCommand({replSetGetStatus: 1
 assert(!secondary.adminCommand('ismaster').ismaster);
 
 jsTest.log("allowing heartbeat stepdown " + secondary.host);
-assert.commandWorked(
-    secondary.adminCommand({configureFailPoint: "blockHeartbeatStepdown", mode: 'off'}));
+blockHeartbeatStepdownFailPoint.off();
 
 jsTestLog("Checking that node successfully stepped down");
 replSet.waitForState(secondary, ReplSetTest.State.SECONDARY);
@@ -151,7 +150,7 @@ assert.soon(function() {
 });
 
 jsTestLog('Ensure new primary is writable.');
-assert.writeOK(secondary.getDB("foo").flag.insert({sentinel: 1}, {writeConcern: {w: 3}}));
+assert.commandWorked(secondary.getDB("foo").flag.insert({sentinel: 1}, {writeConcern: {w: 3}}));
 // Check that no writes were lost.
 assert.eq(secondary.getDB("foo").foo.find().itcount(), numDocuments);
 replSet.stopSet();
