@@ -1002,6 +1002,10 @@ const txnOverrideTests = [
             assert.eq(cmdRes.cursor.ns, coll1.getFullName());
             assert.eq(cmdRes.cursor.firstBatch.length, 1);
 
+            // The transactions override commits the current transaction whenever it sees a DDL
+            // command.
+            assert.commandWorked(testDB.createCollection(collName2));
+
             assert.commandWorked(coll2.insert({_id: 3}));
             assert.eq(coll1.find().itcount(), 2);
             assert.eq(coll2.find().itcount(), 1);
@@ -1144,62 +1148,6 @@ const txnOverrideTests = [
             assert.eq(coll1.find().toArray(), [{_id: 1, x: 1, y: 1}]);
         }
     },
-    {
-        name: "implicit collection creation with stepdown",
-        test: function() {
-            failCommandWithFailPoint(["create"], {errorCode: ErrorCodes.NotMaster});
-            assert.throws(() => coll1.insert({_id: 1}));
-        }
-    },
-    {
-        name: "implicit collection creation with WriteConcernError",
-        test: function() {
-            failCommandWithFailPoint(
-                ["create"],
-                {writeConcernError: {code: ErrorCodes.NotMaster, codeName: "NotMaster"}});
-            assert.throws(() => coll1.insert({_id: 1}));
-        }
-    },
-    {
-        name: "implicit collection creation with WriteConcernError and normal stepdown error",
-        test: function() {
-            failCommandWithErrorAndWCENoRun(
-                "create", ErrorCodes.NotMaster, "NotMaster", ErrorCodes.NotMaster, "NotMaster");
-            assert.throws(() => coll1.insert({_id: 1}));
-        }
-    },
-    {
-        name: "implicit collection creation with WriteConcernError and normal ordinary error",
-        test: function() {
-            failCommandWithErrorAndWCENoRun("create",
-                                            ErrorCodes.OperationFailed,
-                                            "OperationFailed",
-                                            ErrorCodes.NotMaster,
-                                            "NotMaster");
-            assert.throws(() => coll1.insert({_id: 1}));
-        }
-    },
-    {
-        name: "implicit collection creation with ordinary error",
-        test: function() {
-            failCommandWithFailPoint(["create"], {errorCode: ErrorCodes.OperationFailed});
-            assert.throws(() => coll1.insert({_id: 1}));
-        }
-    },
-    {
-        name: "implicit collection creation with network error",
-        test: function() {
-            failCommandWithFailPoint(["create"], {closeConnection: true});
-            assert.throws(() => coll1.insert({_id: 1}));
-        }
-    },
-    {
-        name: "implicit collection creation with WriteConcernError no success",
-        test: function() {
-            failCommandWithWCENoRun("create", ErrorCodes.NotMaster, "NotMaster");
-            assert.throws(() => coll1.insert({_id: 1}));
-        }
-    },
     {
         name: "errors cause the override to abort transactions",
         test: function() {
@@ -1374,6 +1322,69 @@ const txnOverrideTests = [
     },
 ];
 
+const txnOverrideTestsFcv42 = [
+    {
+        name: "implicit collection creation with stepdown",
+        test: function() {
+            // We set a failpoint on "create" since an implicit collection creation via
+            // findAndModify inside of a transaction will fail and this suite will attempt to
+            // explicitly create the collection outside of a transaction, and then retry the
+            // entire transaction.
+            failCommandWithFailPoint(["create"], {errorCode: ErrorCodes.NotMaster});
+            assert.throws(() => coll1.findAndModify(({update: {a: 1}, upsert: true})));
+        }
+    },
+    {
+        name: "implicit collection creation with WriteConcernError",
+        test: function() {
+            failCommandWithFailPoint(
+                ["create"],
+                {writeConcernError: {code: ErrorCodes.NotMaster, codeName: "NotMaster"}});
+            assert.throws(() => coll1.findAndModify(({update: {a: 1}, upsert: true})));
+        }
+    },
+    {
+        name: "implicit collection creation with WriteConcernError and normal stepdown error",
+        test: function() {
+            failCommandWithErrorAndWCENoRun(
+                "create", ErrorCodes.NotMaster, "NotMaster", ErrorCodes.NotMaster, "NotMaster");
+            assert.throws(() => coll1.findAndModify(({update: {a: 1}, upsert: true})));
+        }
+    },
+    {
+        name: "implicit collection creation with WriteConcernError and normal ordinary error",
+        test: function() {
+            failCommandWithErrorAndWCENoRun("create",
+                                            ErrorCodes.OperationFailed,
+                                            "OperationFailed",
+                                            ErrorCodes.NotMaster,
+                                            "NotMaster");
+            assert.throws(() => coll1.findAndModify(({update: {a: 1}, upsert: true})));
+        }
+    },
+    {
+        name: "implicit collection creation with ordinary error",
+        test: function() {
+            failCommandWithFailPoint(["create"], {errorCode: ErrorCodes.OperationFailed});
+            assert.throws(() => coll1.findAndModify(({update: {a: 1}, upsert: true})));
+        }
+    },
+    {
+        name: "implicit collection creation with network error",
+        test: function() {
+            failCommandWithFailPoint(["create"], {closeConnection: true});
+            assert.throws(() => coll1.findAndModify(({update: {a: 1}, upsert: true})));
+        }
+    },
+    {
+        name: "implicit collection creation with WriteConcernError no success",
+        test: function() {
+            failCommandWithWCENoRun("create", ErrorCodes.NotMaster, "NotMaster");
+            assert.throws(() => coll1.findAndModify(({update: {a: 1}, upsert: true})));
+        }
+    }
+];
+
 // Failpoints, overrides, and post-command functions are set by default to only run once, so
 // commands should succeed on retry.
 const txnOverridePlusRetryOnNetworkErrorTests = [
@@ -1450,102 +1461,6 @@ const txnOverridePlusRetryOnNetworkErrorTests = [
             assert.docEq(coll1.find().toArray(), [{_id: 1, x: 5}, {_id: 2, x: 8}]);
         }
     },
-    {
-        name: "implicit collection creation with stepdown",
-        test: function() {
-            assert.commandWorked(testDB.createCollection(collName1));
-            failCommandWithFailPoint(["create"], {errorCode: ErrorCodes.NotMaster});
-            assert.commandWorked(coll1.insert({_id: 1}));
-            assert.commandWorked(coll2.insert({_id: 1}));
-            assert.eq(coll1.find().itcount(), 1);
-            assert.eq(coll2.find().itcount(), 1);
-
-            endCurrentTransactionIfOpen();
-            assert.eq(coll1.find().itcount(), 1);
-            assert.eq(coll2.find().itcount(), 1);
-        }
-    },
-    {
-        name: "implicit collection creation with WriteConcernError",
-        test: function() {
-            assert.commandWorked(testDB.createCollection(collName1));
-            failCommandWithFailPoint(
-                ["create"],
-                {writeConcernError: {code: ErrorCodes.NotMaster, codeName: "NotMaster"}});
-            assert.commandWorked(coll1.insert({_id: 1}));
-            assert.commandWorked(coll2.insert({_id: 1}));
-            assert.eq(coll1.find().itcount(), 1);
-            assert.eq(coll2.find().itcount(), 1);
-
-            endCurrentTransactionIfOpen();
-            assert.eq(coll1.find().itcount(), 1);
-            assert.eq(coll2.find().itcount(), 1);
-        }
-    },
-    {
-        name: "implicit collection creation with WriteConcernError and normal stepdown error",
-        test: function() {
-            assert.commandWorked(testDB.createCollection(collName1));
-            failCommandWithErrorAndWCENoRun(
-                "create", ErrorCodes.NotMaster, "NotMaster", ErrorCodes.NotMaster, "NotMaster");
-            assert.commandWorked(coll1.insert({_id: 1}));
-            assert.commandWorked(coll2.insert({_id: 1}));
-            assert.eq(coll1.find().itcount(), 1);
-            assert.eq(coll2.find().itcount(), 1);
-
-            endCurrentTransactionIfOpen();
-            assert.eq(coll1.find().itcount(), 1);
-            assert.eq(coll2.find().itcount(), 1);
-        }
-    },
-    {
-        name: "implicit collection creation with WriteConcernError and normal ordinary error",
-        test: function() {
-            failCommandWithErrorAndWCENoRun("create",
-                                            ErrorCodes.OperationFailed,
-                                            "OperationFailed",
-                                            ErrorCodes.NotMaster,
-                                            "NotMaster");
-            assert.throws(() => coll1.insert({_id: 1}));
-        }
-    },
-    {
-        name: "implicit collection creation with ordinary error",
-        test: function() {
-            failCommandWithFailPoint(["create"], {errorCode: ErrorCodes.OperationFailed});
-            assert.throws(() => coll1.insert({_id: 1}));
-        }
-    },
-    {
-        name: "implicit collection creation with network error",
-        test: function() {
-            assert.commandWorked(testDB.createCollection(collName1));
-            failCommandWithFailPoint(["create"], {closeConnection: true});
-            assert.commandWorked(coll1.insert({_id: 1}));
-            assert.commandWorked(coll2.insert({_id: 1}));
-            assert.eq(coll1.find().itcount(), 1);
-            assert.eq(coll2.find().itcount(), 1);
-
-            endCurrentTransactionIfOpen();
-            assert.eq(coll1.find().itcount(), 1);
-            assert.eq(coll2.find().itcount(), 1);
-        }
-    },
-    {
-        name: "implicit collection creation with WriteConcernError no success",
-        test: function() {
-            assert.commandWorked(testDB.createCollection(collName1));
-            failCommandWithWCENoRun("create", ErrorCodes.NotMaster, "NotMaster");
-            assert.commandWorked(coll1.insert({_id: 1}));
-            assert.commandWorked(coll2.insert({_id: 1}));
-            assert.eq(coll1.find().itcount(), 1);
-            assert.eq(coll2.find().itcount(), 1);
-
-            endCurrentTransactionIfOpen();
-            assert.eq(coll1.find().itcount(), 1);
-            assert.eq(coll2.find().itcount(), 1);
-        }
-    },
     {
         name: "update with stepdown",
         test: function() {
@@ -1802,32 +1717,6 @@ const txnOverridePlusRetryOnNetworkErrorTests = [
             assert.eq(coll2.find().itcount(), 1);
         }
     },
-    {
-        name: "commitTransaction fails with SERVER-38856",
-        test: function() {
-            assert.commandWorked(testDB.createCollection(collName1));
-            failCommandWithFailPoint(
-                ["create"],
-                {writeConcernError: {code: ErrorCodes.NotMaster, codeName: "NotMaster"}});
-
-            // After commitTransaction fails, abort the transaction and drop the collection
-            // as if the transaction were being retried on a different node.
-            attachPostCmdFunction("commitTransaction", function() {
-                abortCurrentTransaction();
-                assert.commandWorked(mongoRunCommandOriginal.apply(testDB.getMongo(),
-                                                                   [dbName, {drop: collName2}, 0]));
-            });
-            failCommandWithWCENoRun("commitTransaction", ErrorCodes.NotMaster, "NotMaster");
-            assert.commandWorked(coll1.insert({_id: 1, x: 2}));
-            assert.commandWorked(coll2.insert({_id: 2}));
-            assert.commandWorked(coll1.update({_id: 1}, {$inc: {x: 4}}));
-
-            endCurrentTransactionIfOpen();
-
-            assert.docEq(coll1.find().toArray(), [{_id: 1, x: 6}]);
-            assert.docEq(coll2.find().toArray(), [{_id: 2}]);
-        }
-    },
     {
         name: 'Dates are copied correctly for SERVER-41917',
         test: function() {
@@ -1871,6 +1760,304 @@ const txnOverridePlusRetryOnNetworkErrorTests = [
     }
 ];
 
+const txnOverridePlusRetryOnNetworkErrorTestsFcv42 = [
+    {
+        name: "implicit collection creation with stepdown",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            failCommandWithFailPoint(["create"], {errorCode: ErrorCodes.NotMaster});
+            let resDoc1 = coll1.findAndModify(({update: {a: 1}, upsert: true, 'new': true}));
+            assert.eq(resDoc1.a, 1);
+            let resDoc2 = coll2.findAndModify(({update: {a: 1}, upsert: true, 'new': true}));
+            assert.eq(resDoc2.a, 1);
+            assert.eq(coll1.find().itcount(), 1);
+            assert.eq(coll2.find().itcount(), 1);
+
+            endCurrentTransactionIfOpen();
+            assert.eq(coll1.find().itcount(), 1);
+            assert.eq(coll2.find().itcount(), 1);
+        }
+    },
+    {
+        name: "implicit collection creation with WriteConcernError",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            failCommandWithFailPoint(
+                ["create"],
+                {writeConcernError: {code: ErrorCodes.NotMaster, codeName: "NotMaster"}});
+            let resDoc1 = coll1.findAndModify(({update: {a: 1}, upsert: true, 'new': true}));
+            assert.eq(resDoc1.a, 1);
+            let resDoc2 = coll2.findAndModify(({update: {a: 1}, upsert: true, 'new': true}));
+            assert.eq(resDoc2.a, 1);
+            assert.eq(coll1.find().itcount(), 1);
+            assert.eq(coll2.find().itcount(), 1);
+
+            endCurrentTransactionIfOpen();
+            assert.eq(coll1.find().itcount(), 1);
+            assert.eq(coll2.find().itcount(), 1);
+        }
+    },
+    {
+        name: "implicit collection creation with WriteConcernError and normal stepdown error",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            failCommandWithErrorAndWCENoRun(
+                "create", ErrorCodes.NotMaster, "NotMaster", ErrorCodes.NotMaster, "NotMaster");
+            let resDoc1 = coll1.findAndModify(({update: {a: 1}, upsert: true, 'new': true}));
+            assert.eq(resDoc1.a, 1);
+            let resDoc2 = coll2.findAndModify(({update: {a: 1}, upsert: true, 'new': true}));
+            assert.eq(resDoc2.a, 1);
+            assert.eq(coll1.find().itcount(), 1);
+            assert.eq(coll2.find().itcount(), 1);
+
+            endCurrentTransactionIfOpen();
+            assert.eq(coll1.find().itcount(), 1);
+            assert.eq(coll2.find().itcount(), 1);
+        }
+    },
+    {
+        name: "implicit collection creation with WriteConcernError and normal ordinary error",
+        test: function() {
+            failCommandWithErrorAndWCENoRun("create",
+                                            ErrorCodes.OperationFailed,
+                                            "OperationFailed",
+                                            ErrorCodes.NotMaster,
+                                            "NotMaster");
+            assert.throws(() => coll1.findAndModify(({update: {a: 1}, upsert: true})));
+        }
+    },
+    {
+        name: "implicit collection creation with ordinary error",
+        test: function() {
+            failCommandWithFailPoint(["create"], {errorCode: ErrorCodes.OperationFailed});
+            assert.throws(() => coll1.findAndModify(({update: {a: 1}, upsert: true})));
+        }
+    },
+    {
+        name: "implicit collection creation with network error",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            failCommandWithFailPoint(["create"], {closeConnection: true});
+            let resDoc1 = coll1.findAndModify(({update: {a: 1}, upsert: true, 'new': true}));
+            assert.eq(resDoc1.a, 1);
+            let resDoc2 = coll2.findAndModify(({update: {a: 1}, upsert: true, 'new': true}));
+            assert.eq(resDoc2.a, 1);
+            assert.eq(coll1.find().itcount(), 1);
+            assert.eq(coll2.find().itcount(), 1);
+
+            endCurrentTransactionIfOpen();
+            assert.eq(coll1.find().itcount(), 1);
+            assert.eq(coll2.find().itcount(), 1);
+        }
+    },
+    {
+        name: "implicit collection creation with WriteConcernError no success",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            failCommandWithWCENoRun("create", ErrorCodes.NotMaster, "NotMaster");
+            let resDoc1 = coll1.findAndModify(({update: {a: 1}, upsert: true, 'new': true}));
+            assert.eq(resDoc1.a, 1);
+            let resDoc2 = coll2.findAndModify(({update: {a: 1}, upsert: true, 'new': true}));
+            assert.eq(resDoc2.a, 1);
+            assert.eq(coll1.find().itcount(), 1);
+            assert.eq(coll2.find().itcount(), 1);
+
+            endCurrentTransactionIfOpen();
+            assert.eq(coll1.find().itcount(), 1);
+            assert.eq(coll2.find().itcount(), 1);
+        }
+    },
+    {
+        name: "commitTransaction fails with SERVER-38856",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            failCommandWithFailPoint(
+                ["create"],
+                {writeConcernError: {code: ErrorCodes.NotMaster, codeName: "NotMaster"}});
+
+            // After commitTransaction fails, abort the transaction and drop the collection
+            // as if the transaction were being retried on a different node.
+            attachPostCmdFunction("commitTransaction", function() {
+                abortCurrentTransaction();
+                assert.commandWorked(mongoRunCommandOriginal.apply(testDB.getMongo(),
+                                                                   [dbName, {drop: collName2}, 0]));
+            });
+            failCommandWithWCENoRun("commitTransaction", ErrorCodes.NotMaster, "NotMaster");
+            assert.commandWorked(coll1.insert({_id: 1, x: 2}));
+            let resDoc2 = coll2.findAndModify(({update: {_id: 1}, upsert: true, 'new': true}));
+            assert.eq(resDoc2._id, 1);
+            assert.commandWorked(coll1.update({_id: 1}, {$inc: {x: 4}}));
+
+            endCurrentTransactionIfOpen();
+
+            assert.docEq(coll1.find().toArray(), [{_id: 1, x: 6}]);
+            assert.docEq(coll2.find().toArray(), [resDoc2]);
+        }
+    }
+];
+
+const retryOnReadErrorsFromBackgroundReconfigTest = [
+    {
+        name: "find retries on ReadConcernMajorityNotAvailableYet",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            assert.commandWorked(coll1.insert({_id: 1}));
+            failCommandWithFailPoint(["find"],
+                                     {errorCode: ErrorCodes.ReadConcernMajorityNotAvailableYet});
+            assert.eq(coll1.findOne({_id: 1}), {_id: 1});
+        }
+    },
+    {
+        name: "aggregate retries on ReadConcernMajorityNotAvailableYet",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 2}));
+            failCommandWithFailPoint(["aggregate"],
+                                     {errorCode: ErrorCodes.ReadConcernMajorityNotAvailableYet});
+            const cursor = coll1.aggregate([{$match: {a: 1}}]);
+            assert.eq(cursor.toArray().length, 2);
+        }
+    },
+    {
+        name: "distinct retries on ReadConcernMajorityNotAvailableYet",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 2}));
+            failCommandWithFailPoint(["distinct"],
+                                     {errorCode: ErrorCodes.ReadConcernMajorityNotAvailableYet});
+            assert.eq(coll1.distinct("a").sort(), [1, 2]);
+        }
+    },
+    {
+        name: "count retries on ReadConcernMajorityNotAvailableYet",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 2}));
+            failCommandWithFailPoint(["count"],
+                                     {errorCode: ErrorCodes.ReadConcernMajorityNotAvailableYet});
+            assert.eq(coll1.count({a: 1}), 2);
+        }
+    },
+];
+
+const retryReadsOnNetworkErrorsWithNetworkRetryAndBackgroundReconfigTest = [
+    {
+        name: "find retries on network errors",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            assert.commandWorked(coll1.insert({_id: 1}));
+            failCommandWithFailPoint(["find"], {closeConnection: true});
+            assert.eq(coll1.findOne({_id: 1}), {_id: 1});
+        }
+    },
+    {
+        name: "aggregate retries on network errors",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 2}));
+            failCommandWithFailPoint(["aggregate"], {closeConnection: true});
+            const cursor = coll1.aggregate([{$match: {a: 1}}]);
+            assert.eq(cursor.toArray().length, 2);
+        }
+    },
+    {
+        name: "distinct retries on network errors",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 2}));
+            failCommandWithFailPoint(["distinct"], {closeConnection: true});
+            assert.eq(coll1.distinct("a").sort(), [1, 2]);
+        }
+    },
+    {
+        name: "count retries on network errors",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 2}));
+            failCommandWithFailPoint(["count"], {closeConnection: true});
+            assert.eq(coll1.count({a: 1}), 2);
+        }
+    },
+];
+
+const doNotRetryReadErrorWithOutBackgroundReconfigTest = [
+    {
+        name: "find fails on ReadConcernMajorityNotAvailableYet",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            assert.commandWorked(coll1.insert({_id: 1}));
+            failCommandWithFailPoint(["find"],
+                                     {errorCode: ErrorCodes.ReadConcernMajorityNotAvailableYet});
+            assert.commandFailedWithCode(
+                assert.throws(function() {
+                                 coll1.findOne({_id: 1});
+                             }),
+                             ErrorCodes.ReadConcernMajorityNotAvailableYet);
+        }
+    },
+    {
+        name: "aggregate fails on ReadConcernMajorityNotAvailableYet",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 2}));
+            failCommandWithFailPoint(["aggregate"],
+                                     {errorCode: ErrorCodes.ReadConcernMajorityNotAvailableYet});
+            assert.commandFailedWithCode(
+                assert.throws(function() {
+                                 const cursor = coll1.aggregate([{$match: {a: 1}}]);
+                                 assert.eq(cursor.toArray().length, 2);
+                             }),
+                             ErrorCodes.ReadConcernMajorityNotAvailableYet);
+        }
+    },
+    {
+        name: "distinct fails on ReadConcernMajorityNotAvailableYet",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 2}));
+            failCommandWithFailPoint(["distinct"],
+                                     {errorCode: ErrorCodes.ReadConcernMajorityNotAvailableYet});
+            assert.commandFailedWithCode(
+                assert.throws(function() {
+                                 coll1.distinct("a");
+                             }),
+                             ErrorCodes.ReadConcernMajorityNotAvailableYet);
+        }
+    },
+    {
+        name: "count fails on ReadConcernMajorityNotAvailableYet",
+        test: function() {
+            assert.commandWorked(testDB.createCollection(collName1));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 1}));
+            assert.commandWorked(coll1.insert({a: 2}));
+            failCommandWithFailPoint(["count"],
+                                     {errorCode: ErrorCodes.ReadConcernMajorityNotAvailableYet});
+            assert.commandFailedWithCode(
+                assert.throws(function() {
+                                 coll1.count({a: 1});
+                             }),
+                             ErrorCodes.ReadConcernMajorityNotAvailableYet);
+        }
+    },
+];
+
 TestData.networkErrorAndTxnOverrideConfig = {};
 TestData.sessionOptions = new SessionOptions();
 TestData.overrideRetryAttempts = 3;
@@ -1878,12 +2065,16 @@ TestData.overrideRetryAttempts = 3;
 let session = conn.startSession(TestData.sessionOptions);
 let testDB = session.getDatabase(dbName);
 
+const fcv = conn.getDB("admin").system.version.findOne({_id: 'featureCompatibilityVersion'});
+const usingFcv42 = fcv.version === "4.2" && !fcv.hasOwnProperty("targetVersion");
+
 load("jstests/libs/override_methods/network_error_and_txn_override.js");
 
 jsTestLog("=-=-=-=-=-= Testing with 'retry on network error' by itself. =-=-=-=-=-=");
 TestData.sessionOptions = new SessionOptions({retryWrites: true});
 TestData.networkErrorAndTxnOverrideConfig.retryOnNetworkErrors = true;
 TestData.networkErrorAndTxnOverrideConfig.wrapCRUDinTransactions = false;
+TestData.networkErrorAndTxnOverrideConfig.backgroundReconfigs = false;
 
 session = conn.startSession(TestData.sessionOptions);
 testDB = session.getDatabase(dbName);
@@ -1896,6 +2087,7 @@ jsTestLog("=-=-=-=-=-= Testing with 'txn override' by itself. =-=-=-=-=-=");
 TestData.sessionOptions = new SessionOptions({retryWrites: false});
 TestData.networkErrorAndTxnOverrideConfig.retryOnNetworkErrors = false;
 TestData.networkErrorAndTxnOverrideConfig.wrapCRUDinTransactions = true;
+TestData.networkErrorAndTxnOverrideConfig.backgroundReconfigs = false;
 
 session = conn.startSession(TestData.sessionOptions);
 testDB = session.getDatabase(dbName);
@@ -1903,11 +2095,15 @@ coll1 = testDB[collName1];
 coll2 = testDB[collName2];
 
 txnOverrideTests.forEach((testCase) => runTest("txnOverrideTests", testCase));
+if (usingFcv42) {
+    txnOverrideTestsFcv42.forEach((testCase) => runTest("txnOverrideTestsFcv42", testCase));
+}
 
 jsTestLog("=-=-=-=-=-= Testing 'both txn override and retry on network error'. =-=-=-=-=-=");
 TestData.sessionOptions = new SessionOptions({retryWrites: true});
 TestData.networkErrorAndTxnOverrideConfig.retryOnNetworkErrors = true;
 TestData.networkErrorAndTxnOverrideConfig.wrapCRUDinTransactions = true;
+TestData.networkErrorAndTxnOverrideConfig.backgroundReconfigs = false;
 
 session = conn.startSession(TestData.sessionOptions);
 testDB = session.getDatabase(dbName);
@@ -1916,6 +2112,55 @@ coll2 = testDB[collName2];
 
 txnOverridePlusRetryOnNetworkErrorTests.forEach(
     (testCase) => runTest("txnOverridePlusRetryOnNetworkErrorTests", testCase));
+if (usingFcv42) {
+    txnOverridePlusRetryOnNetworkErrorTestsFcv42.forEach(
+        (testCase) => runTest("txnOverridePlusRetryOnNetworkErrorTestsFcv42", testCase));
+}
+
+jsTestLog("=-=-=-=-=-= Testing 'retry on read errors from background reconfigs'. =-=-=-=-=-=");
+TestData.sessionOptions = new SessionOptions({retryWrites: false});
+TestData.networkErrorAndTxnOverrideConfig.retryOnNetworkErrors = false;
+TestData.networkErrorAndTxnOverrideConfig.backgroundReconfigs = true;
+TestData.networkErrorAndTxnOverrideConfig.wrapCRUDinTransactions = false;
+
+session = conn.startSession(TestData.sessionOptions);
+testDB = session.getDatabase(dbName);
+coll1 = testDB[collName1];
+coll2 = testDB[collName2];
+
+retryOnReadErrorsFromBackgroundReconfigTest.forEach(
+    (testCase) => runTest("retryOnReadErrorsFromBackgroundReconfigTest", testCase));
+
+jsTestLog(
+    "=-=-=-=-=-= Testing 'retry on network errors during network error retry and background reconfigs'. =-=-=-=-=-=");
+TestData.sessionOptions = new SessionOptions({retryWrites: true});
+TestData.networkErrorAndTxnOverrideConfig.retryOnNetworkErrors = true;
+TestData.networkErrorAndTxnOverrideConfig.backgroundReconfigs = true;
+TestData.networkErrorAndTxnOverrideConfig.wrapCRUDinTransactions = false;
+
+session = conn.startSession(TestData.sessionOptions);
+testDB = session.getDatabase(dbName);
+coll1 = testDB[collName1];
+coll2 = testDB[collName2];
+
+retryReadsOnNetworkErrorsWithNetworkRetryAndBackgroundReconfigTest.forEach(
+    (testCase) =>
+        runTest("retryReadsOnNetworkErrorsWithNetworkRetryAndBackgroundReconfigTest", testCase));
+
+jsTestLog(
+    "=-=-=-=-=-= Testing 'don't retry on network errors during background reconfigs'. =-=-=-=-=-=");
+TestData.sessionOptions = new SessionOptions({retryWrites: true});
+TestData.networkErrorAndTxnOverrideConfig.retryOnNetworkErrors = true;
+TestData.networkErrorAndTxnOverrideConfig.backgroundReconfigs = false;
+TestData.networkErrorAndTxnOverrideConfig.wrapCRUDinTransactions = false;
+
+session = conn.startSession(TestData.sessionOptions);
+testDB = session.getDatabase(dbName);
+coll1 = testDB[collName1];
+coll2 = testDB[collName2];
+
+doNotRetryReadErrorWithOutBackgroundReconfigTest.forEach(
+    (testCase) => runTest("doNotRetryReadErrorWithOutBackgroundReconfigTest", testCase));
 
 rst.stopSet();
 })();
