@@ -1,8 +1,9 @@
 /**
- * Tests that an updateLookup change stream doesn't throw ChangeStreamFatalError after
- * fixing SERVER-44598
+ * Tests that an updateLookup change stream doesn't throw ChangeStreamFatalError or
+ * TooManyMatchingDocuments after fixing SERVER-44598.
  *
- * @tags: [uses_change_streams]
+ * Remove requires_fcv_44 tag if SERVER-44598 is backported or 4.4 becomes last stable
+ * @tags: [uses_change_streams, requires_fcv_44]
  */
 (function() {
 "use strict";
@@ -32,7 +33,7 @@ st.ensurePrimaryShard(mongosDB.getName(), shard0.getURL());
 st.shardColl(mongosColl, {a: 1}, {a: 0}, {a: 1});
 
 // Open a change stream on the collection.
-const csCursor = mongosColl.watch();
+let csCursor = mongosColl.watch();
 
 // Write one document onto shard0 and obtain its resume token.
 assert.commandWorked(mongosColl.insert({_id: 0, a: -100}));
@@ -58,17 +59,19 @@ mongosColl = st.s.getDB(mongosDB.getName())[mongosColl.getName()];
 assert.soonNoExcept(
     () => assert.commandWorked(mongosColl.update({_id: 0}, {$set: {updated: true}}, false, true)));
 
-// Resume the change stream with {fullDocument: 'updateLookup'}.
-const cmdRes = assert.commandWorked(mongosColl.runCommand("aggregate", {
-    pipeline: [{$changeStream: {resumeAfter: resumeToken, fullDocument: "updateLookup"}}],
-    cursor: {}
-}));
-
+// Resume the change stream with {fullDocument: 'updateLookup'}. Update lookup can successfully
+// identify the document based on its _id alone so long as the _id is unique in the collection, so
+// this alone does not prove that the multi-update actually wrote its shard key into the oplog.
+csCursor = mongosColl.watch([], {resumeAfter: resumeToken, fullDocument: "updateLookup"});
 assert.soon(() => csCursor.hasNext());
+assert.docEq(csCursor.next().fullDocument, {_id: 0, a: -100, updated: true});
 
-const updateObj = csCursor.next();
-
-assert.eq(true, updateObj.updateDescription.updatedFields.updated);
+// Now insert a new document with the same _id on the other shard. Update lookup will be able to
+// distinguish between the two, proving that they both have full shard keys available.
+assert.commandWorked(mongosColl.insert({_id: 0, a: 100}));
+csCursor = mongosColl.watch([], {resumeAfter: resumeToken, fullDocument: "updateLookup"});
+assert.soon(() => csCursor.hasNext());
+assert.docEq(csCursor.next().fullDocument, {_id: 0, a: -100, updated: true});
 
 st.stop();
 })();
\ No newline at end of file
