@@ -15,6 +15,15 @@
  *                 *when the range has been deleted on the donor.*
  * - behavior: Must be one of "unshardedOnly", "targetsPrimaryUsesConnectionVersioning" or
  * "versioned". Determines what system profiler checks are performed.
+ *
+ * Tagged as 'requires_fcv_44', since this test cannot run against versions less then 4.4. This is
+ * because 'planCacheListPlans' and 'planCacheListQueryShapes' were deleted in 4.4, and thus not
+ * tested here. But this test asserts that all commands are covered, so will fail against a version
+ * of the server which implements these commands.
+ *
+ * @tags: [
+ *   requires_fcv_44,
+ * ]
  */
 (function() {
 "use strict";
@@ -38,9 +47,10 @@ let validateTestCase = function(test) {
 
 let testCases = {
     _addShard: {skip: "primary only"},
-    _cloneCatalogData: {skip: "primary only"},
+    _shardsvrCloneCatalogData: {skip: "primary only"},
     _configsvrAddShard: {skip: "primary only"},
     _configsvrAddShardToZone: {skip: "primary only"},
+    _configsvrBalancerCollectionStatus: {skip: "primary only"},
     _configsvrBalancerStart: {skip: "primary only"},
     _configsvrBalancerStatus: {skip: "primary only"},
     _configsvrBalancerStop: {skip: "primary only"},
@@ -60,9 +70,10 @@ let testCases = {
     _getUserCacheGeneration: {skip: "does not return user data"},
     _hashBSONElement: {skip: "does not return user data"},
     _isSelf: {skip: "does not return user data"},
+    _killOperations: {skip: "does not return user data"},
     _mergeAuthzCollections: {skip: "primary only"},
     _migrateClone: {skip: "primary only"},
-    _movePrimary: {skip: "primary only"},
+    _shardsvrMovePrimary: {skip: "primary only"},
     _recvChunkAbort: {skip: "primary only"},
     _recvChunkCommit: {skip: "primary only"},
     _recvChunkStart: {skip: "primary only"},
@@ -73,7 +84,7 @@ let testCases = {
     addShardToZone: {skip: "primary only"},
     aggregate: {
         setUp: function(mongosConn) {
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
         },
         command: {aggregate: coll, pipeline: [{$match: {x: 1}}], cursor: {batchSize: 10}},
         checkResults: function(res) {
@@ -88,6 +99,7 @@ let testCases = {
     authenticate: {skip: "does not return user data"},
     authSchemaUpgrade: {skip: "primary only"},
     availableQueryOptions: {skip: "does not return user data"},
+    balancerCollectionStatus: {skip: "primary only"},
     balancerStart: {skip: "primary only"},
     balancerStatus: {skip: "primary only"},
     balancerStop: {skip: "primary only"},
@@ -98,7 +110,6 @@ let testCases = {
     clearJumboFlag: {skip: "primary only"},
     clearLog: {skip: "does not return user data"},
     clone: {skip: "primary only"},
-    cloneCollection: {skip: "primary only"},
     cloneCollectionAsCapped: {skip: "primary only"},
     collMod: {skip: "primary only"},
     collStats: {skip: "does not return user data"},
@@ -111,7 +122,7 @@ let testCases = {
     convertToCapped: {skip: "primary only"},
     count: {
         setUp: function(mongosConn) {
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
         },
         command: {count: coll, query: {x: 1}},
         checkResults: function(res) {
@@ -133,8 +144,8 @@ let testCases = {
     delete: {skip: "primary only"},
     distinct: {
         setUp: function(mongosConn) {
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
         },
         command: {distinct: coll, key: "x"},
         checkResults: function(res) {
@@ -161,7 +172,7 @@ let testCases = {
     filemd5: {skip: "does not return user data"},
     find: {
         setUp: function(mongosConn) {
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
         },
         command: {find: coll, filter: {x: 1}},
         checkResults: function(res) {
@@ -178,6 +189,7 @@ let testCases = {
     fsyncUnlock: {skip: "does not return user data"},
     geoSearch: {skip: "not supported in mongos"},
     getCmdLineOpts: {skip: "does not return user data"},
+    getDefaultRWConcern: {skip: "does not return user data"},
     getDiagnosticData: {skip: "does not return user data"},
     getLastError: {skip: "primary only"},
     getLog: {skip: "does not return user data"},
@@ -213,8 +225,8 @@ let testCases = {
     makeSnapshot: {skip: "does not return user data"},
     mapReduce: {
         setUp: function(mongosConn) {
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
         },
         command: {
             mapReduce: coll,
@@ -232,7 +244,7 @@ let testCases = {
             assert.eq(1, res.results[0]._id, tojson(res));
             assert.eq(2, res.results[0].value, tojson(res));
         },
-        behavior: "targetsPrimaryUsesConnectionVersioning"
+        behavior: "versioned"
     },
     mergeChunks: {skip: "primary only"},
     moveChunk: {skip: "primary only"},
@@ -243,18 +255,16 @@ let testCases = {
     planCacheClear: {skip: "does not return user data"},
     planCacheClearFilters: {skip: "does not return user data"},
     planCacheListFilters: {skip: "does not return user data"},
-    planCacheListPlans: {skip: "does not return user data"},
-    planCacheListQueryShapes: {skip: "does not return user data"},
     planCacheSetFilter: {skip: "does not return user data"},
     profile: {skip: "primary only"},
     reapLogicalSessionCacheNow: {skip: "does not return user data"},
+    refineCollectionShardKey: {skip: "primary only"},
     refreshLogicalSessionCacheNow: {skip: "does not return user data"},
     refreshSessions: {skip: "does not return user data"},
     refreshSessionsInternal: {skip: "does not return user data"},
     removeShard: {skip: "primary only"},
     removeShardFromZone: {skip: "primary only"},
     renameCollection: {skip: "primary only"},
-    repairCursor: {skip: "does not return user data"},
     replSetAbortPrimaryCatchUp: {skip: "does not return user data"},
     replSetFreeze: {skip: "does not return user data"},
     replSetGetConfig: {skip: "does not return user data"},
@@ -272,7 +282,6 @@ let testCases = {
     replSetUpdatePosition: {skip: "does not return user data"},
     replSetResizeOplog: {skip: "does not return user data"},
     resetError: {skip: "does not return user data"},
-    restartCatalog: {skip: "internal-only command"},
     resync: {skip: "primary only"},
     revokePrivilegesFromRole: {skip: "primary only"},
     revokeRolesFromRole: {skip: "primary only"},
@@ -282,6 +291,8 @@ let testCases = {
     saslStart: {skip: "primary only"},
     serverStatus: {skip: "does not return user data"},
     setCommittedSnapshot: {skip: "does not return user data"},
+    setDefaultRWConcern: {skip: "primary only"},
+    setIndexCommitQuorum: {skip: "primary only"},
     setFeatureCompatibilityVersion: {skip: "primary only"},
     setFreeMonitoring: {skip: "primary only"},
     setParameter: {skip: "does not return user data"},
@@ -299,7 +310,6 @@ let testCases = {
     startSession: {skip: "does not return user data"},
     stopRecordingTraffic: {skip: "does not return user data"},
     top: {skip: "does not return user data"},
-    touch: {skip: "does not return user data"},
     unsetSharding: {skip: "does not return user data"},
     update: {skip: "primary only"},
     updateRole: {skip: "primary only"},
@@ -312,12 +322,12 @@ let testCases = {
     whatsmyuri: {skip: "does not return user data"}
 };
 
-commandsRemovedFromMongosIn42.forEach(function(cmd) {
-    testCases[cmd] = {skip: "must define test coverage for 4.0 backwards compatibility"};
+commandsRemovedFromMongosIn44.forEach(function(cmd) {
+    testCases[cmd] = {skip: "must define test coverage for 4.2 backwards compatibility"};
 });
 
-// Set the secondaries to priority 0 and votes 0 to prevent the primaries from stepping down.
-let rsOpts = {nodes: [{rsConfig: {votes: 1}}, {rsConfig: {priority: 0, votes: 0}}]};
+// Set the secondaries to priority 0 to prevent the primaries from stepping down.
+let rsOpts = {nodes: [{}, {rsConfig: {priority: 0}}]};
 let st = new ShardingTest({mongos: 2, shards: {rs0: rsOpts, rs1: rsOpts}});
 
 let recipientShardPrimary = st.rs1.getPrimary();
