@@ -99,6 +99,10 @@ public:
     virtual TestCaseResult execute() = 0;
 
     virtual const std::string& FilePath() const = 0;
+
+    virtual bool errorIsExpected() {
+        return false;
+    }
 };
 
 /**
@@ -207,21 +211,46 @@ private:
 class JsonServerSelectionTestCase : public JsonTestCase {
 public:
     JsonServerSelectionTestCase(fs::path testFilePath) {
-        parseTest(testFilePath);
+        try {
+            parseTest(testFilePath);
+        } catch (const DBException& ex) {
+            _parseError = TestCaseResult{{std::make_pair("error while parsing", ex.toString())},
+                                         testFilePath.string()};
+        }
     }
     ~JsonServerSelectionTestCase() = default;
 
     TestCaseResult execute() {
         LOGV2(4333504, "### Running Test ###", "testFilePath"_attr = _testFilePath);
-
-        SdamServerSelector serverSelector(
-            SdamConfiguration(std::vector<HostAndPort>{HostAndPort("foo:1234")}));
-        auto selectedServers = serverSelector.selectServers(_topologyDescription, _readPreference);
+        if (_parseError)
+            return *_parseError;
 
         TestCaseResult result{{}, _testFilePath};
-        validateServersInLatencyWindow(&result, selectedServers);
 
-        if (!result.Success()) {
+        try {
+            SdamServerSelector serverSelector(
+                SdamConfiguration(std::vector<HostAndPort>{HostAndPort("foo:1234")}));
+            auto selectedServers =
+                serverSelector.selectServers(_topologyDescription, _readPreference);
+
+            std::vector<std::string> selectedHosts;
+            if (selectedServers) {
+                std::transform((*selectedServers).begin(),
+                               (*selectedServers).end(),
+                               std::back_inserter(selectedHosts),
+                               [](const ServerDescriptionPtr& server) {
+                                   return server->getAddress().toString();
+                               });
+            }
+            LOGV2(5017006, "Servers selected", "servers"_attr = selectedHosts);
+
+            validateServersInLatencyWindow(&result, selectedServers);
+        } catch (const DBException& ex) {
+            auto errorDescription = std::make_pair("exception", ex.toString());
+            result.errorDescriptions.push_back(errorDescription);
+        }
+
+        if (!result.Success() && !_errorExpected) {
             LOGV2(4333505, "Test failed", "testFilePath"_attr = _testFilePath);
         }
 
@@ -232,6 +261,10 @@ public:
         return _testFilePath;
     }
 
+    bool errorIsExpected() {
+        return _errorExpected;
+    }
+
 private:
     void parseTest(fs::path testFilePath) {
         _testFilePath = testFilePath.string();
@@ -243,33 +276,90 @@ private:
             _jsonTest = fromjson(json.str());
         }
 
+        // Do this first to save the state
+        if (_jsonTest.hasField("error")) {
+            LOGV2(5017004, "Expecting test case to generate an error.");
+            _errorExpected = _jsonTest.getBoolField("error");
+        }
+
         // The json tests pass in capitalized keywords for mode, but the server only accepts
         // lowercased keywords. Also, change the key "tags_set" to "tags".
+        // This can throw for test cases that have invalid read preferences.
         auto readPrefObj = _jsonTest.getObjectField("read_preference");
         std::string mode = readPrefObj.getStringField("mode");
         mode[0] = std::tolower(mode[0]);
         auto tagSetsObj = readPrefObj["tag_sets"];
         auto tags = tagSetsObj ? BSONArray(readPrefObj["tag_sets"].Obj()) : BSONArray();
 
-        BSONObj readPref = BSON("mode" << mode << "tags" << tags);
-        _readPreference = uassertStatusOK(ReadPreferenceSetting::fromInnerBSON(readPref));
 
-        // Parse the TopologyDescription and inLatencyWindow objects
         auto topologyDescriptionObj = _jsonTest.getObjectField("topology_description");
+        TopologyType initType =
+            uassertStatusOK(parseTopologyType(topologyDescriptionObj.getStringField("type")));
+        boost::optional<std::string> setName = boost::none;
+        if (initType == TopologyType::kReplicaSetNoPrimary || initType == TopologyType::kSingle)
+            setName = "replset";
+
+        int maxStalenessSeconds = 0;
+        if (readPrefObj.hasField("maxStalenessSeconds")) {
+            maxStalenessSeconds = readPrefObj.getIntField("maxStalenessSeconds");
+        }
+
+        BSONObj readPref =
+            BSON("mode" << mode << "tags" << tags << "maxStalenessSeconds" << maxStalenessSeconds);
+        auto stalenessParseResult = ReadPreferenceSetting::fromInnerBSON(readPref);
+        if (!_errorExpected &&
+            stalenessParseResult.getStatus().code() == ErrorCodes::MaxStalenessOutOfRange &&
+            (initType == TopologyType::kUnknown || initType == TopologyType::kSingle ||
+             initType == TopologyType::kSharded)) {
+            // Drivers tests expect no error for invalid maxStaleness values for Unknown, Single,
+            // and Sharded topology types. The server code doesn't allow read preferences to be
+            // created for invalid values for maxStaleness, so ignore it.
+            readPref = BSON("mode" << mode << "tags" << tags << "maxStalenessSeconds" << 0);
+            stalenessParseResult = ReadPreferenceSetting::fromInnerBSON(readPref);
+        } else if (_errorExpected && maxStalenessSeconds == 0 &&
+                   readPrefObj.hasField("maxStalenessSeconds")) {
+            // Generate an error to pass test cases that set max staleness to zero since the server
+            // interprets this as no maxStaleness is specified.
+            uassert(ErrorCodes::MaxStalenessOutOfRange, "max staleness should be >= 0", false);
+        }
+        _readPreference = uassertStatusOK(stalenessParseResult);
+        LOGV2(5017007, "Read Preference", "value"_attr = _readPreference);
+
+        if (_jsonTest.hasField("heartbeatFrequencyMS")) {
+            sdamHeartBeatFrequencyMs = _jsonTest.getIntField("heartbeatFrequencyMS");
+            LOGV2(5017003, "Set heartbeatFrequencyMs", "value"_attr = sdamHeartBeatFrequencyMs);
+        }
 
         std::vector<ServerDescriptionPtr> serverDescriptions;
         std::vector<HostAndPort> serverAddresses;
         const std::vector<BSONElement>& bsonServers = topologyDescriptionObj["servers"].Array();
+
         for (auto bsonServer : bsonServers) {
             auto server = bsonServer.Obj();
 
+            int maxWireVersion = 9;
+            if (server.hasField("maxWireVersion")) {
+                maxWireVersion = server["maxWireVersion"].numberInt();
+            }
+
             auto serverType = uassertStatusOK(parseServerType(server.getStringField("type")));
             auto serverDescription = ServerDescriptionBuilder()
                                          .withAddress(HostAndPort(server.getStringField("address")))
                                          .withType(serverType)
                                          .withRtt(Milliseconds(server["avg_rtt_ms"].numberInt()))
-                                         .withMinWireVersion(8)
-                                         .withMaxWireVersion(9);
+                                         .withMinWireVersion(5)
+                                         .withMaxWireVersion(maxWireVersion);
+
+            if (server.hasField("lastWrite")) {
+                auto lastWriteDate =
+                    server.getObjectField("lastWrite").getIntField("lastWriteDate");
+                serverDescription.withLastWriteDate(Date_t::fromMillisSinceEpoch(lastWriteDate));
+            }
+
+            if (server.hasField("lastUpdateTime")) {
+                auto lastUpdateTime = server.getIntField("lastUpdateTime");
+                serverDescription.withLastUpdateTime(Date_t::fromMillisSinceEpoch(lastUpdateTime));
+            }
 
             auto tagsObj = server.getObjectField("tags");
             const auto keys = tagsObj.getFieldNames<std::set<std::string>>();
@@ -278,14 +368,12 @@ private:
             }
 
             serverDescriptions.push_back(serverDescription.instance());
+            LOGV2(5017002,
+                  "Server Description",
+                  "description"_attr = serverDescriptions.back()->toBson());
             serverAddresses.push_back(HostAndPort(server.getStringField("address")));
         }
 
-        TopologyType initType =
-            uassertStatusOK(parseTopologyType(topologyDescriptionObj.getStringField("type")));
-        boost::optional<std::string> setName = boost::none;
-        if (initType == TopologyType::kReplicaSetNoPrimary || initType == TopologyType::kSingle)
-            setName = "replset";
 
         boost::optional<std::vector<HostAndPort>> seedList = boost::none;
         if (serverAddresses.size() > 0)
@@ -299,15 +387,18 @@ private:
                                         setName);
         _topologyDescription = std::make_shared<TopologyDescription>(config);
 
-        const std::vector<BSONElement>& bsonLatencyWindow = _jsonTest["in_latency_window"].Array();
-        for (const auto& serverDescription : serverDescriptions) {
-            _topologyDescription->installServerDescription(serverDescription);
+        if (_jsonTest.hasField("in_latency_window")) {
+            const std::vector<BSONElement>& bsonLatencyWindow =
+                _jsonTest["in_latency_window"].Array();
+            for (const auto& serverDescription : serverDescriptions) {
+                _topologyDescription->installServerDescription(serverDescription);
 
-            for (auto bsonServer : bsonLatencyWindow) {
-                auto server = bsonServer.Obj();
-                if (serverDescription->getAddress() ==
-                    HostAndPort(server.getStringField("address"))) {
-                    _inLatencyWindow.push_back(serverDescription);
+                for (auto bsonServer : bsonLatencyWindow) {
+                    auto server = bsonServer.Obj();
+                    if (serverDescription->getAddress() ==
+                        HostAndPort(server.getStringField("address"))) {
+                        _inLatencyWindow.push_back(serverDescription);
+                    }
                 }
             }
         }
@@ -316,50 +407,38 @@ private:
     void validateServersInLatencyWindow(
         TestCaseResult* result,
         boost::optional<std::vector<ServerDescriptionPtr>> selectedServers) {
-        if (!selectedServers && _inLatencyWindow.size() > 0) {
+        // Compare the server addresses of each server in the selectedServers and
+        // _inLatencyWindow vectors. We do not need to compare the entire server description
+        // because we only need to make sure that the correct server was chosen and are not
+        // manipulating the ServerDescriptions at all.
+        std::vector<HostAndPort> selectedHostAndPorts;
+        std::vector<HostAndPort> expectedHostAndPorts;
+
+        auto extractHost = [](const ServerDescriptionPtr& s) { return s->getAddress(); };
+        if (selectedServers) {
+            std::transform(selectedServers->begin(),
+                           selectedServers->end(),
+                           std::back_inserter(selectedHostAndPorts),
+                           extractHost);
+        }
+        std::transform(_inLatencyWindow.begin(),
+                       _inLatencyWindow.end(),
+                       std::back_inserter(expectedHostAndPorts),
+                       extractHost);
+
+        std::sort(selectedHostAndPorts.begin(), selectedHostAndPorts.end());
+        std::sort(expectedHostAndPorts.begin(), expectedHostAndPorts.end());
+        if (!std::equal(selectedHostAndPorts.begin(),
+                        selectedHostAndPorts.end(),
+                        expectedHostAndPorts.begin(),
+                        expectedHostAndPorts.end())) {
             std::stringstream errorMessage;
-            errorMessage << "did not select any servers, but expected '" << _inLatencyWindow.size()
+            errorMessage << "selected servers with addresses '" << selectedHostAndPorts
+                         << "' server(s), but expected '" << expectedHostAndPorts
                          << "' to be selected.";
             auto errorDescription = std::make_pair("servers in latency window", errorMessage.str());
             result->errorDescriptions.push_back(errorDescription);
-        } else if (selectedServers && selectedServers->size() != _inLatencyWindow.size()) {
-            std::stringstream errorMessage;
-            errorMessage << "selected '" << selectedServers->size() << "' server(s), but expected '"
-                         << _inLatencyWindow.size() << "' to be selected.";
-            auto errorDescription = std::make_pair("servers in latency window", errorMessage.str());
-            result->errorDescriptions.push_back(errorDescription);
-        } else {
-            // Compare the server addresses of each server in the selectedServers and
-            // _inLatencyWindow vectors. We do not need to compare the entire server description
-            // because we only need to make sure that the correct server was chosen and are not
-            // manipulating the ServerDescriptions at all.
-            std::vector<HostAndPort> selectedHostAndPortes;
-            std::vector<HostAndPort> expectedHostAndPortes;
-
-            auto selectedServersIt = selectedServers->begin();
-            for (auto expectedServersIt = _inLatencyWindow.begin();
-                 expectedServersIt != _inLatencyWindow.end();
-                 ++expectedServersIt) {
-                selectedHostAndPortes.push_back((*selectedServersIt)->getAddress());
-                expectedHostAndPortes.push_back((*expectedServersIt)->getAddress());
-
-                selectedServersIt++;
-            }
-
-            std::sort(selectedHostAndPortes.begin(), selectedHostAndPortes.end());
-            std::sort(expectedHostAndPortes.begin(), expectedHostAndPortes.end());
-            if (!std::equal(selectedHostAndPortes.begin(),
-                            selectedHostAndPortes.end(),
-                            expectedHostAndPortes.begin())) {
-                std::stringstream errorMessage;
-                errorMessage << "selected servers with addresses '" << selectedHostAndPortes
-                             << "' server(s), but expected '" << expectedHostAndPortes
-                             << "' to be selected.";
-                auto errorDescription =
-                    std::make_pair("servers in latency window", errorMessage.str());
-                result->errorDescriptions.push_back(errorDescription);
-                return;
-            }
+            return;
         }
     }
 
@@ -368,6 +447,8 @@ private:
     TopologyDescriptionPtr _topologyDescription;
     ReadPreferenceSetting _readPreference;
     std::vector<ServerDescriptionPtr> _inLatencyWindow;
+    boost::optional<TestCaseResult> _parseError;
+    bool _errorExpected = false;
 };
 
 /**
@@ -382,29 +463,60 @@ public:
         std::vector<JsonTestCase::TestCaseResult> results;
         const auto testFiles = getTestFiles();
         for (auto jsonTest : testFiles) {
-            auto testCase = [jsonTest]() -> std::unique_ptr<JsonTestCase> {
-                if (jsonTest.string().find("/rtt/")) {
-                    return std::make_unique<JsonRttTestCase>(jsonTest);
-                }
-                return std::make_unique<JsonServerSelectionTestCase>(jsonTest);
-            }();
+            int restoreHeartBeatFrequencyMs = sdamHeartBeatFrequencyMs;
 
+            std::unique_ptr<JsonTestCase> testCase;
             try {
+                testCase = [jsonTest]() -> std::unique_ptr<JsonTestCase> {
+                    if (jsonTest.string().find("/rtt/") != std::string::npos ||
+                        jsonTest.string().find("\\rtt\\") != std::string::npos) {
+                        return std::make_unique<JsonRttTestCase>(jsonTest);
+                    }
+                    return std::make_unique<JsonServerSelectionTestCase>(jsonTest);
+                }();
                 LOGV2(
                     4333508, "### Executing Test ###", "testFilePath"_attr = testCase->FilePath());
-                results.push_back(testCase->execute());
+                auto executionResult = testCase->execute();
+                if (testCase->errorIsExpected() && executionResult.Success()) {
+                    auto errorDescription =
+                        std::make_pair("failure expected", "Expected test to fail, but it didn't");
+                    executionResult.errorDescriptions.push_back(errorDescription);
+                } else if (testCase->errorIsExpected() && !executionResult.Success()) {
+                    // clear the errors, so that it's treated as a success
+                    executionResult.errorDescriptions.clear();
+                }
+                results.push_back(executionResult);
             } catch (const DBException& ex) {
-                std::stringstream error;
-                error << "Exception while executing " << jsonTest.string() << ": " << ex.toString();
-                std::string errorStr = error.str();
-                results.push_back(JsonTestCase::TestCaseResult{
-                    {std::make_pair("exception", errorStr)}, jsonTest.string()});
-                std::cerr << errorStr;
+                if (!testCase || !testCase->errorIsExpected()) {
+                    std::stringstream error;
+                    error << "Exception while executing " << jsonTest.string() << ": "
+                          << ex.toString();
+                    std::string errorStr = error.str();
+                    results.push_back(JsonTestCase::TestCaseResult{
+                        {std::make_pair("exception", errorStr)}, jsonTest.string()});
+                    std::cerr << errorStr;
+                }
             }
+
+            // use default value of sdamHeartBeatFrequencyMs unless the test explicitly sets it.
+            sdamHeartBeatFrequencyMs = restoreHeartBeatFrequencyMs;
         }
         return results;
     }
 
+    std::string collectErrorStr(const std::vector<JsonTestCase::TestErrors>& errors) {
+        std::string result;
+        for (size_t i = 0; i < errors.size(); ++i) {
+            auto error = errors[i];
+            result = result + error.first + " - " + error.second;
+            if (i != errors.size() - 1) {
+                result += "; ";
+            }
+        }
+        return result;
+    }
+
+
     int report(std::vector<JsonTestCase::TestCaseResult> results) {
         int numTestCases = results.size();
         int numSuccess = 0;
@@ -417,12 +529,15 @@ public:
             LOGV2(4333509, "### Failed Test Results ###");
         }
 
-        for (const auto result : results) {
+        for (const auto& result : results) {
             auto file = result.file;
             if (result.Success()) {
                 ++numSuccess;
             } else {
-                LOGV2(4333510, "### Failed Test File ###", "testFilePath"_attr = file);
+                LOGV2(4333510,
+                      "### Failed Test File ###",
+                      "testFilePath"_attr = file,
+                      "errors"_attr = collectErrorStr(result.errorDescriptions));
                 ++numFailed;
             }
         }
