@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kAccessControl
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
 
 #include "mongo/platform/basic.h"
 
@@ -36,6 +36,7 @@
 #include <string>
 #include <vector>
 
+#include "mongo/base/shim.h"
 #include "mongo/base/status.h"
 #include "mongo/db/auth/action_set.h"
 #include "mongo/db/auth/action_type.h"
@@ -52,8 +53,8 @@
 #include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/aggregation_request.h"
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/log.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
@@ -61,14 +62,18 @@ namespace mongo {
 namespace dps = ::mongo::dotted_path_support;
 using std::vector;
 
-MONGO_REGISTER_SHIM(AuthorizationSession::create)
-(AuthorizationManager* authzManager)->std::unique_ptr<AuthorizationSession> {
+namespace {
+
+std::unique_ptr<AuthorizationSession> authorizationSessionCreateImpl(
+    AuthorizationManager* authzManager) {
     return std::make_unique<AuthorizationSessionImpl>(
         AuthzSessionExternalState::create(authzManager),
         AuthorizationSessionImpl::InstallMockForTestingOrAuthImpl{});
 }
 
-namespace {
+auto authorizationSessionCreateRegistration =
+    MONGO_WEAK_FUNCTION_REGISTRATION(AuthorizationSession::create, authorizationSessionCreateImpl);
+
 constexpr StringData ADMIN_DBNAME = "admin"_sd;
 
 // Checks if this connection has the privileges necessary to create or modify the view 'viewNs'
@@ -134,8 +139,10 @@ Status AuthorizationSessionImpl::addAndAuthorizeUser(OperationContext* opCtx,
     Status restrictionStatus =
         restrictionSet.validate(RestrictionEnvironment::get(*opCtx->getClient()));
     if (!restrictionStatus.isOK()) {
-        log() << "Failed to acquire user '" << userName
-              << "' because of unmet authentication restrictions: " << restrictionStatus.reason();
+        LOGV2(20240,
+              "Failed to acquire user because of unmet authentication restrictions",
+              "user"_attr = userName,
+              "reason"_attr = restrictionStatus.reason());
         return AuthorizationManager::authenticationFailedStatus;
     }
 
@@ -150,7 +157,8 @@ Status AuthorizationSessionImpl::addAndAuthorizeUser(OperationContext* opCtx,
 }
 
 User* AuthorizationSessionImpl::lookupUser(const UserName& name) {
-    return _authenticatedUsers.lookup(name).get();
+    auto user = _authenticatedUsers.lookup(name);
+    return user ? user.get() : nullptr;
 }
 
 User* AuthorizationSessionImpl::getSingleUser() {
@@ -184,17 +192,6 @@ RoleNameIterator AuthorizationSessionImpl::getAuthenticatedRoleNames() {
     return makeRoleNameIterator(_authenticatedRoleNames.begin(), _authenticatedRoleNames.end());
 }
 
-std::string AuthorizationSessionImpl::getAuthenticatedUserNamesToken() {
-    std::string ret;
-    for (UserNameIterator nameIter = getAuthenticatedUserNames(); nameIter.more();
-         nameIter.next()) {
-        ret += '\0';  // Using a NUL byte which isn't valid in usernames to separate them.
-        ret += nameIter->getFullName();
-    }
-
-    return ret;
-}
-
 void AuthorizationSessionImpl::grantInternalAuthorization(Client* client) {
     stdx::lock_guard<Client> lk(*client);
     _authenticatedUsers.add(internalSecurity.user);
@@ -285,7 +282,7 @@ StatusWith<PrivilegeVector> AuthorizationSessionImpl::getPrivilegesForAggregate(
 
     // If the first stage of the pipeline is not an initial source, the pipeline is implicitly
     // reading documents from the underlying collection. The client must be authorized to do so.
-    auto liteParsedDocSource = LiteParsedDocumentSource::parse(aggRequest, pipeline[0]);
+    auto liteParsedDocSource = LiteParsedDocumentSource::parse(nss, pipeline[0]);
     if (!liteParsedDocSource->isInitialSource()) {
         Privilege currentPriv =
             Privilege(ResourcePattern::forExactNamespace(nss), ActionType::find);
@@ -294,8 +291,9 @@ StatusWith<PrivilegeVector> AuthorizationSessionImpl::getPrivilegesForAggregate(
 
     // Confirm privileges for the pipeline.
     for (auto&& pipelineStage : pipeline) {
-        liteParsedDocSource = LiteParsedDocumentSource::parse(aggRequest, pipelineStage);
-        PrivilegeVector currentPrivs = liteParsedDocSource->requiredPrivileges(isMongos);
+        liteParsedDocSource = LiteParsedDocumentSource::parse(nss, pipelineStage);
+        PrivilegeVector currentPrivs = liteParsedDocSource->requiredPrivileges(
+            isMongos, aggRequest.shouldBypassDocumentValidation());
         Privilege::addPrivilegesToPrivilegeVector(&privileges, currentPrivs);
     }
     return privileges;
@@ -557,9 +555,11 @@ bool AuthorizationSessionImpl::isAuthorizedToCreateRole(
                 return true;
             }
         }
-        log() << "Not authorized to create the first role in the system '" << args.roleName
-              << "' using the localhost exception. The user needs to acquire the role through "
-                 "external authentication first.";
+        LOGV2(20241,
+              "Not authorized to create the first role in the system using the "
+              "localhost exception. The user needs to acquire the role through "
+              "external authentication first.",
+              "role"_attr = args.roleName);
     }
 
     return false;
@@ -618,6 +618,15 @@ static const int resourceSearchListCapacity = 5;
 /**
  * Builds from "target" an exhaustive list of all ResourcePatterns that match "target".
  *
+ * Some resources are considered to be "normal resources", and are matched by the
+ * forAnyNormalResource pattern. Collections which are not prefixed with "system.",
+ * and which do not belong inside of the "local" or "config" databases are "normal".
+ * Database other than "local" and "config" are normal.
+ *
+ * Most collections are matched by their database's resource. Collections prefixed with "system."
+ * are not. Neither are collections on the "local" database, whose name are prefixed with "replset."
+ *
+ *
  * Stores the resulting list into resourceSearchList, and returns the length.
  *
  * The seach lists are as follows, depending on the type of "target":
@@ -646,15 +655,19 @@ static int buildResourceSearchList(const ResourcePattern& target,
     int size = 0;
     resourceSearchList[size++] = ResourcePattern::forAnyResource();
     if (target.isExactNamespacePattern()) {
-        if (!target.ns().isSystem()) {
-            // Some databases should not be matchable with ResourcePattern::forAnyNormalResource.
-            // 'local' and 'config' are used to store special system collections, which user level
+        // Normal collections can be matched by anyNormalResource, or their database's resource.
+        if (target.ns().isNormalCollection()) {
+            // But even normal collections in non-normal databases should not be matchable with
+            // ResourcePattern::forAnyNormalResource. 'local' and 'config' are
+            // used to store special system collections, which user level
             // administrators should not be able to manipulate.
             if (target.ns().db() != "local" && target.ns().db() != "config") {
                 resourceSearchList[size++] = ResourcePattern::forAnyNormalResource();
             }
             resourceSearchList[size++] = ResourcePattern::forDatabaseName(target.ns().db());
         }
+
+        // All collections can be matched by a collection resource for their name
         resourceSearchList[size++] = ResourcePattern::forCollectionName(target.ns().coll());
     } else if (target.isDatabasePattern()) {
         if (target.ns().db() != "local" && target.ns().db() != "config") {
@@ -729,7 +742,7 @@ void AuthorizationSessionImpl::_refreshUserInfoAsNeeded(OperationContext* opCtx)
 
     while (it != _authenticatedUsers.end()) {
         auto& user = *it;
-        if (!user->isValid()) {
+        if (!user.isValid()) {
             // Make a good faith effort to acquire an up-to-date user object, since the one
             // we've cached is marked "out-of-date."
             UserName name = user->getName();
@@ -756,45 +769,57 @@ void AuthorizationSessionImpl::_refreshUserInfoAsNeeded(OperationContext* opCtx)
                         Status restrictionStatus = restrictionSet.validate(
                             RestrictionEnvironment::get(*opCtx->getClient()));
                         if (!restrictionStatus.isOK()) {
-                            log() << "Removed user " << name
-                                  << " with unmet authentication restrictions from session cache of"
-                                  << " user information. Restriction failed because: "
-                                  << restrictionStatus.reason();
+                            LOGV2(20242,
+                                  "Removed user with unmet authentication restrictions from "
+                                  "session cache of user information. Restriction failed",
+                                  "user"_attr = name,
+                                  "reason"_attr = restrictionStatus.reason());
                             // If we remove from the UserSet, we cannot increment the iterator.
                             continue;
                         }
                     } catch (...) {
-                        log() << "Evaluating authentication restrictions for " << name
-                              << " resulted in an unknown exception. Removing user from the"
-                              << " session cache.";
+                        LOGV2(20243,
+                              "Evaluating authentication restrictions for user resulted in an "
+                              "unknown exception. Removing user from the session cache",
+                              "user"_attr = name);
                         continue;
                     }
 
                     // Success! Replace the old User object with the updated one.
                     removeGuard.dismiss();
                     _authenticatedUsers.replaceAt(it, std::move(updatedUser));
-                    LOG(1) << "Updated session cache of user information for " << name;
+                    LOGV2_DEBUG(20244,
+                                1,
+                                "Updated session cache of user information for user",
+                                "user"_attr = name);
                     break;
                 }
                 case ErrorCodes::UserNotFound: {
                     // User does not exist anymore; remove it from _authenticatedUsers.
-                    log() << "Removed deleted user " << name
-                          << " from session cache of user information.";
+                    LOGV2(20245,
+                          "Removed deleted user from session cache of user information",
+                          "user"_attr = name);
                     continue;  // No need to advance "it" in this case.
                 }
                 case ErrorCodes::UnsupportedFormat: {
                     // An auth subsystem has explicitly indicated a failure.
-                    log() << "Removed user " << name
-                          << " from session cache of user information because of refresh failure:"
-                          << " '" << status << "'.";
+                    LOGV2(20246,
+                          "Removed user from session cache of user information because of "
+                          "refresh failure",
+                          "user"_attr = name,
+                          "error"_attr = status);
                     continue;  // No need to advance "it" in this case.
                 }
                 default:
                     // Unrecognized error; assume that it's transient, and continue working with the
                     // out-of-date privilege data.
-                    warning() << "Could not fetch updated user privilege information for " << name
-                              << "; continuing to use old information.  Reason is "
-                              << redact(status);
+                    LOGV2_WARNING(20247,
+                                  "Could not fetch updated user privilege information for {user}; "
+                                  "continuing to use old information. Reason is {error}",
+                                  "Could not fetch updated user privilege information, continuing "
+                                  "to use old information"
+                                  "user"_attr = name,
+                                  "error"_attr = redact(status));
                     removeGuard.dismiss();
                     break;
             }
