@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kCommand
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
@@ -41,13 +41,14 @@
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/db_raii.h"
+#include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/op_observer.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/ops/insert.h"
 #include "mongo/db/repl/replication_coordinator.h"
-#include "mongo/logger/redaction.h"
-#include "mongo/util/log.h"
+#include "mongo/db/views/view_catalog.h"
+#include "mongo/logv2/log.h"
 
 namespace mongo {
 namespace {
@@ -67,12 +68,6 @@ Status _createView(OperationContext* opCtx,
 
         Database* db = autoDb.getDb();
 
-        AutoStatsTracker statsTracker(opCtx,
-                                      nss,
-                                      Top::LockType::NotLocked,
-                                      AutoStatsTracker::LogMode::kUpdateTopAndCurOp,
-                                      db->getProfilingLevel());
-
         if (opCtx->writesAreReplicated() &&
             !repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, nss)) {
             return Status(ErrorCodes::NotMaster,
@@ -81,10 +76,28 @@ Status _createView(OperationContext* opCtx,
 
         // Create 'system.views' in a separate WUOW if it does not exist.
         WriteUnitOfWork wuow(opCtx);
-        db->getOrCreateCollection(opCtx, NamespaceString(db->getSystemViewsName()));
+        Collection* coll = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(
+            opCtx, NamespaceString(db->getSystemViewsName()));
+        if (!coll) {
+            coll = db->createCollection(opCtx, NamespaceString(db->getSystemViewsName()));
+        }
+        invariant(coll);
         wuow.commit();
 
         WriteUnitOfWork wunit(opCtx);
+
+        AutoStatsTracker statsTracker(opCtx,
+                                      nss,
+                                      Top::LockType::NotLocked,
+                                      AutoStatsTracker::LogMode::kUpdateTopAndCurOp,
+                                      db->getProfilingLevel());
+
+        // If the view creation rolls back, ensure that the Top entry created for the view is
+        // deleted.
+        opCtx->recoveryUnit()->onRollback([nss, serviceContext = opCtx->getServiceContext()]() {
+            Top::get(serviceContext).collectionDropped(nss);
+        });
+
         Status status = db->userCreateNS(opCtx, nss, collectionOptions, true, idIndex);
         if (!status.isOK()) {
             return status;
@@ -101,13 +114,18 @@ Status _createCollection(OperationContext* opCtx,
                          const BSONObj& idIndex) {
     return writeConflictRetry(opCtx, "create", nss.ns(), [&] {
         AutoGetOrCreateDb autoDb(opCtx, nss.db(), MODE_IX);
-        Lock::CollectionLock collLock(opCtx, nss, MODE_X);
-
-        AutoStatsTracker statsTracker(opCtx,
-                                      nss,
-                                      Top::LockType::NotLocked,
-                                      AutoStatsTracker::LogMode::kUpdateTopAndCurOp,
-                                      autoDb.getDb()->getProfilingLevel());
+        Lock::CollectionLock collLock(opCtx, nss, MODE_IX);
+        // This is a top-level handler for collection creation name conflicts. New commands coming
+        // in, or commands that generated a WriteConflict must return a NamespaceExists error here
+        // on conflict.
+        if (CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss) != nullptr) {
+            return Status(ErrorCodes::NamespaceExists,
+                          str::stream() << "Collection already exists. NS: " << nss);
+        }
+        if (ViewCatalog::get(autoDb.getDb())->lookup(opCtx, nss.ns())) {
+            return Status(ErrorCodes::NamespaceExists,
+                          str::stream() << "A view already exists. NS: " << nss);
+        }
 
         if (opCtx->writesAreReplicated() &&
             !repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, nss)) {
@@ -116,6 +134,19 @@ Status _createCollection(OperationContext* opCtx,
         }
 
         WriteUnitOfWork wunit(opCtx);
+
+        AutoStatsTracker statsTracker(opCtx,
+                                      nss,
+                                      Top::LockType::NotLocked,
+                                      AutoStatsTracker::LogMode::kUpdateTopAndCurOp,
+                                      autoDb.getDb()->getProfilingLevel());
+
+        // If the collection creation rolls back, ensure that the Top entry created for the
+        // collection is deleted.
+        opCtx->recoveryUnit()->onRollback([nss, serviceContext = opCtx->getServiceContext()]() {
+            Top::get(serviceContext).collectionDropped(nss);
+        });
+
         Status status = autoDb.getDb()->userCreateNS(opCtx, nss, collectionOptions, true, idIndex);
         if (!status.isOK()) {
             return status;
@@ -161,20 +192,28 @@ Status createCollection(OperationContext* opCtx,
     BSONObj options = optionsBuilder.obj();
     uassert(14832,
             "specify size:<n> when capped is true",
-            !options["capped"].trueValue() || options["size"].isNumber() ||
-                options.hasField("$nExtents"));
+            !options["capped"].trueValue() || options["size"].isNumber());
 
     CollectionOptions collectionOptions;
     {
-        Status status = collectionOptions.parse(options, kind);
-        if (!status.isOK()) {
-            return status;
+        StatusWith<CollectionOptions> statusWith = CollectionOptions::parse(options, kind);
+        if (!statusWith.isOK()) {
+            return statusWith.getStatus();
         }
+        collectionOptions = statusWith.getValue();
     }
 
     if (collectionOptions.isView()) {
+        uassert(ErrorCodes::OperationNotSupportedInTransaction,
+                str::stream() << "Cannot create a view in a multi-document "
+                                 "transaction.",
+                !opCtx->inMultiDocumentTransaction());
         return _createView(opCtx, nss, collectionOptions, idIndex);
     } else {
+        uassert(ErrorCodes::OperationNotSupportedInTransaction,
+                str::stream() << "Cannot create system collection " << nss.toString()
+                              << " within a transaction.",
+                !opCtx->inMultiDocumentTransaction() || !nss.isSystem());
         return _createCollection(opCtx, nss, collectionOptions, idIndex);
     }
 }
@@ -194,8 +233,9 @@ Status createCollection(OperationContext* opCtx,
 
 Status createCollectionForApplyOps(OperationContext* opCtx,
                                    const std::string& dbName,
-                                   const BSONElement& ui,
+                                   const OptionalCollectionUUID& ui,
                                    const BSONObj& cmdObj,
+                                   const bool allowRenameOutOfTheWay,
                                    const BSONObj& idIndex) {
     invariant(opCtx->lockState()->isDbLockedForMode(dbName, MODE_IX));
 
@@ -211,30 +251,34 @@ Status createCollectionForApplyOps(OperationContext* opCtx,
     // We need to do the renaming part in a separate transaction, as we cannot transactionally
     // create a database on MMAPv1, which could result in createCollection failing if the database
     // does not yet exist.
-    if (ui.ok()) {
+    if (ui) {
         // Return an optional, indicating whether we need to early return (if the collection already
         // exists, or in case of an error).
         using Result = boost::optional<Status>;
         auto result =
             writeConflictRetry(opCtx, "createCollectionForApplyOps", newCollName.ns(), [&] {
                 WriteUnitOfWork wunit(opCtx);
-                // Options need the field to be named "uuid", so parse/recreate.
-                auto uuid = uassertStatusOK(UUID::parse(ui));
+                auto uuid = ui.get();
                 uassert(ErrorCodes::InvalidUUID,
                         "Invalid UUID in applyOps create command: " + uuid.toString(),
                         uuid.isRFC4122v4());
 
                 auto& catalog = CollectionCatalog::get(opCtx);
-                const auto currentName = catalog.lookupNSSByUUID(uuid);
+                const auto currentName = catalog.lookupNSSByUUID(opCtx, uuid);
                 auto serviceContext = opCtx->getServiceContext();
                 auto opObserver = serviceContext->getOpObserver();
                 if (currentName && *currentName == newCollName)
                     return Result(Status::OK());
 
                 if (currentName && currentName->isDropPendingNamespace()) {
-                    log() << "CMD: create " << newCollName
-                          << " - existing collection with conflicting UUID " << uuid
-                          << " is in a drop-pending state: " << *currentName;
+                    LOGV2(20308,
+                          "CMD: create {newCollection} - existing collection with conflicting UUID "
+                          "{conflictingUUID} is in a drop-pending state: {existingCollection}",
+                          "CMD: create -- existing collection with conflicting UUID "
+                          "is in a drop-pending state",
+                          "newCollection"_attr = newCollName,
+                          "conflictingUUID"_attr = uuid,
+                          "existingCollection"_attr = *currentName);
                     return Result(Status(ErrorCodes::NamespaceExists,
                                          str::stream()
                                              << "existing collection " << currentName->toString()
@@ -251,8 +295,12 @@ Status createCollectionForApplyOps(OperationContext* opCtx,
                 // of the initial sync or result in rollback to fassert, requiring a resync of that
                 // node.
                 const bool stayTemp = true;
-                auto futureColl = db ? db->getCollection(opCtx, newCollName) : nullptr;
+                auto futureColl = db
+                    ? CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, newCollName)
+                    : nullptr;
                 bool needsRenaming = static_cast<bool>(futureColl);
+                invariant(!needsRenaming || allowRenameOutOfTheWay);
+
                 for (int tries = 0; needsRenaming && tries < 10; ++tries) {
                     auto tmpNameResult =
                         db->makeUniqueCollectionNamespace(opCtx, "tmp%%%%%.create");
@@ -272,9 +320,14 @@ Status createCollectionForApplyOps(OperationContext* opCtx,
                     }
 
                     // It is ok to log this because this doesn't happen very frequently.
-                    log() << "CMD: create " << newCollName
-                          << " - renaming existing collection with conflicting UUID " << uuid
-                          << " to temporary collection " << tmpName;
+                    LOGV2(20309,
+                          "CMD: create {newCollection} - renaming existing collection with "
+                          "conflicting UUID {conflictingUUID} to temporary collection {tempName}",
+                          "CMD: create -- renaming existing collection with "
+                          "conflicting UUID to temporary collection",
+                          "newCollection"_attr = newCollName,
+                          "conflictingUUID"_attr = uuid,
+                          "tempName"_attr = tmpName);
                     Status status = db->renameCollection(opCtx, newCollName, tmpName, stayTemp);
                     if (!status.isOK())
                         return Result(status);
@@ -285,6 +338,14 @@ Status createCollectionForApplyOps(OperationContext* opCtx,
                                                    /*dropTargetUUID*/ {},
                                                    /*numRecords*/ 0U,
                                                    stayTemp);
+
+                    // Abort any remaining index builds on the temporary collection.
+                    IndexBuildsCoordinator::get(opCtx)->abortCollectionIndexBuilds(
+                        opCtx,
+                        tmpName,
+                        futureColl->uuid(),
+                        "Aborting index builds on temporary collection");
+
                     // The existing collection has been successfully moved out of the way.
                     needsRenaming = false;
                 }
@@ -298,7 +359,7 @@ Status createCollectionForApplyOps(OperationContext* opCtx,
 
                 // If the collection with the requested UUID already exists, but with a different
                 // name, just rename it to 'newCollName'.
-                if (catalog.lookupCollectionByUUID(uuid)) {
+                if (catalog.lookupCollectionByUUID(opCtx, uuid)) {
                     invariant(currentName);
                     uassert(40655,
                             str::stream() << "Invalid name " << newCollName << " for UUID " << uuid,
