@@ -96,8 +96,6 @@ public:
 
     virtual int getProfilingLevel() const = 0;
 
-    virtual const NamespaceString& getProfilingNS() const = 0;
-
     /**
      * Sets the 'drop-pending' state of this Database.
      * This is done at the beginning of a dropDatabase operation and is used to reject subsequent
@@ -108,7 +106,6 @@ public:
 
     /**
      * Returns the 'drop-pending' state of this Database.
-     * The database must be locked in MODE_X when calling this function.
      */
     virtual bool isDropPending(OperationContext* opCtx) const = 0;
 
@@ -136,6 +133,14 @@ public:
 
     virtual Status dropView(OperationContext* const opCtx, NamespaceString viewName) const = 0;
 
+    /**
+     * A MODE_IX collection lock must be held for this call. Throws a WriteConflictException error
+     * if the collection already exists (say if another thread raced to create it).
+     *
+     * Surrounding writeConflictRetry loops must encompass checking that the collection exists as
+     * well as creating it. Otherwise the loop will endlessly throw WCEs: the caller must check that
+     * the collection exists to break free.
+     */
     virtual Collection* createCollection(OperationContext* const opCtx,
                                          const NamespaceString& nss,
                                          const CollectionOptions& options = CollectionOptions(),
@@ -146,12 +151,6 @@ public:
                               const NamespaceString& viewName,
                               const CollectionOptions& options) const = 0;
 
-    virtual Collection* getCollection(OperationContext* opCtx,
-                                      const NamespaceString& nss) const = 0;
-
-    virtual Collection* getOrCreateCollection(OperationContext* const opCtx,
-                                              const NamespaceString& nss) const = 0;
-
     /**
      * Arguments are passed by value as they otherwise would be changing as result of renaming.
      */
@@ -170,10 +169,10 @@ public:
      * Returns NamespaceExists if we are unable to generate a collection name that does not conflict
      * with an existing collection in this database.
      *
-     * The database must be locked in MODE_X when calling this function.
+     * The database must be locked in MODE_IX when calling this function.
      */
     virtual StatusWith<NamespaceString> makeUniqueCollectionNamespace(
-        OperationContext* opCtx, StringData collectionNameModel) = 0;
+        OperationContext* opCtx, StringData collectionNameModel) const = 0;
 
     /**
      * If we are in a replset, every replicated collection must have an _id index.  As we scan each
