@@ -29,15 +29,16 @@
 
 #pragma once
 
+#include <functional>
 #include <map>
 #include <string>
 #include <vector>
 
 #include "mongo/db/catalog/multi_index_block.h"
 #include "mongo/db/namespace_string.h"
-#include "mongo/db/repair_database.h"
+#include "mongo/db/rebuild_indexes.h"
+#include "mongo/db/repl_index_build_state.h"
 #include "mongo/platform/mutex.h"
-#include "mongo/stdx/functional.h"
 
 namespace mongo {
 
@@ -68,7 +69,7 @@ public:
     struct SetupOptions {
         SetupOptions();
         IndexConstraints indexConstraints = IndexConstraints::kEnforce;
-        bool forRecovery = false;
+        IndexBuildProtocol protocol = IndexBuildProtocol::kSinglePhase;
     };
 
     IndexBuildsManager() = default;
@@ -86,21 +87,12 @@ public:
                            SetupOptions options = {});
 
     /**
-     * Recovers the index build from its persisted state and sets it up to run again.
-     *
-     * Returns an enum reflecting the point up to which the build was recovered, so the caller knows
-     * where to recommence.
-     *
-     * TODO: Not yet implemented.
+     * Unregisters the builder associated with the given buildUUID from the _builders map.
      */
-    StatusWith<IndexBuildRecoveryState> recoverIndexBuild(const NamespaceString& nss,
-                                                          const UUID& buildUUID,
-                                                          std::vector<std::string> indexNames);
+    void unregisterIndexBuild(const UUID& buildUUID);
 
     /**
      * Runs the scanning/insertion phase of the index build..
-     *
-     * TODO: Not yet implemented.
      */
     Status startBuildingIndex(OperationContext* opCtx,
                               Collection* collection,
@@ -121,26 +113,19 @@ public:
      */
     Status drainBackgroundWrites(OperationContext* opCtx,
                                  const UUID& buildUUID,
-                                 RecoveryUnit::ReadSource readSource);
+                                 RecoveryUnit::ReadSource readSource,
+                                 IndexBuildInterceptor::DrainYieldPolicy drainYieldPolicy);
 
     /**
-     * For the given index build, check whether all the side writes have been applied from all the
-     * side write tables.
+     * Retries the key generation and insertion of records that were skipped during the scanning
+     * phase due to error suppression.
      */
-    bool areAllWritesApplied(OperationContext* opCtx, const UUID& buildUUID);
-
-    /**
-     * Persists information in the index catalog entry to reflect the successful completion of the
-     * scanning/insertion phase.
-     *
-     * TODO: Not yet implemented.
-     */
-    Status finishBuildingPhase(const UUID& buildUUID);
+    Status retrySkippedRecords(OperationContext* opCtx,
+                               const UUID& buildUUID,
+                               Collection* collection);
 
     /**
      * Runs the index constraint violation checking phase of the index build..
-     *
-     * TODO: Not yet implemented.
      */
     Status checkIndexConstraintViolations(OperationContext* opCtx, const UUID& buildUUID);
 
@@ -158,31 +143,25 @@ public:
                             OnCommitFn onCommitFn);
 
     /**
-     * Signals the index build to be aborted and returns without waiting for completion.
-     *
-     * Returns true if a build existed to be signaled, as opposed to having already finished and
-     * been cleared away, or not having yet started..
-     *
-     * TODO: Not yet fully implemented. The MultiIndexBlock::abort function that is called is
-     * not yet implemented.
+     * Deletes the index entry from the durable catalog.
      */
-    bool abortIndexBuild(const UUID& buildUUID, const std::string& reason);
+    using OnCleanUpFn = MultiIndexBlock::OnCleanUpFn;
+    bool abortIndexBuild(OperationContext* opCtx,
+                         Collection* collection,
+                         const UUID& buildUUID,
+                         OnCleanUpFn onCleanUpFn);
 
     /**
-     * Signals the index build to be interrupted and returns without waiting for it to stop. Does
-     * nothing if the index build has already been cleared away.
+     * Signals the index build to be aborted without being cleaned up and returns without waiting
+     * for it to stop. Does nothing if the index build has already been cleared away.
      *
      * Returns true if a build existed to be signaled, as opposed to having already finished and
      * been cleared away, or not having yet started..
      */
-    bool interruptIndexBuild(OperationContext* opCtx,
-                             const UUID& buildUUID,
-                             const std::string& reason);
-
-    /**
-     * Cleans up the index build state and unregisters it from the manager.
-     */
-    void tearDownIndexBuild(OperationContext* opCtx, Collection* collection, const UUID& buildUUID);
+    bool abortIndexBuildWithoutCleanup(OperationContext* opCtx,
+                                       Collection* collection,
+                                       const UUID& buildUUID,
+                                       const std::string& reason);
 
     /**
      * Returns true if the index build supports background writes while building an index. This is
@@ -202,21 +181,16 @@ private:
     void _registerIndexBuild(UUID buildUUID);
 
     /**
-     * Unregisters the builder associcated with the given buildUUID from the _builders map.
+     * Returns a pointer to the builder. Returns a bad status if the builder does not exist.
      */
-    void _unregisterIndexBuild(const UUID& buildUUID);
-
-    /**
-     * Returns a shared pointer to the builder. Invariants if the builder does not exist.
-     */
-    std::shared_ptr<MultiIndexBlock> _getBuilder(const UUID& buildUUID);
+    StatusWith<MultiIndexBlock*> _getBuilder(const UUID& buildUUID);
 
     // Protects the map data structures below.
     mutable Mutex _mutex = MONGO_MAKE_LATCH("IndexBuildsManager::_mutex");
 
     // Map of index builders by build UUID. Allows access to the builders so that actions can be
     // taken on and information passed to and from index builds.
-    std::map<UUID, std::shared_ptr<MultiIndexBlock>> _builders;
+    std::map<UUID, std::unique_ptr<MultiIndexBlock>> _builders;
 };
 
 }  // namespace mongo
