@@ -42,6 +42,7 @@
 #include "mongo/db/storage/record_store.h"
 
 namespace mongo {
+
 class Client;
 class Collection;
 
@@ -76,6 +77,36 @@ enum class IndexBuildMethod {
     kForeground,
 };
 
+StringData toString(IndexBuildMethod method);
+
+enum class CreateIndexEntryFlags : int {
+    kNone = 0x0,
+    /**
+     * kInitFromDisk avoids registering a change to undo this operation when set to true. You
+     * must set this flag if calling this function outside of a WriteUnitOfWork.
+     */
+    kInitFromDisk = 0x1,
+    /**
+     * kIsReady controls whether the index will be directly available for query usage without
+     * needing to complete the IndexBuildBlock process.
+     */
+    kIsReady = 0x2,
+    /**
+     * kFrozen indicates that the index is not usable and that it is not currently being
+     * built. This is used when starting a node in standalone mode and a two-phase index build
+     * is incomplete. kIsReady must not also be set.
+     */
+    kFrozen = 0x4
+};
+
+inline bool operator&(CreateIndexEntryFlags lhs, CreateIndexEntryFlags rhs) {
+    return (static_cast<int>(lhs) & static_cast<int>(rhs)) != 0;
+}
+
+inline CreateIndexEntryFlags operator|(CreateIndexEntryFlags lhs, CreateIndexEntryFlags rhs) {
+    return CreateIndexEntryFlags(static_cast<int>(lhs) | static_cast<int>(rhs));
+}
+
 /**
  * The IndexCatalog is owned by the Collection and is responsible for the lookup and lifetimes of
  * the indexes in a collection. Every collection has exactly one instance of this class.
@@ -85,8 +116,8 @@ enum class IndexBuildMethod {
  * To inspect the contents of this IndexCatalog, callers may obtain an iterator from
  * getIndexIterator().
  *
- * Index building functionality is supported by the IndexBuildBlockInterface interface. However, it
- * is recommended to use the higher level MultiIndexBlock interface.
+ * Index building functionality is supported by the IndexBuildBlock interface. However, it is
+ * recommended to use the higher level MultiIndexBlock interface.
  *
  * Due to the resource-intensive nature of the index building process, this interface also provides
  * information on which indexes are available for queries through the following functions:
@@ -148,61 +179,6 @@ public:
         std::unique_ptr<std::vector<IndexCatalogEntry*>> _ownedContainer;
     };
 
-    /**
-     * Interface for building a single index from an index spec and persisting its state to disk.
-     */
-    class IndexBuildBlockInterface {
-    public:
-        virtual ~IndexBuildBlockInterface() = default;
-
-        /**
-         * Must be called before the object is destructed if init() has been called.
-         * Cleans up the temporary tables that are created for an index build.
-         */
-        virtual void deleteTemporaryTables(OperationContext* opCtx) = 0;
-
-        /**
-         * Initializes a new entry for the index in the IndexCatalog.
-         *
-         * On success, holds pointer to newly created IndexCatalogEntry that can be accessed using
-         * getEntry(). IndexCatalog will still own the entry.
-         *
-         * Must be called from within a `WriteUnitOfWork`
-         */
-        virtual Status init(OperationContext* opCtx, Collection* collection) = 0;
-
-        /**
-         * Marks the state of the index as 'ready' and commits the index to disk.
-         *
-         * Must be called from within a `WriteUnitOfWork`
-         */
-        virtual void success(OperationContext* opCtx, Collection* collection) = 0;
-
-        /**
-         * Aborts the index build and removes any on-disk state where applicable.
-         *
-         * Must be called from within a `WriteUnitOfWork`
-         */
-        virtual void fail(OperationContext* opCtx, const Collection* collection) = 0;
-
-        /**
-         * Returns the IndexCatalogEntry that was created in init().
-         *
-         * This entry is owned by the IndexCatalog.
-         */
-        virtual IndexCatalogEntry* getEntry() = 0;
-
-        /**
-         * Returns the name of the index managed by this index builder.
-         */
-        virtual const std::string& getIndexName() const = 0;
-
-        /**
-         * Returns the index spec used to build this index.
-         */
-        virtual const BSONObj& getSpec() const = 0;
-    };
-
     IndexCatalog() = default;
     virtual ~IndexCatalog() = default;
 
@@ -212,8 +188,6 @@ public:
     // Must be called before used.
     virtual Status init(OperationContext* const opCtx) = 0;
 
-    virtual bool ok() const = 0;
-
     // ---- accessors -----
 
     virtual bool haveAnyIndexes() const = 0;
@@ -329,12 +303,6 @@ public:
     virtual std::vector<std::shared_ptr<const IndexCatalogEntry>> getAllReadyEntriesShared()
         const = 0;
 
-    /**
-     * Returns a not-ok Status if there are any unfinished index builds. No new indexes should
-     * be built when in this state.
-     */
-    virtual Status checkUnfinished() const = 0;
-
     /**
      * Returns an iterator for the index descriptors in this IndexCatalog.
      */
@@ -343,6 +311,15 @@ public:
 
     // ---- index set modifiers ------
 
+    /*
+     * Creates an index entry with the provided descriptor on the catalog's collection.
+     *
+     */
+
+    virtual IndexCatalogEntry* createIndexEntry(OperationContext* opCtx,
+                                                std::unique_ptr<IndexDescriptor> descriptor,
+                                                CreateIndexEntryFlags flags) = 0;
+
     /**
      * Call this only on an empty collection from inside a WriteUnitOfWork. Index creation on an
      * empty collection can be rolled back as part of a larger WUOW. Returns the full specification
@@ -401,11 +378,11 @@ public:
      */
     virtual void dropAllIndexes(OperationContext* opCtx,
                                 bool includingIdIndex,
-                                stdx::function<void(const IndexDescriptor*)> onDropFn) = 0;
+                                std::function<void(const IndexDescriptor*)> onDropFn) = 0;
     virtual void dropAllIndexes(OperationContext* opCtx, bool includingIdIndex) = 0;
 
     /**
-     * Drops the index.
+     * Drops the index given its descriptor.
      *
      * The caller must hold the collection X lock and ensure no index builds are in progress on the
      * collection.
@@ -413,16 +390,31 @@ public:
     virtual Status dropIndex(OperationContext* const opCtx, const IndexDescriptor* const desc) = 0;
 
     /**
-     * Drops all incomplete indexes and returns specs. After this, the indexes can be rebuilt.
+     * Drops an unfinished index given its descriptor.
+     *
+     * The caller must hold the collection X lock.
      */
-    virtual std::vector<BSONObj> getAndClearUnfinishedIndexes(OperationContext* const opCtx) = 0;
+    virtual Status dropUnfinishedIndex(OperationContext* const opCtx,
+                                       const IndexDescriptor* const desc) = 0;
+
+    /**
+     * Drops the index given its catalog entry.
+     *
+     * The caller must hold the collection X lock.
+     */
+    virtual Status dropIndexEntry(OperationContext* opCtx, IndexCatalogEntry* entry) = 0;
+
+    /**
+     * Deletes the index from the durable catalog on disk.
+     */
+    virtual void deleteIndexFromDisk(OperationContext* opCtx, const std::string& indexName) = 0;
 
     // ---- modify single index
 
     /**
      * Returns true if the index 'idx' is multikey, and returns false otherwise.
      */
-    virtual bool isMultikey(OperationContext* const opCtx, const IndexDescriptor* const idx) = 0;
+    virtual bool isMultikey(const IndexDescriptor* const idx) = 0;
 
     /**
      * Returns the path components that cause the index 'idx' to be multikey if the index supports
@@ -488,13 +480,6 @@ public:
 
     virtual std::string getAccessMethodName(const BSONObj& keyPattern) = 0;
 
-    /**
-     * Creates an instance of IndexBuildBlockInterface for building an index with the provided index
-     * spex and OperationContext.
-     */
-    virtual std::unique_ptr<IndexBuildBlockInterface> createIndexBuildBlock(
-        OperationContext* opCtx, const BSONObj& spec, IndexBuildMethod method) = 0;
-
     // public helpers
 
     /**
@@ -516,8 +501,6 @@ public:
                                             const IndexDescriptor* desc,
                                             InsertDeleteOptions* options) const = 0;
 
-    virtual void setNs(NamespaceString ns) = 0;
-
     virtual void indexBuildSuccess(OperationContext* opCtx, IndexCatalogEntry* index) = 0;
 };
 }  // namespace mongo
