@@ -48,14 +48,6 @@ class KillCursorsCmd final : public KillCursorsCmdBase {
 public:
     KillCursorsCmd() = default;
 
-    bool supportsReadConcern(const std::string& dbName,
-                             const BSONObj& cmdObj,
-                             repl::ReadConcernLevel level) const final {
-        // killCursors must support snapshot read concern in order to be run in transactions.
-        return level == repl::ReadConcernLevel::kLocalReadConcern ||
-            level == repl::ReadConcernLevel::kSnapshotReadConcern;
-    }
-
     bool run(OperationContext* opCtx,
              const std::string& dbname,
              const BSONObj& cmdObj,
