@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kStorage
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #include "mongo/platform/basic.h"
 
@@ -39,7 +39,8 @@
 #include "mongo/db/db_raii_gen.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/collection_sharding_state.h"
-#include "mongo/util/log.h"
+#include "mongo/db/storage/snapshot_helper.h"
+#include "mongo/logv2/log.h"
 
 namespace mongo {
 namespace {
@@ -112,7 +113,7 @@ AutoGetCollectionForRead::AutoGetCollectionForRead(OperationContext* opCtx,
     _autoColl.emplace(opCtx, nsOrUUID, collectionLockMode, viewMode, deadline);
 
     // If the read source is explicitly set to kNoTimestamp, we read the most up to date data and do
-    // not consider reading at last applied (e.g. FTDC needs that).
+    // not consider changing our ReadSource (e.g. FTDC needs that).
     if (opCtx->recoveryUnit()->getTimestampReadSource() == RecoveryUnit::ReadSource::kNoTimestamp)
         return;
 
@@ -123,102 +124,95 @@ AutoGetCollectionForRead::AutoGetCollectionForRead(OperationContext* opCtx,
     // need to check for pending catalog changes.
     while (auto coll = _autoColl->getCollection()) {
 
-        auto readSource = opCtx->recoveryUnit()->getTimestampReadSource();
-        auto minSnapshot = coll->getMinimumVisibleSnapshot();
-        auto mySnapshot = opCtx->recoveryUnit()->getPointInTimeReadTimestamp();
-
-        // If we are reading at a provided timestamp earlier than the latest catalog changes, then
-        // we must return an error.
-        if (readSource == RecoveryUnit::ReadSource::kProvided && minSnapshot &&
-            (*mySnapshot < *minSnapshot)) {
-            uasserted(ErrorCodes::SnapshotUnavailable,
-                      str::stream()
-                          << "Unable to read from a snapshot due to pending collection catalog "
-                             "changes; please retry the operation. Snapshot timestamp is "
-                          << mySnapshot->toString() << ". Collection minimum is "
-                          << minSnapshot->toString());
-        }
-
         // During batch application on secondaries, there is a potential to read inconsistent states
         // that would normally be protected by the PBWM lock. In order to serve secondary reads
         // during this period, we default to not acquiring the lock (by setting
         // _shouldNotConflictWithSecondaryBatchApplicationBlock). On primaries, we always read at a
         // consistent time, so not taking the PBWM lock is not a problem. On secondaries, we have to
-        // guarantee we read at a consistent state, so we must read at the last applied timestamp,
+        // guarantee we read at a consistent state, so we must read at the lastApplied timestamp,
         // which is set after each complete batch.
         //
-        // If an attempt to read at the last applied timestamp is unsuccessful because there are
-        // pending catalog changes that occur after the last applied timestamp, we release our locks
-        // and try again with the PBWM lock (by unsetting
+        // If an attempt to read at the lastApplied timestamp is unsuccessful because there are
+        // pending catalog changes that occur after that timestamp, we release our locks and try
+        // again with the PBWM lock (by unsetting
         // _shouldNotConflictWithSecondaryBatchApplicationBlock).
 
         const NamespaceString nss = coll->ns();
+        auto readSource = opCtx->recoveryUnit()->getTimestampReadSource();
 
-        bool readAtLastAppliedTimestamp =
-            _shouldReadAtLastAppliedTimestamp(opCtx, nss, readConcernLevel);
-
-        if (readAtLastAppliedTimestamp) {
-            opCtx->recoveryUnit()->setTimestampReadSource(RecoveryUnit::ReadSource::kLastApplied);
-            readSource = opCtx->recoveryUnit()->getTimestampReadSource();
+        // Once we have our locks, check whether or not we should override the ReadSource that was
+        // set before acquiring locks.
+        if (auto newReadSource = SnapshotHelper::getNewReadSource(opCtx, nss)) {
+            opCtx->recoveryUnit()->setTimestampReadSource(*newReadSource);
+            readSource = *newReadSource;
         }
 
-        // This timestamp could be earlier than the timestamp seen when the transaction is opened
-        // because it is set asynchonously. This is not problematic because holding the collection
-        // lock guarantees no metadata changes will occur in that time.
-        auto lastAppliedTimestamp = readAtLastAppliedTimestamp
-            ? boost::optional<Timestamp>(replCoord->getMyLastAppliedOpTime().getTimestamp())
-            : boost::none;
-
-        if (!_conflictingCatalogChanges(opCtx, minSnapshot, lastAppliedTimestamp)) {
+        const auto readTimestamp = opCtx->recoveryUnit()->getPointInTimeReadTimestamp();
+        const auto minSnapshot = coll->getMinimumVisibleSnapshot();
+        if (!SnapshotHelper::collectionChangesConflictWithRead(minSnapshot, readTimestamp)) {
             return;
         }
 
-        invariant(lastAppliedTimestamp ||
-                  // The kMajorityCommitted and kNoOverlap read sources already read from timestamps
-                  // that are safe with respect to concurrent secondary batch application.
-                  readSource == RecoveryUnit::ReadSource::kMajorityCommitted ||
-                  readSource == RecoveryUnit::ReadSource::kNoOverlap);
+        // If we are reading at a provided timestamp earlier than the latest catalog changes,
+        // then we must return an error.
+        if (readSource == RecoveryUnit::ReadSource::kProvided) {
+            uasserted(ErrorCodes::SnapshotUnavailable,
+                      str::stream()
+                          << "Unable to read from a snapshot due to pending collection catalog "
+                             "changes; please retry the operation. Snapshot timestamp is "
+                          << readTimestamp->toString() << ". Collection minimum is "
+                          << minSnapshot->toString());
+        }
+
+        invariant(
+            // The kMajorityCommitted and kLastApplied read sources already read from timestamps
+            // that are safe with respect to concurrent secondary batch application, and are
+            // eligible for retrying.
+            readSource == RecoveryUnit::ReadSource::kMajorityCommitted ||
+            readSource == RecoveryUnit::ReadSource::kNoOverlap ||
+            readSource == RecoveryUnit::ReadSource::kLastApplied);
         invariant(readConcernLevel != repl::ReadConcernLevel::kSnapshotReadConcern);
 
         // Yield locks in order to do the blocking call below.
         _autoColl = boost::none;
 
-        // If there are pending catalog changes, we should conflict with any in-progress batches (by
-        // taking the PBWM lock) and choose not to read from the last applied timestamp by unsetting
-        // _shouldNotConflictWithSecondaryBatchApplicationBlock. Index builds on secondaries can
-        // complete at timestamps later than the lastAppliedTimestamp during initial sync. After
-        // initial sync finishes, if we waited instead of retrying, readers would block indefinitely
-        // waiting for the lastAppliedTimestamp to move forward. Instead we force the reader take
-        // the PBWM lock and retry.
-        if (lastAppliedTimestamp) {
-            LOG(0) << "tried reading at last-applied time: " << *lastAppliedTimestamp
-                   << " on ns: " << nss.ns() << ", but future catalog changes are pending at time "
-                   << *minSnapshot << ". Trying again without reading at last-applied time.";
+        // If there are pending catalog changes when using a no-overlap or lastApplied read source,
+        // we choose to take the PBWM lock to conflict with any in-progress batches. This prevents
+        // us from idly spinning in this loop trying to get a new read timestamp ahead of the
+        // minimum visible snapshot. This helps us guarantee liveness (i.e. we can eventually get a
+        // suitable read timestamp) but should not be necessary for correctness. After initial sync
+        // finishes, if we waited instead of retrying, readers would block indefinitely waiting for
+        // their read timstamp to move forward. Instead we force the reader take the PBWM lock and
+        // retry.
+        if (readSource == RecoveryUnit::ReadSource::kLastApplied ||
+            readSource == RecoveryUnit::ReadSource::kNoOverlap) {
+            invariant(readTimestamp);
+            LOGV2(20576,
+                  "Tried reading at a timestamp, but future catalog changes are pending. "
+                  "Trying again without reading at a timestamp",
+                  "readTimestamp"_attr = *readTimestamp,
+                  "collection"_attr = nss.ns(),
+                  "collectionMinSnapshot"_attr = *minSnapshot);
             // Destructing the block sets _shouldConflictWithSecondaryBatchApplication back to the
             // previous value. If the previous value is false (because there is another
             // shouldNotConflictWithSecondaryBatchApplicationBlock outside of this function), this
             // does not take the PBWM lock.
             _shouldNotConflictWithSecondaryBatchApplicationBlock = boost::none;
-            invariant(opCtx->lockState()->shouldConflictWithSecondaryBatchApplication());
 
-            // Cannot change ReadSource while a RecoveryUnit is active, which may result from
-            // calling getPointInTimeReadTimestamp().
-            opCtx->recoveryUnit()->abandonSnapshot();
-            opCtx->recoveryUnit()->setTimestampReadSource(RecoveryUnit::ReadSource::kUnset);
-        }
+            // As alluded to above, if we are AutoGetting multiple collections, it
+            // is possible that our "reaquire the PBWM" trick doesn't work, since we've already done
+            // some reads and locked in our snapshot.  At this point, the only way out is to fail
+            // the operation. The client application will need to retry.
+            uassert(
+                ErrorCodes::SnapshotUnavailable,
+                str::stream() << "Unable to read from a snapshot due to pending collection catalog "
+                                 "changes; please retry the operation. Snapshot timestamp is "
+                              << readTimestamp->toString() << ". Collection minimum is "
+                              << minSnapshot->toString(),
+                opCtx->lockState()->shouldConflictWithSecondaryBatchApplication());
 
-        // If there are pending catalog changes when using a no-overlap read source, we choose to
-        // take the PBWM lock to conflict with any in-progress batches. This prevents us from idly
-        // spinning in this loop trying to get a new read timestamp ahead of the minimum visible
-        // snapshot. This helps us guarantee liveness (i.e. we can eventually get a suitable read
-        // timestamp) but should not be necessary for correctness.
-        if (readSource == RecoveryUnit::ReadSource::kNoOverlap) {
-            invariant(!lastAppliedTimestamp);  // no-overlap read source selects its own timestamp.
-            _shouldNotConflictWithSecondaryBatchApplicationBlock = boost::none;
-            invariant(opCtx->lockState()->shouldConflictWithSecondaryBatchApplication());
-
-            // Abandon our snapshot but don't change our read source, so that we can select a new
-            // read timestamp on the next loop iteration.
+            // Abandon our snapshot. We may select a new read timestamp or ReadSource in the next
+            // loop iteration.
             opCtx->recoveryUnit()->abandonSnapshot();
         }
 
@@ -236,84 +230,6 @@ AutoGetCollectionForRead::AutoGetCollectionForRead(OperationContext* opCtx,
     }
 }
 
-bool AutoGetCollectionForRead::_shouldReadAtLastAppliedTimestamp(
-    OperationContext* opCtx,
-    const NamespaceString& nss,
-    repl::ReadConcernLevel readConcernLevel) const {
-
-    // If this block is unset, then the operation did not opt-out of the PBWM lock, implying that it
-    // cannot read at lastApplied. It's important to note that it is possible for this to be set,
-    // but still be holding the PBWM lock, explained below.
-    if (!_shouldNotConflictWithSecondaryBatchApplicationBlock) {
-        return false;
-    }
-
-    // If we are already holding the PBWM lock, do not read at last-applied. This is because once an
-    // operation reads without a timestamp (effectively seeing all writes), it is no longer safe to
-    // start reading at a timestamp, as writes or catalog operations may appear to vanish.
-    // This may occur when multiple collection locks are held concurrently, which is often the case
-    // when DBDirectClient is used.
-    if (opCtx->lockState()->isLockHeldForMode(resourceIdParallelBatchWriterMode, MODE_IS)) {
-        LOG(1) << "not reading at last-applied because the PBWM lock is held";
-        return false;
-    }
-
-    // Majority and snapshot readConcern levels should not read from lastApplied; these read
-    // concerns already have a designated timestamp to read from.
-    if (readConcernLevel != repl::ReadConcernLevel::kLocalReadConcern &&
-        readConcernLevel != repl::ReadConcernLevel::kAvailableReadConcern) {
-        return false;
-    }
-
-    // If we are in a replication state (like secondary or primary catch-up) where we are not
-    // accepting writes, we should read at lastApplied. If this node can accept writes, then no
-    // conflicting replication batches are being applied and we can read from the default snapshot.
-    if (repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesForDatabase(opCtx, "admin")) {
-        return false;
-    }
-
-    // Non-replicated collections do not need to read at lastApplied, as those collections are not
-    // written by the replication system.  However, the oplog is special, as it *is* written by the
-    // replication system.
-    if (!nss.isReplicated() && !nss.isOplog()) {
-        return false;
-    }
-
-    return true;
-}
-
-bool AutoGetCollectionForRead::_conflictingCatalogChanges(
-    OperationContext* opCtx,
-    boost::optional<Timestamp> minSnapshot,
-    boost::optional<Timestamp> lastAppliedTimestamp) const {
-    // This is the timestamp of the most recent catalog changes to this collection. If this is
-    // greater than any point in time read timestamps, we should either wait or return an error.
-    if (!minSnapshot) {
-        return false;
-    }
-
-    // If we are reading from the lastAppliedTimestamp and it is up-to-date with any catalog
-    // changes, we can return.
-    if (lastAppliedTimestamp &&
-        (lastAppliedTimestamp->isNull() || *lastAppliedTimestamp >= *minSnapshot)) {
-        return false;
-    }
-
-    // This can be set when readConcern is "snapshot" or "majority".
-    auto mySnapshot = opCtx->recoveryUnit()->getPointInTimeReadTimestamp();
-
-    // If we do not have a point in time to conflict with minSnapshot, return.
-    if (!mySnapshot && !lastAppliedTimestamp) {
-        return false;
-    }
-
-    // Return if there are no conflicting catalog changes with mySnapshot.
-    if (mySnapshot && *mySnapshot >= *minSnapshot) {
-        return false;
-    }
-
-    return true;
-}
 
 AutoGetCollectionForReadCommand::AutoGetCollectionForReadCommand(
     OperationContext* opCtx,
@@ -329,20 +245,19 @@ AutoGetCollectionForReadCommand::AutoGetCollectionForReadCommand(
                     _autoCollForRead.getDb() ? _autoCollForRead.getDb()->getProfilingLevel()
                                              : kDoNotChangeProfilingLevel,
                     deadline) {
-    if (!_autoCollForRead.getView()) {
-        // Perform the check early so the query planner would be able to extract the correct
-        // shard key. Also make sure that version is compatible if query planner decides to
-        // use an empty plan.
-        auto css = CollectionShardingState::get(opCtx, _autoCollForRead.getNss());
-        css->checkShardVersionOrThrow(opCtx);
-    }
+
+    // Perform the check early so the query planner would be able to extract the correct
+    // shard key. Also make sure that version is compatible if query planner decides to
+    // use an empty plan.
+    invariant(!_autoCollForRead.getView() || !_autoCollForRead.getCollection());
+    auto css = CollectionShardingState::get(opCtx, _autoCollForRead.getNss());
+    css->checkShardVersionOrThrow(opCtx);
 }
 
 OldClientContext::OldClientContext(OperationContext* opCtx, const std::string& ns, bool doVersion)
     : _opCtx(opCtx), _db(DatabaseHolder::get(opCtx)->getDb(opCtx, ns)) {
     if (!_db) {
         const auto dbName = nsToDatabaseSubstring(ns);
-        invariant(_opCtx->lockState()->isDbLockedForMode(dbName, MODE_X));
         _db = DatabaseHolder::get(opCtx)->openDb(_opCtx, dbName, &_justCreated);
         invariant(_db);
     }
