@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kDefault
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/platform/basic.h"
 
@@ -53,7 +53,6 @@
 #include "mongo/db/service_context.h"
 #include "mongo/db/write_concern.h"
 #include "mongo/db/write_concern_options.h"
-#include "mongo/util/log.h"
 #include "mongo/util/scopeguard.h"
 #include "mongo/util/str.h"
 
@@ -88,7 +87,7 @@ RecordId Helpers::findOne(OperationContext* opCtx,
     if (!collection)
         return RecordId();
 
-    auto qr = stdx::make_unique<QueryRequest>(collection->ns());
+    auto qr = std::make_unique<QueryRequest>(collection->ns());
     qr->setFilter(query);
     return findOne(opCtx, collection, std::move(qr), requireIndex);
 }
@@ -138,7 +137,8 @@ bool Helpers::findById(OperationContext* opCtx,
                        bool* indexFound) {
     invariant(database);
 
-    Collection* collection = database->getCollection(opCtx, NamespaceString(ns));
+    Collection* collection =
+        CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, NamespaceString(ns));
     if (!collection) {
         return false;
     }
@@ -176,7 +176,7 @@ bool Helpers::getSingleton(OperationContext* opCtx, const char* ns, BSONObj& res
     AutoGetCollectionForReadCommand ctx(opCtx, NamespaceString(ns));
     auto exec =
         InternalPlanner::collectionScan(opCtx, ns, ctx.getCollection(), PlanExecutor::NO_YIELD);
-    PlanExecutor::ExecState state = exec->getNext(&result, NULL);
+    PlanExecutor::ExecState state = exec->getNext(&result, nullptr);
 
     CurOp::get(opCtx)->done();
 
@@ -195,7 +195,7 @@ bool Helpers::getLast(OperationContext* opCtx, const char* ns, BSONObj& result)
     AutoGetCollectionForReadCommand autoColl(opCtx, NamespaceString(ns));
     auto exec = InternalPlanner::collectionScan(
         opCtx, ns, autoColl.getCollection(), PlanExecutor::NO_YIELD, InternalPlanner::BACKWARD);
-    PlanExecutor::ExecState state = exec->getNext(&result, NULL);
+    PlanExecutor::ExecState state = exec->getNext(&result, nullptr);
 
     // Non-yielding collection scans from InternalPlanner will never error.
     invariant(PlanExecutor::ADVANCED == state || PlanExecutor::IS_EOF == state);
@@ -215,16 +215,24 @@ void Helpers::upsert(OperationContext* opCtx,
     BSONElement e = o["_id"];
     verify(e.type());
     BSONObj id = e.wrap();
+    upsert(opCtx, ns, id, o, fromMigrate);
+}
 
+void Helpers::upsert(OperationContext* opCtx,
+                     const string& ns,
+                     const BSONObj& filter,
+                     const BSONObj& updateMod,
+                     bool fromMigrate) {
     OldClientContext context(opCtx, ns);
 
     const NamespaceString requestNs(ns);
     UpdateRequest request(requestNs);
 
-    request.setQuery(id);
-    request.setUpdateModification(o);
+    request.setQuery(filter);
+    request.setUpdateModification(updateMod);
     request.setUpsert();
     request.setFromMigration(fromMigrate);
+    request.setYieldPolicy(PlanExecutor::NO_YIELD);
 
     update(opCtx, context.db(), request);
 }
@@ -262,7 +270,9 @@ BSONObj Helpers::inferKeyPattern(const BSONObj& o) {
 void Helpers::emptyCollection(OperationContext* opCtx, const NamespaceString& nss) {
     OldClientContext context(opCtx, nss.ns());
     repl::UnreplicatedWritesBlock uwb(opCtx);
-    Collection* collection = context.db() ? context.db()->getCollection(opCtx, nss) : nullptr;
+    Collection* collection = context.db()
+        ? CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss)
+        : nullptr;
     deleteObjects(opCtx, collection, nss, BSONObj(), false);
 }
 
