@@ -27,10 +27,12 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kQuery
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/db/exec/collection_scan.h"
 
+#include <memory>
+
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/database.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
@@ -40,9 +42,9 @@
 #include "mongo/db/exec/working_set.h"
 #include "mongo/db/exec/working_set_common.h"
 #include "mongo/db/repl/optime.h"
-#include "mongo/stdx/memory.h"
-#include "mongo/util/fail_point_service.h"
-#include "mongo/util/log.h"
+#include "mongo/db/storage/oplog_hack.h"
+#include "mongo/logv2/log.h"
+#include "mongo/util/fail_point.h"
 
 #include "mongo/db/client.h"  // XXX-ERH
 
@@ -50,29 +52,44 @@ namespace mongo {
 
 using std::unique_ptr;
 using std::vector;
-using stdx::make_unique;
 
 // static
 const char* CollectionScan::kStageType = "COLLSCAN";
 
-CollectionScan::CollectionScan(OperationContext* opCtx,
+CollectionScan::CollectionScan(ExpressionContext* expCtx,
                                const Collection* collection,
                                const CollectionScanParams& params,
                                WorkingSet* workingSet,
                                const MatchExpression* filter)
-    : RequiresCollectionStage(kStageType, opCtx, collection),
+    : RequiresCollectionStage(kStageType, expCtx, collection),
       _workingSet(workingSet),
-      _filter(filter),
+      _filter((filter && !filter->isTriviallyTrue()) ? filter : nullptr),
       _params(params) {
     // Explain reports the direction of the collection scan.
     _specificStats.direction = params.direction;
+    _specificStats.minTs = params.minTs;
     _specificStats.maxTs = params.maxTs;
+    _specificStats.tailable = params.tailable;
+    if (params.minTs || params.maxTs) {
+        // The 'minTs' and 'maxTs' parameters are used for a special optimization that
+        // applies only to forwards scans of the oplog.
+        invariant(params.direction == CollectionScanParams::FORWARD);
+        invariant(collection->ns().isOplog());
+        invariant(!params.resumeAfterRecordId);
+    }
     invariant(!_params.shouldTrackLatestOplogTimestamp || collection->ns().isOplog());
 
+    if (params.resumeAfterRecordId) {
+        // The 'resumeAfterRecordId' parameter is used for resumable collection scans, which we
+        // only support in the forward direction.
+        invariant(params.direction == CollectionScanParams::FORWARD);
+    }
+
+    // Set early stop condition.
     if (params.maxTs) {
-        _endConditionBSON = BSON("$gte" << *(params.maxTs));
-        _endCondition = stdx::make_unique<GTEMatchExpression>(repl::OpTime::kTimestampFieldName,
-                                                              _endConditionBSON.firstElement());
+        _endConditionBSON = BSON("$gte"_sd << *(params.maxTs));
+        _endCondition = std::make_unique<GTEMatchExpression>(repl::OpTime::kTimestampFieldName,
+                                                             _endConditionBSON.firstElement());
     }
 }
 
@@ -100,11 +117,11 @@ PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
                 // snapshot where the oplog entries are not yet visible even after the wait.
                 invariant(!_params.tailable && collection()->ns().isOplog());
 
-                getOpCtx()->recoveryUnit()->abandonSnapshot();
-                collection()->getRecordStore()->waitForAllEarlierOplogWritesToBeVisible(getOpCtx());
+                opCtx()->recoveryUnit()->abandonSnapshot();
+                collection()->getRecordStore()->waitForAllEarlierOplogWritesToBeVisible(opCtx());
             }
 
-            _cursor = collection()->getCursor(getOpCtx(), forward);
+            _cursor = collection()->getCursor(opCtx(), forward);
 
             if (!_lastSeenId.isNull()) {
                 invariant(_params.tailable);
@@ -112,8 +129,9 @@ PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
                 // want to signal an error rather than silently dropping data from the stream.
                 //
                 // Note that we want to return the record *after* this one since we have already
-                // returned this one. This is only possible in the tailing case because that is the
-                // only time we'd need to create a cursor after already getting a record out of it.
+                // returned this one. This is possible in the tailing case. Notably, tailing is the
+                // only time we'd need to create a cursor after already getting a record out of it
+                // and updating our _lastSeenId.
                 if (!_cursor->seekExact(_lastSeenId)) {
                     Status status(ErrorCodes::CappedPositionLost,
                                   str::stream() << "CollectionScan died due to failure to restore "
@@ -124,12 +142,44 @@ PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
                 }
             }
 
+            if (_params.resumeAfterRecordId) {
+                invariant(!_params.tailable);
+                invariant(_lastSeenId.isNull());
+                // Seek to where we are trying to resume the scan from. Signal a KeyNotFound error
+                // if the record no longer exists.
+                //
+                // Note that we want to return the record *after* this one since we have already
+                // returned this one prior to the resume.
+                auto recordIdToSeek = *_params.resumeAfterRecordId;
+                if (!_cursor->seekExact(recordIdToSeek)) {
+                    Status status(
+                        ErrorCodes::KeyNotFound,
+                        str::stream()
+                            << "Failed to resume collection scan: the recordId from which we are "
+                            << "attempting to resume no longer exists in the collection. "
+                            << "recordId: " << recordIdToSeek);
+                    *out = WorkingSetCommon::allocateStatusMember(_workingSet, status);
+                    return PlanStage::FAILURE;
+                }
+            }
+
             return PlanStage::NEED_TIME;
         }
 
-        if (_lastSeenId.isNull() && !_params.start.isNull()) {
-            record = _cursor->seekExact(_params.start);
-        } else {
+        if (_lastSeenId.isNull() && _params.minTs) {
+            // See if the RecordStore supports the oplogStartHack.
+            StatusWith<RecordId> goal = oploghack::keyForOptime(*_params.minTs);
+            if (goal.isOK()) {
+                boost::optional<RecordId> startLoc =
+                    collection()->getRecordStore()->oplogStartHack(opCtx(), goal.getValue());
+                if (startLoc && !startLoc->isNull()) {
+                    LOGV2_DEBUG(20584, 3, "Using direct oplog seek");
+                    record = _cursor->seekExact(*startLoc);
+                }
+            }
+        }
+
+        if (!record) {
             record = _cursor->next();
         }
     } catch (const WriteConflictException&) {
@@ -165,7 +215,7 @@ PlanStage::StageState CollectionScan::doWork(WorkingSetID* out) {
     WorkingSetID id = _workingSet->allocate();
     WorkingSetMember* member = _workingSet->get(id);
     member->recordId = record->id;
-    member->obj = {getOpCtx()->recoveryUnit()->getSnapshotId(), record->data.releaseToBson()};
+    member->resetDocument(opCtx()->recoveryUnit()->getSnapshotId(), record->data.releaseToBson());
     _workingSet->transitionToRecordIdAndObj(id);
 
     return returnIfMatches(member, id, out);
@@ -188,7 +238,6 @@ PlanStage::StageState CollectionScan::returnIfMatches(WorkingSetMember* member,
                                                       WorkingSetID memberID,
                                                       WorkingSetID* out) {
     ++_specificStats.docsTested;
-
     if (Filter::passes(member, _filter)) {
         if (_params.stopApplyingFilterAfterFirstMatch) {
             _filter = nullptr;
@@ -233,19 +282,19 @@ void CollectionScan::doDetachFromOperationContext() {
 
 void CollectionScan::doReattachToOperationContext() {
     if (_cursor)
-        _cursor->reattachToOperationContext(getOpCtx());
+        _cursor->reattachToOperationContext(opCtx());
 }
 
 unique_ptr<PlanStageStats> CollectionScan::getStats() {
     // Add a BSON representation of the filter to the stats tree, if there is one.
-    if (NULL != _filter) {
+    if (nullptr != _filter) {
         BSONObjBuilder bob;
         _filter->serialize(&bob);
         _commonStats.filter = bob.obj();
     }
 
-    unique_ptr<PlanStageStats> ret = make_unique<PlanStageStats>(_commonStats, STAGE_COLLSCAN);
-    ret->specific = make_unique<CollectionScanStats>(_specificStats);
+    unique_ptr<PlanStageStats> ret = std::make_unique<PlanStageStats>(_commonStats, STAGE_COLLSCAN);
+    ret->specific = std::make_unique<CollectionScanStats>(_specificStats);
     return ret;
 }
 
