@@ -31,6 +31,7 @@
 
 #include "mongo/db/curop.h"
 #include "mongo/db/pipeline/document_source_check_resume_token.h"
+#include "mongo/db/repl/oplog_entry.h"
 
 using boost::intrusive_ptr;
 namespace mongo {
@@ -153,7 +154,8 @@ ResumeStatus compareAgainstClientResumeToken(const intrusive_ptr<ExpressionConte
 
     // In order for the relaxed comparison to be applicable, the client token must have a single _id
     // field, and the resumed stream token must have additional fields beyond _id.
-    if (!(documentKeyFromClient.size() == 1 && documentKeyFromResumedStream.size() > 1)) {
+    if (!(documentKeyFromClient.computeSize() == 1 &&
+          documentKeyFromResumedStream.computeSize() > 1)) {
         return defaultResumeStatus;
     }
 
@@ -168,14 +170,15 @@ ResumeStatus compareAgainstClientResumeToken(const intrusive_ptr<ExpressionConte
 }  // namespace
 
 const char* DocumentSourceEnsureResumeTokenPresent::getSourceName() const {
-    return "$_ensureResumeTokenPresent";
+    return kStageName.rawData();
 }
 
 Value DocumentSourceEnsureResumeTokenPresent::serialize(
     boost::optional<ExplainOptions::Verbosity> explain) const {
-    // This stage is created by the DocumentSourceChangeStream stage, so serializing it here
-    // would result in it being created twice.
-    return Value();
+    // We only serialize this stage in the context of explain.
+    return explain
+        ? Value(DOC(kStageName << DOC("resumeToken" << ResumeToken(_tokenFromClient).toDocument())))
+        : Value();
 }
 
 intrusive_ptr<DocumentSourceEnsureResumeTokenPresent>
@@ -186,11 +189,9 @@ DocumentSourceEnsureResumeTokenPresent::create(const intrusive_ptr<ExpressionCon
 
 DocumentSourceEnsureResumeTokenPresent::DocumentSourceEnsureResumeTokenPresent(
     const intrusive_ptr<ExpressionContext>& expCtx, ResumeTokenData token)
-    : DocumentSource(expCtx), _tokenFromClient(std::move(token)) {}
-
-DocumentSource::GetNextResult DocumentSourceEnsureResumeTokenPresent::getNext() {
-    pExpCtx->checkForInterrupt();
+    : DocumentSource(kStageName, expCtx), _tokenFromClient(std::move(token)) {}
 
+DocumentSource::GetNextResult DocumentSourceEnsureResumeTokenPresent::doGetNext() {
     if (_resumeStatus == ResumeStatus::kSurpassedToken) {
         // We've already verified the resume token is present.
         return pSource->getNext();
@@ -256,14 +257,15 @@ DocumentSourceEnsureResumeTokenPresent::_checkNextDocAndSwallowResumeToken(
 }
 
 const char* DocumentSourceShardCheckResumability::getSourceName() const {
-    return "$_checkShardResumability";
+    return kStageName.rawData();
 }
 
 Value DocumentSourceShardCheckResumability::serialize(
     boost::optional<ExplainOptions::Verbosity> explain) const {
-    // This stage is created by the DocumentSourceChangeStream stage, so serializing it here
-    // would result in it being created twice.
-    return Value();
+    // We only serialize this stage in the context of explain.
+    return explain
+        ? Value(DOC(kStageName << DOC("resumeToken" << ResumeToken(_tokenFromClient).toDocument())))
+        : Value();
 }
 
 intrusive_ptr<DocumentSourceShardCheckResumability> DocumentSourceShardCheckResumability::create(
@@ -279,11 +281,9 @@ intrusive_ptr<DocumentSourceShardCheckResumability> DocumentSourceShardCheckResu
 
 DocumentSourceShardCheckResumability::DocumentSourceShardCheckResumability(
     const intrusive_ptr<ExpressionContext>& expCtx, ResumeTokenData token)
-    : DocumentSource(expCtx), _tokenFromClient(std::move(token)) {}
-
-DocumentSource::GetNextResult DocumentSourceShardCheckResumability::getNext() {
-    pExpCtx->checkForInterrupt();
+    : DocumentSource(kStageName, expCtx), _tokenFromClient(std::move(token)) {}
 
+DocumentSource::GetNextResult DocumentSourceShardCheckResumability::doGetNext() {
     if (_surpassedResumeToken)
         return pSource->getNext();
 
@@ -328,17 +328,18 @@ void DocumentSourceShardCheckResumability::_assertOplogHasEnoughHistory(
     // Look up the first document in the oplog and compare it with the resume token's clusterTime.
     auto firstEntryExpCtx = pExpCtx->copyWith(NamespaceString::kRsOplogNamespace);
     auto matchSpec = BSON("$match" << BSONObj());
-    auto pipeline = pExpCtx->mongoProcessInterface->makePipeline({matchSpec}, firstEntryExpCtx);
+    auto pipeline = Pipeline::makePipeline({matchSpec}, firstEntryExpCtx);
     if (auto first = pipeline->getNext()) {
         auto firstOplogEntry = Value(*first);
         // If the first entry in the oplog is the replset initialization, then it doesn't matter
         // if its timestamp is later than the resume token. No events earlier than the token can
         // have fallen off this oplog, and it is therefore safe to resume. Otherwise, verify that
         // the timestamp of the first oplog entry is earlier than that of the resume token.
+        using repl::kInitiatingSetMsg;
         const bool isNewRS =
-            Value::compare(firstOplogEntry["o"]["msg"], Value("initiating set"_sd), nullptr) == 0 &&
+            Value::compare(firstOplogEntry["o"]["msg"], Value(kInitiatingSetMsg), nullptr) == 0 &&
             Value::compare(firstOplogEntry["op"], Value("n"_sd), nullptr) == 0;
-        uassert(ErrorCodes::ChangeStreamFatalError,
+        uassert(ErrorCodes::ChangeStreamHistoryLost,
                 "Resume of change stream was not possible, as the resume point may no longer be in "
                 "the oplog. ",
                 isNewRS || firstOplogEntry["ts"].getTimestamp() < _tokenFromClient.clusterTime);
