@@ -59,7 +59,8 @@ namespace mongo {
  */
 class DocumentSourceShardCheckResumability final : public DocumentSource {
 public:
-    GetNextResult getNext() final;
+    static constexpr StringData kStageName = "$_internalCheckShardResumability"_sd;
+
     const char* getSourceName() const final;
 
     StageConstraints constraints(Pipeline::SplitState pipeState) const final {
@@ -70,6 +71,7 @@ public:
                 FacetRequirement::kNotAllowed,
                 TransactionRequirement::kNotAllowed,
                 LookupRequirement::kNotAllowed,
+                UnionRequirement::kNotAllowed,
                 ChangeStreamRequirement::kChangeStreamStage};
     }
 
@@ -77,7 +79,7 @@ public:
         return boost::none;
     }
 
-    Value serialize(boost::optional<ExplainOptions::Verbosity> explain = boost::none) const final;
+    Value serialize(boost::optional<ExplainOptions::Verbosity> explain) const final;
 
     static boost::intrusive_ptr<DocumentSourceShardCheckResumability> create(
         const boost::intrusive_ptr<ExpressionContext>& expCtx, Timestamp ts);
@@ -92,9 +94,11 @@ private:
     DocumentSourceShardCheckResumability(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                                          ResumeTokenData token);
 
+    GetNextResult doGetNext() final;
+
     void _assertOplogHasEnoughHistory(const GetNextResult& nextInput);
 
-    ResumeTokenData _tokenFromClient;
+    const ResumeTokenData _tokenFromClient;
     bool _verifiedOplogHasEnoughHistory = false;
     bool _surpassedResumeToken = false;
 };
@@ -105,6 +109,7 @@ private:
  */
 class DocumentSourceEnsureResumeTokenPresent final : public DocumentSource {
 public:
+    static constexpr StringData kStageName = "$_internalEnsureResumeTokenPresent"_sd;
     // Used to record the results of comparing the token data extracted from documents in the
     // resumed stream against the client's resume token.
     enum class ResumeStatus {
@@ -113,7 +118,6 @@ public:
         kCheckNextDoc     // The next document produced by the stream may contain the resume token.
     };
 
-    GetNextResult getNext() final;
     const char* getSourceName() const final;
 
     StageConstraints constraints(Pipeline::SplitState) const final {
@@ -126,6 +130,7 @@ public:
                 FacetRequirement::kNotAllowed,
                 TransactionRequirement::kNotAllowed,
                 LookupRequirement::kNotAllowed,
+                UnionRequirement::kNotAllowed,
                 ChangeStreamRequirement::kChangeStreamStage};
     }
 
@@ -142,7 +147,7 @@ public:
         return logic;
     };
 
-    Value serialize(boost::optional<ExplainOptions::Verbosity> explain = boost::none) const final;
+    Value serialize(boost::optional<ExplainOptions::Verbosity> explain) const final;
 
     static boost::intrusive_ptr<DocumentSourceEnsureResumeTokenPresent> create(
         const boost::intrusive_ptr<ExpressionContext>& expCtx, ResumeTokenData token);
@@ -154,6 +159,8 @@ private:
     DocumentSourceEnsureResumeTokenPresent(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                                            ResumeTokenData token);
 
+    GetNextResult doGetNext() final;
+
     /**
      * Check the given event to determine whether it matches the client's resume token. If so, we
      * swallow this event and return the next event in the stream. Otherwise, return boost::none.
@@ -162,7 +169,7 @@ private:
         const DocumentSource::GetNextResult& nextInput);
 
     ResumeStatus _resumeStatus = ResumeStatus::kCheckNextDoc;
-    ResumeTokenData _tokenFromClient;
+    const ResumeTokenData _tokenFromClient;
 };
 
 }  // namespace mongo
