@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kQuery
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/platform/basic.h"
 
@@ -54,7 +54,7 @@
 #include "mongo/db/query/planner_ixselect.h"
 #include "mongo/db/query/query_planner_common.h"
 #include "mongo/db/query/query_solution.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
 
 namespace mongo {
 
@@ -108,9 +108,6 @@ string optionString(size_t options) {
             case QueryPlannerParams::INCLUDE_SHARD_FILTER:
                 ss << "INCLUDE_SHARD_FILTER ";
                 break;
-            case QueryPlannerParams::NO_BLOCKING_SORT:
-                ss << "NO_BLOCKING_SORT ";
-                break;
             case QueryPlannerParams::INDEX_INTERSECTION:
                 ss << "INDEX_INTERSECTION ";
                 break;
@@ -120,9 +117,6 @@ string optionString(size_t options) {
             case QueryPlannerParams::SPLIT_LIMITED_SORT:
                 ss << "SPLIT_LIMITED_SORT ";
                 break;
-            case QueryPlannerParams::NO_UNCOVERED_PROJECTIONS:
-                ss << "NO_UNCOVERED_PROJECTIONS ";
-                break;
             case QueryPlannerParams::GENERATE_COVERED_IXSCANS:
                 ss << "GENERATE_COVERED_IXSCANS ";
                 break;
@@ -135,6 +129,9 @@ string optionString(size_t options) {
             case QueryPlannerParams::STRICT_DISTINCT_ONLY:
                 ss << "STRICT_DISTINCT_ONLY ";
                 break;
+            case QueryPlannerParams::PRESERVE_RECORD_ID:
+                ss << "PRESERVE_RECORD_ID ";
+                break;
             case QueryPlannerParams::DEFAULT:
                 MONGO_UNREACHABLE;
                 break;
@@ -145,7 +142,7 @@ string optionString(size_t options) {
 }
 
 static BSONObj getKeyFromQuery(const BSONObj& keyPattern, const BSONObj& query) {
-    return query.extractFieldsUnDotted(keyPattern);
+    return query.extractFieldsUndotted(keyPattern);
 }
 
 static bool indexCompatibleMaxMin(const BSONObj& obj,
@@ -303,7 +300,7 @@ StatusWith<std::unique_ptr<PlanCacheIndexTree>> QueryPlanner::cacheDataFromTagge
         return Status(ErrorCodes::BadValue, "Cannot produce cache data: tree is NULL.");
     }
 
-    auto indexTree = stdx::make_unique<PlanCacheIndexTree>();
+    auto indexTree = std::make_unique<PlanCacheIndexTree>();
 
     if (taggedTree->getTag() &&
         taggedTree->getTag()->getType() == MatchExpression::TagData::Type::IndexTag) {
@@ -341,7 +338,7 @@ StatusWith<std::unique_ptr<PlanCacheIndexTree>> QueryPlanner::cacheDataFromTagge
             }
 
             std::unique_ptr<IndexEntry> indexEntry =
-                stdx::make_unique<IndexEntry>(relevantIndices[itag->index]);
+                std::make_unique<IndexEntry>(relevantIndices[itag->index]);
             indexTree->entry.reset(indexEntry.release());
             indexTree->index_pos = itag->pos;
             indexTree->canCombineBounds = itag->canCombineBounds;
@@ -382,7 +379,7 @@ Status QueryPlanner::tagAccordingToCache(MatchExpression* filter,
 
     // We're tagging the tree here, so it shouldn't have
     // any tags hanging off yet.
-    verify(NULL == filter->getTag());
+    verify(nullptr == filter->getTag());
 
     if (filter->numChildren() != indexTree->children.size()) {
         str::stream ss;
@@ -411,7 +408,7 @@ Status QueryPlanner::tagAccordingToCache(MatchExpression* filter,
             }
             OrPushdownTag::Destination dest;
             dest.route = orPushdown.route;
-            dest.tagData = stdx::make_unique<IndexTag>(
+            dest.tagData = std::make_unique<IndexTag>(
                 index->second, orPushdown.position, orPushdown.canCombineBounds);
             orPushdownTag->addDestination(std::move(dest));
         }
@@ -478,10 +475,13 @@ StatusWith<std::unique_ptr<QuerySolution>> QueryPlanner::planFromCache(
     // Create a copy of the expression tree.  We use cachedSoln to annotate this with indices.
     unique_ptr<MatchExpression> clone = query.root()->shallowClone();
 
-    LOG(5) << "Tagging the match expression according to cache data: " << endl
-           << "Filter:" << endl
-           << redact(clone->debugString()) << "Cache data:" << endl
-           << redact(winnerCacheData.toString());
+    LOGV2_DEBUG(20963,
+                5,
+                "Tagging the match expression according to cache data: \nFilter:\n{filter}Cache "
+                "data:\n{cacheData}",
+                "Tagging the match expression according to cache data",
+                "filter"_attr = redact(clone->debugString()),
+                "cacheData"_attr = redact(winnerCacheData.toString()));
 
     stdx::unordered_set<string> fields;
     QueryPlannerIXSelect::getFields(query.root(), &fields);
@@ -495,7 +495,12 @@ StatusWith<std::unique_ptr<QuerySolution>> QueryPlanner::planFromCache(
         const auto insertionRes = indexMap.insert(std::make_pair(ie.identifier, i));
         // Be sure the key was not already in the map.
         invariant(insertionRes.second);
-        LOG(5) << "Index " << i << ": " << ie.identifier;
+        LOGV2_DEBUG(20964,
+                    5,
+                    "Index {indexNumber}: {id}",
+                    "Index mapping: number and identifier",
+                    "indexNumber"_attr = i,
+                    "id"_attr = ie.identifier);
     }
 
     Status s = tagAccordingToCache(clone.get(), winnerCacheData.tree.get(), indexMap);
@@ -506,7 +511,11 @@ StatusWith<std::unique_ptr<QuerySolution>> QueryPlanner::planFromCache(
     // The MatchExpression tree is in canonical order. We must order the nodes for access planning.
     prepareForAccessPlanning(clone.get());
 
-    LOG(5) << "Tagged tree:" << endl << redact(clone->debugString());
+    LOGV2_DEBUG(20965,
+                5,
+                "Tagged tree:\n{tree}",
+                "Tagged tree",
+                "tree"_attr = redact(clone->debugString()));
 
     // Use the cached index assignments to build solnRoot.
     std::unique_ptr<QuerySolutionNode> solnRoot(QueryPlannerAccess::buildIndexedDataAccess(
@@ -525,23 +534,32 @@ StatusWith<std::unique_ptr<QuerySolution>> QueryPlanner::planFromCache(
                           << "Failed to analyze plan from cache. Query: " << query.toStringShort());
     }
 
-    LOG(5) << "Planner: solution constructed from the cache:\n" << redact(soln->toString());
+    LOGV2_DEBUG(20966,
+                5,
+                "Planner: solution constructed from the cache:\n{solution}",
+                "Planner: solution constructed from the cache",
+                "solution"_attr = redact(soln->toString()));
     return {std::move(soln)};
 }
 
 // static
 StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
     const CanonicalQuery& query, const QueryPlannerParams& params) {
-    LOG(5) << "Beginning planning..." << endl
-           << "=============================" << endl
-           << "Options = " << optionString(params.options) << endl
-           << "Canonical query:" << endl
-           << redact(query.toString()) << "=============================";
-
-    std::vector<std::unique_ptr<QuerySolution>> out;
+    LOGV2_DEBUG(20967,
+                5,
+                "Beginning planning...\n=============================\nOptions = "
+                "{options}\nCanonical query:\n{query}=============================",
+                "Beginning planning",
+                "options"_attr = optionString(params.options),
+                "query"_attr = redact(query.toString()));
 
     for (size_t i = 0; i < params.indices.size(); ++i) {
-        LOG(5) << "Index " << i << " is " << params.indices[i].toString();
+        LOGV2_DEBUG(20968,
+                    5,
+                    "Index {indexNumber} is {value}",
+                    "Index number and details",
+                    "indexNumber"_attr = i,
+                    "index"_attr = params.indices[i].toString());
     }
 
     const bool canTableScan = !(params.options & QueryPlannerParams::NO_TABLE_SCAN);
@@ -550,37 +568,44 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
     // If the query requests a tailable cursor, the only solution is a collscan + filter with
     // tailable set on the collscan.
     if (isTailable) {
-        if (canTableScan) {
-            auto soln = buildCollscanSoln(query, isTailable, params);
-            if (soln) {
-                out.push_back(std::move(soln));
-            }
+        if (!canTableScan) {
+            return Status(
+                ErrorCodes::NoQueryExecutionPlans,
+                "Running with 'notablescan', so tailable cursors (which always do a table "
+                "scan) are not allowed");
         }
+        auto soln = buildCollscanSoln(query, isTailable, params);
+        if (!soln) {
+            return Status(ErrorCodes::NoQueryExecutionPlans,
+                          "Failed to build collection scan soln");
+        }
+        std::vector<std::unique_ptr<QuerySolution>> out;
+        out.push_back(std::move(soln));
         return {std::move(out)};
     }
 
-    // The hint or sort can be $natural: 1.  If this happens, output a collscan. If both
-    // a $natural hint and a $natural sort are specified, then the direction of the collscan
-    // is determined by the sign of the sort (not the sign of the hint).
-    if (!query.getQueryRequest().getHint().isEmpty() ||
-        !query.getQueryRequest().getSort().isEmpty()) {
-        BSONObj hintObj = query.getQueryRequest().getHint();
-        BSONObj sortObj = query.getQueryRequest().getSort();
-        BSONElement naturalHint = dps::extractElementAtPath(hintObj, "$natural");
-        BSONElement naturalSort = dps::extractElementAtPath(sortObj, "$natural");
-
-        // A hint overrides a $natural sort. This means that we don't force a table
-        // scan if there is a $natural sort with a non-$natural hint.
-        if (!naturalHint.eoo() || (!naturalSort.eoo() && hintObj.isEmpty())) {
-            LOG(5) << "Forcing a table scan due to hinted $natural";
-            // min/max are incompatible with $natural.
-            if (canTableScan && query.getQueryRequest().getMin().isEmpty() &&
-                query.getQueryRequest().getMax().isEmpty()) {
-                auto soln = buildCollscanSoln(query, isTailable, params);
-                if (soln) {
-                    out.push_back(std::move(soln));
-                }
+    // The hint can be {$natural: +/-1}. If this happens, output a collscan. We expect any $natural
+    // sort to have been normalized to a $natural hint upstream.
+    if (!query.getQueryRequest().getHint().isEmpty()) {
+        const BSONObj& hintObj = query.getQueryRequest().getHint();
+        if (hintObj[QueryRequest::kNaturalSortField]) {
+            LOGV2_DEBUG(20969, 5, "Forcing a table scan due to hinted $natural");
+            if (!canTableScan) {
+                return Status(ErrorCodes::NoQueryExecutionPlans,
+                              "hint $natural is not allowed, because 'notablescan' is enabled");
             }
+            if (!query.getQueryRequest().getMin().isEmpty() ||
+                !query.getQueryRequest().getMax().isEmpty()) {
+                return Status(ErrorCodes::NoQueryExecutionPlans,
+                              "min and max are incompatible with $natural");
+            }
+            auto soln = buildCollscanSoln(query, isTailable, params);
+            if (!soln) {
+                return Status(ErrorCodes::NoQueryExecutionPlans,
+                              "Failed to build collection scan soln");
+            }
+            std::vector<std::unique_ptr<QuerySolution>> out;
+            out.push_back(std::move(soln));
             return {std::move(out)};
         }
     }
@@ -624,7 +649,11 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
     stdx::unordered_set<string> fields;
     QueryPlannerIXSelect::getFields(query.root(), &fields);
     for (auto&& field : fields) {
-        LOG(5) << "Predicate over field '" << field << "'";
+        LOGV2_DEBUG(20970,
+                    5,
+                    "Predicate over field '{field}'",
+                    "Predicate over field",
+                    "field"_attr = field);
     }
 
     fullIndexList = QueryPlannerIXSelect::expandIndexes(fields, std::move(fullIndexList));
@@ -688,15 +717,22 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
         invariant(solnRoot);
 
         auto soln = QueryPlannerAnalysis::analyzeDataAccess(query, params, std::move(solnRoot));
-        if (soln) {
-            out.push_back(std::move(soln));
+        if (!soln) {
+            return Status(ErrorCodes::NoQueryExecutionPlans,
+                          "Sort and covering analysis failed while planning hint/min/max query");
         }
-
+        std::vector<std::unique_ptr<QuerySolution>> out;
+        out.push_back(std::move(soln));
         return {std::move(out)};
     }
 
     for (size_t i = 0; i < relevantIndices.size(); ++i) {
-        LOG(2) << "Relevant index " << i << " is " << relevantIndices[i].toString();
+        LOGV2_DEBUG(20971,
+                    2,
+                    "Relevant index {indexNumber} is {value}",
+                    "Relevant index",
+                    "indexNumber"_attr = i,
+                    "index"_attr = relevantIndices[i].toString());
     }
 
     // Figure out how useful each index is to each predicate.
@@ -718,26 +754,34 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
     }
 
     // query.root() is now annotated with RelevantTag(s).
-    LOG(5) << "Rated tree:" << endl << redact(query.root()->debugString());
+    LOGV2_DEBUG(20972,
+                5,
+                "Rated tree:\n{tree}",
+                "Rated tree",
+                "tree"_attr = redact(query.root()->debugString()));
 
     // If there is a GEO_NEAR it must have an index it can use directly.
-    const MatchExpression* gnNode = NULL;
+    const MatchExpression* gnNode = nullptr;
     if (QueryPlannerCommon::hasNode(query.root(), MatchExpression::GEO_NEAR, &gnNode)) {
         // No index for GEO_NEAR?  No query.
         RelevantTag* tag = static_cast<RelevantTag*>(gnNode->getTag());
         if (!tag || (0 == tag->first.size() && 0 == tag->notFirst.size())) {
-            LOG(5) << "Unable to find index for $geoNear query.";
+            LOGV2_DEBUG(20973, 5, "Unable to find index for $geoNear query");
             // Don't leave tags on query tree.
             query.root()->resetTag();
             return Status(ErrorCodes::NoQueryExecutionPlans,
                           "unable to find index for $geoNear query");
         }
 
-        LOG(5) << "Rated tree after geonear processing:" << redact(query.root()->debugString());
+        LOGV2_DEBUG(20974,
+                    5,
+                    "Rated tree after geonear processing:{tree}",
+                    "Rated tree after geonear processing",
+                    "tree"_attr = redact(query.root()->debugString()));
     }
 
     // Likewise, if there is a TEXT it must have an index it can use directly.
-    const MatchExpression* textNode = NULL;
+    const MatchExpression* textNode = nullptr;
     if (QueryPlannerCommon::hasNode(query.root(), MatchExpression::TEXT, &textNode)) {
         RelevantTag* tag = static_cast<RelevantTag*>(textNode->getTag());
 
@@ -770,9 +814,15 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
         // assigned to it.
         invariant(1 == tag->first.size() + tag->notFirst.size());
 
-        LOG(5) << "Rated tree after text processing:" << redact(query.root()->debugString());
+        LOGV2_DEBUG(20975,
+                    5,
+                    "Rated tree after text processing:{tree}",
+                    "Rated tree after text processing",
+                    "tree"_attr = redact(query.root()->debugString()));
     }
 
+    std::vector<std::unique_ptr<QuerySolution>> out;
+
     // If we have any relevant indices, we try to create indexed plans.
     if (0 < relevantIndices.size()) {
         // The enumerator spits out trees tagged with IndexTag(s).
@@ -786,8 +836,11 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
 
         unique_ptr<MatchExpression> nextTaggedTree;
         while ((nextTaggedTree = isp.getNext()) && (out.size() < params.maxIndexedSolutions)) {
-            LOG(5) << "About to build solntree from tagged tree:" << endl
-                   << redact(nextTaggedTree->debugString());
+            LOGV2_DEBUG(20976,
+                        5,
+                        "About to build solntree from tagged tree:\n{tree}",
+                        "About to build solntree from tagged tree",
+                        "tree"_attr = redact(nextTaggedTree->debugString()));
 
             // Store the plan cache index tree before calling prepareForAccessingPlanning(), so that
             // the PlanCacheIndexTree has the same sort as the MatchExpression used to generate the
@@ -796,8 +849,11 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
             std::unique_ptr<PlanCacheIndexTree> cacheData;
             auto statusWithCacheData = cacheDataFromTaggedTree(clone.get(), relevantIndices);
             if (!statusWithCacheData.isOK()) {
-                LOG(5) << "Query is not cachable: "
-                       << redact(statusWithCacheData.getStatus().reason());
+                LOGV2_DEBUG(20977,
+                            5,
+                            "Query is not cachable: {reason}",
+                            "Query is not cachable",
+                            "reason"_attr = redact(statusWithCacheData.getStatus().reason()));
             } else {
                 cacheData = std::move(statusWithCacheData.getValue());
             }
@@ -816,7 +872,11 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
 
             auto soln = QueryPlannerAnalysis::analyzeDataAccess(query, params, std::move(solnRoot));
             if (soln) {
-                LOG(5) << "Planner: adding solution:" << endl << redact(soln->toString());
+                LOGV2_DEBUG(20978,
+                            5,
+                            "Planner: adding solution:\n{solution}",
+                            "Planner: adding solution",
+                            "solution"_attr = redact(soln->toString()));
                 if (statusWithCacheData.isOK()) {
                     SolutionCacheData* scd = new SolutionCacheData();
                     scd->tree = std::move(cacheData);
@@ -830,11 +890,16 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
     // Don't leave tags on query tree.
     query.root()->resetTag();
 
-    LOG(5) << "Planner: outputted " << out.size() << " indexed solutions.";
+    LOGV2_DEBUG(20979,
+                5,
+                "Planner: outputted {numSolutions} indexed solutions",
+                "Planner: outputted indexed solutions",
+                "numSolutions"_attr = out.size());
 
     // Produce legible error message for failed OR planning with a TEXT child.
     // TODO: support collection scan for non-TEXT children of OR.
-    if (out.size() == 0 && textNode != NULL && MatchExpression::OR == query.root()->matchType()) {
+    if (out.size() == 0 && textNode != nullptr &&
+        MatchExpression::OR == query.root()->matchType()) {
         MatchExpression* root = query.root();
         for (size_t i = 0; i < root->numChildren(); ++i) {
             if (textNode == root->getChild(i)) {
@@ -850,23 +915,30 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
     // desired behavior when an index is hinted that is not relevant to the query. In the case that
     // $** index is hinted, we do not want this behavior.
     if (!hintedIndex.isEmpty() && relevantIndices.size() == 1) {
-        if (0 == out.size() && relevantIndices.front().type != IndexType::INDEX_WILDCARD) {
-            // Push hinted index solution to output list if found. It is possible to end up without
-            // a solution in the case where a filtering QueryPlannerParams argument, such as
-            // NO_BLOCKING_SORT, leads to its exclusion.
-            auto soln = buildWholeIXSoln(relevantIndices.front(), query, params);
-            if (soln) {
-                LOG(5) << "Planner: outputting soln that uses hinted index as scan.";
-                out.push_back(std::move(soln));
-            }
+        if (out.size() > 0) {
+            return {std::move(out)};
         }
+        if (relevantIndices.front().type == IndexType::INDEX_WILDCARD) {
+            return Status(
+                ErrorCodes::NoQueryExecutionPlans,
+                "$hint: refusing to build whole-index solution, because it's a wildcard index");
+        }
+        // Return hinted index solution if found.
+        auto soln = buildWholeIXSoln(relevantIndices.front(), query, params);
+        if (!soln) {
+            return Status(ErrorCodes::NoQueryExecutionPlans,
+                          "Failed to build whole-index solution for $hint");
+        }
+        LOGV2_DEBUG(20980, 5, "Planner: outputting soln that uses hinted index as scan");
+        std::vector<std::unique_ptr<QuerySolution>> out;
+        out.push_back(std::move(soln));
         return {std::move(out)};
     }
 
     // If a sort order is requested, there may be an index that provides it, even if that
     // index is not over any predicates in the query.
     //
-    if (!query.getQueryRequest().getSort().isEmpty() &&
+    if (query.getSortPattern() &&
         !QueryPlannerCommon::hasNode(query.root(), MatchExpression::GEO_NEAR) &&
         !QueryPlannerCommon::hasNode(query.root(), MatchExpression::TEXT)) {
         // See if we have a sort provided from an index already.
@@ -883,8 +955,8 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
         if (!usingIndexToSort) {
             for (size_t i = 0; i < fullIndexList.size(); ++i) {
                 const IndexEntry& index = fullIndexList[i];
-                // Only regular (non-plugin) indexes can be used to provide a sort, and only
-                // non-sparse indexes can be used to provide a sort.
+                // Only a regular index or the non-hashed prefix of a compound hashed index can be
+                // used to provide a sort. In addition, the index needs to be a non-sparse index.
                 //
                 // TODO: Sparse indexes can't normally provide a sort, because non-indexed
                 // documents could potentially be missing from the result set.  However, if the
@@ -897,7 +969,7 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
                 // - Index {a: 1, b: "2dsphere"} (which is "geo-sparse", if
                 //   2dsphereIndexVersion=2) should be able to provide a sort for
                 //   find({b: GEO}).sort({a:1}).  SERVER-10801.
-                if (index.type != INDEX_BTREE) {
+                if (index.type != INDEX_BTREE && index.type != INDEX_HASHED) {
                     continue;
                 }
                 if (index.sparse) {
@@ -918,7 +990,8 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
 
                 const BSONObj kp = QueryPlannerAnalysis::getSortPattern(index.keyPattern);
                 if (providesSort(query, kp)) {
-                    LOG(5) << "Planner: outputting soln that uses index to provide sort.";
+                    LOGV2_DEBUG(
+                        20981, 5, "Planner: outputting soln that uses index to provide sort");
                     auto soln = buildWholeIXSoln(fullIndexList[i], query, params);
                     if (soln) {
                         PlanCacheIndexTree* indexTree = new PlanCacheIndexTree();
@@ -930,12 +1003,13 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
 
                         soln->cacheData.reset(scd);
                         out.push_back(std::move(soln));
-                        break;
                     }
                 }
                 if (providesSort(query, QueryPlannerCommon::reverseSortObj(kp))) {
-                    LOG(5) << "Planner: outputting soln that uses (reverse) index "
-                           << "to provide sort.";
+                    LOGV2_DEBUG(
+                        20982,
+                        5,
+                        "Planner: outputting soln that uses (reverse) index to provide sort");
                     auto soln = buildWholeIXSoln(fullIndexList[i], query, params, -1);
                     if (soln) {
                         PlanCacheIndexTree* indexTree = new PlanCacheIndexTree();
@@ -947,7 +1021,6 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
 
                         soln->cacheData.reset(scd);
                         out.push_back(std::move(soln));
-                        break;
                     }
                 }
             }
@@ -968,11 +1041,10 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
             }
 
             QueryPlannerParams paramsForCoveredIxScan;
-            paramsForCoveredIxScan.options =
-                params.options | QueryPlannerParams::NO_UNCOVERED_PROJECTIONS;
             auto soln = buildWholeIXSoln(index, query, paramsForCoveredIxScan);
-            if (soln) {
-                LOG(5) << "Planner: outputting soln that uses index to provide projection.";
+            if (soln && !soln->root->fetched()) {
+                LOGV2_DEBUG(
+                    20983, 5, "Planner: outputting soln that uses index to provide projection");
                 PlanCacheIndexTree* indexTree = new PlanCacheIndexTree();
                 indexTree->setIndexEntry(index);
 
@@ -988,22 +1060,37 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
         }
     }
 
+    // The caller can explicitly ask for a collscan.
+    bool collscanRequested = (params.options & QueryPlannerParams::INCLUDE_COLLSCAN);
+
+    // No indexed plans?  We must provide a collscan if possible or else we can't run the query.
+    bool collScanRequired = 0 == out.size();
+    if (collScanRequired && !canTableScan) {
+        return Status(ErrorCodes::NoQueryExecutionPlans,
+                      "No indexed plans available, and running with 'notablescan'");
+    }
+
     // geoNear and text queries *require* an index.
     // Also, if a hint is specified it indicates that we MUST use it.
     bool possibleToCollscan =
         !QueryPlannerCommon::hasNode(query.root(), MatchExpression::GEO_NEAR) &&
         !QueryPlannerCommon::hasNode(query.root(), MatchExpression::TEXT) && hintedIndex.isEmpty();
+    if (collScanRequired && !possibleToCollscan) {
+        return Status(ErrorCodes::NoQueryExecutionPlans, "No query solutions");
+    }
 
-    // The caller can explicitly ask for a collscan.
-    bool collscanRequested = (params.options & QueryPlannerParams::INCLUDE_COLLSCAN);
-
-    // No indexed plans?  We must provide a collscan if possible or else we can't run the query.
-    bool collscanNeeded = (0 == out.size() && canTableScan);
-
-    if (possibleToCollscan && (collscanRequested || collscanNeeded)) {
+    if (possibleToCollscan && (collscanRequested || collScanRequired)) {
         auto collscan = buildCollscanSoln(query, isTailable, params);
+        if (!collscan && collScanRequired) {
+            return Status(ErrorCodes::NoQueryExecutionPlans,
+                          "Failed to build collection scan soln");
+        }
         if (collscan) {
-            LOG(5) << "Planner: outputting a collscan:" << endl << redact(collscan->toString());
+            LOGV2_DEBUG(20984,
+                        5,
+                        "Planner: outputting a collection scan:\n{collectionScan}",
+                        "Planner: outputting a collection scan",
+                        "collectionScan"_attr = redact(collscan->toString()));
             SolutionCacheData* scd = new SolutionCacheData();
             scd->solnType = SolutionCacheData::COLLSCAN_SOLN;
             collscan->cacheData.reset(scd);
@@ -1011,6 +1098,7 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
         }
     }
 
+    invariant(out.size() > 0);
     return {std::move(out)};
 }
 
