@@ -31,6 +31,8 @@
 
 #include "mongo/db/query/query_request.h"
 
+#include <memory>
+
 #include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
 #include "mongo/bson/simple_bsonobj_comparator.h"
@@ -40,7 +42,6 @@
 #include "mongo/db/dbmessage.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/repl/read_concern_args.h"
-#include "mongo/stdx/memory.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/str.h"
 
@@ -57,11 +58,12 @@ const char QueryRequest::queryOptionMaxTimeMS[] = "$maxTimeMS";
 
 const string QueryRequest::metaGeoNearDistance("geoNearDistance");
 const string QueryRequest::metaGeoNearPoint("geoNearPoint");
-const string QueryRequest::metaIndexKey("indexKey");
 const string QueryRequest::metaRecordId("recordId");
 const string QueryRequest::metaSortKey("sortKey");
 const string QueryRequest::metaTextScore("textScore");
 
+const string QueryRequest::kAllowDiskUseField("allowDiskUse");
+
 const long long QueryRequest::kDefaultBatchSize = 101;
 
 namespace {
@@ -77,38 +79,41 @@ Status checkFieldType(const BSONElement& el, BSONType type) {
     return Status::OK();
 }
 
+}  // namespace
+
 // Find command field names.
-const char kFilterField[] = "filter";
-const char kProjectionField[] = "projection";
-const char kSortField[] = "sort";
-const char kHintField[] = "hint";
-const char kCollationField[] = "collation";
-const char kSkipField[] = "skip";
-const char kLimitField[] = "limit";
-const char kBatchSizeField[] = "batchSize";
-const char kNToReturnField[] = "ntoreturn";
-const char kSingleBatchField[] = "singleBatch";
-const char kCommentField[] = "comment";
-const char kMaxField[] = "max";
-const char kMinField[] = "min";
-const char kReturnKeyField[] = "returnKey";
-const char kShowRecordIdField[] = "showRecordId";
-const char kTailableField[] = "tailable";
-const char kOplogReplayField[] = "oplogReplay";
-const char kNoCursorTimeoutField[] = "noCursorTimeout";
-const char kAwaitDataField[] = "awaitData";
-const char kPartialResultsField[] = "allowPartialResults";
-const char kRuntimeConstantsField[] = "runtimeConstants";
-const char kTermField[] = "term";
-const char kOptionsField[] = "options";
-const char kReadOnceField[] = "readOnce";
-const char kAllowSpeculativeMajorityReadField[] = "allowSpeculativeMajorityRead";
-const char kInternalReadAtClusterTimeField[] = "$_internalReadAtClusterTime";
+const char QueryRequest::kFilterField[] = "filter";
+const char QueryRequest::kProjectionField[] = "projection";
+const char QueryRequest::kSortField[] = "sort";
+const char QueryRequest::kHintField[] = "hint";
+const char QueryRequest::kCollationField[] = "collation";
+const char QueryRequest::kSkipField[] = "skip";
+const char QueryRequest::kLimitField[] = "limit";
+const char QueryRequest::kBatchSizeField[] = "batchSize";
+const char QueryRequest::kNToReturnField[] = "ntoreturn";
+const char QueryRequest::kSingleBatchField[] = "singleBatch";
+const char QueryRequest::kMaxField[] = "max";
+const char QueryRequest::kMinField[] = "min";
+const char QueryRequest::kReturnKeyField[] = "returnKey";
+const char QueryRequest::kShowRecordIdField[] = "showRecordId";
+const char QueryRequest::kTailableField[] = "tailable";
+const char QueryRequest::kOplogReplayField[] = "oplogReplay";
+const char QueryRequest::kNoCursorTimeoutField[] = "noCursorTimeout";
+const char QueryRequest::kAwaitDataField[] = "awaitData";
+const char QueryRequest::kPartialResultsField[] = "allowPartialResults";
+const char QueryRequest::kRuntimeConstantsField[] = "runtimeConstants";
+const char QueryRequest::kTermField[] = "term";
+const char QueryRequest::kOptionsField[] = "options";
+const char QueryRequest::kReadOnceField[] = "readOnce";
+const char QueryRequest::kAllowSpeculativeMajorityReadField[] = "allowSpeculativeMajorityRead";
+const char QueryRequest::kInternalReadAtClusterTimeField[] = "$_internalReadAtClusterTime";
+const char QueryRequest::kRequestResumeTokenField[] = "$_requestResumeToken";
+const char QueryRequest::kResumeAfterField[] = "$_resumeAfter";
+const char QueryRequest::kUse44SortKeys[] = "_use44SortKeys";
+const char QueryRequest::kMaxTimeMSOpOnlyField[] = "maxTimeMSOpOnly";
 
 // Field names for sorting options.
-const char kNaturalSortField[] = "$natural";
-
-}  // namespace
+const char QueryRequest::kNaturalSortField[] = "$natural";
 
 const char QueryRequest::kFindCommandName[] = "find";
 const char QueryRequest::kShardVersionField[] = "shardVersion";
@@ -119,7 +124,7 @@ QueryRequest::QueryRequest(NamespaceStringOrUUID nssOrUuid)
 void QueryRequest::refreshNSS(OperationContext* opCtx) {
     if (_uuid) {
         const CollectionCatalog& catalog = CollectionCatalog::get(opCtx);
-        auto foundColl = catalog.lookupCollectionByUUID(_uuid.get());
+        auto foundColl = catalog.lookupCollectionByUUID(opCtx, _uuid.get());
         uassert(ErrorCodes::NamespaceNotFound,
                 str::stream() << "UUID " << _uuid.get() << " specified in query request not found",
                 foundColl);
@@ -261,13 +266,13 @@ StatusWith<unique_ptr<QueryRequest>> QueryRequest::parseFromFindCommand(unique_p
             }
 
             qr->_wantMore = !el.boolean();
-        } else if (fieldName == kCommentField) {
-            Status status = checkFieldType(el, String);
+        } else if (fieldName == kAllowDiskUseField) {
+            Status status = checkFieldType(el, Bool);
             if (!status.isOK()) {
                 return status;
             }
 
-            qr->_comment = el.str();
+            qr->_allowDiskUse = el.boolean();
         } else if (fieldName == cmdOptionMaxTimeMS) {
             StatusWith<int> maxTimeMS = parseMaxTimeMS(el);
             if (!maxTimeMS.isOK()) {
@@ -396,6 +401,24 @@ StatusWith<unique_ptr<QueryRequest>> QueryRequest::parseFromFindCommand(unique_p
                 return status;
             }
             qr->_internalReadAtClusterTime = el.timestamp();
+        } else if (fieldName == kResumeAfterField) {
+            Status status = checkFieldType(el, Object);
+            if (!status.isOK()) {
+                return status;
+            }
+            qr->_resumeAfter = el.embeddedObject();
+        } else if (fieldName == kRequestResumeTokenField) {
+            Status status = checkFieldType(el, Bool);
+            if (!status.isOK()) {
+                return status;
+            }
+            qr->_requestResumeToken = el.boolean();
+        } else if (fieldName == kUse44SortKeys) {
+            Status status = checkFieldType(el, Bool);
+            if (!status.isOK()) {
+                return status;
+            }
+            qr->_use44SortKeys = el.boolean();
         } else if (!isGenericArgument(fieldName)) {
             return Status(ErrorCodes::FailedToParse,
                           str::stream() << "Failed to parse: " << cmdObj.toString() << ". "
@@ -424,10 +447,10 @@ StatusWith<unique_ptr<QueryRequest>> QueryRequest::makeFromFindCommand(Namespace
     BSONElement first = cmdObj.firstElement();
     if (first.type() == BinData && first.binDataType() == BinDataType::newUUID) {
         auto uuid = uassertStatusOK(UUID::parse(first));
-        auto qr = stdx::make_unique<QueryRequest>(NamespaceStringOrUUID(nss.db().toString(), uuid));
+        auto qr = std::make_unique<QueryRequest>(NamespaceStringOrUUID(nss.db().toString(), uuid));
         return parseFromFindCommand(std::move(qr), cmdObj, isExplain);
     } else {
-        auto qr = stdx::make_unique<QueryRequest>(nss);
+        auto qr = std::make_unique<QueryRequest>(nss);
         return parseFromFindCommand(std::move(qr), cmdObj, isExplain);
     }
 }
@@ -472,8 +495,8 @@ void QueryRequest::asFindCommandInternal(BSONObjBuilder* cmdBuilder) const {
         cmdBuilder->append(kHintField, _hint);
     }
 
-    if (!_readConcern.isEmpty()) {
-        cmdBuilder->append(repl::ReadConcernArgs::kReadConcernFieldName, _readConcern);
+    if (_readConcern) {
+        cmdBuilder->append(repl::ReadConcernArgs::kReadConcernFieldName, *_readConcern);
     }
 
     if (!_collation.isEmpty()) {
@@ -492,6 +515,10 @@ void QueryRequest::asFindCommandInternal(BSONObjBuilder* cmdBuilder) const {
         cmdBuilder->append(kLimitField, *_limit);
     }
 
+    if (_allowDiskUse) {
+        cmdBuilder->append(kAllowDiskUseField, true);
+    }
+
     if (_batchSize) {
         cmdBuilder->append(kBatchSizeField, *_batchSize);
     }
@@ -500,10 +527,6 @@ void QueryRequest::asFindCommandInternal(BSONObjBuilder* cmdBuilder) const {
         cmdBuilder->append(kSingleBatchField, true);
     }
 
-    if (!_comment.empty()) {
-        cmdBuilder->append(kCommentField, _comment);
-    }
-
     if (_maxTimeMS > 0) {
         cmdBuilder->append(cmdOptionMaxTimeMS, _maxTimeMS);
     }
@@ -572,19 +595,26 @@ void QueryRequest::asFindCommandInternal(BSONObjBuilder* cmdBuilder) const {
     if (_internalReadAtClusterTime) {
         cmdBuilder->append(kInternalReadAtClusterTimeField, *_internalReadAtClusterTime);
     }
-}
 
-void QueryRequest::addReturnKeyMetaProj() {
-    BSONObjBuilder projBob;
-    projBob.appendElements(_proj);
-    // We use $$ because it's never going to show up in a user's projection.
-    // The exact text doesn't matter.
-    BSONObj indexKey = BSON("$$" << BSON("$meta" << QueryRequest::metaIndexKey));
-    projBob.append(indexKey.firstElement());
-    _proj = projBob.obj();
+    if (_requestResumeToken) {
+        cmdBuilder->append(kRequestResumeTokenField, _requestResumeToken);
+    }
+
+    if (!_resumeAfter.isEmpty()) {
+        cmdBuilder->append(kResumeAfterField, _resumeAfter);
+    }
+
+    if (_use44SortKeys) {
+        cmdBuilder->append(kUse44SortKeys, true);
+    }
 }
 
 void QueryRequest::addShowRecordIdMetaProj() {
+    if (_proj["$recordId"]) {
+        // There's already some projection on $recordId. Don't overwrite it.
+        return;
+    }
+
     BSONObjBuilder projBob;
     projBob.appendElements(_proj);
     BSONObj metaRecordId = BSON("$recordId" << BSON("$meta" << QueryRequest::metaRecordId));
@@ -600,42 +630,11 @@ Status QueryRequest::validate() const {
         }
     }
 
-    // Can't combine a normal sort and a $meta projection on the same field.
-    BSONObjIterator projIt(_proj);
-    while (projIt.more()) {
-        BSONElement projElt = projIt.next();
-        if (isTextScoreMeta(projElt)) {
-            BSONElement sortElt = _sort[projElt.fieldName()];
-            if (!sortElt.eoo() && !isTextScoreMeta(sortElt)) {
-                return Status(ErrorCodes::BadValue,
-                              "can't have a non-$meta sort on a $meta projection");
-            }
-        }
-    }
-
-    if (!isValidSortOrder(_sort)) {
-        return Status(ErrorCodes::BadValue, "bad sort specification");
-    }
-
-    // All fields with a $meta sort must have a corresponding $meta projection.
-    BSONObjIterator sortIt(_sort);
-    while (sortIt.more()) {
-        BSONElement sortElt = sortIt.next();
-        if (isTextScoreMeta(sortElt)) {
-            BSONElement projElt = _proj[sortElt.fieldName()];
-            if (projElt.eoo() || !isTextScoreMeta(projElt)) {
-                return Status(ErrorCodes::BadValue,
-                              "must have $meta projection for all $meta sort keys");
-            }
-        }
-    }
-
     if ((_limit || _batchSize) && _ntoreturn) {
         return Status(ErrorCodes::BadValue,
                       "'limit' or 'batchSize' fields can not be set with 'ntoreturn' field.");
     }
 
-
     if (_skip && *_skip < 0) {
         return Status(ErrorCodes::BadValue,
                       str::stream() << "Skip value must be non-negative, but received: " << *_skip);
@@ -681,6 +680,29 @@ Status QueryRequest::validate() const {
         }
     }
 
+    if (_requestResumeToken) {
+        if (SimpleBSONObjComparator::kInstance.evaluate(_hint != BSON(kNaturalSortField << 1))) {
+            return Status(ErrorCodes::BadValue,
+                          "hint must be {$natural:1} if 'requestResumeToken' is enabled");
+        }
+        if (!_sort.isEmpty() &&
+            SimpleBSONObjComparator::kInstance.evaluate(_sort != BSON(kNaturalSortField << 1))) {
+            return Status(ErrorCodes::BadValue,
+                          "sort must be unset or {$natural:1} if 'requestResumeToken' is enabled");
+        }
+        if (!_resumeAfter.isEmpty()) {
+            if (_resumeAfter.nFields() != 1 ||
+                _resumeAfter["$recordId"].type() != BSONType::NumberLong) {
+                return Status(ErrorCodes::BadValue,
+                              "Malformed resume token: the '_resumeAfter' object must contain"
+                              " exactly one field named '$recordId', of type NumberLong.");
+            }
+        }
+    } else if (!_resumeAfter.isEmpty()) {
+        return Status(ErrorCodes::BadValue,
+                      "'requestResumeToken' must be true if 'resumeAfter' is"
+                      " specified");
+    }
     return Status::OK();
 }
 
@@ -707,7 +729,6 @@ StatusWith<int> QueryRequest::parseMaxTimeMS(BSONElement maxTimeMSElt) {
     return StatusWith<int>(static_cast<int>(maxTimeMSLongLong));
 }
 
-// static
 bool QueryRequest::isTextScoreMeta(BSONElement elt) {
     // elt must be foo: {$meta: "textScore"}
     if (mongo::Object != elt.type()) {
@@ -736,34 +757,13 @@ bool QueryRequest::isTextScoreMeta(BSONElement elt) {
     return true;
 }
 
-// static
-bool QueryRequest::isValidSortOrder(const BSONObj& sortObj) {
-    BSONObjIterator i(sortObj);
-    while (i.more()) {
-        BSONElement e = i.next();
-        // fieldNameSize() includes NULL terminator. For empty field name,
-        // we should be checking for 1 instead of 0.
-        if (1 == e.fieldNameSize()) {
-            return false;
-        }
-        if (isTextScoreMeta(e)) {
-            continue;
-        }
-        long long n = e.safeNumberLong();
-        if (!(e.isNumber() && (n == -1LL || n == 1LL))) {
-            return false;
-        }
-    }
-    return true;
-}
-
 //
 // Old QueryRequest parsing code: SOON TO BE DEPRECATED.
 //
 
 // static
 StatusWith<unique_ptr<QueryRequest>> QueryRequest::fromLegacyQueryMessage(const QueryMessage& qm) {
-    auto qr = stdx::make_unique<QueryRequest>(NamespaceString(qm.ns));
+    auto qr = std::make_unique<QueryRequest>(NamespaceString(qm.ns));
 
     Status status = qr->init(qm.ntoskip, qm.ntoreturn, qm.queryOptions, qm.query, qm.fields, true);
     if (!status.isOK()) {
@@ -779,7 +779,7 @@ StatusWith<unique_ptr<QueryRequest>> QueryRequest::fromLegacyQuery(NamespaceStri
                                                                    int ntoskip,
                                                                    int ntoreturn,
                                                                    int queryOptions) {
-    auto qr = stdx::make_unique<QueryRequest>(nsOrUuid);
+    auto qr = std::make_unique<QueryRequest>(nsOrUuid);
 
     Status status = qr->init(ntoskip, ntoreturn, queryOptions, queryObj, proj, true);
     if (!status.isOK()) {
@@ -836,6 +836,10 @@ Status QueryRequest::init(int ntoskip,
         } else {
             _filter = queryObj.getOwned();
         }
+        // It's not possible to specify readConcern in a legacy query message, so initialize it to
+        // an empty readConcern object, ie. equivalent to `readConcern: {}`.  This ensures that
+        // mongos passes this empty readConcern to shards.
+        _readConcern = BSONObj();
     } else {
         // This is the debugging code path.
         _filter = queryObj.getOwned();
@@ -919,7 +923,6 @@ Status QueryRequest::initFullQuery(const BSONObj& top) {
                 // Won't throw.
                 if (e.trueValue()) {
                     _returnKey = true;
-                    addReturnKeyMetaProj();
                 }
             } else if (name == "showDiskLoc") {
                 // Won't throw.
@@ -933,14 +936,6 @@ Status QueryRequest::initFullQuery(const BSONObj& top) {
                     return maxTimeMS.getStatus();
                 }
                 _maxTimeMS = maxTimeMS.getValue();
-            } else if (name == "comment") {
-                // Legacy $comment can be any BSON element. Convert to string if it isn't
-                // already.
-                if (e.type() == BSONType::String) {
-                    _comment = e.str();
-                } else {
-                    _comment = e.toString(false);
-                }
             }
         }
     }
@@ -986,11 +981,6 @@ void QueryRequest::initFromInt(int options) {
 }
 
 void QueryRequest::addMetaProjection() {
-    // We might need to update the projection object with a $meta projection.
-    if (returnKey()) {
-        addReturnKeyMetaProj();
-    }
-
     if (showRecordId()) {
         addShowRecordIdMetaProj();
     }
@@ -1073,6 +1063,18 @@ StatusWith<BSONObj> QueryRequest::asAggregationCommand() const {
                               << " not supported in aggregation."};
     }
 
+    if (_requestResumeToken) {
+        return {ErrorCodes::InvalidPipelineOperator,
+                str::stream() << "Option " << kRequestResumeTokenField
+                              << " not supported in aggregation."};
+    }
+
+    if (!_resumeAfter.isEmpty()) {
+        return {ErrorCodes::InvalidPipelineOperator,
+                str::stream() << "Option " << kResumeAfterField
+                              << " not supported in aggregation."};
+    }
+
     // Now that we've successfully validated this QR, begin building the aggregation command.
     aggregationBuilder.append("aggregate", _nss.coll());
 
@@ -1120,15 +1122,15 @@ StatusWith<BSONObj> QueryRequest::asAggregationCommand() const {
     if (!_hint.isEmpty()) {
         aggregationBuilder.append("hint", _hint);
     }
-    if (!_comment.empty()) {
-        aggregationBuilder.append("comment", _comment);
-    }
-    if (!_readConcern.isEmpty()) {
-        aggregationBuilder.append("readConcern", _readConcern);
+    if (_readConcern) {
+        aggregationBuilder.append("readConcern", *_readConcern);
     }
     if (!_unwrappedReadPref.isEmpty()) {
         aggregationBuilder.append(QueryRequest::kUnwrappedReadPrefField, _unwrappedReadPref);
     }
+    if (_allowDiskUse) {
+        aggregationBuilder.append(QueryRequest::kAllowDiskUseField, _allowDiskUse);
+    }
     if (_runtimeConstants) {
         BSONObjBuilder rtcBuilder(aggregationBuilder.subobjStart(kRuntimeConstantsField));
         _runtimeConstants->serialize(&rtcBuilder);
