@@ -27,12 +27,14 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kQuery
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/query/stage_builder.h"
 
+#include <memory>
+
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/index_catalog.h"
@@ -50,6 +52,7 @@
 #include "mongo/db/exec/merge_sort.h"
 #include "mongo/db/exec/or.h"
 #include "mongo/db/exec/projection.h"
+#include "mongo/db/exec/return_key.h"
 #include "mongo/db/exec/shard_filter.h"
 #include "mongo/db/exec/skip.h"
 #include "mongo/db/exec/sort.h"
@@ -58,20 +61,20 @@
 #include "mongo/db/index/fts_access_method.h"
 #include "mongo/db/matcher/extensions_callback_real.h"
 #include "mongo/db/s/collection_sharding_state.h"
-#include "mongo/stdx/memory.h"
-#include "mongo/util/log.h"
+#include "mongo/db/storage/oplog_hack.h"
+#include "mongo/logv2/log.h"
 
 namespace mongo {
 
-using std::unique_ptr;
-using stdx::make_unique;
-
-PlanStage* buildStages(OperationContext* opCtx,
-                       const Collection* collection,
-                       const CanonicalQuery& cq,
-                       const QuerySolution& qsol,
-                       const QuerySolutionNode* root,
-                       WorkingSet* ws) {
+// Returns a non-null pointer to the root of a plan tree, or a non-OK status if the PlanStage tree
+// could not be constructed.
+std::unique_ptr<PlanStage> buildStages(OperationContext* opCtx,
+                                       const Collection* collection,
+                                       const CanonicalQuery& cq,
+                                       const QuerySolution& qsol,
+                                       const QuerySolutionNode* root,
+                                       WorkingSet* ws) {
+    auto* const expCtx = cq.getExpCtx().get();
     switch (root->getType()) {
         case STAGE_COLLSCAN: {
             const CollectionScanNode* csn = static_cast<const CollectionScanNode*>(root);
@@ -81,16 +84,18 @@ PlanStage* buildStages(OperationContext* opCtx,
             params.direction = (csn->direction == 1) ? CollectionScanParams::FORWARD
                                                      : CollectionScanParams::BACKWARD;
             params.shouldWaitForOplogVisibility = csn->shouldWaitForOplogVisibility;
-            return new CollectionScan(opCtx, collection, params, ws, csn->filter.get());
+            params.minTs = csn->minTs;
+            params.maxTs = csn->maxTs;
+            params.requestResumeToken = csn->requestResumeToken;
+            params.resumeAfterRecordId = csn->resumeAfterRecordId;
+            params.stopApplyingFilterAfterFirstMatch = csn->stopApplyingFilterAfterFirstMatch;
+            return std::make_unique<CollectionScan>(
+                expCtx, collection, params, ws, csn->filter.get());
         }
         case STAGE_IXSCAN: {
             const IndexScanNode* ixn = static_cast<const IndexScanNode*>(root);
 
-            if (nullptr == collection) {
-                warning() << "Can't ixscan null namespace";
-                return nullptr;
-            }
-
+            invariant(collection);
             auto descriptor = collection->getIndexCatalog()->findIndexByName(
                 opCtx, ixn->index.identifier.catalogName);
             invariant(descriptor,
@@ -109,124 +114,118 @@ PlanStage* buildStages(OperationContext* opCtx,
             params.direction = ixn->direction;
             params.addKeyMetadata = ixn->addKeyMetadata;
             params.shouldDedup = ixn->shouldDedup;
-            return new IndexScan(opCtx, std::move(params), ws, ixn->filter.get());
+            return std::make_unique<IndexScan>(expCtx, std::move(params), ws, ixn->filter.get());
         }
         case STAGE_FETCH: {
             const FetchNode* fn = static_cast<const FetchNode*>(root);
-            PlanStage* childStage = buildStages(opCtx, collection, cq, qsol, fn->children[0], ws);
-            if (nullptr == childStage) {
-                return nullptr;
-            }
-            return new FetchStage(opCtx, ws, childStage, fn->filter.get(), collection);
+            auto childStage = buildStages(opCtx, collection, cq, qsol, fn->children[0], ws);
+            return std::make_unique<FetchStage>(
+                expCtx, ws, std::move(childStage), fn->filter.get(), collection);
         }
-        case STAGE_SORT: {
-            const SortNode* sn = static_cast<const SortNode*>(root);
-            PlanStage* childStage = buildStages(opCtx, collection, cq, qsol, sn->children[0], ws);
-            if (nullptr == childStage) {
-                return nullptr;
-            }
-            SortStageParams params;
-            params.pattern = sn->pattern;
-            params.limit = sn->limit;
-            return new SortStage(opCtx, params, ws, childStage);
+        case STAGE_SORT_DEFAULT: {
+            auto snDefault = static_cast<const SortNodeDefault*>(root);
+            auto childStage = buildStages(opCtx, collection, cq, qsol, snDefault->children[0], ws);
+            return std::make_unique<SortStageDefault>(
+                cq.getExpCtx(),
+                ws,
+                SortPattern{snDefault->pattern, cq.getExpCtx()},
+                snDefault->limit,
+                internalQueryMaxBlockingSortMemoryUsageBytes.load(),
+                snDefault->addSortKeyMetadata,
+                std::move(childStage));
+        }
+        case STAGE_SORT_SIMPLE: {
+            auto snSimple = static_cast<const SortNodeSimple*>(root);
+            auto childStage = buildStages(opCtx, collection, cq, qsol, snSimple->children[0], ws);
+            return std::make_unique<SortStageSimple>(
+                cq.getExpCtx(),
+                ws,
+                SortPattern{snSimple->pattern, cq.getExpCtx()},
+                snSimple->limit,
+                internalQueryMaxBlockingSortMemoryUsageBytes.load(),
+                snSimple->addSortKeyMetadata,
+                std::move(childStage));
         }
         case STAGE_SORT_KEY_GENERATOR: {
             const SortKeyGeneratorNode* keyGenNode = static_cast<const SortKeyGeneratorNode*>(root);
-            PlanStage* childStage =
-                buildStages(opCtx, collection, cq, qsol, keyGenNode->children[0], ws);
-            if (nullptr == childStage) {
-                return nullptr;
-            }
-            return new SortKeyGeneratorStage(
-                opCtx, childStage, ws, keyGenNode->sortSpec, cq.getCollator());
+            auto childStage = buildStages(opCtx, collection, cq, qsol, keyGenNode->children[0], ws);
+            return std::make_unique<SortKeyGeneratorStage>(
+                cq.getExpCtx(), std::move(childStage), ws, keyGenNode->sortSpec);
+        }
+        case STAGE_RETURN_KEY: {
+            auto returnKeyNode = static_cast<const ReturnKeyNode*>(root);
+            auto childStage =
+                buildStages(opCtx, collection, cq, qsol, returnKeyNode->children[0], ws);
+            return std::make_unique<ReturnKeyStage>(expCtx,
+                                                    std::move(returnKeyNode->sortKeyMetaFields),
+                                                    ws,
+                                                    cq.getExpCtx()->sortKeyFormat,
+                                                    std::move(childStage));
         }
         case STAGE_PROJECTION_DEFAULT: {
             auto pn = static_cast<const ProjectionNodeDefault*>(root);
-            unique_ptr<PlanStage> childStage{
-                buildStages(opCtx, collection, cq, qsol, pn->children[0], ws)};
-            if (nullptr == childStage) {
-                return nullptr;
-            }
-            return new ProjectionStageDefault(opCtx,
-                                              pn->projection,
-                                              ws,
-                                              std::move(childStage),
-                                              pn->fullExpression,
-                                              cq.getCollator());
+            auto childStage = buildStages(opCtx, collection, cq, qsol, pn->children[0], ws);
+            return std::make_unique<ProjectionStageDefault>(cq.getExpCtx(),
+                                                            cq.getQueryRequest().getProj(),
+                                                            cq.getProj(),
+                                                            ws,
+                                                            std::move(childStage));
         }
         case STAGE_PROJECTION_COVERED: {
             auto pn = static_cast<const ProjectionNodeCovered*>(root);
-            unique_ptr<PlanStage> childStage{
-                buildStages(opCtx, collection, cq, qsol, pn->children[0], ws)};
-            if (nullptr == childStage) {
-                return nullptr;
-            }
-            return new ProjectionStageCovered(
-                opCtx, pn->projection, ws, std::move(childStage), pn->coveredKeyObj);
+            auto childStage = buildStages(opCtx, collection, cq, qsol, pn->children[0], ws);
+            return std::make_unique<ProjectionStageCovered>(cq.getExpCtx().get(),
+                                                            cq.getQueryRequest().getProj(),
+                                                            cq.getProj(),
+                                                            ws,
+                                                            std::move(childStage),
+                                                            pn->coveredKeyObj);
         }
         case STAGE_PROJECTION_SIMPLE: {
             auto pn = static_cast<const ProjectionNodeSimple*>(root);
-            unique_ptr<PlanStage> childStage{
-                buildStages(opCtx, collection, cq, qsol, pn->children[0], ws)};
-            if (nullptr == childStage) {
-                return nullptr;
-            }
-            return new ProjectionStageSimple(opCtx, pn->projection, ws, std::move(childStage));
+            auto childStage = buildStages(opCtx, collection, cq, qsol, pn->children[0], ws);
+            return std::make_unique<ProjectionStageSimple>(cq.getExpCtx().get(),
+                                                           cq.getQueryRequest().getProj(),
+                                                           cq.getProj(),
+                                                           ws,
+                                                           std::move(childStage));
         }
         case STAGE_LIMIT: {
             const LimitNode* ln = static_cast<const LimitNode*>(root);
-            PlanStage* childStage = buildStages(opCtx, collection, cq, qsol, ln->children[0], ws);
-            if (nullptr == childStage) {
-                return nullptr;
-            }
-            return new LimitStage(opCtx, ln->limit, ws, childStage);
+            auto childStage = buildStages(opCtx, collection, cq, qsol, ln->children[0], ws);
+            return std::make_unique<LimitStage>(expCtx, ln->limit, ws, std::move(childStage));
         }
         case STAGE_SKIP: {
             const SkipNode* sn = static_cast<const SkipNode*>(root);
-            PlanStage* childStage = buildStages(opCtx, collection, cq, qsol, sn->children[0], ws);
-            if (nullptr == childStage) {
-                return nullptr;
-            }
-            return new SkipStage(opCtx, sn->skip, ws, childStage);
+            auto childStage = buildStages(opCtx, collection, cq, qsol, sn->children[0], ws);
+            return std::make_unique<SkipStage>(expCtx, sn->skip, ws, std::move(childStage));
         }
         case STAGE_AND_HASH: {
             const AndHashNode* ahn = static_cast<const AndHashNode*>(root);
-            auto ret = make_unique<AndHashStage>(opCtx, ws);
+            auto ret = std::make_unique<AndHashStage>(expCtx, ws);
             for (size_t i = 0; i < ahn->children.size(); ++i) {
-                PlanStage* childStage =
-                    buildStages(opCtx, collection, cq, qsol, ahn->children[i], ws);
-                if (nullptr == childStage) {
-                    return nullptr;
-                }
-                ret->addChild(childStage);
+                auto childStage = buildStages(opCtx, collection, cq, qsol, ahn->children[i], ws);
+                ret->addChild(std::move(childStage));
             }
-            return ret.release();
+            return ret;
         }
         case STAGE_OR: {
             const OrNode* orn = static_cast<const OrNode*>(root);
-            auto ret = make_unique<OrStage>(opCtx, ws, orn->dedup, orn->filter.get());
+            auto ret = std::make_unique<OrStage>(expCtx, ws, orn->dedup, orn->filter.get());
             for (size_t i = 0; i < orn->children.size(); ++i) {
-                PlanStage* childStage =
-                    buildStages(opCtx, collection, cq, qsol, orn->children[i], ws);
-                if (nullptr == childStage) {
-                    return nullptr;
-                }
-                ret->addChild(childStage);
+                auto childStage = buildStages(opCtx, collection, cq, qsol, orn->children[i], ws);
+                ret->addChild(std::move(childStage));
             }
-            return ret.release();
+            return ret;
         }
         case STAGE_AND_SORTED: {
             const AndSortedNode* asn = static_cast<const AndSortedNode*>(root);
-            auto ret = make_unique<AndSortedStage>(opCtx, ws);
+            auto ret = std::make_unique<AndSortedStage>(expCtx, ws);
             for (size_t i = 0; i < asn->children.size(); ++i) {
-                PlanStage* childStage =
-                    buildStages(opCtx, collection, cq, qsol, asn->children[i], ws);
-                if (nullptr == childStage) {
-                    return nullptr;
-                }
-                ret->addChild(childStage);
+                auto childStage = buildStages(opCtx, collection, cq, qsol, asn->children[i], ws);
+                ret->addChild(std::move(childStage));
             }
-            return ret.release();
+            return ret;
         }
         case STAGE_SORT_MERGE: {
             const MergeSortNode* msn = static_cast<const MergeSortNode*>(root);
@@ -234,16 +233,12 @@ PlanStage* buildStages(OperationContext* opCtx,
             params.dedup = msn->dedup;
             params.pattern = msn->sort;
             params.collator = cq.getCollator();
-            auto ret = make_unique<MergeSortStage>(opCtx, params, ws);
+            auto ret = std::make_unique<MergeSortStage>(expCtx, params, ws);
             for (size_t i = 0; i < msn->children.size(); ++i) {
-                PlanStage* childStage =
-                    buildStages(opCtx, collection, cq, qsol, msn->children[i], ws);
-                if (nullptr == childStage) {
-                    return nullptr;
-                }
-                ret->addChild(childStage);
+                auto childStage = buildStages(opCtx, collection, cq, qsol, msn->children[i], ws);
+                ret->addChild(std::move(childStage));
             }
-            return ret.release();
+            return ret;
         }
         case STAGE_GEO_NEAR_2D: {
             const GeoNear2DNode* node = static_cast<const GeoNear2DNode*>(root);
@@ -255,13 +250,12 @@ PlanStage* buildStages(OperationContext* opCtx,
             params.addPointMeta = node->addPointMeta;
             params.addDistMeta = node->addDistMeta;
 
+            invariant(collection);
             const IndexDescriptor* twoDIndex = collection->getIndexCatalog()->findIndexByName(
                 opCtx, node->index.identifier.catalogName);
             invariant(twoDIndex);
 
-            GeoNear2DStage* nearStage = new GeoNear2DStage(params, opCtx, ws, twoDIndex);
-
-            return nearStage;
+            return std::make_unique<GeoNear2DStage>(params, expCtx, ws, twoDIndex);
         }
         case STAGE_GEO_NEAR_2DSPHERE: {
             const GeoNear2DSphereNode* node = static_cast<const GeoNear2DSphereNode*>(root);
@@ -273,14 +267,16 @@ PlanStage* buildStages(OperationContext* opCtx,
             params.addPointMeta = node->addPointMeta;
             params.addDistMeta = node->addDistMeta;
 
+            invariant(collection);
             const IndexDescriptor* s2Index = collection->getIndexCatalog()->findIndexByName(
                 opCtx, node->index.identifier.catalogName);
             invariant(s2Index);
 
-            return new GeoNear2DSphereStage(params, opCtx, ws, s2Index);
+            return std::make_unique<GeoNear2DSphereStage>(params, expCtx, ws, s2Index);
         }
         case STAGE_TEXT: {
             const TextNode* node = static_cast<const TextNode*>(root);
+            invariant(collection);
             const IndexDescriptor* desc = collection->getIndexCatalog()->findIndexByName(
                 opCtx, node->index.identifier.catalogName);
             invariant(desc);
@@ -292,32 +288,24 @@ PlanStage* buildStages(OperationContext* opCtx,
             params.index = desc;
             params.indexPrefix = node->indexPrefix;
             // We assume here that node->ftsQuery is an FTSQueryImpl, not an FTSQueryNoop. In
-            // practice,
-            // this means that it is illegal to use the StageBuilder on a QuerySolution created by
-            // planning a query that contains "no-op" expressions. TODO: make StageBuilder::build()
-            // fail in this case (this improvement is being tracked by SERVER-21510).
+            // practice, this means that it is illegal to use the StageBuilder on a QuerySolution
+            // created by planning a query that contains "no-op" expressions.
             params.query = static_cast<FTSQueryImpl&>(*node->ftsQuery);
-            params.wantTextScore = (cq.getProj() && cq.getProj()->wantTextScore());
-            return new TextStage(opCtx, params, ws, node->filter.get());
+            params.wantTextScore = cq.metadataDeps()[DocumentMetadataFields::kTextScore];
+            return std::make_unique<TextStage>(expCtx, params, ws, node->filter.get());
         }
         case STAGE_SHARDING_FILTER: {
             const ShardingFilterNode* fn = static_cast<const ShardingFilterNode*>(root);
-            PlanStage* childStage = buildStages(opCtx, collection, cq, qsol, fn->children[0], ws);
-            if (nullptr == childStage) {
-                return nullptr;
-            }
+            auto childStage = buildStages(opCtx, collection, cq, qsol, fn->children[0], ws);
 
             auto css = CollectionShardingState::get(opCtx, collection->ns());
-            return new ShardFilterStage(opCtx, css->getOrphansFilter(opCtx), ws, childStage);
+            return std::make_unique<ShardFilterStage>(
+                expCtx, css->getOwnershipFilter(opCtx), ws, std::move(childStage));
         }
         case STAGE_DISTINCT_SCAN: {
             const DistinctNode* dn = static_cast<const DistinctNode*>(root);
 
-            if (nullptr == collection) {
-                warning() << "Can't distinct-scan null namespace";
-                return nullptr;
-            }
-
+            invariant(collection);
             auto descriptor = collection->getIndexCatalog()->findIndexByName(
                 opCtx, dn->index.identifier.catalogName);
             invariant(descriptor);
@@ -333,16 +321,12 @@ PlanStage* buildStages(OperationContext* opCtx,
             params.scanDirection = dn->direction;
             params.bounds = dn->bounds;
             params.fieldNo = dn->fieldNo;
-            return new DistinctScan(opCtx, std::move(params), ws);
+            return std::make_unique<DistinctScan>(expCtx, std::move(params), ws);
         }
         case STAGE_COUNT_SCAN: {
             const CountScanNode* csn = static_cast<const CountScanNode*>(root);
 
-            if (nullptr == collection) {
-                warning() << "Can't fast-count null namespace (collection null)";
-                return nullptr;
-            }
-
+            invariant(collection);
             auto descriptor = collection->getIndexCatalog()->findIndexByName(
                 opCtx, csn->index.identifier.catalogName);
             invariant(descriptor);
@@ -359,15 +343,13 @@ PlanStage* buildStages(OperationContext* opCtx,
             params.startKeyInclusive = csn->startKeyInclusive;
             params.endKey = csn->endKey;
             params.endKeyInclusive = csn->endKeyInclusive;
-            return new CountScan(opCtx, std::move(params), ws);
+            return std::make_unique<CountScan>(expCtx, std::move(params), ws);
         }
         case STAGE_ENSURE_SORTED: {
             const EnsureSortedNode* esn = static_cast<const EnsureSortedNode*>(root);
-            PlanStage* childStage = buildStages(opCtx, collection, cq, qsol, esn->children[0], ws);
-            if (nullptr == childStage) {
-                return nullptr;
-            }
-            return new EnsureSortedStage(opCtx, esn->pattern, ws, childStage);
+            auto childStage = buildStages(opCtx, collection, cq, qsol, esn->children[0], ws);
+            return std::make_unique<EnsureSortedStage>(
+                expCtx, esn->pattern, ws, std::move(childStage));
         }
         case STAGE_CACHED_PLAN:
         case STAGE_CHANGE_STREAM_PROXY:
@@ -386,36 +368,32 @@ PlanStage* buildStages(OperationContext* opCtx,
         case STAGE_TRIAL:
         case STAGE_UNKNOWN:
         case STAGE_UPDATE: {
-            str::stream ss;
-            root->appendToString(&ss, 0);
-            string nodeStr(ss);
-            warning() << "Can't build exec tree for node " << nodeStr << endl;
+            LOGV2_WARNING(4615604,
+                          "Can't build exec tree for node {node}",
+                          "Can't build exec tree for node",
+                          "node"_attr = *root);
         }
     }
-    return nullptr;
+
+    MONGO_UNREACHABLE;
 }
 
-// static (this one is used for Cached and MultiPlanStage)
-bool StageBuilder::build(OperationContext* opCtx,
-                         const Collection* collection,
-                         const CanonicalQuery& cq,
-                         const QuerySolution& solution,
-                         WorkingSet* wsIn,
-                         PlanStage** rootOut) {
+std::unique_ptr<PlanStage> StageBuilder::build(OperationContext* opCtx,
+                                               const Collection* collection,
+                                               const CanonicalQuery& cq,
+                                               const QuerySolution& solution,
+                                               WorkingSet* wsIn) {
     // Only QuerySolutions derived from queries parsed with context, or QuerySolutions derived from
     // queries that disallow extensions, can be properly executed. If the query does not have
     // $text/$where context (and $text/$where are allowed), then no attempt should be made to
     // execute the query.
     invariant(!cq.canHaveNoopMatchNodes());
 
-    if (nullptr == wsIn || nullptr == rootOut) {
-        return false;
-    }
+    invariant(wsIn);
+    invariant(solution.root);
+
     QuerySolutionNode* solutionNode = solution.root.get();
-    if (nullptr == solutionNode) {
-        return false;
-    }
-    return nullptr != (*rootOut = buildStages(opCtx, collection, cq, solution, solutionNode, wsIn));
+    return buildStages(opCtx, collection, cq, solution, solutionNode, wsIn);
 }
 
 }  // namespace mongo
