@@ -42,6 +42,7 @@
 #include "mongo/db/repl/replication_coordinator_test_fixture.h"
 #include "mongo/executor/network_interface_mock.h"
 #include "mongo/logv2/log.h"
+#include "mongo/stdx/future.h"
 #include "mongo/unittest/log_test.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/fail_point.h"
@@ -329,7 +330,7 @@ void doReplSetReconfig(ReplicationCoordinatorImpl* replCoord,
     *status = replCoord->processReplSetReconfig(opCtx, args, &garbage);
 }
 
-TEST_F(ReplCoordTest, QuorumCheckFailsWhenOtherNodesHaveHigherTerm) {
+TEST_F(ReplCoordTest, QuorumCheckSucceedsWhenOtherNodesHaveHigherTerm) {
     // Initiate a config without a term to avoid stepup auto-reconfig.
     auto configObj =
         configWithMembers(2, -1, BSON_ARRAY(member(1, "node1:12345") << member(2, "node2:12345")));
@@ -356,7 +357,9 @@ TEST_F(ReplCoordTest, QuorumCheckFailsWhenOtherNodesHaveHigherTerm) {
     repl::ReplSetHeartbeatResponse hbResp;
     hbResp.setSetName("mySet");
     hbResp.setState(MemberState::RS_SECONDARY);
-    // The config version from the remote is lower, but its term is higher.
+    // The config version from the remote is lower, but its term is higher. The quorum checker
+    // does not compare config versions or terms of remotes nodes, so this should always
+    // succeed.
     hbResp.setConfigVersion(2);
     hbResp.setConfigTerm(getReplCoord()->getTerm() + 1);
     hbResp.setAppliedOpTimeAndWallTime({opTime, net->now()});
@@ -365,7 +368,7 @@ TEST_F(ReplCoordTest, QuorumCheckFailsWhenOtherNodesHaveHigherTerm) {
     net->runReadyNetworkOperations();
     net->exitNetwork();
     reconfigThread.join();
-    ASSERT_EQUALS(ErrorCodes::NewReplicaSetConfigurationIncompatible, status);
+    ASSERT_OK(status);
 }
 
 TEST_F(ReplCoordTest, QuorumCheckSucceedsWhenOtherNodesHaveLowerTerm) {
@@ -395,7 +398,9 @@ TEST_F(ReplCoordTest, QuorumCheckSucceedsWhenOtherNodesHaveLowerTerm) {
     repl::ReplSetHeartbeatResponse hbResp;
     hbResp.setSetName("mySet");
     hbResp.setState(MemberState::RS_SECONDARY);
-    // The config version from the remote is higher, but its term is lower.
+    // The config version from the remote is higher, but its term is lower. The quorum checker
+    // does not compare config versions or terms of remotes nodes, so this should always
+    // succeed.
     hbResp.setConfigVersion(4);
     hbResp.setConfigTerm(getReplCoord()->getTerm() - 1);
     hbResp.setAppliedOpTimeAndWallTime({opTime, net->now()});
@@ -1385,9 +1390,9 @@ TEST_F(ReplCoordReconfigTest, StepdownShouldInterruptConfigWrite) {
     BSONObjBuilder result;
     Status status(ErrorCodes::InternalError, "Not Set");
     const auto opCtx = makeOperationContext();
-    stdx::thread reconfigThread;
-    reconfigThread = stdx::thread(
-        [&] { status = getReplCoord()->processReplSetReconfig(opCtx.get(), args, &result); });
+    auto reconfigResult = stdx::async(stdx::launch::async, [&] {
+        status = getReplCoord()->processReplSetReconfig(opCtx.get(), args, &result);
+    });
 
     getNet()->enterNetwork();
     // Wait for the next heartbeat of quorum check and blackhole it.
@@ -1403,10 +1408,13 @@ TEST_F(ReplCoordReconfigTest, StepdownShouldInterruptConfigWrite) {
     ASSERT(updateTermEvh.isValid());
     getReplExec()->waitForEvent(updateTermEvh);
 
-    // Respond to quorum check to resume the reconfig.
-    respondToAllHeartbeats();
+    // Respond to quorum check to resume the reconfig. We keep responding until the reconfig thread
+    // finishes.
+    while (stdx::future_status::ready !=
+           reconfigResult.wait_for(Milliseconds::zero().toSystemDuration())) {
+        respondToAllHeartbeats();
+    }
 
-    reconfigThread.join();
     ASSERT_EQ(status.code(), ErrorCodes::NotMaster);
     ASSERT_EQ(status.reason(), "Stepped down when persisting new config");
 }
