@@ -26,13 +26,14 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kFTDC
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kFTDC
 
 #include "mongo/platform/basic.h"
 
 #include <list>
 #include <vector>
 
+#include "mongo/bson/util/bson_extract.h"
 #include "mongo/client/connpool.h"
 #include "mongo/client/dbclient_connection.h"
 #include "mongo/db/auth/sasl_mechanism_registry.h"
@@ -49,22 +50,29 @@
 #include "mongo/db/query/internal_plans.h"
 #include "mongo/db/repl/is_master_response.h"
 #include "mongo/db/repl/oplog.h"
-#include "mongo/db/repl/oplogreader.h"
+#include "mongo/db/repl/replication_auth.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_process.h"
+#include "mongo/db/repl/speculative_auth.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/storage/storage_options.h"
 #include "mongo/db/wire_version.h"
 #include "mongo/executor/network_interface.h"
+#include "mongo/logv2/log.h"
 #include "mongo/rpc/metadata/client_metadata.h"
 #include "mongo/rpc/metadata/client_metadata_ismaster.h"
-#include "mongo/util/fail_point_service.h"
-#include "mongo/util/log.h"
+#include "mongo/transport/ismaster_metrics.h"
+#include "mongo/util/decimal_counter.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/map_util.h"
 
 namespace mongo {
 
+// Hangs in the beginning of each isMaster command when set.
 MONGO_FAIL_POINT_DEFINE(waitInIsMaster);
+// Awaitable isMaster requests with the proper topologyVersions will sleep for maxAwaitTimeMS on
+// standalones. This failpoint will hang right before doing this sleep when set.
+MONGO_FAIL_POINT_DEFINE(hangWaitingForIsMasterResponseOnStandalone);
 
 using std::list;
 using std::string;
@@ -73,17 +81,59 @@ using std::unique_ptr;
 
 namespace repl {
 namespace {
-void appendReplicationInfo(OperationContext* opCtx, BSONObjBuilder& result, int level) {
+/**
+ * Appends replication-related fields to the isMaster response. Returns the topology version that
+ * was included in the response.
+ */
+TopologyVersion appendReplicationInfo(OperationContext* opCtx,
+                                      BSONObjBuilder& result,
+                                      int level,
+                                      boost::optional<TopologyVersion> clientTopologyVersion,
+                                      boost::optional<long long> maxAwaitTimeMS) {
+    TopologyVersion topologyVersion;
     ReplicationCoordinator* replCoord = ReplicationCoordinator::get(opCtx);
     if (replCoord->getSettings().usingReplSets()) {
         const auto& horizonParams = SplitHorizon::getParameters(opCtx->getClient());
-        IsMasterResponse isMasterResponse;
-        replCoord->fillIsMasterForReplSet(&isMasterResponse, horizonParams);
-        result.appendElements(isMasterResponse.toBSON());
+
+        boost::optional<Date_t> deadline;
+        if (maxAwaitTimeMS) {
+            deadline = opCtx->getServiceContext()->getPreciseClockSource()->now() +
+                Milliseconds(*maxAwaitTimeMS);
+        }
+        auto isMasterResponse =
+            replCoord->awaitIsMasterResponse(opCtx, horizonParams, clientTopologyVersion, deadline);
+        result.appendElements(isMasterResponse->toBSON());
         if (level) {
             replCoord->appendSlaveInfoData(&result);
         }
-        return;
+        invariant(isMasterResponse->getTopologyVersion());
+        return isMasterResponse->getTopologyVersion().get();
+    }
+
+    auto currentTopologyVersion = replCoord->getTopologyVersion();
+
+    if (clientTopologyVersion &&
+        clientTopologyVersion->getProcessId() == currentTopologyVersion.getProcessId()) {
+        uassert(51764,
+                str::stream() << "Received a topology version with counter: "
+                              << clientTopologyVersion->getCounter()
+                              << " which is greater than the server topology version counter: "
+                              << currentTopologyVersion.getCounter(),
+                clientTopologyVersion->getCounter() == currentTopologyVersion.getCounter());
+
+        // The topologyVersion never changes on a running standalone process, so just sleep for
+        // maxAwaitTimeMS.
+        invariant(maxAwaitTimeMS);
+
+        IsMasterMetrics::get(opCtx)->incrementNumAwaitingTopologyChanges();
+        ON_BLOCK_EXIT([&] { IsMasterMetrics::get(opCtx)->decrementNumAwaitingTopologyChanges(); });
+        if (MONGO_unlikely(hangWaitingForIsMasterResponseOnStandalone.shouldFail())) {
+            // Used in tests that wait for this failpoint to be entered to guarantee that the
+            // request is waiting and metrics have been updated.
+            LOGV2(31462, "Hanging due to hangWaitingForIsMasterResponseOnStandalone failpoint.");
+            hangWaitingForIsMasterResponseOnStandalone.pauseWhileSet(opCtx);
+        }
+        opCtx->sleepFor(Milliseconds(*maxAwaitTimeMS));
     }
 
     result.appendBool("ismaster",
@@ -92,7 +142,7 @@ void appendReplicationInfo(OperationContext* opCtx, BSONObjBuilder& result, int
     if (level) {
         BSONObjBuilder sources(result.subarrayStart("sources"));
 
-        int n = 0;
+        DecimalCounter<unsigned> n;
         list<BSONObj> src;
         {
             const NamespaceString localSources{"local.sources"};
@@ -101,7 +151,7 @@ void appendReplicationInfo(OperationContext* opCtx, BSONObjBuilder& result, int
                 opCtx, localSources.ns(), ctx.getCollection(), PlanExecutor::NO_YIELD);
             BSONObj obj;
             PlanExecutor::ExecState state;
-            while (PlanExecutor::ADVANCED == (state = exec->getNext(&obj, NULL))) {
+            while (PlanExecutor::ADVANCED == (state = exec->getNext(&obj, nullptr))) {
                 src.push_back(obj.getOwned());
             }
 
@@ -120,7 +170,7 @@ void appendReplicationInfo(OperationContext* opCtx, BSONObjBuilder& result, int
                 BSONElement e = s["syncedTo"];
                 BSONObjBuilder t(bb.subobjStart("syncedTo"));
                 t.appendDate("time", e.timestampTime());
-                t.append("inc", e.timestampInc());
+                t.append("inc", static_cast<int>(e.timestampInc()));
                 t.done();
             }
 
@@ -131,11 +181,17 @@ void appendReplicationInfo(OperationContext* opCtx, BSONObjBuilder& result, int
                 ScopedDbConnection conn(s["host"].valuestr());
 
                 DBClientConnection* cliConn = dynamic_cast<DBClientConnection*>(&conn.conn());
-                if (cliConn && replAuthenticate(cliConn)) {
+                if (cliConn && replAuthenticate(cliConn).isOK()) {
                     BSONObj first = conn->findOne((string) "local.oplog.$" + sourcename,
-                                                  Query().sort(BSON("$natural" << 1)));
+                                                  Query().sort(BSON("$natural" << 1)),
+                                                  nullptr /* fieldsToReturn */,
+                                                  0 /* queryOptions */,
+                                                  ReadConcernArgs::kImplicitDefault);
                     BSONObj last = conn->findOne((string) "local.oplog.$" + sourcename,
-                                                 Query().sort(BSON("$natural" << -1)));
+                                                 Query().sort(BSON("$natural" << -1)),
+                                                 nullptr /* fieldsToReturn */,
+                                                 0 /* queryOptions */,
+                                                 ReadConcernArgs::kImplicitDefault);
                     bb.appendDate("masterFirst", first["ts"].timestampTime());
                     bb.appendDate("masterLast", last["ts"].timestampTime());
                     const auto lag = (last["ts"].timestampTime() - s["syncedTo"].timestampTime());
@@ -144,13 +200,19 @@ void appendReplicationInfo(OperationContext* opCtx, BSONObjBuilder& result, int
                 conn.done();
             }
 
-            sources.append(BSONObjBuilder::numStr(n++), bb.obj());
+            sources.append(StringData{n}, bb.obj());
+            ++n;
         }
 
         sources.done();
 
         replCoord->appendSlaveInfoData(&result);
     }
+
+    BSONObjBuilder topologyVersionBuilder(result.subobjStart("topologyVersion"));
+    currentTopologyVersion.serialize(&topologyVersionBuilder);
+
+    return currentTopologyVersion;
 }
 
 class ReplicationInfoServerStatus : public ServerStatusSection {
@@ -170,7 +232,11 @@ public:
         int level = configElement.numberInt();
 
         BSONObjBuilder result;
-        appendReplicationInfo(opCtx, result, level);
+        appendReplicationInfo(opCtx,
+                              result,
+                              level,
+                              boost::none /* clientTopologyVersion */,
+                              boost::none /* maxAwaitTimeMS */);
 
         auto rbid = ReplicationProcess::get(opCtx)->getRollbackID();
         if (ReplicationProcess::kUninitializedRollbackId != rbid) {
@@ -201,18 +267,34 @@ public:
         // TODO(siyuan) Output term of OpTime
         result.append("latestOptime", replCoord->getMyLastAppliedOpTime().getTimestamp());
 
-        BSONObj o;
-        uassert(17347,
-                "Problem reading earliest entry from oplog",
-                Helpers::getSingleton(opCtx, NamespaceString::kRsOplogNamespace.ns().c_str(), o));
-        result.append("earliestOptime", o["ts"].timestamp());
+        auto earliestOplogTimestampFetch = [&] {
+            AutoGetCollection oplog(opCtx, NamespaceString::kRsOplogNamespace, MODE_IS);
+            if (!oplog.getCollection()) {
+                return StatusWith<Timestamp>(ErrorCodes::NamespaceNotFound, "oplog doesn't exist");
+            }
+            return oplog.getCollection()->getRecordStore()->getEarliestOplogTimestamp(opCtx);
+        }();
+
+        if (earliestOplogTimestampFetch.getStatus() == ErrorCodes::OplogOperationUnsupported) {
+            // Falling back to use getSingleton if the storage engine does not support
+            // getEarliestOplogTimestamp.
+            BSONObj o;
+            if (Helpers::getSingleton(opCtx, NamespaceString::kRsOplogNamespace.ns().c_str(), o)) {
+                earliestOplogTimestampFetch = o["ts"].timestamp();
+            }
+        }
+
+        uassert(
+            17347, "Problem reading earliest entry from oplog", earliestOplogTimestampFetch.isOK());
+        result.append("earliestOptime", earliestOplogTimestampFetch.getValue());
+
         return result.obj();
     }
 } oplogInfoServerStatus;
 
-class CmdIsMaster final : public BasicCommand {
+class CmdIsMaster final : public BasicCommandWithReplyBuilderInterface {
 public:
-    CmdIsMaster() : BasicCommand("isMaster", "ismaster") {}
+    CmdIsMaster() : BasicCommandWithReplyBuilderInterface("isMaster", "ismaster") {}
 
     bool requiresAuth() const final {
         return false;
@@ -235,13 +317,13 @@ public:
                                const BSONObj& cmdObj,
                                std::vector<Privilege>* out) const final {}  // No auth required
 
-    bool run(OperationContext* opCtx,
-             const string&,
-             const BSONObj& cmdObj,
-             BSONObjBuilder& result) final {
+    bool runWithReplyBuilder(OperationContext* opCtx,
+                             const string&,
+                             const BSONObj& cmdObj,
+                             rpc::ReplyBuilderInterface* replyBuilder) final {
         CommandHelpers::handleMarkKillOnClientDisconnect(opCtx);
 
-        MONGO_FAIL_POINT_PAUSE_WHILE_SET_OR_INTERRUPTED(opCtx, waitInIsMaster);
+        waitInIsMaster.pauseWhileSet(opCtx);
 
         /* currently request to arbiter is (somewhat arbitrarily) an ismaster request that is not
            authenticated.
@@ -357,7 +439,43 @@ public:
                 });
         }
 
-        appendReplicationInfo(opCtx, result, 0);
+        // If a client is following the awaitable isMaster protocol, maxAwaitTimeMS should be
+        // present if and only if topologyVersion is present in the request.
+        auto topologyVersionElement = cmdObj["topologyVersion"];
+        auto maxAwaitTimeMSField = cmdObj["maxAwaitTimeMS"];
+        boost::optional<TopologyVersion> clientTopologyVersion;
+        boost::optional<long long> maxAwaitTimeMS;
+        if (topologyVersionElement && maxAwaitTimeMSField) {
+            clientTopologyVersion = TopologyVersion::parse(IDLParserErrorContext("TopologyVersion"),
+                                                           topologyVersionElement.Obj());
+            uassert(31372,
+                    "topologyVersion must have a non-negative counter",
+                    clientTopologyVersion->getCounter() >= 0);
+
+            {
+                long long parsedMaxAwaitTimeMS;
+                uassertStatusOK(
+                    bsonExtractIntegerField(cmdObj, "maxAwaitTimeMS", &parsedMaxAwaitTimeMS));
+                maxAwaitTimeMS = parsedMaxAwaitTimeMS;
+            }
+
+            uassert(31373, "maxAwaitTimeMS must be a non-negative integer", *maxAwaitTimeMS >= 0);
+
+            LOGV2_DEBUG(23904, 3, "Using maxAwaitTimeMS for awaitable isMaster protocol.");
+
+            // Awaitable isMaster commands have high latency by design.
+            opCtx->setShouldIncrementLatencyStats(false);
+        } else {
+            uassert(31368,
+                    (topologyVersionElement
+                         ? "A request with a 'topologyVersion' must include 'maxAwaitTimeMS'"
+                         : "A request with 'maxAwaitTimeMS' must include a 'topologyVersion'"),
+                    !topologyVersionElement && !maxAwaitTimeMSField);
+        }
+
+        auto result = replyBuilder->getBodyBuilder();
+        auto currentTopologyVersion =
+            appendReplicationInfo(opCtx, result, 0, clientTopologyVersion, maxAwaitTimeMS);
 
         if (serverGlobalParams.clusterRole == ClusterRole::ConfigServer) {
             const int configServerModeNumber = 2;
@@ -399,6 +517,39 @@ public:
         auto& saslMechanismRegistry = SASLServerMechanismRegistry::get(opCtx->getServiceContext());
         saslMechanismRegistry.advertiseMechanismNamesForUser(opCtx, cmdObj, &result);
 
+        if (opCtx->isExhaust()) {
+            LOGV2_DEBUG(23905, 3, "Using exhaust for isMaster protocol");
+
+            uassert(51756,
+                    "An isMaster request with exhaust must specify 'maxAwaitTimeMS'",
+                    maxAwaitTimeMSField);
+            invariant(clientTopologyVersion);
+
+            InExhaustIsMaster::get(opCtx->getClient()->session().get())
+                ->setInExhaustIsMaster(true /* inExhaustIsMaster */);
+
+            if (clientTopologyVersion->getProcessId() == currentTopologyVersion.getProcessId() &&
+                clientTopologyVersion->getCounter() == currentTopologyVersion.getCounter()) {
+                // Indicate that an exhaust message should be generated and the previous BSONObj
+                // command parameters should be reused as the next BSONObj command parameters.
+                replyBuilder->setNextInvocation(boost::none);
+            } else {
+                BSONObjBuilder nextInvocationBuilder;
+                for (auto&& elt : cmdObj) {
+                    if (elt.fieldNameStringData() == "topologyVersion"_sd) {
+                        BSONObjBuilder topologyVersionBuilder(
+                            nextInvocationBuilder.subobjStart("topologyVersion"));
+                        currentTopologyVersion.serialize(&topologyVersionBuilder);
+                    } else {
+                        nextInvocationBuilder.append(elt);
+                    }
+                }
+                replyBuilder->setNextInvocation(nextInvocationBuilder.obj());
+            }
+        }
+
+        handleIsMasterSpeculativeAuth(opCtx, cmdObj, &result);
+
         return true;
     }
 } cmdismaster;
