@@ -27,8 +27,12 @@
  *    it in the license file.
  */
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
 #include "mongo/platform/basic.h"
 
+#include <memory>
+
 #include "mongo/db/client.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/namespace_string.h"
@@ -36,20 +40,19 @@
 #include "mongo/db/op_observer_registry.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/drop_pending_collection_reaper.h"
+#include "mongo/db/repl/oplog_applier_impl_test_fixture.h"
 #include "mongo/db/repl/oplog_entry.h"
 #include "mongo/db/repl/oplog_interface_local.h"
 #include "mongo/db/repl/replication_consistency_markers_mock.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/repl/replication_recovery.h"
 #include "mongo/db/repl/storage_interface_impl.h"
-#include "mongo/db/repl/sync_tail_test_fixture.h"
 #include "mongo/db/service_context_d_test_fixture.h"
 #include "mongo/db/session_catalog_mongod.h"
 #include "mongo/db/session_txn_record_gen.h"
 #include "mongo/db/storage/storage_options.h"
 #include "mongo/db/storage/storage_parameters_gen.h"
 #include "mongo/db/transaction_participant.h"
-#include "mongo/stdx/memory.h"
 #include "mongo/unittest/death_test.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/assert_util.h"
@@ -171,15 +174,15 @@ private:
         ServiceContextMongoDTest::setUp();
 
         auto service = getServiceContext();
-        StorageInterface::set(service, stdx::make_unique<StorageInterfaceRecovery>());
+        StorageInterface::set(service, std::make_unique<StorageInterfaceRecovery>());
         _storageInterface = static_cast<StorageInterfaceRecovery*>(StorageInterface::get(service));
 
         _createOpCtx();
-        _consistencyMarkers = stdx::make_unique<ReplicationConsistencyMarkersMock>();
+        _consistencyMarkers = std::make_unique<ReplicationConsistencyMarkersMock>();
 
 
         ReplicationCoordinator::set(
-            service, stdx::make_unique<ReplicationCoordinatorMock>(service, getStorageInterface()));
+            service, std::make_unique<ReplicationCoordinatorMock>(service, getStorageInterface()));
 
         ASSERT_OK(
             ReplicationCoordinator::get(_opCtx.get())->setFollowerMode(MemberState::RS_PRIMARY));
@@ -193,7 +196,7 @@ private:
         observerRegistry->addObserver(std::make_unique<ReplicationRecoveryTestObObserver>());
 
         repl::DropPendingCollectionReaper::set(
-            service, stdx::make_unique<repl::DropPendingCollectionReaper>(_storageInterface));
+            service, std::make_unique<repl::DropPendingCollectionReaper>(_storageInterface));
 
         setOplogCollectionName(service);
     }
@@ -231,7 +234,7 @@ repl::OplogEntry _makeOplogEntry(repl::OpTime opTime,
                                  BSONObj object,
                                  boost::optional<BSONObj> object2 = boost::none,
                                  OperationSessionInfo sessionInfo = {},
-                                 boost::optional<Date_t> wallTime = boost::none) {
+                                 Date_t wallTime = Date_t()) {
     return repl::OplogEntry(opTime,                           // optime
                             boost::none,                      // hash
                             opType,                           // opType
@@ -424,9 +427,9 @@ TEST_F(ReplicationRecoveryTest, RecoveryWithEmptyOplogSucceedsWithStableTimestam
     _assertDocsInTestCollection(opCtx, {});
 }
 
-DEATH_TEST_F(ReplicationRecoveryTest,
-             TruncateFassertsWithoutOplogCollection,
-             "Fatal assertion 34418 NamespaceNotFound: Can't find local.oplog.rs") {
+DEATH_TEST_REGEX_F(ReplicationRecoveryTest,
+                   TruncateFassertsWithoutOplogCollection,
+                   "Fatal assertion.*34418.*NamespaceNotFound: Can't find local.oplog.rs") {
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
 
@@ -452,7 +455,7 @@ DEATH_TEST_F(ReplicationRecoveryTest,
     recovery.recoverFromOplog(opCtx, Timestamp(1, 1));
 }
 
-DEATH_TEST_F(ReplicationRecoveryTest, TruncateEntireOplogFasserts, "Fatal Assertion 40296") {
+DEATH_TEST_REGEX_F(ReplicationRecoveryTest, TruncateEntireOplogFasserts, "Fatal assertion.*40296") {
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
 
@@ -467,7 +470,7 @@ TEST_F(ReplicationRecoveryTest, RecoveryTruncatesOplogAtOplogTruncateAfterPoint)
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
 
-    getConsistencyMarkers()->setOplogTruncateAfterPoint(opCtx, Timestamp(4, 4));
+    getConsistencyMarkers()->setOplogTruncateAfterPoint(opCtx, Timestamp(3, 3));
     getConsistencyMarkers()->setAppliedThrough(opCtx, OpTime(Timestamp(3, 3), 1));
     _setUpOplog(opCtx, getStorageInterface(), {1, 2, 3, 4, 5});
 
@@ -479,6 +482,79 @@ TEST_F(ReplicationRecoveryTest, RecoveryTruncatesOplogAtOplogTruncateAfterPoint)
     ASSERT_EQ(getConsistencyMarkers()->getAppliedThrough(opCtx), OpTime(Timestamp(3, 3), 1));
 }
 
+TEST_F(ReplicationRecoveryTest, RecoveryDoesNotTruncateOplogAtOrBeforeStableTimestamp) {
+    ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
+    auto opCtx = getOperationContext();
+
+    getConsistencyMarkers()->setOplogTruncateAfterPoint(opCtx, Timestamp(2, 2));
+    getStorageInterfaceRecovery()->setRecoveryTimestamp(Timestamp(4, 4));
+    getConsistencyMarkers()->setAppliedThrough(opCtx, OpTime(Timestamp(4, 4), 1));
+    _setUpOplog(opCtx, getStorageInterface(), {1, 2, 3, 4, 5, 6});
+
+    recovery.recoverFromOplog(opCtx, boost::none);
+
+    _assertDocsInOplog(opCtx, {1, 2, 3, 4});
+    _assertDocsInTestCollection(opCtx, {});
+
+    ASSERT_EQ(getConsistencyMarkers()->getOplogTruncateAfterPoint(opCtx), Timestamp());
+    ASSERT_EQ(getConsistencyMarkers()->getAppliedThrough(opCtx), OpTime(Timestamp(4, 4), 1));
+}
+
+TEST_F(ReplicationRecoveryTest, RecoveryTruncatesNothingIfNothingIsAfterStableTimestamp) {
+    ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
+    auto opCtx = getOperationContext();
+
+    getConsistencyMarkers()->setOplogTruncateAfterPoint(opCtx, Timestamp(2, 2));
+    getStorageInterfaceRecovery()->setRecoveryTimestamp(Timestamp(4, 4));
+    getConsistencyMarkers()->setAppliedThrough(opCtx, OpTime(Timestamp(4, 4), 1));
+    _setUpOplog(opCtx, getStorageInterface(), {1, 2, 3, 4});
+
+    recovery.recoverFromOplog(opCtx, boost::none);
+
+    _assertDocsInOplog(opCtx, {1, 2, 3, 4});
+    _assertDocsInTestCollection(opCtx, {});
+
+    ASSERT_EQ(getConsistencyMarkers()->getOplogTruncateAfterPoint(opCtx), Timestamp());
+    ASSERT_EQ(getConsistencyMarkers()->getAppliedThrough(opCtx), OpTime(Timestamp(4, 4), 1));
+}
+
+TEST_F(ReplicationRecoveryTest,
+       RecoveryTruncatesNothingIfTruncatePointEqualsStableWithNoLaterEntries) {
+    ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
+    auto opCtx = getOperationContext();
+
+    getConsistencyMarkers()->setOplogTruncateAfterPoint(opCtx, Timestamp(4, 4));
+    getStorageInterfaceRecovery()->setRecoveryTimestamp(Timestamp(4, 4));
+    getConsistencyMarkers()->setAppliedThrough(opCtx, OpTime(Timestamp(4, 4), 1));
+    _setUpOplog(opCtx, getStorageInterface(), {1, 2, 3, 4});
+
+    recovery.recoverFromOplog(opCtx, boost::none);
+
+    _assertDocsInOplog(opCtx, {1, 2, 3, 4});
+    _assertDocsInTestCollection(opCtx, {});
+
+    ASSERT_EQ(getConsistencyMarkers()->getOplogTruncateAfterPoint(opCtx), Timestamp());
+    ASSERT_EQ(getConsistencyMarkers()->getAppliedThrough(opCtx), OpTime(Timestamp(4, 4), 1));
+}
+
+TEST_F(ReplicationRecoveryTest, RecoveryTruncatesAfterStableIfTruncatePointEqualsStable) {
+    ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
+    auto opCtx = getOperationContext();
+
+    getConsistencyMarkers()->setOplogTruncateAfterPoint(opCtx, Timestamp(4, 4));
+    getStorageInterfaceRecovery()->setRecoveryTimestamp(Timestamp(4, 4));
+    getConsistencyMarkers()->setAppliedThrough(opCtx, OpTime(Timestamp(4, 4), 1));
+    _setUpOplog(opCtx, getStorageInterface(), {1, 2, 3, 4, 5, 6});
+
+    recovery.recoverFromOplog(opCtx, boost::none);
+
+    _assertDocsInOplog(opCtx, {1, 2, 3, 4});
+    _assertDocsInTestCollection(opCtx, {});
+
+    ASSERT_EQ(getConsistencyMarkers()->getOplogTruncateAfterPoint(opCtx), Timestamp());
+    ASSERT_EQ(getConsistencyMarkers()->getAppliedThrough(opCtx), OpTime(Timestamp(4, 4), 1));
+}
+
 TEST_F(ReplicationRecoveryTest, RecoverySucceedsWithOplogTruncatePointTooHigh) {
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
@@ -619,6 +695,10 @@ TEST_F(ReplicationRecoveryTest,
 DEATH_TEST_F(ReplicationRecoveryTest,
              RecoveryFailsWithUnmatchedAppliedThrough,
              "Invariant failure") {
+    ASSERT_OK(ServerParameterSet::getGlobal()
+                  ->getMap()
+                  .find("assertStableTimestampEqualsAppliedThroughOnRecovery")
+                  ->second->setFromString("true"));
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
 
@@ -658,7 +738,7 @@ TEST_F(ReplicationRecoveryTest, RecoveryAppliesDocumentsWhenAppliedThroughIsBehi
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
 
-    getConsistencyMarkers()->setOplogTruncateAfterPoint(opCtx, Timestamp(4, 4));
+    getConsistencyMarkers()->setOplogTruncateAfterPoint(opCtx, Timestamp(3, 3));
     getConsistencyMarkers()->setAppliedThrough(opCtx, OpTime(Timestamp(1, 1), 1));
     _setUpOplog(opCtx, getStorageInterface(), {1, 2, 3, 4, 5});
 
@@ -675,7 +755,7 @@ void ReplicationRecoveryTest::testRecoveryAppliesDocumentsWithNoAppliedThroughAf
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
 
-    getConsistencyMarkers()->setOplogTruncateAfterPoint(opCtx, Timestamp(4, 4));
+    getConsistencyMarkers()->setOplogTruncateAfterPoint(opCtx, Timestamp(3, 3));
     _setUpOplog(opCtx, getStorageInterface(), {1, 2, 3, 4, 5});
 
     auto startingTS = Timestamp(1, 1);
@@ -703,7 +783,9 @@ TEST_F(ReplicationRecoveryTest,
     testRecoveryAppliesDocumentsWithNoAppliedThroughAfterTruncation(false);
 }
 
-DEATH_TEST_F(ReplicationRecoveryTest, AppliedThroughBehindOplogFasserts, "Fatal Assertion 40292") {
+DEATH_TEST_REGEX_F(ReplicationRecoveryTest,
+                   AppliedThroughBehindOplogFasserts,
+                   "Fatal assertion.*40292") {
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
 
@@ -713,9 +795,9 @@ DEATH_TEST_F(ReplicationRecoveryTest, AppliedThroughBehindOplogFasserts, "Fatal
     recovery.recoverFromOplog(opCtx, boost::none);
 }
 
-DEATH_TEST_F(ReplicationRecoveryTest,
-             AppliedThroughAheadOfTopOfOplogCausesFassert,
-             "Fatal Assertion 40313") {
+DEATH_TEST_REGEX_F(ReplicationRecoveryTest,
+                   AppliedThroughAheadOfTopOfOplogCausesFassert,
+                   "Fatal assertion.*40313") {
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
 
@@ -725,9 +807,9 @@ DEATH_TEST_F(ReplicationRecoveryTest,
     recovery.recoverFromOplog(opCtx, boost::none);
 }
 
-DEATH_TEST_F(ReplicationRecoveryTest,
-             AppliedThroughNotInOplogCausesFassert,
-             "Fatal Assertion 40292") {
+DEATH_TEST_REGEX_F(ReplicationRecoveryTest,
+                   AppliedThroughNotInOplogCausesFassert,
+                   "Fatal assertion.*40292") {
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
 
@@ -749,14 +831,17 @@ TEST_F(ReplicationRecoveryTest, RecoveryDoesNotApplyOperationsIfAppliedThroughIs
     _assertDocsInOplog(opCtx, {5});
     _assertDocsInTestCollection(opCtx, {});
     ASSERT(getConsistencyMarkers()->getOplogTruncateAfterPoint(opCtx).isNull());
-    // In 4.0 with RTT, recovering without a `recoverTimestamp` will set `appliedThrough` to the
-    // top of oplog.
+    // Recovering without a `recoverTimestamp` will set `appliedThrough` to the top of oplog.
     ASSERT_EQ(OpTime(Timestamp(5, 5), 1), getConsistencyMarkers()->getAppliedThrough(opCtx));
 }
 
 DEATH_TEST_F(ReplicationRecoveryTest,
              RecoveryInvariantsWithUnequalStableTimestampAndAppliedThrough,
              "Invariant failure") {
+    ASSERT_OK(ServerParameterSet::getGlobal()
+                  ->getMap()
+                  .find("assertStableTimestampEqualsAppliedThroughOnRecovery")
+                  ->second->setFromString("true"));
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
 
@@ -1004,9 +1089,9 @@ TEST_F(ReplicationRecoveryTest, AbortTransactionOplogEntryCorrectlyUpdatesConfig
     ASSERT_EQ(getConsistencyMarkers()->getAppliedThrough(opCtx), OpTime(Timestamp(3, 0), 1));
 }
 
-DEATH_TEST_F(ReplicationRecoveryTest,
-             RecoveryFailsWithPrepareAndEnableReadConcernMajorityFalse,
-             "Fatal Assertion 51146") {
+DEATH_TEST_REGEX_F(ReplicationRecoveryTest,
+                   RecoveryFailsWithPrepareAndEnableReadConcernMajorityFalse,
+                   "Fatal assertion.*51146") {
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
 
@@ -1247,7 +1332,8 @@ TEST_F(ReplicationRecoveryTest, RecoverFromOplogUpToWithEmptyOplog) {
     recovery.recoverFromOplogUpTo(opCtx, Timestamp(5, 5));
     stopCapturingLogMessages();
 
-    ASSERT_EQUALS(1, countLogLinesContaining("No stored oplog entries to apply for recovery"));
+    ASSERT_EQUALS(
+        1, countTextFormatLogLinesContaining("No stored oplog entries to apply for recovery"));
     _assertDocsInTestCollection(opCtx, {});
     ASSERT_EQ(getConsistencyMarkers()->getAppliedThrough(opCtx), OpTime(Timestamp(0, 0), 1));
 }
@@ -1289,8 +1375,9 @@ TEST_F(ReplicationRecoveryTest, RecoverFromOplogUpToWithNoOperationsToRecover) {
     recovery.recoverFromOplogUpTo(opCtx, Timestamp(1580148193, 1));
     stopCapturingLogMessages();
 
-    ASSERT_EQUALS(1,
-                  countLogLinesContaining("No stored oplog entries to apply for recovery between"));
+    ASSERT_EQUALS(
+        1,
+        countTextFormatLogLinesContaining("No stored oplog entries to apply for recovery between"));
 }
 
 TEST_F(ReplicationRecoveryTest, RecoverFromOplogUpToReconstructsPreparedTransactions) {
@@ -1377,8 +1464,9 @@ TEST_F(ReplicationRecoveryTest,
     recovery.recoverFromOplogUpTo(opCtx, Timestamp(5, 1));
     stopCapturingLogMessages();
 
-    ASSERT_EQUALS(1,
-                  countLogLinesContaining("No stored oplog entries to apply for recovery between"));
+    ASSERT_EQUALS(
+        1,
+        countTextFormatLogLinesContaining("No stored oplog entries to apply for recovery between"));
 
     {
         MongoDOperationContextSession ocs(opCtx);
@@ -1389,9 +1477,9 @@ TEST_F(ReplicationRecoveryTest,
     }
 }
 
-DEATH_TEST_F(ReplicationRecoveryTest,
-             RecoverFromOplogUpToWithoutStableCheckpoint,
-             "Fatal Assertion 31399") {
+DEATH_TEST_REGEX_F(ReplicationRecoveryTest,
+                   RecoverFromOplogUpToWithoutStableCheckpoint,
+                   "Fatal assertion.*31399") {
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
 
@@ -1400,9 +1488,9 @@ DEATH_TEST_F(ReplicationRecoveryTest,
     recovery.recoverFromOplogUpTo(opCtx, Timestamp(5, 5));
 }
 
-DEATH_TEST_F(ReplicationRecoveryTest,
-             RecoverFromOplogAsStandaloneFailsWithoutStableCheckpoint,
-             "Fatal Assertion 31229") {
+DEATH_TEST_REGEX_F(ReplicationRecoveryTest,
+                   RecoverFromOplogAsStandaloneFailsWithoutStableCheckpoint,
+                   "Fatal assertion.*31229") {
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
 
@@ -1411,9 +1499,9 @@ DEATH_TEST_F(ReplicationRecoveryTest,
     recovery.recoverFromOplogAsStandalone(opCtx);
 }
 
-DEATH_TEST_F(ReplicationRecoveryTest,
-             RecoverFromOplogAsStandaloneFailsWithNullStableCheckpoint,
-             "Fatal Assertion 50806") {
+DEATH_TEST_REGEX_F(ReplicationRecoveryTest,
+                   RecoverFromOplogAsStandaloneFailsWithNullStableCheckpoint,
+                   "Fatal assertion.*50806") {
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
 
@@ -1423,9 +1511,9 @@ DEATH_TEST_F(ReplicationRecoveryTest,
     recovery.recoverFromOplogAsStandalone(opCtx);
 }
 
-DEATH_TEST_F(ReplicationRecoveryTest,
-             RecoverFromOplogUpToFailsWithNullStableCheckpoint,
-             "Fatal Assertion 50806") {
+DEATH_TEST_REGEX_F(ReplicationRecoveryTest,
+                   RecoverFromOplogUpToFailsWithNullStableCheckpoint,
+                   "Fatal assertion.*50806") {
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
     auto opCtx = getOperationContext();
 
@@ -1479,10 +1567,10 @@ TEST_F(ReplicationRecoveryTest,
     _assertDocsInTestCollection(opCtx, {});
 }
 
-DEATH_TEST_F(
+DEATH_TEST_REGEX_F(
     ReplicationRecoveryTest,
     RecoverFromOplogAsStandaloneWithTakeUnstableCheckpointOnShutdownFailsWithInitialSyncFlag,
-    "Fatal Assertion 31362") {
+    "Fatal assertion.*31362") {
     gTakeUnstableCheckpointOnShutdown = true;
 
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
@@ -1494,10 +1582,10 @@ DEATH_TEST_F(
     recovery.recoverFromOplogAsStandalone(opCtx);
 }
 
-DEATH_TEST_F(
+DEATH_TEST_REGEX_F(
     ReplicationRecoveryTest,
     RecoverFromOplogAsStandaloneWithTakeUnstableCheckpointOnShutdownFailsWithOplogTruncateAfterPoint,
-    "Fatal Assertion 31363") {
+    "Fatal assertion.*31363") {
     gTakeUnstableCheckpointOnShutdown = true;
 
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
@@ -1509,9 +1597,10 @@ DEATH_TEST_F(
     recovery.recoverFromOplogAsStandalone(opCtx);
 }
 
-DEATH_TEST_F(ReplicationRecoveryTest,
-             RecoverFromOplogAsStandaloneWithTakeUnstableCheckpointOnShutdownFailsWithEmptyOplog,
-             "Fatal Assertion 31364") {
+DEATH_TEST_REGEX_F(
+    ReplicationRecoveryTest,
+    RecoverFromOplogAsStandaloneWithTakeUnstableCheckpointOnShutdownFailsWithEmptyOplog,
+    "Fatal assertion.*31364") {
     gTakeUnstableCheckpointOnShutdown = true;
 
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
@@ -1522,10 +1611,10 @@ DEATH_TEST_F(ReplicationRecoveryTest,
     recovery.recoverFromOplogAsStandalone(opCtx);
 }
 
-DEATH_TEST_F(
+DEATH_TEST_REGEX_F(
     ReplicationRecoveryTest,
     RecoverFromOplogAsStandaloneWithTakeUnstableCheckpointOnShutdownFailsWithMismatchedAppliedThrough,
-    "Fatal Assertion 31365") {
+    "Fatal assertion.*31365") {
     gTakeUnstableCheckpointOnShutdown = true;
 
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
@@ -1537,9 +1626,10 @@ DEATH_TEST_F(
     recovery.recoverFromOplogAsStandalone(opCtx);
 }
 
-DEATH_TEST_F(ReplicationRecoveryTest,
-             RecoverFromOplogAsStandaloneWithTakeUnstableCheckpointOnShutdownFailsWithHighMinValid,
-             "Fatal Assertion 31366") {
+DEATH_TEST_REGEX_F(
+    ReplicationRecoveryTest,
+    RecoverFromOplogAsStandaloneWithTakeUnstableCheckpointOnShutdownFailsWithHighMinValid,
+    "Fatal assertion.*31366") {
     gTakeUnstableCheckpointOnShutdown = true;
 
     ReplicationRecoveryImpl recovery(getStorageInterface(), getConsistencyMarkers());
