@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kReplication
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
 #include "mongo/platform/basic.h"
 
@@ -39,8 +39,8 @@
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/reporter.h"
 #include "mongo/executor/task_executor.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/concurrency/idle_thread_block.h"
-#include "mongo/util/log.h"
 #include "mongo/util/net/hostandport.h"
 #include "mongo/util/scopeguard.h"
 #include "mongo/util/time_support.h"
@@ -104,8 +104,11 @@ void SyncSourceFeedback::forwardSlaveProgress() {
         if (_reporter) {
             auto triggerStatus = _reporter->trigger();
             if (!triggerStatus.isOK()) {
-                warning() << "unable to forward slave progress to " << _reporter->getTarget()
-                          << ": " << triggerStatus;
+                LOGV2_WARNING(21764,
+                              "unable to forward progress to {target}: {error}",
+                              "Unable to forward progress",
+                              "target"_attr = _reporter->getTarget(),
+                              "error"_attr = triggerStatus);
             }
         }
     }
@@ -116,15 +119,23 @@ Status SyncSourceFeedback::_updateUpstream(Reporter* reporter) {
 
     auto triggerStatus = reporter->trigger();
     if (!triggerStatus.isOK()) {
-        warning() << "unable to schedule reporter to update replication progress on " << syncTarget
-                  << ": " << triggerStatus;
+        LOGV2_WARNING(21765,
+                      "unable to schedule reporter to update replication progress on {syncTarget}: "
+                      "{error}",
+                      "Unable to schedule reporter to update replication progress",
+                      "syncTarget"_attr = syncTarget,
+                      "error"_attr = triggerStatus);
         return triggerStatus;
     }
 
     auto status = reporter->join();
 
     if (!status.isOK()) {
-        log() << "SyncSourceFeedback error sending update to " << syncTarget << ": " << status;
+        LOGV2(21760,
+              "SyncSourceFeedback error sending update to {syncTarget}: {error}",
+              "SyncSourceFeedback error sending update",
+              "syncTarget"_attr = syncTarget,
+              "error"_attr = status);
     }
 
     // Sync source blacklisting will be done in BackgroundSync and SyncSourceResolver.
@@ -202,15 +213,24 @@ void SyncSourceFeedback::run(executor::TaskExecutor* executor,
         }
 
         if (syncTarget != target) {
-            LOG(1) << "setting syncSourceFeedback to " << target;
+            LOGV2_DEBUG(21761,
+                        1,
+                        "setting syncSourceFeedback to {target}",
+                        "Setting syncSourceFeedback",
+                        "target"_attr = target);
             syncTarget = target;
 
             // Update keepalive value from config.
             auto oldKeepAliveInterval = keepAliveInterval;
             keepAliveInterval = calculateKeepAliveInterval(replCoord->getConfig());
             if (oldKeepAliveInterval != keepAliveInterval) {
-                LOG(1) << "new syncSourceFeedback keep alive duration = " << keepAliveInterval
-                       << " (previously " << oldKeepAliveInterval << ")";
+                LOGV2_DEBUG(21762,
+                            1,
+                            "new syncSourceFeedback keep alive duration = {newKeepAliveInterval} "
+                            "(previously {oldKeepAliveInterval})",
+                            "New syncSourceFeedback keep alive duration",
+                            "newKeepAliveInterval"_attr = keepAliveInterval,
+                            "oldKeepAliveInterval"_attr = oldKeepAliveInterval);
             }
         }
 
@@ -233,9 +253,14 @@ void SyncSourceFeedback::run(executor::TaskExecutor* executor,
 
         auto status = _updateUpstream(&reporter);
         if (!status.isOK()) {
-            LOG(1) << "The replication progress command (replSetUpdatePosition) failed and will be "
-                      "retried: "
-                   << status;
+            LOGV2_DEBUG(
+                21763,
+                1,
+                "The replication progress command (replSetUpdatePosition) failed and will be "
+                "retried: {error}",
+                "The replication progress command (replSetUpdatePosition) failed and will be "
+                "retried",
+                "error"_attr = status);
         }
     }
 }
