@@ -29,6 +29,7 @@
 
 #pragma once
 
+#include <functional>
 #include <iosfwd>
 #include <string>
 
@@ -43,7 +44,7 @@
 #include "mongo/db/repl/split_horizon.h"
 #include "mongo/db/repl/update_position_args.h"
 #include "mongo/db/server_options.h"
-#include "mongo/stdx/functional.h"
+#include "mongo/rpc/topology_version_gen.h"
 #include "mongo/util/net/hostandport.h"
 #include "mongo/util/time_support.h"
 
@@ -112,6 +113,11 @@ public:
      */
     Role getRole() const;
 
+    /**
+     * Gets the current topology version of this member.
+     */
+    TopologyVersion getTopologyVersion() const;
+
     /**
      * Gets the MemberState of this member in the replica set.
      */
@@ -165,6 +171,16 @@ public:
 
     enum class UpdateTermResult { kAlreadyUpToDate, kTriggerStepDown, kUpdatedTerm };
 
+    /**
+     * Returns true if we are a one-node replica set, we're the one member,
+     * we're electable, we're not in maintenance mode, and we are currently in followerMode
+     * SECONDARY.
+     *
+     * This is used to decide if we should start an election in a one-node replica set.
+     */
+    bool isElectableNodeInSingleNodeReplicaSet() const;
+
+
     ////////////////////////////////////////////////////////////
     //
     // Basic state manipulation methods.
@@ -222,12 +238,10 @@ public:
      * ("syncSourceHasSyncSource" is false), and only has data up to "myLastOpTime", returns true.
      *
      * "now" is used to skip over currently blacklisted sync sources.
-     *
-     * TODO (SERVER-27668): Make OplogQueryMetadata non-optional in mongodb 3.8.
      */
     bool shouldChangeSyncSource(const HostAndPort& currentSource,
                                 const rpc::ReplSetMetadata& replMetadata,
-                                boost::optional<rpc::OplogQueryMetadata> oqMetadata,
+                                const rpc::OplogQueryMetadata& oqMetadata,
                                 Date_t now) const;
 
     /**
@@ -314,11 +328,6 @@ public:
         const BSONObj electionCandidateMetrics;
         const BSONObj electionParticipantMetrics;
 
-        // boost::none if the storage engine does not support RTT, or if it does but does not
-        // persist data to necessitate taking checkpoints. Timestamp::min() if a checkpoint is yet
-        // to be taken.
-        const boost::optional<Timestamp> lastStableCheckpointTimestampDeprecated;
-
         // boost::none if the storage engine does not support recovery to a timestamp.
         // Timestamp::min() if a stable recovery timestamp is yet to be taken.
         //
@@ -326,6 +335,7 @@ public:
         // as well as startup recovery without re-initial syncing in the case of durable storage
         // engines.
         const boost::optional<Timestamp> lastStableRecoveryTimestamp;
+        bool tooStale;
     };
 
     // produce a reply to a status request
@@ -340,8 +350,8 @@ public:
     // Produce a reply to an ismaster request.  It is only valid to call this if we are a
     // replset.  Drivers interpret the isMaster fields according to the Server Discovery and
     // Monitoring Spec, see the "Parsing an isMaster response" section.
-    void fillIsMasterForReplSet(IsMasterResponse* response,
-                                const SplitHorizon::Parameters& horizonParams);
+    void fillIsMasterForReplSet(std::shared_ptr<IsMasterResponse> response,
+                                const StringData& horizonString) const;
 
     // Produce member data for the serverStatus command and diagnostic logging.
     void fillMemberData(BSONObjBuilder* result);
@@ -417,20 +427,39 @@ public:
         const StatusWith<ReplSetHeartbeatResponse>& hbResponse);
 
     /**
-     *  Returns whether or not at least 'numNodes' have reached the given opTime.
+     *  Returns whether or not at least 'numNodes' have reached the given opTime with the same term.
      * "durablyWritten" indicates whether the operation has to be durably applied.
      */
     bool haveNumNodesReachedOpTime(const OpTime& opTime, int numNodes, bool durablyWritten);
 
     /**
-     * Returns whether or not at least one node matching the tagPattern has reached
-     * the given opTime.
+     * Returns whether or not at least one node matching the tagPattern has reached the given opTime
+     * with the same term.
      * "durablyWritten" indicates whether the operation has to be durably applied.
      */
     bool haveTaggedNodesReachedOpTime(const OpTime& opTime,
                                       const ReplSetTagPattern& tagPattern,
                                       bool durablyWritten);
 
+    using MemberPredicate = std::function<bool(const MemberData&)>;
+
+    /**
+     * Return the predicate that tests if a member has reached the target OpTime.
+     */
+    MemberPredicate makeOpTimePredicate(const OpTime& opTime, bool durablyWritten);
+
+    /**
+     * Return the predicate that tests if a member has replicated the given config.
+     */
+    MemberPredicate makeConfigPredicate();
+
+    /**
+     * Returns whether or not at least one node matching the tagPattern has satisfied the given
+     * condition.
+     */
+    bool haveTaggedNodesSatisfiedCondition(MemberPredicate pred,
+                                           const ReplSetTagPattern& tagPattern);
+
     /**
      * Returns a vector of members that have applied the operation with OpTime 'op'.
      * "durablyWritten" indicates whether the operation has to be durably applied.
@@ -512,6 +541,22 @@ public:
                                    Date_t now,
                                    long long* configVersion);
 
+    /**
+     * Sets the latest optime committed in the previous config to the current lastCommitted optime.
+     */
+    void updateLastCommittedInPrevConfig();
+
+    /**
+     * Returns the latest optime committed in the previous config.
+     */
+    OpTime getLastCommittedInPrevConfig();
+
+    /**
+     * Returns an optime that must become majority committed in the current config before it is safe
+     * for a primary to move to a new config.
+     */
+    OpTime getConfigOplogCommitmentOpTime();
+
     /**
      * Sets lastVote to be for ourself in this term.
      */
@@ -543,7 +588,7 @@ public:
     void processLoseElection();
 
 
-    using StepDownAttemptAbortFn = stdx::function<void()>;
+    using StepDownAttemptAbortFn = std::function<void()>;
     /**
      * Readies the TopologyCoordinator for an attempt to stepdown that may fail.  This is used
      * when we receive a stepdown command (which can fail if not enough secondaries are caught up)
@@ -677,6 +722,11 @@ public:
      */
     void restartHeartbeats();
 
+    /**
+     * Increments the counter field of the current TopologyVersion.
+     */
+    void incrementTopologyVersion();
+
     // Scans through all members that are 'up' and returns the latest known optime.
     OpTime latestKnownOpTime() const;
 
@@ -699,21 +749,10 @@ public:
         const;
 
     /**
-     * Checks if the 'commitQuorum' can be satisifed by 'members'. Returns true if it can be
-     * satisfied.
-     *
-     * 'members' must be part of the replica set configuration.
+     * Checks if the 'commitQuorum' can be satisifed by the current replica set config. Returns true
+     * if it can be satisfied.
      */
-    bool checkIfCommitQuorumCanBeSatisfied(const CommitQuorumOptions& commitQuorum,
-                                           const std::vector<MemberConfig>& members) const;
-
-    /**
-     * Returns 'true' if the 'commitQuorum' is satisifed by the 'commitReadyMembers'.
-     *
-     * 'commitReadyMembers' must be part of the replica set configuration.
-     */
-    bool checkIfCommitQuorumIsSatisfied(const CommitQuorumOptions& commitQuorum,
-                                        const std::vector<HostAndPort>& commitReadyMembers) const;
+    bool checkIfCommitQuorumCanBeSatisfied(const CommitQuorumOptions& commitQuorum) const;
 
     /**
      * Returns nullptr if there is no primary, or the MemberConfig* for the current primary.
@@ -739,6 +778,12 @@ public:
     void setCurrentPrimary_forTest(int primaryIndex,
                                    const Timestamp& electionTime = Timestamp(0, 0));
 
+    /**
+     * Change config (version, term) of each member in the initial test config so that
+     * it will be majority replicated without having to mock heartbeats.
+     */
+    void populateAllMembersConfigVersionAndTerm_forTest();
+
     // Returns _electionTime.  Only used in unittests.
     Timestamp getElectionTime() const;
 
@@ -873,13 +918,14 @@ private:
     MemberData* _findMemberDataByMemberId(const int memberId);
 
     /**
-     * Performs updating "_currentPrimaryIndex" for processHeartbeatResponse(), and determines if an
-     * election or stepdown should commence.
+     * Performs updating "_currentPrimaryIndex" for processHeartbeatResponse().
      */
-    HeartbeatResponseAction _updatePrimaryFromHBDataV1(int updatedConfigIndex,
-                                                       const MemberState& originalState,
-                                                       Date_t now);
+    void _updatePrimaryFromHBDataV1(Date_t now);
 
+    /**
+     * Determine if the node should run PriorityTakeover or CatchupTakeover.
+     */
+    HeartbeatResponseAction _shouldTakeOverPrimary(int updatedConfigIndex);
     /**
      * Updates _memberData based on the newConfig, ensuring that every member in the newConfig
      * has an entry in _memberData.  If any nodes in the newConfig are also present in
@@ -910,21 +956,15 @@ private:
      **/
     bool _memberIsBlacklisted(const MemberConfig& memberConfig, Date_t now) const;
 
-    /**
-     * Returns true if we are a one-node replica set, we're the one member,
-     * we're electable, we're not in maintenance mode, and we are currently in followerMode
-     * SECONDARY.
-     *
-     * This is used to decide if we should transition to Role::candidate in a one-node replica set.
-     */
-    bool _isElectableNodeInSingleNodeReplicaSet() const;
-
     // Returns a string representation of the current replica set status for logging purposes.
     std::string _getReplSetStatusString();
 
     // This node's role in the replication protocol.
     Role _role;
 
+    // This node's topology version. This is updated upon a significant topology change.
+    TopologyVersion _topologyVersion;
+
     // This is a unique id that is generated and set each time we transition to PRIMARY, as the
     // result of an election.
     OID _electionId;
@@ -983,6 +1023,9 @@ private:
     // _lastCommittedOpTime cannot be set to an earlier OpTime.
     OpTime _firstOpTimeOfMyTerm;
 
+    // Latest committed optime in the previous config.
+    OpTime _lastCommittedInPrevConfig;
+
     // The number of calls we have had to enter maintenance mode
     int _maintenanceModeCalls;
 
