@@ -27,13 +27,14 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/s/balancer/balancer.h"
 
 #include <algorithm>
+#include <memory>
 #include <string>
 
 #include "mongo/base/status_with.h"
@@ -45,17 +46,17 @@
 #include "mongo/db/s/balancer/balancer_chunk_selection_policy_impl.h"
 #include "mongo/db/s/balancer/cluster_statistics_impl.h"
 #include "mongo/db/s/sharding_logging.h"
+#include "mongo/logv2/log.h"
 #include "mongo/s/balancer_configuration.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/catalog_cache.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/request_types/balancer_collection_status_gen.h"
 #include "mongo/s/shard_util.h"
-#include "mongo/stdx/memory.h"
 #include "mongo/util/concurrency/idle_thread_block.h"
 #include "mongo/util/exit.h"
-#include "mongo/util/fail_point_service.h"
-#include "mongo/util/log.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/timer.h"
 #include "mongo/util/version.h"
 
@@ -79,6 +80,13 @@ const Seconds kShortBalanceRoundInterval(1);
 
 const auto getBalancer = ServiceContext::declareDecoration<std::unique_ptr<Balancer>>();
 
+/**
+ * Balancer status response
+ */
+static constexpr StringData kBalancerPolicyStatusDraining = "draining"_sd;
+static constexpr StringData kBalancerPolicyStatusZoneViolation = "zoneViolation"_sd;
+static constexpr StringData kBalancerPolicyStatusChunksImbalance = "chunksImbalance"_sd;
+
 /**
  * Utility class to generate timing and statistics for a single balancer round.
  */
@@ -149,7 +157,7 @@ void warnOnMultiVersion(const vector<ClusterStatistics::ShardStatistics>& cluste
         sb << stat.shardId << " is at " << stat.mongoVersion << "; ";
     }
 
-    warning() << sb.str();
+    LOGV2_WARNING(21875, "{sb_str}", "sb_str"_attr = sb.str());
 }
 
 }  // namespace
@@ -157,9 +165,9 @@ void warnOnMultiVersion(const vector<ClusterStatistics::ShardStatistics>& cluste
 Balancer::Balancer(ServiceContext* serviceContext)
     : _balancedLastTime(0),
       _random(std::random_device{}()),
-      _clusterStats(stdx::make_unique<ClusterStatisticsImpl>(_random)),
+      _clusterStats(std::make_unique<ClusterStatisticsImpl>(_random)),
       _chunkSelectionPolicy(
-          stdx::make_unique<BalancerChunkSelectionPolicyImpl>(_clusterStats.get(), _random)),
+          std::make_unique<BalancerChunkSelectionPolicyImpl>(_clusterStats.get(), _random)),
       _migrationManager(serviceContext) {}
 
 Balancer::~Balancer() {
@@ -170,7 +178,7 @@ Balancer::~Balancer() {
 
 void Balancer::create(ServiceContext* serviceContext) {
     invariant(!getBalancer(serviceContext));
-    getBalancer(serviceContext) = stdx::make_unique<Balancer>(serviceContext);
+    getBalancer(serviceContext) = std::make_unique<Balancer>(serviceContext);
 }
 
 Balancer* Balancer::get(ServiceContext* serviceContext) {
@@ -199,6 +207,7 @@ void Balancer::interruptBalancer() {
         return;
 
     _state = kStopping;
+    _thread.detach();
 
     // Interrupt the balancer thread if it has been started. We are guaranteed that the operation
     // context of that thread is still alive, because we hold the balancer mutex.
@@ -217,22 +226,9 @@ void Balancer::interruptBalancer() {
 }
 
 void Balancer::waitForBalancerToStop() {
-    {
-        stdx::lock_guard<Latch> scopedLock(_mutex);
-        if (_state == kStopped)
-            return;
+    stdx::unique_lock<Latch> scopedLock(_mutex);
 
-        invariant(_state == kStopping);
-        invariant(_thread.joinable());
-    }
-
-    _thread.join();
-
-    stdx::lock_guard<Latch> scopedLock(_mutex);
-    _state = kStopped;
-    _thread = {};
-
-    LOG(1) << "Balancer thread terminated";
+    _joinCond.wait(scopedLock, [this] { return _state == kStopped; });
 }
 
 void Balancer::joinCurrentRound(OperationContext* opCtx) {
@@ -251,7 +247,11 @@ Status Balancer::rebalanceSingleChunk(OperationContext* opCtx, const ChunkType&
 
     auto migrateInfo = std::move(migrateStatus.getValue());
     if (!migrateInfo) {
-        LOG(1) << "Unable to find more appropriate location for chunk " << redact(chunk.toString());
+        LOGV2_DEBUG(21854,
+                    1,
+                    "Unable to find more appropriate location for chunk {chunk}",
+                    "Unable to find more appropriate location for chunk",
+                    "chunk"_attr = redact(chunk.toString()));
         return Status::OK();
     }
 
@@ -273,14 +273,23 @@ Status Balancer::moveSingleChunk(OperationContext* opCtx,
                                  const ShardId& newShardId,
                                  uint64_t maxChunkSizeBytes,
                                  const MigrationSecondaryThrottleOptions& secondaryThrottle,
-                                 bool waitForDelete) {
+                                 bool waitForDelete,
+                                 bool forceJumbo) {
     auto moveAllowedStatus = _chunkSelectionPolicy->checkMoveAllowed(opCtx, chunk, newShardId);
     if (!moveAllowedStatus.isOK()) {
         return moveAllowedStatus;
     }
 
     return _migrationManager.executeManualMigration(
-        opCtx, MigrateInfo(newShardId, chunk), maxChunkSizeBytes, secondaryThrottle, waitForDelete);
+        opCtx,
+        MigrateInfo(newShardId,
+                    chunk,
+                    forceJumbo ? MoveChunkRequest::ForceJumbo::kForceManual
+                               : MoveChunkRequest::ForceJumbo::kDoNotForce,
+                    MigrateInfo::chunksImbalance),
+        maxChunkSizeBytes,
+        secondaryThrottle,
+        waitForDelete);
 }
 
 void Balancer::report(OperationContext* opCtx, BSONObjBuilder* builder) {
@@ -296,11 +305,20 @@ void Balancer::report(OperationContext* opCtx, BSONObjBuilder* builder) {
 }
 
 void Balancer::_mainThread() {
+    ON_BLOCK_EXIT([this] {
+        stdx::lock_guard<Latch> scopedLock(_mutex);
+
+        _state = kStopped;
+        _joinCond.notify_all();
+
+        LOGV2_DEBUG(21855, 1, "Balancer thread terminated");
+    });
+
     Client::initThread("Balancer");
     auto opCtx = cc().makeOperationContext();
     auto shardingContext = Grid::get(opCtx.get());
 
-    log() << "CSRS balancer is starting";
+    LOGV2(21856, "CSRS balancer is starting");
 
     {
         stdx::lock_guard<Latch> scopedLock(_mutex);
@@ -313,9 +331,13 @@ void Balancer::_mainThread() {
     while (!_stopRequested()) {
         Status refreshStatus = balancerConfig->refreshAndCheck(opCtx.get());
         if (!refreshStatus.isOK()) {
-            warning() << "Balancer settings could not be loaded and will be retried in "
-                      << durationCount<Seconds>(kInitBackoffInterval) << " seconds"
-                      << causedBy(refreshStatus);
+            LOGV2_WARNING(
+                21876,
+                "Balancer settings could not be loaded because of {error} and will be retried in "
+                "{backoffInterval}",
+                "Got error while refreshing balancer settings, will retry with a backoff",
+                "backoffInterval"_attr = Milliseconds(kInitBackoffInterval),
+                "error"_attr = refreshStatus);
 
             _sleepFor(opCtx.get(), kInitBackoffInterval);
             continue;
@@ -324,13 +346,13 @@ void Balancer::_mainThread() {
         break;
     }
 
-    log() << "CSRS balancer thread is recovering";
+    LOGV2(21857, "CSRS balancer thread is recovering");
 
     _migrationManager.finishRecovery(opCtx.get(),
                                      balancerConfig->getMaxChunkSizeBytes(),
                                      balancerConfig->getSecondaryThrottle());
 
-    log() << "CSRS balancer thread is recovered";
+    LOGV2(21858, "CSRS balancer thread is recovered");
 
     // Main balancer loop
     while (!_stopRequested()) {
@@ -345,22 +367,29 @@ void Balancer::_mainThread() {
 
             Status refreshStatus = balancerConfig->refreshAndCheck(opCtx.get());
             if (!refreshStatus.isOK()) {
-                warning() << "Skipping balancing round" << causedBy(refreshStatus);
+                LOGV2_WARNING(21877,
+                              "Skipping balancing round due to {error}",
+                              "Skipping balancing round",
+                              "error"_attr = refreshStatus);
                 _endRound(opCtx.get(), kBalanceRoundDefaultInterval);
                 continue;
             }
 
             if (!balancerConfig->shouldBalance()) {
-                LOG(1) << "Skipping balancing round because balancing is disabled";
+                LOGV2_DEBUG(21859, 1, "Skipping balancing round because balancing is disabled");
                 _endRound(opCtx.get(), kBalanceRoundDefaultInterval);
                 continue;
             }
 
             {
-                LOG(1) << "*** start balancing round. "
-                       << "waitForDelete: " << balancerConfig->waitForDelete()
-                       << ", secondaryThrottle: "
-                       << balancerConfig->getSecondaryThrottle().toBSON();
+                LOGV2_DEBUG(21860,
+                            1,
+                            "Start balancing round. waitForDelete: {waitForDelete}, "
+                            "secondaryThrottle: {secondaryThrottle}",
+                            "Start balancing round",
+                            "waitForDelete"_attr = balancerConfig->waitForDelete(),
+                            "secondaryThrottle"_attr =
+                                balancerConfig->getSecondaryThrottle().toBSON());
 
                 static Occasionally sampler;
                 if (sampler.tick()) {
@@ -369,16 +398,19 @@ void Balancer::_mainThread() {
 
                 Status status = _splitChunksIfNeeded(opCtx.get());
                 if (!status.isOK()) {
-                    warning() << "Failed to split chunks" << causedBy(status);
+                    LOGV2_WARNING(21878,
+                                  "Failed to split chunks due to {error}",
+                                  "Failed to split chunks",
+                                  "error"_attr = status);
                 } else {
-                    LOG(1) << "Done enforcing tag range boundaries.";
+                    LOGV2_DEBUG(21861, 1, "Done enforcing tag range boundaries.");
                 }
 
                 const auto candidateChunks =
                     uassertStatusOK(_chunkSelectionPolicy->selectChunksToMove(opCtx.get()));
 
                 if (candidateChunks.empty()) {
-                    LOG(1) << "no need to move any chunk";
+                    LOGV2_DEBUG(21862, 1, "No need to move any chunk");
                     _balancedLastTime = 0;
                 } else {
                     _balancedLastTime = _moveChunks(opCtx.get(), candidateChunks);
@@ -391,28 +423,30 @@ void Balancer::_mainThread() {
                         .ignore();
                 }
 
-                LOG(1) << "*** End of balancing round";
+                LOGV2_DEBUG(21863, 1, "End balancing round");
             }
 
-            auto balancerInterval = [&]() -> Milliseconds {
-                MONGO_FAIL_POINT_BLOCK(overrideBalanceRoundInterval, data) {
-                    int interval = data.getData()["intervalMs"].numberInt();
-                    log() << "overrideBalanceRoundInterval: using shorter balancing interval: "
-                          << interval << "ms";
+            Milliseconds balancerInterval =
+                _balancedLastTime ? kShortBalanceRoundInterval : kBalanceRoundDefaultInterval;
 
-                    return Milliseconds(interval);
-                }
-
-                return _balancedLastTime ? kShortBalanceRoundInterval
-                                         : kBalanceRoundDefaultInterval;
-            }();
+            overrideBalanceRoundInterval.execute([&](const BSONObj& data) {
+                balancerInterval = Milliseconds(data["intervalMs"].numberInt());
+                LOGV2(21864,
+                      "overrideBalanceRoundInterval: using shorter balancing interval: "
+                      "{balancerInterval}",
+                      "overrideBalanceRoundInterval: using shorter balancing interval",
+                      "balancerInterval"_attr = balancerInterval);
+            });
 
             _endRound(opCtx.get(), balancerInterval);
         } catch (const DBException& e) {
-            log() << "caught exception while doing balance: " << e.what();
+            LOGV2(21865,
+                  "caught exception while doing balance: {error}",
+                  "Error while doing balance",
+                  "error"_attr = e);
 
             // Just to match the opening statement if in log level 1
-            LOG(1) << "*** End of balancing round";
+            LOGV2_DEBUG(21866, 1, "End balancing round");
 
             // This round failed, tell the world!
             roundDetails.setFailed(e.what());
@@ -441,7 +475,7 @@ void Balancer::_mainThread() {
         _threadOperationContext = nullptr;
     }
 
-    log() << "CSRS balancer is now stopped";
+    LOGV2(21867, "CSRS balancer is now stopped");
 }
 
 bool Balancer::_stopRequested() {
@@ -506,8 +540,13 @@ bool Balancer::_checkOIDs(OperationContext* opCtx) {
             if (oids.count(x) == 0) {
                 oids[x] = shardId;
             } else {
-                log() << "error: 2 machines have " << x << " as oid machine piece: " << shardId
-                      << " and " << oids[x];
+                LOGV2(21868,
+                      "error: 2 machines have {oidMachine} as oid machine piece: {firstShardId} "
+                      "and {secondShardId}",
+                      "Two machines have the same oidMachine value",
+                      "oidMachine"_attr = x,
+                      "firstShardId"_attr = shardId,
+                      "secondShardId"_attr = oids[x]);
 
                 result = uassertStatusOK(s->runCommandWithFixedRetryAttempts(
                     opCtx,
@@ -532,7 +571,10 @@ bool Balancer::_checkOIDs(OperationContext* opCtx) {
                 return false;
             }
         } else {
-            log() << "warning: oidMachine not set on: " << s->toString();
+            LOGV2(21869,
+                  "warning: oidMachine not set on: {shard}",
+                  "warning: oidMachine not set on shard",
+                  "shard"_attr = s->toString());
         }
     }
 
@@ -564,8 +606,11 @@ Status Balancer::_splitChunksIfNeeded(OperationContext* opCtx) {
                                                   ChunkRange(splitInfo.minKey, splitInfo.maxKey),
                                                   splitInfo.splitKeys);
         if (!splitStatus.isOK()) {
-            warning() << "Failed to split chunk " << redact(splitInfo.toString())
-                      << causedBy(redact(splitStatus.getStatus()));
+            LOGV2_WARNING(21879,
+                          "Failed to split chunk {splitInfo} {error}",
+                          "Failed to split chunk",
+                          "splitInfo"_attr = redact(splitInfo.toString()),
+                          "error"_attr = redact(splitStatus.getStatus()));
         }
     }
 
@@ -578,7 +623,7 @@ int Balancer::_moveChunks(OperationContext* opCtx,
 
     // If the balancer was disabled since we started this round, don't start new chunk moves
     if (_stopRequested() || !balancerConfig->shouldBalance()) {
-        LOG(1) << "Skipping balancing round because balancer was stopped";
+        LOGV2_DEBUG(21870, 1, "Skipping balancing round because balancer was stopped");
         return 0;
     }
 
@@ -607,18 +652,30 @@ int Balancer::_moveChunks(OperationContext* opCtx,
                                             });
         invariant(requestIt != candidateChunks.end());
 
-        if (status == ErrorCodes::ChunkTooBig) {
+        // ChunkTooBig is returned by the source shard during the cloning phase if the migration
+        // manager finds that the chunk is larger than some calculated size, the source shard is
+        // *not* in draining mode, and the 'forceJumbo' balancer setting is 'kDoNotForce'.
+        // ExceededMemoryLimit is returned when the transfer mods queue surpasses 500MB regardless
+        // of whether the source shard is in draining mode or the value if the 'froceJumbo' balancer
+        // setting.
+        if (status == ErrorCodes::ChunkTooBig || status == ErrorCodes::ExceededMemoryLimit) {
             numChunksProcessed++;
 
-            log() << "Performing a split because migration " << redact(requestIt->toString())
-                  << " failed for size reasons" << causedBy(redact(status));
+            LOGV2(21871,
+                  "Migration {migrateInfo} failed with {error}, going to try splitting the chunk",
+                  "Migration failed, going to try splitting the chunk",
+                  "migrateInfo"_attr = redact(requestIt->toString()),
+                  "error"_attr = redact(status));
 
             _splitOrMarkJumbo(opCtx, requestIt->nss, requestIt->minKey);
             continue;
         }
 
-        log() << "Balancer move " << redact(requestIt->toString()) << " failed"
-              << causedBy(redact(status));
+        LOGV2(21872,
+              "Migration {migrateInfo} failed with {error}",
+              "Migration failed",
+              "migrateInfo"_attr = redact(requestIt->toString()),
+              "error"_attr = redact(status));
     }
 
     return numChunksProcessed;
@@ -644,21 +701,27 @@ void Balancer::_splitOrMarkJumbo(OperationContext* opCtx,
             boost::none));
 
         if (splitPoints.empty()) {
-            log() << "Marking chunk " << redact(chunk.toString()) << " as jumbo.";
+            LOGV2(21873,
+                  "Marking chunk {chunk} as jumbo",
+                  "Marking chunk as jumbo",
+                  "chunk"_attr = redact(chunk.toString()));
             chunk.markAsJumbo();
 
-            const std::string chunkName = ChunkType::genID(nss, chunk.getMin());
-
             auto status = Grid::get(opCtx)->catalogClient()->updateConfigDocument(
                 opCtx,
                 ChunkType::ConfigNS,
-                BSON(ChunkType::name(chunkName)),
+                BSON(ChunkType::ns(nss.ns()) << ChunkType::min(chunk.getMin())),
                 BSON("$set" << BSON(ChunkType::jumbo(true))),
                 false,
                 ShardingCatalogClient::kMajorityWriteConcern);
             if (!status.isOK()) {
-                log() << "Couldn't set jumbo for chunk: " << redact(chunkName)
-                      << causedBy(redact(status.getStatus()));
+                LOGV2(21874,
+                      "Couldn't mark chunk with namespace {namespace} and min key {minKey} as "
+                      "jumbo due to {error}",
+                      "Couldn't mark chunk as jumbo",
+                      "namespace"_attr = redact(nss.ns()),
+                      "minKey"_attr = redact(chunk.getMin()),
+                      "error"_attr = redact(status.getStatus()));
             }
 
             return;
@@ -681,4 +744,28 @@ void Balancer::notifyPersistedBalancerSettingsChanged() {
     _condVar.notify_all();
 }
 
+Balancer::BalancerStatus Balancer::getBalancerStatusForNs(OperationContext* opCtx,
+                                                          const NamespaceString& ns) {
+    auto splitChunks = uassertStatusOK(_chunkSelectionPolicy->selectChunksToSplit(opCtx, ns));
+    if (!splitChunks.empty()) {
+        return {false, kBalancerPolicyStatusZoneViolation.toString()};
+    }
+    auto chunksToMove = uassertStatusOK(_chunkSelectionPolicy->selectChunksToMove(opCtx, ns));
+    if (chunksToMove.empty()) {
+        return {true, boost::none};
+    }
+    const auto& migrationInfo = chunksToMove.front();
+
+    switch (migrationInfo.reason) {
+        case MigrateInfo::drain:
+            return {false, kBalancerPolicyStatusDraining.toString()};
+        case MigrateInfo::zoneViolation:
+            return {false, kBalancerPolicyStatusZoneViolation.toString()};
+        case MigrateInfo::chunksImbalance:
+            return {false, kBalancerPolicyStatusChunksImbalance.toString()};
+    }
+
+    return {true, boost::none};
+}
+
 }  // namespace mongo
