@@ -29,261 +29,81 @@
 
 #include "mongo/platform/basic.h"
 
-#include "mongo/client/remote_command_targeter_mock.h"
+#include <memory>
+
 #include "mongo/db/commands.h"
+#include "mongo/db/read_write_concern_defaults.h"
+#include "mongo/db/read_write_concern_defaults_cache_lookup_mock.h"
 #include "mongo/db/s/balancer/migration_manager.h"
-#include "mongo/db/s/balancer/type_migration.h"
-#include "mongo/db/write_concern_options.h"
-#include "mongo/s/catalog/dist_lock_manager_mock.h"
-#include "mongo/s/catalog/type_collection.h"
-#include "mongo/s/catalog/type_database.h"
-#include "mongo/s/catalog/type_locks.h"
-#include "mongo/s/catalog/type_shard.h"
-#include "mongo/s/config_server_test_fixture.h"
-#include "mongo/s/database_version_helpers.h"
+#include "mongo/db/s/balancer/migration_test_fixture.h"
+#include "mongo/db/s/config/sharding_catalog_manager.h"
 #include "mongo/s/request_types/move_chunk_request.h"
-#include "mongo/stdx/memory.h"
-#include "mongo/util/scopeguard.h"
 
 namespace mongo {
 namespace {
 
-using executor::RemoteCommandRequest;
-using executor::RemoteCommandResponse;
-using std::vector;
-using unittest::assertGet;
-
-const auto kShardId0 = ShardId("shard0");
-const auto kShardId1 = ShardId("shard1");
-const auto kShardId2 = ShardId("shard2");
-const auto kShardId3 = ShardId("shard3");
-
-const HostAndPort kShardHost0 = HostAndPort("TestHost0", 12345);
-const HostAndPort kShardHost1 = HostAndPort("TestHost1", 12346);
-const HostAndPort kShardHost2 = HostAndPort("TestHost2", 12347);
-const HostAndPort kShardHost3 = HostAndPort("TestHost3", 12348);
-
 const MigrationSecondaryThrottleOptions kDefaultSecondaryThrottle =
     MigrationSecondaryThrottleOptions::create(MigrationSecondaryThrottleOptions::kDefault);
 
-const long long kMaxSizeMB = 100;
-const std::string kPattern = "_id";
-
-const WriteConcernOptions kMajorityWriteConcern(WriteConcernOptions::kMajority,
-                                                WriteConcernOptions::SyncMode::UNSET,
-                                                Seconds(15));
-
-class MigrationManagerTest : public ConfigServerTestFixture {
+class MigrationManagerTest : public MigrationTestFixture {
 protected:
-    /**
-     * Returns the mock targeter for the specified shard. Useful to use like so
-     *
-     *     shardTargeterMock(opCtx, shardId)->setFindHostReturnValue(shardHost);
-     *
-     * Then calls to RemoteCommandTargeterMock::findHost will return HostAndPort "shardHost" for
-     * Shard "shardId".
-     *
-     * Scheduling a command requires a shard host target. The command will be caught by the mock
-     * network, but sending the command requires finding the shard's host.
-     */
-    std::shared_ptr<RemoteCommandTargeterMock> shardTargeterMock(OperationContext* opCtx,
-                                                                 ShardId shardId);
+    void setUp() override {
+        MigrationTestFixture::setUp();
+        _migrationManager = std::make_unique<MigrationManager>(getServiceContext());
+        _migrationManager->startRecoveryAndAcquireDistLocks(operationContext());
+        _migrationManager->finishRecovery(operationContext(), 0, kDefaultSecondaryThrottle);
+
+        // Necessary because the migration manager may take a dist lock, which calls serverStatus
+        // and will attempt to return the latest read write concern defaults.
+        ReadWriteConcernDefaults::create(getServiceContext(), _lookupMock.getFetchDefaultsFn());
+    }
+
+    void tearDown() override {
+        checkMigrationsCollectionIsEmptyAndLocksAreUnlocked();
+        _migrationManager->interruptAndDisableMigrations();
+        _migrationManager->drainActiveMigrations();
+        _migrationManager.reset();
+        ConfigServerTestFixture::tearDown();
+    }
 
     /**
-     * Inserts a document into the config.databases collection to indicate that "dbName" is sharded
-     * with primary "primaryShard".
-     */
-    void setUpDatabase(const std::string& dbName, const ShardId primaryShard);
-
-    /**
-     * Inserts a document into the config.collections collection to indicate that "collName" is
-     * sharded with version "version". The shard key pattern defaults to "_id".
-     */
-    void setUpCollection(const NamespaceString& collName, ChunkVersion version);
-
-    /**
-     * Inserts a document into the config.chunks collection so that the chunk defined by the
-     * parameters exists. Returns a ChunkType defined by the parameters.
-     */
-    ChunkType setUpChunk(const NamespaceString& collName,
-                         const BSONObj& chunkMin,
-                         const BSONObj& chunkMax,
-                         const ShardId& shardId,
-                         const ChunkVersion& version);
-
-    /**
-     * Inserts a document into the config.migrations collection as an active migration.
-     */
-    void setUpMigration(const ChunkType& chunk, const ShardId& toShard);
-
-    /**
-     * Asserts that config.migrations is empty and config.locks contains no locked documents other
-     * than the balancer's, both of which should be true if the MigrationManager is inactive and
-     * behaving properly.
-     */
-    void checkMigrationsCollectionIsEmptyAndLocksAreUnlocked();
-
-    /**
-     * Sets up mock network to expect a moveChunk command and return a fixed BSON response or a
+     * Sets up mock network to expect a moveChunk command and returns a fixed BSON response or a
      * "returnStatus".
      */
     void expectMoveChunkCommand(const ChunkType& chunk,
                                 const ShardId& toShardId,
-                                const BSONObj& response);
+                                const BSONObj& response) {
+        onCommand([&chunk, &toShardId, &response](const RemoteCommandRequest& request) {
+            NamespaceString nss(request.cmdObj.firstElement().valueStringData());
+            ASSERT_EQ(chunk.getNS(), nss);
+
+            const StatusWith<MoveChunkRequest> moveChunkRequestWithStatus =
+                MoveChunkRequest::createFromCommand(nss, request.cmdObj);
+            ASSERT_OK(moveChunkRequestWithStatus.getStatus());
+
+            ASSERT_EQ(chunk.getNS(), moveChunkRequestWithStatus.getValue().getNss());
+            ASSERT_BSONOBJ_EQ(chunk.getMin(), moveChunkRequestWithStatus.getValue().getMinKey());
+            ASSERT_BSONOBJ_EQ(chunk.getMax(), moveChunkRequestWithStatus.getValue().getMaxKey());
+            ASSERT_EQ(chunk.getShard(), moveChunkRequestWithStatus.getValue().getFromShardId());
+
+            ASSERT_EQ(toShardId, moveChunkRequestWithStatus.getValue().getToShardId());
+
+            return response;
+        });
+    }
+
     void expectMoveChunkCommand(const ChunkType& chunk,
                                 const ShardId& toShardId,
-                                const Status& returnStatus);
-
-    // Random static initialization order can result in X constructor running before Y constructor
-    // if X and Y are defined in different source files. Defining variables here to enforce order.
-    const BSONObj kShard0 =
-        BSON(ShardType::name(kShardId0.toString())
-             << ShardType::host(kShardHost0.toString()) << ShardType::maxSizeMB(kMaxSizeMB));
-    const BSONObj kShard1 =
-        BSON(ShardType::name(kShardId1.toString())
-             << ShardType::host(kShardHost1.toString()) << ShardType::maxSizeMB(kMaxSizeMB));
-    const BSONObj kShard2 =
-        BSON(ShardType::name(kShardId2.toString())
-             << ShardType::host(kShardHost2.toString()) << ShardType::maxSizeMB(kMaxSizeMB));
-    const BSONObj kShard3 =
-        BSON(ShardType::name(kShardId3.toString())
-             << ShardType::host(kShardHost3.toString()) << ShardType::maxSizeMB(kMaxSizeMB));
-
-    const KeyPattern kKeyPattern = KeyPattern(BSON(kPattern << 1));
+                                const Status& returnStatus) {
+        BSONObjBuilder resultBuilder;
+        CommandHelpers::appendCommandStatusNoThrow(resultBuilder, returnStatus);
+        expectMoveChunkCommand(chunk, toShardId, resultBuilder.obj());
+    }
 
     std::unique_ptr<MigrationManager> _migrationManager;
-
-private:
-    void setUp() override;
-    void tearDown() override;
+    ReadWriteConcernDefaultsLookupMock _lookupMock;
 };
 
-void MigrationManagerTest::setUp() {
-    ConfigServerTestFixture::setUp();
-    _migrationManager = stdx::make_unique<MigrationManager>(getServiceContext());
-    _migrationManager->startRecoveryAndAcquireDistLocks(operationContext());
-    _migrationManager->finishRecovery(operationContext(), 0, kDefaultSecondaryThrottle);
-}
-
-void MigrationManagerTest::tearDown() {
-    checkMigrationsCollectionIsEmptyAndLocksAreUnlocked();
-    _migrationManager->interruptAndDisableMigrations();
-    _migrationManager->drainActiveMigrations();
-    _migrationManager.reset();
-    ConfigServerTestFixture::tearDown();
-}
-
-std::shared_ptr<RemoteCommandTargeterMock> MigrationManagerTest::shardTargeterMock(
-    OperationContext* opCtx, ShardId shardId) {
-    return RemoteCommandTargeterMock::get(
-        uassertStatusOK(shardRegistry()->getShard(opCtx, shardId))->getTargeter());
-}
-
-void MigrationManagerTest::setUpDatabase(const std::string& dbName, const ShardId primaryShard) {
-    DatabaseType db(dbName, primaryShard, true, databaseVersion::makeNew());
-    ASSERT_OK(catalogClient()->insertConfigDocument(
-        operationContext(), DatabaseType::ConfigNS, db.toBSON(), kMajorityWriteConcern));
-}
-
-void MigrationManagerTest::setUpCollection(const NamespaceString& collName, ChunkVersion version) {
-    CollectionType coll;
-    coll.setNs(collName);
-    coll.setEpoch(version.epoch());
-    coll.setUpdatedAt(Date_t::fromMillisSinceEpoch(version.toLong()));
-    coll.setKeyPattern(kKeyPattern);
-    coll.setUnique(false);
-    ASSERT_OK(catalogClient()->insertConfigDocument(
-        operationContext(), CollectionType::ConfigNS, coll.toBSON(), kMajorityWriteConcern));
-}
-
-ChunkType MigrationManagerTest::setUpChunk(const NamespaceString& collName,
-                                           const BSONObj& chunkMin,
-                                           const BSONObj& chunkMax,
-                                           const ShardId& shardId,
-                                           const ChunkVersion& version) {
-    ChunkType chunk;
-    chunk.setNS(collName);
-    chunk.setMin(chunkMin);
-    chunk.setMax(chunkMax);
-    chunk.setShard(shardId);
-    chunk.setVersion(version);
-    ASSERT_OK(catalogClient()->insertConfigDocument(
-        operationContext(), ChunkType::ConfigNS, chunk.toConfigBSON(), kMajorityWriteConcern));
-    return chunk;
-}
-
-void MigrationManagerTest::setUpMigration(const ChunkType& chunk, const ShardId& toShard) {
-    BSONObjBuilder builder;
-    builder.append(MigrationType::ns(), chunk.getNS().ns());
-    builder.append(MigrationType::min(), chunk.getMin());
-    builder.append(MigrationType::max(), chunk.getMax());
-    builder.append(MigrationType::toShard(), toShard.toString());
-    builder.append(MigrationType::fromShard(), chunk.getShard().toString());
-    chunk.getVersion().appendWithField(&builder, "chunkVersion");
-
-    MigrationType migrationType = assertGet(MigrationType::fromBSON(builder.obj()));
-    ASSERT_OK(catalogClient()->insertConfigDocument(operationContext(),
-                                                    MigrationType::ConfigNS,
-                                                    migrationType.toBSON(),
-                                                    kMajorityWriteConcern));
-}
-
-void MigrationManagerTest::checkMigrationsCollectionIsEmptyAndLocksAreUnlocked() {
-    auto statusWithMigrationsQueryResponse =
-        shardRegistry()->getConfigShard()->exhaustiveFindOnConfig(
-            operationContext(),
-            ReadPreferenceSetting{ReadPreference::PrimaryOnly},
-            repl::ReadConcernLevel::kMajorityReadConcern,
-            MigrationType::ConfigNS,
-            BSONObj(),
-            BSONObj(),
-            boost::none);
-    Shard::QueryResponse migrationsQueryResponse =
-        uassertStatusOK(statusWithMigrationsQueryResponse);
-    ASSERT_EQUALS(0U, migrationsQueryResponse.docs.size());
-
-    auto statusWithLocksQueryResponse = shardRegistry()->getConfigShard()->exhaustiveFindOnConfig(
-        operationContext(),
-        ReadPreferenceSetting{ReadPreference::PrimaryOnly},
-        repl::ReadConcernLevel::kMajorityReadConcern,
-        LocksType::ConfigNS,
-        BSON(LocksType::state(LocksType::LOCKED) << LocksType::name("{ '$ne' : 'balancer'}")),
-        BSONObj(),
-        boost::none);
-    Shard::QueryResponse locksQueryResponse = uassertStatusOK(statusWithLocksQueryResponse);
-    ASSERT_EQUALS(0U, locksQueryResponse.docs.size());
-}
-
-void MigrationManagerTest::expectMoveChunkCommand(const ChunkType& chunk,
-                                                  const ShardId& toShardId,
-                                                  const BSONObj& response) {
-    onCommand([&chunk, &toShardId, &response](const RemoteCommandRequest& request) {
-        NamespaceString nss(request.cmdObj.firstElement().valueStringData());
-        ASSERT_EQ(chunk.getNS(), nss);
-
-        const StatusWith<MoveChunkRequest> moveChunkRequestWithStatus =
-            MoveChunkRequest::createFromCommand(nss, request.cmdObj);
-        ASSERT_OK(moveChunkRequestWithStatus.getStatus());
-
-        ASSERT_EQ(chunk.getNS(), moveChunkRequestWithStatus.getValue().getNss());
-        ASSERT_BSONOBJ_EQ(chunk.getMin(), moveChunkRequestWithStatus.getValue().getMinKey());
-        ASSERT_BSONOBJ_EQ(chunk.getMax(), moveChunkRequestWithStatus.getValue().getMaxKey());
-        ASSERT_EQ(chunk.getShard(), moveChunkRequestWithStatus.getValue().getFromShardId());
-
-        ASSERT_EQ(toShardId, moveChunkRequestWithStatus.getValue().getToShardId());
-
-        return response;
-    });
-}
-
-void MigrationManagerTest::expectMoveChunkCommand(const ChunkType& chunk,
-                                                  const ShardId& toShardId,
-                                                  const Status& returnStatus) {
-    BSONObjBuilder resultBuilder;
-    CommandHelpers::appendCommandStatusNoThrow(resultBuilder, returnStatus);
-    expectMoveChunkCommand(chunk, toShardId, resultBuilder.obj());
-}
-
 TEST_F(MigrationManagerTest, OneCollectionTwoMigrations) {
     // Set up two shards in the metadata.
     ASSERT_OK(catalogClient()->insertConfigDocument(
@@ -307,7 +127,14 @@ TEST_F(MigrationManagerTest, OneCollectionTwoMigrations) {
         setUpChunk(collName, BSON(kPattern << 49), kKeyPattern.globalMax(), kShardId2, version);
 
     // Going to request that these two chunks get migrated.
-    const std::vector<MigrateInfo> migrationRequests{{kShardId1, chunk1}, {kShardId3, chunk2}};
+    const std::vector<MigrateInfo> migrationRequests{{kShardId1,
+                                                      chunk1,
+                                                      MoveChunkRequest::ForceJumbo::kDoNotForce,
+                                                      MigrateInfo::chunksImbalance},
+                                                     {kShardId3,
+                                                      chunk2,
+                                                      MoveChunkRequest::ForceJumbo::kDoNotForce,
+                                                      MigrateInfo::chunksImbalance}};
 
     auto future = launchAsync([this, migrationRequests] {
         ThreadClient tc("Test", getGlobalServiceContext());
@@ -366,10 +193,22 @@ TEST_F(MigrationManagerTest, TwoCollectionsTwoMigrationsEach) {
         setUpChunk(collName2, BSON(kPattern << 49), kKeyPattern.globalMax(), kShardId2, version2);
 
     // Going to request that these four chunks get migrated.
-    const std::vector<MigrateInfo> migrationRequests{{kShardId1, chunk1coll1},
-                                                     {kShardId3, chunk2coll1},
-                                                     {kShardId1, chunk1coll2},
-                                                     {kShardId3, chunk2coll2}};
+    const std::vector<MigrateInfo> migrationRequests{{kShardId1,
+                                                      chunk1coll1,
+                                                      MoveChunkRequest::ForceJumbo::kDoNotForce,
+                                                      MigrateInfo::chunksImbalance},
+                                                     {kShardId3,
+                                                      chunk2coll1,
+                                                      MoveChunkRequest::ForceJumbo::kDoNotForce,
+                                                      MigrateInfo::chunksImbalance},
+                                                     {kShardId1,
+                                                      chunk1coll2,
+                                                      MoveChunkRequest::ForceJumbo::kDoNotForce,
+                                                      MigrateInfo::chunksImbalance},
+                                                     {kShardId3,
+                                                      chunk2coll2,
+                                                      MoveChunkRequest::ForceJumbo::kDoNotForce,
+                                                      MigrateInfo::chunksImbalance}};
 
     auto future = launchAsync([this, migrationRequests] {
         ThreadClient tc("Test", getGlobalServiceContext());
@@ -423,7 +262,14 @@ TEST_F(MigrationManagerTest, SourceShardNotFound) {
         setUpChunk(collName, BSON(kPattern << 49), kKeyPattern.globalMax(), kShardId2, version);
 
     // Going to request that these two chunks get migrated.
-    const std::vector<MigrateInfo> migrationRequests{{kShardId1, chunk1}, {kShardId3, chunk2}};
+    const std::vector<MigrateInfo> migrationRequests{{kShardId1,
+                                                      chunk1,
+                                                      MoveChunkRequest::ForceJumbo::kDoNotForce,
+                                                      MigrateInfo::chunksImbalance},
+                                                     {kShardId3,
+                                                      chunk2,
+                                                      MoveChunkRequest::ForceJumbo::kDoNotForce,
+                                                      MigrateInfo::chunksImbalance}};
 
     auto future = launchAsync([this, chunk1, chunk2, migrationRequests] {
         ThreadClient tc("Test", getGlobalServiceContext());
@@ -439,8 +285,9 @@ TEST_F(MigrationManagerTest, SourceShardNotFound) {
         MigrationStatuses migrationStatuses = _migrationManager->executeMigrationsForAutoBalance(
             opCtx.get(), migrationRequests, 0, kDefaultSecondaryThrottle, false);
 
-        ASSERT_OK(migrationStatuses.at(chunk1.getName()));
-        ASSERT_EQ(ErrorCodes::ReplicaSetNotFound, migrationStatuses.at(chunk2.getName()));
+        ASSERT_OK(migrationStatuses.at(migrationRequests.front().getName()));
+        ASSERT_EQ(ErrorCodes::ReplicaSetNotFound,
+                  migrationStatuses.at(migrationRequests.back().getName()));
     });
 
     // Expect only one moveChunk command to be called.
@@ -469,7 +316,10 @@ TEST_F(MigrationManagerTest, JumboChunkResponseBackwardsCompatibility) {
         setUpChunk(collName, kKeyPattern.globalMin(), kKeyPattern.globalMax(), kShardId0, version);
 
     // Going to request that this chunk gets migrated.
-    const std::vector<MigrateInfo> migrationRequests{{kShardId1, chunk1}};
+    const std::vector<MigrateInfo> migrationRequests{{kShardId1,
+                                                      chunk1,
+                                                      MoveChunkRequest::ForceJumbo::kDoNotForce,
+                                                      MigrateInfo::chunksImbalance}};
 
     auto future = launchAsync([this, chunk1, migrationRequests] {
         ThreadClient tc("Test", getGlobalServiceContext());
@@ -482,7 +332,8 @@ TEST_F(MigrationManagerTest, JumboChunkResponseBackwardsCompatibility) {
         MigrationStatuses migrationStatuses = _migrationManager->executeMigrationsForAutoBalance(
             opCtx.get(), migrationRequests, 0, kDefaultSecondaryThrottle, false);
 
-        ASSERT_EQ(ErrorCodes::ChunkTooBig, migrationStatuses.at(chunk1.getName()));
+        ASSERT_EQ(ErrorCodes::ChunkTooBig,
+                  migrationStatuses.at(migrationRequests.front().getName()));
     });
 
     // Expect only one moveChunk command to be called.
@@ -517,9 +368,16 @@ TEST_F(MigrationManagerTest, InterruptMigration) {
         // up a dummy host for kShardHost0.
         shardTargeterMock(opCtx.get(), kShardId0)->setFindHostReturnValue(kShardHost0);
 
-        ASSERT_EQ(ErrorCodes::BalancerInterrupted,
-                  _migrationManager->executeManualMigration(
-                      opCtx.get(), {kShardId1, chunk}, 0, kDefaultSecondaryThrottle, false));
+        ASSERT_EQ(
+            ErrorCodes::BalancerInterrupted,
+            _migrationManager->executeManualMigration(opCtx.get(),
+                                                      {kShardId1,
+                                                       chunk,
+                                                       MoveChunkRequest::ForceJumbo::kDoNotForce,
+                                                       MigrateInfo::chunksImbalance},
+                                                      0,
+                                                      kDefaultSecondaryThrottle,
+                                                      false));
     });
 
     // Wait till the move chunk request gets sent and pretend that it is stuck by never responding
@@ -542,8 +400,14 @@ TEST_F(MigrationManagerTest, InterruptMigration) {
 
     // Ensure that no new migrations can be scheduled
     ASSERT_EQ(ErrorCodes::BalancerInterrupted,
-              _migrationManager->executeManualMigration(
-                  operationContext(), {kShardId1, chunk}, 0, kDefaultSecondaryThrottle, false));
+              _migrationManager->executeManualMigration(operationContext(),
+                                                        {kShardId1,
+                                                         chunk,
+                                                         MoveChunkRequest::ForceJumbo::kDoNotForce,
+                                                         MigrateInfo::chunksImbalance},
+                                                        0,
+                                                        kDefaultSecondaryThrottle,
+                                                        false));
 
     // Ensure that the migration manager is no longer handling any migrations.
     _migrationManager->drainActiveMigrations();
@@ -557,17 +421,18 @@ TEST_F(MigrationManagerTest, InterruptMigration) {
             ReadPreferenceSetting{ReadPreference::PrimaryOnly},
             repl::ReadConcernLevel::kMajorityReadConcern,
             MigrationType::ConfigNS,
-            BSON(MigrationType::name(chunk.getName())),
+            BSON(MigrationType::ns(collName.ns()) << MigrationType::min(chunk.getMin())),
             BSONObj(),
             boost::none);
     Shard::QueryResponse migrationsQueryResponse =
         uassertStatusOK(statusWithMigrationsQueryResponse);
     ASSERT_EQUALS(1U, migrationsQueryResponse.docs.size());
 
-    ASSERT_OK(catalogClient()->removeConfigDocuments(operationContext(),
-                                                     MigrationType::ConfigNS,
-                                                     BSON(MigrationType::name(chunk.getName())),
-                                                     kMajorityWriteConcern));
+    ASSERT_OK(catalogClient()->removeConfigDocuments(
+        operationContext(),
+        MigrationType::ConfigNS,
+        BSON(MigrationType::ns(collName.ns()) << MigrationType::min(chunk.getMin())),
+        kMajorityWriteConcern));
 
     // Restore the migration manager back to the started state, which is expected by tearDown
     _migrationManager->startRecoveryAndAcquireDistLocks(operationContext());
@@ -605,8 +470,15 @@ TEST_F(MigrationManagerTest, RestartMigrationManager) {
         // up a dummy host for kShardHost0.
         shardTargeterMock(opCtx.get(), kShardId0)->setFindHostReturnValue(kShardHost0);
 
-        ASSERT_OK(_migrationManager->executeManualMigration(
-            opCtx.get(), {kShardId1, chunk1}, 0, kDefaultSecondaryThrottle, false));
+        ASSERT_OK(
+            _migrationManager->executeManualMigration(opCtx.get(),
+                                                      {kShardId1,
+                                                       chunk1,
+                                                       MoveChunkRequest::ForceJumbo::kDoNotForce,
+                                                       MigrateInfo::chunksImbalance},
+                                                      0,
+                                                      kDefaultSecondaryThrottle,
+                                                      false));
     });
 
     // Expect only one moveChunk command to be called.
@@ -752,6 +624,16 @@ TEST_F(MigrationManagerTest, RemoteCallErrorConversionToOperationFailed) {
     ChunkType chunk2 =
         setUpChunk(collName, BSON(kPattern << 49), kKeyPattern.globalMax(), kShardId2, version);
 
+    // Going to request that these two chunks get migrated.
+    const std::vector<MigrateInfo> migrationRequests{{kShardId1,
+                                                      chunk1,
+                                                      MoveChunkRequest::ForceJumbo::kDoNotForce,
+                                                      MigrateInfo::chunksImbalance},
+                                                     {kShardId3,
+                                                      chunk2,
+                                                      MoveChunkRequest::ForceJumbo::kDoNotForce,
+                                                      MigrateInfo::chunksImbalance}};
+
     auto future = launchAsync([&] {
         ThreadClient tc("Test", getGlobalServiceContext());
         auto opCtx = cc().makeOperationContext();
@@ -762,14 +644,12 @@ TEST_F(MigrationManagerTest, RemoteCallErrorConversionToOperationFailed) {
         shardTargeterMock(opCtx.get(), kShardId2)->setFindHostReturnValue(kShardHost2);
 
         MigrationStatuses migrationStatuses = _migrationManager->executeMigrationsForAutoBalance(
-            opCtx.get(),
-            {{kShardId1, chunk1}, {kShardId3, chunk2}},
-            0,
-            kDefaultSecondaryThrottle,
-            false);
+            opCtx.get(), migrationRequests, 0, kDefaultSecondaryThrottle, false);
 
-        ASSERT_EQ(ErrorCodes::OperationFailed, migrationStatuses.at(chunk1.getName()));
-        ASSERT_EQ(ErrorCodes::OperationFailed, migrationStatuses.at(chunk2.getName()));
+        ASSERT_EQ(ErrorCodes::OperationFailed,
+                  migrationStatuses.at(migrationRequests.front().getName()));
+        ASSERT_EQ(ErrorCodes::OperationFailed,
+                  migrationStatuses.at(migrationRequests.back().getName()));
     });
 
     // Expect a moveChunk command that will fail with a retriable error.
