@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kStorage
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #if defined(__linux__)
 #include <sys/vfs.h>
@@ -51,7 +51,7 @@
 #include "mongo/db/storage/wiredtiger/wiredtiger_record_store.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_server_status.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/processinfo.h"
 
 #if __has_feature(address_sanitizer)
@@ -67,7 +67,7 @@ public:
     virtual StorageEngine* create(const StorageGlobalParams& params,
                                   const StorageEngineLockFile* lockFile) const {
         if (lockFile && lockFile->createdByUncleanShutdown()) {
-            warning() << "Recovering data from the last clean checkpoint.";
+            LOGV2_WARNING(22302, "Recovering data from the last clean checkpoint.");
         }
 
 #if defined(__linux__)
@@ -79,13 +79,11 @@ public:
             int ret = statfs(params.dbpath.c_str(), &fs_stats);
 
             if (ret == 0 && fs_stats.f_type == EXT4_SUPER_MAGIC) {
-                log() << startupWarningsLog;
-                log() << "** WARNING: Using the XFS filesystem is strongly recommended with the "
-                         "WiredTiger storage engine"
-                      << startupWarningsLog;
-                log() << "**          See "
-                         "http://dochub.mongodb.org/core/prodnotes-filesystem"
-                      << startupWarningsLog;
+                LOGV2_OPTIONS(22297,
+                              {logv2::LogTag::kStartupWarnings},
+                              "Using the XFS filesystem is strongly recommended with the "
+                              "WiredTiger storage engine. See "
+                              "http://dochub.mongodb.org/core/prodnotes-filesystem");
             }
         }
 #endif
@@ -95,40 +93,41 @@ public:
         ProcessInfo p;
         if (p.supported()) {
             if (cacheMB > memoryThresholdPercentage * p.getMemSizeMB()) {
-                log() << startupWarningsLog;
-                log() << "** WARNING: The configured WiredTiger cache size is more than "
-                      << memoryThresholdPercentage * 100 << "% of available RAM."
-                      << startupWarningsLog;
-                log() << "**          See "
-                         "http://dochub.mongodb.org/core/faq-memory-diagnostics-wt"
-                      << startupWarningsLog;
+                LOGV2_OPTIONS(22300,
+                              {logv2::LogTag::kStartupWarnings},
+                              "The configured WiredTiger cache size is more than 80% of available "
+                              "RAM. See http://dochub.mongodb.org/core/faq-memory-diagnostics-wt");
             }
         }
         const bool ephemeral = false;
-        const auto maxCacheOverflowMB =
-            static_cast<size_t>(1024 * wiredTigerGlobalOptions.maxCacheOverflowFileSizeGB);
         WiredTigerKVEngine* kv =
             new WiredTigerKVEngine(getCanonicalName().toString(),
                                    params.dbpath,
                                    getGlobalServiceContext()->getFastClockSource(),
                                    wiredTigerGlobalOptions.engineConfig,
                                    cacheMB,
-                                   maxCacheOverflowMB,
+                                   wiredTigerGlobalOptions.getMaxHistoryFileSizeMB(),
                                    params.dur,
                                    ephemeral,
                                    params.repair,
                                    params.readOnly);
         kv->setRecordStoreExtraOptions(wiredTigerGlobalOptions.collectionConfig);
         kv->setSortedDataInterfaceExtraOptions(wiredTigerGlobalOptions.indexConfig);
-        // Intentionally leaked.
-        MONGO_COMPILER_VARIABLE_UNUSED auto leakedSection = new WiredTigerServerStatusSection(kv);
-        MONGO_COMPILER_VARIABLE_UNUSED auto leakedSection2 = new OplogStonesServerStatusSection();
 
-        // This allows unit tests to run this code without encountering memory leaks
+        // We must only add the server parameters to the global registry once during unit testing.
+        static int setupCountForUnitTests = 0;
+        if (setupCountForUnitTests == 0) {
+            ++setupCountForUnitTests;
+
+            // Intentionally leaked.
+            MONGO_COMPILER_VARIABLE_UNUSED auto leakedSection =
+                new WiredTigerServerStatusSection(kv);
+
+            // This allows unit tests to run this code without encountering memory leaks
 #if __has_feature(address_sanitizer)
-        __lsan_ignore_object(leakedSection);
-        __lsan_ignore_object(leakedSection2);
+            __lsan_ignore_object(leakedSection);
 #endif
+        }
 
         StorageEngineOptions options;
         options.directoryPerDB = params.directoryperdb;
