@@ -33,6 +33,7 @@
 #include <iostream>
 #include <map>
 
+#include "mongo/db/catalog/uncommitted_collections.h"
 #include "mongo/db/commands/txn_cmds_gen.h"
 #include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/locker.h"
@@ -51,6 +52,7 @@
 #include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/transaction_metrics_observer.h"
 #include "mongo/idl/mutable_observer_registry.h"
+#include "mongo/logv2/attribute_storage.h"
 #include "mongo/stdx/unordered_map.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/concurrency/with_lock.h"
@@ -218,13 +220,16 @@ public:
         std::unique_ptr<RecoveryUnit> _recoveryUnit;
         repl::ReadConcernArgs _readConcernArgs;
         WriteUnitOfWork::RecoveryUnitState _ruState;
+        std::shared_ptr<UncommittedCollections::UncommittedCollectionsMap> _uncommittedCollections;
     };
 
     /**
-     *  An RAII object that stashes the recovery unit from the `opCtx` onto the stack and keeps
-     *  using the same locker of `opCtx`. The locker opts out of two-phase locking of the
-     *  current WUOW. At destruction it unstashes the recovery unit back onto the `opCtx` and
-     *  restores the locker state relevant to the original WUOW.
+     *  An RAII object that will allow the current transaction to be set aside so that a separate
+     *  transaction can be created. It stashes the recovery unit from the `opCtx` onto the stack and
+     *  keeps using the same locker of `opCtx`. The locker opts out of two-phase locking of the
+     *  current WUOW. At destruction the original transaction will be restored by unstashing the
+     *  recovery unit back onto the `opCtx` and restoring the locker state relevant to the original
+     *  WUOW.
      */
     class SideTransactionBlock {
     public:
@@ -514,24 +519,6 @@ public:
          */
         void refreshLocksForPreparedTransaction(OperationContext* opCtx, bool yieldLocks);
 
-        /**
-         * May only be called while a multi-document transaction is not committed and adds the
-         * multi-key path info to the set of path infos to be updated at commit time.
-         */
-        void addUncommittedMultikeyPathInfo(MultikeyPathInfo info) {
-            invariant(transactionIsOpen());
-            p().multikeyPathInfo.emplace_back(std::move(info));
-        }
-
-        /**
-         * May only be called while a multi-document transaction is not committed and returns the
-         * path infos which have been added so far.
-         */
-        const std::vector<MultikeyPathInfo>& getUncommittedMultikeyPathInfos() const {
-            invariant(transactionIsOpen());
-            return p().multikeyPathInfo;
-        }
-
         /**
          * Called after a write under the specified transaction completes while the node is a
          * primary and specifies the statement ids which were written. Must be called while the
@@ -612,6 +599,18 @@ public:
             return _transactionInfoForLog(opCtx, lockStats, terminationCause, readConcernArgs);
         }
 
+        BSONObj getTransactionInfoBSONForLogForTest(
+            OperationContext* opCtx,
+            const SingleThreadedLockStats* lockStats,
+            bool committed,
+            const repl::ReadConcernArgs& readConcernArgs) const {
+
+            TerminationCause terminationCause =
+                committed ? TerminationCause::kCommitted : TerminationCause::kAborted;
+            return _transactionInfoBSONForLog(opCtx, lockStats, terminationCause, readConcernArgs);
+        }
+
+
         SingleTransactionStats getSingleTransactionStatsForTest() const {
             return o().transactionMetricsObserver.getSingleTransactionStats();
         }
@@ -629,6 +628,7 @@ public:
             stdx::lock_guard<Client> lk(*opCtx->getClient());
             o(lk).prepareOpTime = prepareOpTime;
             o(lk).txnState.transitionTo(TransactionState::kPrepared);
+            opCtx->lockState()->unlockRSTLforPrepare();
         }
 
         void transitionToAbortedWithoutPrepareforTest(OperationContext* opCtx) {
@@ -662,7 +662,7 @@ public:
         // has been updated.
         void _finishCommitTransaction(OperationContext* opCtx,
                                       size_t operationCount,
-                                      size_t oplogOperationBytes);
+                                      size_t oplogOperationBytes) noexcept;
 
         // Commits the storage-transaction on the OperationContext.
         //
@@ -712,6 +712,17 @@ public:
                                            TerminationCause terminationCause,
                                            repl::ReadConcernArgs readConcernArgs) const;
 
+        void _transactionInfoForLog(OperationContext* opCtx,
+                                    const SingleThreadedLockStats* lockStats,
+                                    TerminationCause terminationCause,
+                                    repl::ReadConcernArgs readConcernArgs,
+                                    logv2::DynamicAttributes* pAttrs) const;
+
+        BSONObj _transactionInfoBSONForLog(OperationContext* opCtx,
+                                           const SingleThreadedLockStats* lockStats,
+                                           TerminationCause terminationCause,
+                                           repl::ReadConcernArgs readConcernArgs) const;
+
         // Bumps up the transaction number of this transaction and perform the necessary cleanup.
         void _setNewTxnNumber(OperationContext* opCtx, const TxnNumber& txnNumber);
 
@@ -913,15 +924,13 @@ private:
         // Total size in bytes of all operations within the _transactionOperations vector.
         size_t transactionOperationBytes{0};
 
+        // Number of operations that have pre-images to be written to noop oplog entries.
+        size_t numberOfPreImagesToWrite{0};
+
         // The autocommit setting of this transaction. Should always be false for multi-statement
         // transaction. Currently only needed for diagnostics reporting.
         boost::optional<bool> autoCommit;
 
-        // Contains uncommitted multi-key path info entries which were modified under this
-        // transaction so they can be applied to subsequent opreations before the transaction
-        // commits
-        std::vector<MultikeyPathInfo> multikeyPathInfo;
-
         //
         // Retryable writes state
         //
