@@ -29,6 +29,8 @@
 
 #include "mongo/platform/basic.h"
 
+#include <memory>
+
 #include "mongo/db/client.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbdirectclient.h"
@@ -46,7 +48,6 @@
 #include "mongo/db/session_catalog_mongod.h"
 #include "mongo/db/transaction_participant.h"
 #include "mongo/stdx/future.h"
-#include "mongo/stdx/memory.h"
 #include "mongo/unittest/death_test.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/net/socket_utils.h"
@@ -63,7 +64,7 @@ repl::OplogEntry makeOplogEntry(repl::OpTime opTime,
                                 repl::OpTypeEnum opType,
                                 BSONObj object,
                                 OperationSessionInfo sessionInfo,
-                                boost::optional<Date_t> wallClockTime,
+                                Date_t wallClockTime,
                                 boost::optional<StmtId> stmtId,
                                 boost::optional<repl::OpTime> prevWriteOpTimeInTransaction) {
     return repl::OplogEntry(
@@ -89,9 +90,11 @@ class OpObserverMock : public OpObserverNoop {
 public:
     void onTransactionPrepare(OperationContext* opCtx,
                               const std::vector<OplogSlot>& reservedSlots,
-                              std::vector<repl::ReplOperation>& statements) override {
+                              std::vector<repl::ReplOperation>* statements,
+                              size_t numberOfPreImagesToWrite) override {
         ASSERT_TRUE(opCtx->lockState()->inAWriteUnitOfWork());
-        OpObserverNoop::onTransactionPrepare(opCtx, reservedSlots, statements);
+        OpObserverNoop::onTransactionPrepare(
+            opCtx, reservedSlots, statements, numberOfPreImagesToWrite);
 
         uassert(ErrorCodes::OperationFailed,
                 "onTransactionPrepare() failed",
@@ -102,12 +105,13 @@ public:
 
     bool onTransactionPrepareThrowsException = false;
     bool transactionPrepared = false;
-    stdx::function<void()> onTransactionPrepareFn = [this]() { transactionPrepared = true; };
+    std::function<void()> onTransactionPrepareFn = [this]() { transactionPrepared = true; };
 
-    void onUnpreparedTransactionCommit(
-        OperationContext* opCtx, const std::vector<repl::ReplOperation>& statements) override {
+    void onUnpreparedTransactionCommit(OperationContext* opCtx,
+                                       std::vector<repl::ReplOperation>* statements,
+                                       size_t numberOfPreImagesToWrite) override {
         ASSERT_TRUE(opCtx->lockState()->inAWriteUnitOfWork());
-        OpObserverNoop::onUnpreparedTransactionCommit(opCtx, statements);
+        OpObserverNoop::onUnpreparedTransactionCommit(opCtx, statements, numberOfPreImagesToWrite);
 
         uassert(ErrorCodes::OperationFailed,
                 "onUnpreparedTransactionCommit() failed",
@@ -119,7 +123,7 @@ public:
     bool onUnpreparedTransactionCommitThrowsException = false;
     bool unpreparedTransactionCommitted = false;
 
-    stdx::function<void()> onUnpreparedTransactionCommitFn = [this]() {
+    std::function<void()> onUnpreparedTransactionCommitFn = [this]() {
         unpreparedTransactionCommitted = true;
     };
 
@@ -142,7 +146,7 @@ public:
 
     bool onPreparedTransactionCommitThrowsException = false;
     bool preparedTransactionCommitted = false;
-    stdx::function<void(OplogSlot, Timestamp)> onPreparedTransactionCommitFn =
+    std::function<void(OplogSlot, Timestamp)> onPreparedTransactionCommitFn =
         [this](OplogSlot commitOplogEntryOpTime, Timestamp commitTimestamp) {
             preparedTransactionCommitted = true;
         };
@@ -172,7 +176,7 @@ protected:
         MongoDSessionCatalog::onStepUp(opCtx());
 
         const auto opObserverRegistry = dynamic_cast<OpObserverRegistry*>(service->getOpObserver());
-        opObserverRegistry->addObserver(stdx::make_unique<OpObserverMock>());
+        opObserverRegistry->addObserver(std::make_unique<OpObserverMock>());
 
         opCtx()->setLogicalSessionId(makeLogicalSessionIdForTest());
         _opContextSession.emplace(opCtx());
@@ -200,25 +204,19 @@ protected:
                               TxnNumber txnNumber,
                               StmtId stmtId,
                               repl::OpTime prevOpTime) {
-        OperationSessionInfo osi;
-        osi.setSessionId(lsid);
-        osi.setTxnNumber(txnNumber);
-
-        repl::OplogLink link;
-        link.prevOpTime = prevOpTime;
-
-        return repl::logOp(opCtx,
-                           "n",
-                           nss,
-                           uuid,
-                           BSON("TestValue" << 0),
-                           nullptr,
-                           false,
-                           Date_t::now(),
-                           osi,
-                           stmtId,
-                           link,
-                           OplogSlot());
+        repl::MutableOplogEntry oplogEntry;
+        oplogEntry.setOpType(repl::OpTypeEnum::kNoop);
+        oplogEntry.setNss(nss);
+        oplogEntry.setUuid(uuid);
+        oplogEntry.setObject(BSON("TestValue" << 0));
+        oplogEntry.setWallClockTime(Date_t::now());
+        if (stmtId != kUninitializedStmtId) {
+            oplogEntry.setSessionId(lsid);
+            oplogEntry.setTxnNumber(txnNumber);
+            oplogEntry.setStatementId(stmtId);
+            oplogEntry.setPrevWriteOpTimeInTransaction(prevOpTime);
+        }
+        return repl::logOp(opCtx, &oplogEntry);
     }
 
     repl::OpTime writeTxnRecord(TxnNumber txnNum,
@@ -476,17 +474,18 @@ TEST_F(TransactionParticipantRetryableWritesTest, CheckStatementExecuted) {
     ASSERT(txnParticipant.checkStatementExecutedNoOplogEntryFetch(2000));
 }
 
-DEATH_TEST_F(TransactionParticipantRetryableWritesTest,
-             CheckStatementExecutedForInvalidatedTransactionInvariants,
-             "Invariant failure p().isValid") {
+DEATH_TEST_REGEX_F(TransactionParticipantRetryableWritesTest,
+                   CheckStatementExecutedForInvalidatedTransactionInvariants,
+                   R"#(Invariant failure.*p\(\).isValid)#") {
     auto txnParticipant = TransactionParticipant::get(opCtx());
     txnParticipant.invalidate(opCtx());
     txnParticipant.checkStatementExecuted(opCtx(), 0);
 }
 
-DEATH_TEST_F(TransactionParticipantRetryableWritesTest,
-             WriteOpCompletedOnPrimaryForOldTransactionInvariants,
-             "Invariant failure sessionTxnRecord.getTxnNum() == o().activeTxnNumber") {
+DEATH_TEST_REGEX_F(
+    TransactionParticipantRetryableWritesTest,
+    WriteOpCompletedOnPrimaryForOldTransactionInvariants,
+    R"#(Invariant failure.*sessionTxnRecord.getTxnNum\(\) == o\(\).activeTxnNumber)#") {
     auto txnParticipant = TransactionParticipant::get(opCtx());
     txnParticipant.refreshFromStorageIfNeeded(opCtx());
 
@@ -524,9 +523,10 @@ DEATH_TEST_F(TransactionParticipantRetryableWritesTest,
     }
 }
 
-DEATH_TEST_F(TransactionParticipantRetryableWritesTest,
-             WriteOpCompletedOnPrimaryForInvalidatedTransactionInvariants,
-             "Invariant failure sessionTxnRecord.getTxnNum() == o().activeTxnNumber") {
+DEATH_TEST_REGEX_F(
+    TransactionParticipantRetryableWritesTest,
+    WriteOpCompletedOnPrimaryForInvalidatedTransactionInvariants,
+    R"#(Invariant failure.*sessionTxnRecord.getTxnNum\(\) == o\(\).activeTxnNumber)#") {
     auto txnParticipant = TransactionParticipant::get(opCtx());
     txnParticipant.refreshFromStorageIfNeeded(opCtx());
 
@@ -595,7 +595,7 @@ TEST_F(TransactionParticipantRetryableWritesTest, IncompleteHistoryDueToOpLogTru
             sessionRecord.setSessionId(sessionId);
             sessionRecord.setTxnNum(txnNum);
             sessionRecord.setLastWriteOpTime(entry2.getOpTime());
-            sessionRecord.setLastWriteDate(*entry2.getWallClockTime());
+            sessionRecord.setLastWriteDate(entry2.getWallClockTime());
             return sessionRecord.toBSON();
         }());
     }
@@ -625,34 +625,32 @@ TEST_F(TransactionParticipantRetryableWritesTest, ErrorOnlyWhenStmtIdBeingChecke
     txnParticipant.refreshFromStorageIfNeeded(opCtx());
     txnParticipant.beginOrContinue(opCtx(), txnNum, boost::none, boost::none);
 
-    OperationSessionInfo osi;
-    osi.setSessionId(sessionId);
-    osi.setTxnNumber(txnNum);
+    repl::MutableOplogEntry oplogEntry;
+    oplogEntry.setSessionId(sessionId);
+    oplogEntry.setTxnNumber(txnNum);
+    oplogEntry.setNss(kNss);
+    oplogEntry.setUuid(uuid);
 
     auto firstOpTime = ([&]() {
+        oplogEntry.setOpType(repl::OpTypeEnum::kInsert);
+        oplogEntry.setObject(BSON("x" << 1));
+        oplogEntry.setObject2(TransactionParticipant::kDeadEndSentinel);
+        oplogEntry.setPrevWriteOpTimeInTransaction(repl::OpTime());
+        oplogEntry.setStatementId(1);
+
         AutoGetCollection autoColl(opCtx(), kNss, MODE_IX);
         WriteUnitOfWork wuow(opCtx());
 
         const auto wallClockTime = Date_t::now();
+        oplogEntry.setWallClockTime(wallClockTime);
 
-        auto opTime = repl::logOp(opCtx(),
-                                  "i",
-                                  kNss,
-                                  uuid,
-                                  BSON("x" << 1),
-                                  &TransactionParticipant::kDeadEndSentinel,
-                                  false,
-                                  wallClockTime,
-                                  osi,
-                                  1,
-                                  {},
-                                  OplogSlot());
+        auto opTime = repl::logOp(opCtx(), &oplogEntry);
 
         SessionTxnRecord sessionTxnRecord;
         sessionTxnRecord.setSessionId(sessionId);
         sessionTxnRecord.setTxnNum(txnNum);
         sessionTxnRecord.setLastWriteOpTime(opTime);
-        sessionTxnRecord.setLastWriteDate(Date_t::now());
+        sessionTxnRecord.setLastWriteDate(wallClockTime);
         txnParticipant.onWriteOpCompletedOnPrimary(opCtx(), {1}, sessionTxnRecord);
         wuow.commit();
 
@@ -660,26 +658,19 @@ TEST_F(TransactionParticipantRetryableWritesTest, ErrorOnlyWhenStmtIdBeingChecke
     })();
 
     {
-        repl::OplogLink link;
-        link.prevOpTime = firstOpTime;
+        oplogEntry.setOpType(repl::OpTypeEnum::kNoop);
+        oplogEntry.setObject({});
+        oplogEntry.setObject2(TransactionParticipant::kDeadEndSentinel);
+        oplogEntry.setPrevWriteOpTimeInTransaction(firstOpTime);
+        oplogEntry.setStatementId(kIncompleteHistoryStmtId);
 
         AutoGetCollection autoColl(opCtx(), kNss, MODE_IX);
         WriteUnitOfWork wuow(opCtx());
 
         const auto wallClockTime = Date_t::now();
+        oplogEntry.setWallClockTime(wallClockTime);
 
-        auto opTime = repl::logOp(opCtx(),
-                                  "n",
-                                  kNss,
-                                  uuid,
-                                  {},
-                                  &TransactionParticipant::kDeadEndSentinel,
-                                  false,
-                                  wallClockTime,
-                                  osi,
-                                  kIncompleteHistoryStmtId,
-                                  link,
-                                  OplogSlot());
+        auto opTime = repl::logOp(opCtx(), &oplogEntry);
 
         SessionTxnRecord sessionTxnRecord;
         sessionTxnRecord.setSessionId(sessionId);
