@@ -1,7 +1,3 @@
-// repltests.cpp : Unit tests for replication
-//
-
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -31,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kDefault
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/platform/basic.h"
 
@@ -41,7 +37,6 @@
 #include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/client.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
-#include "mongo/db/db.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/json.h"
@@ -50,20 +45,23 @@
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
-#include "mongo/db/repl/sync_tail.h"
 #include "mongo/db/s/op_observer_sharding_impl.h"
 #include "mongo/dbtests/dbtests.h"
+#include "mongo/logger/logger.h"
+#include "mongo/logv2/log.h"
 #include "mongo/transport/transport_layer_asio.h"
-#include "mongo/util/log.h"
 
 using namespace mongo::repl;
 
 namespace ReplTests {
 
-using std::unique_ptr;
+using mongo::logger::globalLogDomain;
+using mongo::logv2::LogComponent;
+using mongo::logv2::LogSeverity;
 using std::endl;
 using std::string;
 using std::stringstream;
+using std::unique_ptr;
 using std::vector;
 
 /**
@@ -85,7 +83,7 @@ repl::OplogEntry makeOplogEntry(repl::OpTime opTime,
                             object2,                    // o2
                             {},                         // sessionInfo
                             boost::none,                // upsert
-                            boost::none,                // wall clock time
+                            Date_t(),                   // wall clock time
                             boost::none,                // statement id
                             boost::none,   // optime of previous write within same transaction
                             boost::none,   // pre-image optime
@@ -108,11 +106,6 @@ public:
         : _client(&_opCtx),
           _defaultReplSettings(
               ReplicationCoordinator::get(getGlobalServiceContext())->getSettings()) {
-        // Replication is not supported by mobile SE.
-        if (mongo::storageGlobalParams.engine == "mobile") {
-            return;
-        }
-
         transport::TransportLayerASIO::Options opts;
         opts.mode = transport::TransportLayerASIO::Options::kEgress;
         auto sc = getGlobalServiceContext();
@@ -135,27 +128,28 @@ public:
         // to avoid the invariant in ReplClientInfo::setLastOp that the optime only goes forward.
         repl::ReplClientInfo::forClient(_opCtx.getClient()).clearLastOp_forTest();
 
-        getGlobalServiceContext()->setOpObserver(stdx::make_unique<OpObserverShardingImpl>());
+        getGlobalServiceContext()->setOpObserver(std::make_unique<OpObserverShardingImpl>());
 
         setOplogCollectionName(getGlobalServiceContext());
         createOplog(&_opCtx);
 
-        OldClientWriteContext ctx(&_opCtx, ns());
+        dbtests::WriteContextForTests ctx(&_opCtx, ns());
         WriteUnitOfWork wuow(&_opCtx);
 
-        Collection* c = ctx.db()->getCollection(&_opCtx, ns());
+        Collection* c = CollectionCatalog::get(&_opCtx).lookupCollectionByNamespace(&_opCtx, nss());
         if (!c) {
-            c = ctx.db()->createCollection(&_opCtx, ns());
+            c = ctx.db()->createCollection(&_opCtx, nss());
         }
 
         ASSERT(c->getIndexCatalog()->haveIdIndex(&_opCtx));
         wuow.commit();
+
+        _opCtx.getServiceContext()->getStorageEngine()->setOldestTimestamp(Timestamp(1, 1));
+
+        // Start with a fresh oplog.
+        deleteAll(cllNS());
     }
     ~Base() {
-        // Replication is not supported by mobile SE.
-        if (mongo::storageGlobalParams.engine == "mobile") {
-            return;
-        }
         try {
             deleteAll(ns());
             deleteAll(cllNS());
@@ -178,6 +172,9 @@ protected:
     static const char* ns() {
         return "unittests.repltests";
     }
+    static NamespaceString nss() {
+        return NamespaceString(ns());
+    }
     static const char* cllNS() {
         return "local.oplog.rs";
     }
@@ -188,7 +185,7 @@ protected:
         check(o, one(o));
     }
     void checkAll(const BSONObj& o) const {
-        unique_ptr<DBClientCursor> c = _client.query(ns(), o);
+        unique_ptr<DBClientCursor> c = _client.query(NamespaceString(ns()), o);
         verify(c->more());
         while (c->more()) {
             check(o, c->next());
@@ -196,22 +193,22 @@ protected:
     }
     void check(const BSONObj& expected, const BSONObj& got) const {
         if (expected.woCompare(got)) {
-            ::mongo::log() << "expected: " << expected.toString() << ", got: " << got.toString()
-                           << endl;
+            LOGV2(22500,
+                  "expected: {expected}, got: {got}",
+                  "expected"_attr = expected.toString(),
+                  "got"_attr = got.toString());
         }
         ASSERT_BSONOBJ_EQ(expected, got);
     }
-    BSONObj oneOp() const {
-        return _client.findOne(cllNS(), BSONObj());
-    }
     int count() const {
         Lock::GlobalWrite lk(&_opCtx);
         OldClientContext ctx(&_opCtx, ns());
         Database* db = ctx.db();
-        Collection* coll = db->getCollection(&_opCtx, ns());
+        Collection* coll =
+            CollectionCatalog::get(&_opCtx).lookupCollectionByNamespace(&_opCtx, nss());
         if (!coll) {
             WriteUnitOfWork wunit(&_opCtx);
-            coll = db->createCollection(&_opCtx, ns());
+            coll = db->createCollection(&_opCtx, nss());
             wunit.commit();
         }
 
@@ -223,14 +220,14 @@ protected:
         return count;
     }
     int opCount() {
-        return DBDirectClient(&_opCtx).query(cllNS(), BSONObj())->itcount();
+        return DBDirectClient(&_opCtx).query(NamespaceString(cllNS()), BSONObj())->itcount();
     }
     void applyAllOperations() {
         Lock::GlobalWrite lk(&_opCtx);
         vector<BSONObj> ops;
         {
             DBDirectClient db(&_opCtx);
-            auto cursor = db.query(cllNS(), BSONObj());
+            auto cursor = db.query(NamespaceString(cllNS()), BSONObj());
             while (cursor->more()) {
                 ops.push_back(cursor->nextSafe());
             }
@@ -248,42 +245,27 @@ protected:
             OldClientContext ctx(&_opCtx, ns());
             for (vector<BSONObj>::iterator i = ops.begin(); i != ops.end(); ++i) {
                 if (0) {
-                    mongo::unittest::log() << "op: " << *i << endl;
+                    LOGV2(22501, "op: {i}", "i"_attr = *i);
                 }
                 repl::UnreplicatedWritesBlock uwb(&_opCtx);
+                auto entry = uassertStatusOK(OplogEntry::parse(*i));
                 uassertStatusOK(applyOperation_inlock(
-                    &_opCtx, ctx.db(), *i, false, OplogApplication::Mode::kSecondary));
+                    &_opCtx, ctx.db(), &entry, false, OplogApplication::Mode::kSecondary));
             }
         }
     }
-    void printAll(const char* ns) {
-        Lock::GlobalWrite lk(&_opCtx);
-        OldClientContext ctx(&_opCtx, ns);
-
-        Database* db = ctx.db();
-        Collection* coll = db->getCollection(&_opCtx, ns);
-        if (!coll) {
-            WriteUnitOfWork wunit(&_opCtx);
-            coll = db->createCollection(&_opCtx, ns);
-            wunit.commit();
-        }
-
-        auto cursor = coll->getCursor(&_opCtx);
-        ::mongo::log() << "all for " << ns << endl;
-        while (auto record = cursor->next()) {
-            ::mongo::log() << record->data.releaseToBson() << endl;
-        }
-    }
     // These deletes don't get logged.
     void deleteAll(const char* ns) const {
         ::mongo::writeConflictRetry(&_opCtx, "deleteAll", ns, [&] {
+            NamespaceString nss(ns);
             Lock::GlobalWrite lk(&_opCtx);
             OldClientContext ctx(&_opCtx, ns);
             WriteUnitOfWork wunit(&_opCtx);
             Database* db = ctx.db();
-            Collection* coll = db->getCollection(&_opCtx, ns);
+            Collection* coll =
+                CollectionCatalog::get(&_opCtx).lookupCollectionByNamespace(&_opCtx, nss);
             if (!coll) {
-                coll = db->createCollection(&_opCtx, ns);
+                coll = db->createCollection(&_opCtx, nss);
             }
 
             ASSERT_OK(coll->truncate(&_opCtx));
@@ -295,16 +277,29 @@ protected:
         OldClientContext ctx(&_opCtx, ns());
         WriteUnitOfWork wunit(&_opCtx);
         Database* db = ctx.db();
-        Collection* coll = db->getCollection(&_opCtx, ns());
+        Collection* coll =
+            CollectionCatalog::get(&_opCtx).lookupCollectionByNamespace(&_opCtx, nss());
         if (!coll) {
-            coll = db->createCollection(&_opCtx, ns());
+            coll = db->createCollection(&_opCtx, nss());
         }
 
+        auto lastApplied = repl::ReplicationCoordinator::get(getGlobalServiceContext())
+                               ->getMyLastAppliedOpTime()
+                               .getTimestamp();
+        // The oplog collection may already have some oplog entries for writes prior to this insert.
+        // And the oplog visibility timestamp may already reflect those entries (e.g. no holes exist
+        // before lastApplied). Thus, it is invalid to do any timestamped writes using a timestamp
+        // less than or equal to the WT "all_durable" timestamp. Therefore, we use the next
+        // timestamp of the lastApplied to be safe. In the case where there is no oplog entries in
+        // the oplog collection, we will use a non-zero timestamp (Timestamp(1, 1)) for the insert.
+        auto nextTimestamp =
+            std::max(Timestamp(lastApplied.getSecs(), lastApplied.getInc() + 1), Timestamp(1, 1));
         OpDebug* const nullOpDebug = nullptr;
         if (o.hasField("_id")) {
             repl::UnreplicatedWritesBlock uwb(&_opCtx);
             coll->insertDocument(&_opCtx, InsertStatement(o), nullOpDebug, true)
                 .transitional_ignore();
+            ASSERT_OK(_opCtx.recoveryUnit()->setTimestamp(nextTimestamp));
             wunit.commit();
             return;
         }
@@ -317,6 +312,7 @@ protected:
         repl::UnreplicatedWritesBlock uwb(&_opCtx);
         coll->insertDocument(&_opCtx, InsertStatement(b.obj()), nullOpDebug, true)
             .transitional_ignore();
+        ASSERT_OK(_opCtx.recoveryUnit()->setTimestamp(nextTimestamp));
         wunit.commit();
     }
     static BSONObj wid(const char* json) {
@@ -333,13 +329,9 @@ protected:
 class LogBasic : public Base {
 public:
     void run() {
-        // Replication is not supported by mobile SE.
-        if (mongo::storageGlobalParams.engine == "mobile") {
-            return;
-        }
-        ASSERT_EQUALS(1, opCount());
+        ASSERT_EQUALS(0, opCount());
         _client.insert(ns(), fromjson("{\"a\":\"b\"}"));
-        ASSERT_EQUALS(2, opCount());
+        ASSERT_EQUALS(1, opCount());
     }
 };
 
@@ -349,10 +341,6 @@ class Base : public ReplTests::Base {
 public:
     virtual ~Base() {}
     void run() {
-        // Replication is not supported by mobile SE.
-        if (mongo::storageGlobalParams.engine == "mobile") {
-            return;
-        }
         reset();
         doIt();
         int nOps = opCount();
@@ -773,7 +761,8 @@ class MultiInc : public Base {
 public:
     string s() const {
         stringstream ss;
-        unique_ptr<DBClientCursor> cc = _client.query(ns(), Query().sort(BSON("_id" << 1)));
+        unique_ptr<DBClientCursor> cc =
+            _client.query(NamespaceString(ns()), Query().sort(BSON("_id" << 1)));
         bool first = true;
         while (cc->more()) {
             if (first)
@@ -969,21 +958,6 @@ public:
     }
 };
 
-class PushWithDollarSigns : public Base {
-    void doIt() const {
-        _client.update(ns(), BSON("_id" << 0), BSON("$push" << BSON("a" << BSON("$foo" << 1))));
-    }
-    using ReplTests::Base::check;
-    void check() const {
-        ASSERT_EQUALS(1, count());
-        check(fromjson("{'_id':0, a:[0, {'$foo':1}]}"), one(fromjson("{'_id':0}")));
-    }
-    void reset() const {
-        deleteAll(ns());
-        insert(BSON("_id" << 0 << "a" << BSON_ARRAY(0)));
-    }
-};
-
 class PushSlice : public Base {
     void doIt() const {
         _client.update(
@@ -1269,21 +1243,6 @@ public:
     }
 };
 
-class AddToSetWithDollarSigns : public Base {
-    void doIt() const {
-        _client.update(ns(), BSON("_id" << 0), BSON("$addToSet" << BSON("a" << BSON("$foo" << 1))));
-    }
-    using ReplTests::Base::check;
-    void check() const {
-        ASSERT_EQUALS(1, count());
-        check(fromjson("{'_id':0, a:[0, {'$foo':1}]}"), one(fromjson("{'_id':0}")));
-    }
-    void reset() const {
-        deleteAll(ns());
-        insert(BSON("_id" << 0 << "a" << BSON_ARRAY(0)));
-    }
-};
-
 //
 // replay cases
 //
@@ -1334,95 +1293,27 @@ public:
 class DeleteOpIsIdBased : public Base {
 public:
     void run() {
-        // Replication is not supported by mobile SE.
-        if (mongo::storageGlobalParams.engine == "mobile") {
-            return;
-        }
+        // These inserts don't write oplog entries.
         insert(BSON("_id" << 0 << "a" << 10));
         insert(BSON("_id" << 1 << "a" << 11));
         insert(BSON("_id" << 3 << "a" << 10));
         _client.remove(ns(), BSON("a" << 10));
-        ASSERT_EQUALS(1U, _client.count(ns(), BSONObj()));
+        ASSERT_EQUALS(1U, _client.count(nss(), BSONObj()));
         insert(BSON("_id" << 0 << "a" << 11));
         insert(BSON("_id" << 2 << "a" << 10));
         insert(BSON("_id" << 3 << "a" << 10));
-
+        // Now the collection has _ids 0, 1, 2, 3. Apply the delete oplog entries for _id 0 and 3.
         applyAllOperations();
-        ASSERT_EQUALS(2U, _client.count(ns(), BSONObj()));
+        // _id 1 and 2 remain.
+        ASSERT_EQUALS(2U, _client.count(nss(), BSONObj()));
         ASSERT(!one(BSON("_id" << 1)).isEmpty());
         ASSERT(!one(BSON("_id" << 2)).isEmpty());
     }
 };
 
-class SyncTest : public SyncTail {
+class All : public OldStyleSuiteSpecification {
 public:
-    bool returnEmpty;
-    SyncTest()
-        : SyncTail(nullptr, nullptr, nullptr, SyncTail::MultiSyncApplyFunc(), nullptr),
-          returnEmpty(false) {}
-    virtual ~SyncTest() {}
-    BSONObj getMissingDoc(OperationContext* opCtx, const OplogEntry& oplogEntry) override {
-        if (returnEmpty) {
-            BSONObj o;
-            return o;
-        }
-        return BSON("_id"
-                    << "on remote"
-                    << "foo"
-                    << "baz");
-    }
-};
-
-class FetchAndInsertMissingDocument : public Base {
-public:
-    void run() {
-        // Replication is not supported by mobile SE.
-        if (mongo::storageGlobalParams.engine == "mobile") {
-            return;
-        }
-        bool threw = false;
-        auto oplogEntry = makeOplogEntry(OpTime(Timestamp(100, 1), 1LL),  // optime
-                                         OpTypeEnum::kUpdate,             // op type
-                                         NamespaceString(ns()),           // namespace
-                                         BSON("foo"
-                                              << "bar"),  // o
-                                         BSON("_id"
-                                              << "in oplog"
-                                              << "foo"
-                                              << "bar"));  // o2
-
-        Lock::GlobalWrite lk(&_opCtx);
-
-        // this should fail because we can't connect
-        try {
-            SyncTail badSource(nullptr, nullptr, nullptr, SyncTail::MultiSyncApplyFunc(), nullptr);
-            badSource.setHostname("localhost:123");
-
-            OldClientContext ctx(&_opCtx, ns());
-            badSource.getMissingDoc(&_opCtx, oplogEntry);
-        } catch (DBException&) {
-            threw = true;
-        }
-        verify(threw);
-
-        // now this should succeed
-        SyncTest t;
-        verify(t.fetchAndInsertMissingDocument(&_opCtx, oplogEntry));
-        verify(!_client
-                    .findOne(ns(),
-                             BSON("_id"
-                                  << "on remote"))
-                    .isEmpty());
-
-        // force it not to find an obj
-        t.returnEmpty = true;
-        verify(!t.fetchAndInsertMissingDocument(&_opCtx, oplogEntry));
-    }
-};
-
-class All : public Suite {
-public:
-    All() : Suite("repl") {}
+    All() : OldStyleSuiteSpecification("repl") {}
 
     void setupTests() {
         add<LogBasic>();
@@ -1477,10 +1368,9 @@ public:
         add<Idempotence::ReplaySetPreexistingNoOpPull>();
         add<Idempotence::ReplayArrayFieldNotAppended>();
         add<DeleteOpIsIdBased>();
-        add<FetchAndInsertMissingDocument>();
     }
 };
 
-SuiteInstance<All> myall;
+OldStyleSuiteInitializer<All> myall;
 
 }  // namespace ReplTests
