@@ -37,7 +37,6 @@
 
 #include "mongo/base/status_with.h"
 #include "mongo/client/connection_string.h"
-#include "mongo/db/commands/test_commands_enabled.h"
 #include "mongo/db/wire_version.h"
 #include "mongo/executor/connection_pool_stats.h"
 #include "mongo/executor/network_connection_hook.h"
@@ -161,10 +160,13 @@ public:
     }
 
     void setUp() override {
-        setTestCommandsEnabled(true);
         startNet(std::make_unique<WaitForIsMasterHook>(this));
     }
 
+    // NetworkInterfaceIntegrationFixture::tearDown() shuts down the NetworkInterface. We always
+    // need to do it even if we have additional tearDown tasks.
+    using NetworkInterfaceIntegrationFixture::tearDown;
+
     RemoteCommandRequest makeTestCommand(
         Milliseconds timeout,
         BSONObj cmd,
@@ -295,6 +297,19 @@ private:
     boost::optional<IsMasterData> _isMasterResult;
 };
 
+class NetworkInterfaceInternalClientTest : public NetworkInterfaceTest {
+public:
+    void setUp() override {
+        WireSpec::instance().isInternalClient = true;
+        NetworkInterfaceTest::setUp();
+    }
+
+    void tearDown() override {
+        NetworkInterfaceTest::tearDown();
+        WireSpec::instance().isInternalClient = false;
+    }
+};
+
 TEST_F(NetworkInterfaceTest, CancelMissingOperation) {
     // This is just a sanity check, this action should have no effect.
     net().cancelCommand(makeCallbackHandle());
@@ -322,7 +337,7 @@ TEST_F(NetworkInterfaceTest, CancelLocally) {
     // Wait for op to complete, assert that it was canceled.
     auto result = deferred.get();
     ASSERT_EQ(ErrorCodes::CallbackCanceled, result.status);
-    ASSERT(result.elapsedMillis);
+    ASSERT(result.elapsed);
 
     assertNumOps(1u, 0u, 0u, 0u);
 }
@@ -373,7 +388,7 @@ TEST_F(NetworkInterfaceTest, CancelRemotely) {
     // Wait for the command to return, assert that it was canceled.
     auto result = deferred.get();
     ASSERT_EQ(ErrorCodes::CallbackCanceled, result.status);
-    ASSERT(result.elapsedMillis);
+    ASSERT(result.elapsed);
 
     // Wait for the operation to be killed on the remote host.
     numCurrentOpRan += waitForCommandToStop("echo", kMaxWait);
@@ -439,7 +454,7 @@ TEST_F(NetworkInterfaceTest, CancelRemotelyTimedOut) {
     // Wait for the command to return, assert that it was canceled.
     auto result = deferred.get();
     ASSERT_EQ(ErrorCodes::CallbackCanceled, result.status);
-    ASSERT(result.elapsedMillis);
+    ASSERT(result.elapsed);
 
     // We have one canceled operation (echo), one timedout operation (_killOperations),
     // and one succeeded operation on top of the currentOp operations (configureFailPoint).
@@ -467,7 +482,7 @@ TEST_F(NetworkInterfaceTest, ImmediateCancel) {
     // Wait for op to complete, assert that it was canceled.
     auto result = deferred.get();
     ASSERT_EQ(ErrorCodes::CallbackCanceled, result.status);
-    ASSERT(result.elapsedMillis);
+    ASSERT(result.elapsed);
     assertNumOps(1u, 0u, 0u, 0u);
 }
 
@@ -481,7 +496,7 @@ TEST_F(NetworkInterfaceTest, LateCancel) {
     net().cancelCommand(cbh);
 
     ASSERT_OK(result.status);
-    ASSERT(result.elapsedMillis);
+    ASSERT(result.elapsed);
     assertNumOps(0u, 0u, 0u, 1u);
 }
 
@@ -499,7 +514,7 @@ TEST_F(NetworkInterfaceTest, AsyncOpTimeout) {
     // check that we've timed out.
     if (!pingCommandMissing(result)) {
         ASSERT_EQ(ErrorCodes::NetworkInterfaceExceededTimeLimit, result.status);
-        ASSERT(result.elapsedMillis);
+        ASSERT(result.elapsed);
         assertNumOps(0u, 1u, 0u, 0u);
     }
 }
@@ -531,11 +546,11 @@ TEST_F(NetworkInterfaceTest, AsyncOpTimeoutWithOpCtxDeadlineSooner) {
     }
 
     ASSERT_EQ(ErrorCodes::NetworkInterfaceExceededTimeLimit, result.status);
-    ASSERT(result.elapsedMillis);
+    ASSERT(result.elapsed);
     // check that the request timeout uses the smaller of the operation context deadline and
     // the timeout specified in the request constructor.
-    ASSERT_GTE(result.elapsedMillis.value(), opCtxDeadline);
-    ASSERT_LT(result.elapsedMillis.value(), requestTimeout);
+    ASSERT_GTE(result.elapsed.value(), opCtxDeadline);
+    ASSERT_LT(result.elapsed.value(), requestTimeout);
     assertNumOps(0u, 1u, 0u, 0u);
 }
 
@@ -565,11 +580,12 @@ TEST_F(NetworkInterfaceTest, AsyncOpTimeoutWithOpCtxDeadlineLater) {
     }
 
     ASSERT_EQ(ErrorCodes::NetworkInterfaceExceededTimeLimit, result.status);
-    ASSERT(result.elapsedMillis);
+    ASSERT(result.elapsed);
     // check that the request timeout uses the smaller of the operation context deadline and
     // the timeout specified in the request constructor.
-    ASSERT_GTE(result.elapsedMillis.value(), requestTimeout);
-    ASSERT_LT(result.elapsedMillis.value(), opCtxDeadline);
+    ASSERT_GTE(duration_cast<Milliseconds>(result.elapsed.value()), requestTimeout);
+    ASSERT_LT(duration_cast<Milliseconds>(result.elapsed.value()), opCtxDeadline);
+
     assertNumOps(0u, 1u, 0u, 0u);
 }
 
@@ -581,7 +597,7 @@ TEST_F(NetworkInterfaceTest, StartCommand) {
 
     auto res = deferred.get();
 
-    ASSERT(res.elapsedMillis);
+    ASSERT(res.elapsed);
     uassertStatusOK(res.status);
 
     // This opmsg request expect the following reply, which is generated below
@@ -629,7 +645,7 @@ TEST_F(NetworkInterfaceTest, FireAndForget) {
 
     for (auto& future : futures) {
         auto result = future.get();
-        ASSERT(result.elapsedMillis);
+        ASSERT(result.elapsed);
         uassertStatusOK(result.status);
         ASSERT_EQ(1, result.data.getIntField("ok"));
     }
@@ -637,14 +653,14 @@ TEST_F(NetworkInterfaceTest, FireAndForget) {
     // Run a non-fireAndForget command and verify that we get a CommandFailed response.
     auto nonFireAndForgetRequest = makeTestCommand(kNoTimeout, makeEchoCmdObj());
     auto result = runCommandSync(nonFireAndForgetRequest);
-    ASSERT(result.elapsedMillis);
+    ASSERT(result.elapsed);
     uassertStatusOK(result.status);
     ASSERT_EQ(0, result.data.getIntField("ok"));
     ASSERT_EQ(ErrorCodes::CommandFailed, result.data.getIntField("code"));
     assertNumOps(0u, 0u, 0u, 5u);
 }
 
-TEST_F(NetworkInterfaceTest, StartCommandOnAny) {
+TEST_F(NetworkInterfaceInternalClientTest, StartCommandOnAny) {
     // The echo command below uses hedging so after a response is returned, we will issue
     // a _killOperations command to kill the pending operation. As a result, the number of
     // successful commands can sometimes be 2 (echo and _killOperations) instead 1 when the
@@ -669,8 +685,8 @@ TEST_F(NetworkInterfaceTest, StartCommandOnAny) {
     auto deferred = runCommandOnAny(makeCallbackHandle(), std::move(request));
     auto res = deferred.get();
 
-    auto cmdObj = res.data.getObjectField("echo");
     uassertStatusOK(res.status);
+    auto cmdObj = res.data.getObjectField("echo");
     ASSERT_EQ(1, cmdObj.getIntField("echo"));
     ASSERT_EQ("bar"_sd, cmdObj.getStringField("foo"));
     ASSERT_EQ("admin"_sd, cmdObj.getStringField("$db"));
@@ -711,9 +727,8 @@ TEST_F(NetworkInterfaceTest, SetAlarm) {
     ASSERT_FALSE(swResult.isOK());
 }
 
-TEST_F(NetworkInterfaceTest, IsMasterRequestContainsOutgoingWireVersionInternalClientInfo) {
-    WireSpec::instance().isInternalClient = true;
-
+TEST_F(NetworkInterfaceInternalClientTest,
+       IsMasterRequestContainsOutgoingWireVersionInternalClientInfo) {
     auto deferred = runCommand(makeCallbackHandle(), makeTestCommand(kNoTimeout, makeEchoCmdObj()));
     auto isMasterHandshake = waitForIsMaster();
 
@@ -729,7 +744,7 @@ TEST_F(NetworkInterfaceTest, IsMasterRequestContainsOutgoingWireVersionInternalC
 
     // Verify that the ping op is counted as a success.
     auto res = deferred.get();
-    ASSERT(res.elapsedMillis);
+    ASSERT(res.elapsed);
     assertNumOps(0u, 0u, 0u, 1u);
 }
 
@@ -743,7 +758,7 @@ TEST_F(NetworkInterfaceTest, IsMasterRequestMissingInternalClientInfoWhenNotInte
     ASSERT_FALSE(isMasterHandshake.request["internalClient"]);
     // Verify that the ping op is counted as a success.
     auto res = deferred.get();
-    ASSERT(res.elapsedMillis);
+    ASSERT(res.elapsed);
     assertNumOps(0u, 0u, 0u, 1u);
 }
 
