@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
@@ -57,7 +57,6 @@
 #include "mongo/s/sharding_router_test_fixture.h"
 #include "mongo/s/write_ops/batched_command_response.h"
 #include "mongo/stdx/future.h"
-#include "mongo/util/log.h"
 #include "mongo/util/time_support.h"
 
 namespace mongo {
@@ -122,9 +121,11 @@ TEST_F(ShardingCatalogClientTest, GetCollectionExisting) {
                                      {newOpTime, Date_t() + Seconds(newOpTime.getSecs())},
                                      newOpTime,
                                      100,
+                                     0,
                                      OID(),
                                      30,
-                                     -1);
+                                     -1,
+                                     true);
             BSONObjBuilder builder;
             metadata.writeToMetadata(&builder).transitional_ignore();
 
@@ -195,9 +196,11 @@ TEST_F(ShardingCatalogClientTest, GetDatabaseExisting) {
                                  {newOpTime, Date_t() + Seconds(newOpTime.getSecs())},
                                  newOpTime,
                                  100,
+                                 0,
                                  OID(),
                                  30,
-                                 -1);
+                                 -1,
+                                 true);
         BSONObjBuilder builder;
         metadata.writeToMetadata(&builder).transitional_ignore();
 
@@ -361,6 +364,7 @@ TEST_F(ShardingCatalogClientTest, GetChunksForNSWithSortAndLimit) {
     OID oid = OID::gen();
 
     ChunkType chunkA;
+    chunkA.setName(OID::gen());
     chunkA.setNS(kNamespace);
     chunkA.setMin(BSON("a" << 1));
     chunkA.setMax(BSON("a" << 100));
@@ -368,6 +372,7 @@ TEST_F(ShardingCatalogClientTest, GetChunksForNSWithSortAndLimit) {
     chunkA.setShard(ShardId("shard0000"));
 
     ChunkType chunkB;
+    chunkB.setName(OID::gen());
     chunkB.setNS(kNamespace);
     chunkB.setMin(BSON("a" << 100));
     chunkB.setMax(BSON("a" << 200));
@@ -420,9 +425,11 @@ TEST_F(ShardingCatalogClientTest, GetChunksForNSWithSortAndLimit) {
                                      {newOpTime, Date_t() + Seconds(newOpTime.getSecs())},
                                      newOpTime,
                                      100,
+                                     0,
                                      OID(),
                                      30,
-                                     -1);
+                                     -1,
+                                     true);
             BSONObjBuilder builder;
             metadata.writeToMetadata(&builder).transitional_ignore();
 
@@ -821,9 +828,11 @@ TEST_F(ShardingCatalogClientTest, GetCollectionsValidResultsNoDb) {
                                  {newOpTime, Date_t() + Seconds(newOpTime.getSecs())},
                                  newOpTime,
                                  100,
+                                 0,
                                  OID(),
                                  30,
-                                 -1);
+                                 -1,
+                                 true);
         BSONObjBuilder builder;
         metadata.writeToMetadata(&builder).transitional_ignore();
 
@@ -1127,7 +1136,7 @@ TEST_F(ShardingCatalogClientTest, UpdateDatabase) {
     future.default_timed_get();
 }
 
-TEST_F(ShardingCatalogClientTest, UpdateConfigDocumentExceededTimeLimit) {
+TEST_F(ShardingCatalogClientTest, UpdateConfigDocumentNonRetryableError) {
     HostAndPort host1("TestHost1");
     configTargeter()->setFindHostReturnValue(host1);
 
@@ -1141,7 +1150,7 @@ TEST_F(ShardingCatalogClientTest, UpdateConfigDocumentExceededTimeLimit) {
                                                   dbt.toBSON(),
                                                   true,
                                                   ShardingCatalogClient::kMajorityWriteConcern);
-        ASSERT_EQ(ErrorCodes::ExceededTimeLimit, status);
+        ASSERT_EQ(ErrorCodes::Interrupted, status);
     });
 
     onCommand([host1](const RemoteCommandRequest& request) {
@@ -1149,7 +1158,7 @@ TEST_F(ShardingCatalogClientTest, UpdateConfigDocumentExceededTimeLimit) {
             ASSERT_EQUALS(host1, request.target);
 
             BatchedCommandResponse response;
-            response.setStatus({ErrorCodes::ExceededTimeLimit, "operation timed out"});
+            response.setStatus({ErrorCodes::Interrupted, "operation interrupted"});
 
             return response.toBSON();
         } catch (const DBException& ex) {
@@ -1241,12 +1250,12 @@ TEST_F(ShardingCatalogClientTest, ApplyChunkOpsDeprecatedSuccessfulWithCheck) {
     });
 
     onFindCommand([this](const RemoteCommandRequest& request) {
-        OID oid = OID::gen();
         ChunkType chunk;
+        chunk.setName(OID::gen());
         chunk.setNS(kNamespace);
         chunk.setMin(BSON("a" << 1));
         chunk.setMax(BSON("a" << 100));
-        chunk.setVersion({1, 2, oid});
+        chunk.setVersion({1, 2, OID::gen()});
         chunk.setShard(ShardId("shard0000"));
         return vector<BSONObj>{chunk.toConfigBSON()};
     });
