@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
@@ -47,12 +47,12 @@
 #include "mongo/db/query/query_request.h"
 #include "mongo/db/repl/read_concern_args.h"
 #include "mongo/executor/task_executor_pool.h"
+#include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/metadata/repl_set_metadata.h"
 #include "mongo/rpc/metadata/tracking_metadata.h"
 #include "mongo/s/client/shard_remote_gen.h"
 #include "mongo/s/grid.h"
-#include "mongo/util/log.h"
 #include "mongo/util/str.h"
 #include "mongo/util/time_support.h"
 
@@ -112,7 +112,7 @@ bool ShardRemote::isRetriableError(ErrorCodes::Error code, RetryPolicy options)
         } break;
 
         case RetryPolicy::kIdempotent: {
-            return ErrorCodes::isRetriableError(code);
+            return isMongosRetriableError(code);
         } break;
 
         case RetryPolicy::kNotIdempotent: {
@@ -165,15 +165,17 @@ std::string ShardRemote::toString() const {
 BSONObj ShardRemote::_appendMetadataForCommand(OperationContext* opCtx,
                                                const ReadPreferenceSetting& readPref) {
     BSONObjBuilder builder;
-    if (logger::globalLogDomain()->shouldLog(
-            logger::LogComponent::kTracking,
-            logger::LogSeverity::Debug(1))) {  // avoid performance overhead if not logging
+    if (shouldLog(logv2::LogComponent::kTracking,
+                  logv2::LogSeverity::Debug(1))) {  // avoid performance overhead if not logging
         if (!TrackingMetadata::get(opCtx).getIsLogged()) {
             if (!TrackingMetadata::get(opCtx).getOperId()) {
                 TrackingMetadata::get(opCtx).initWithOperName("NotSet");
             }
-            MONGO_LOG_COMPONENT(1, logger::LogComponent::kTracking)
-                << TrackingMetadata::get(opCtx).toString();
+            LOGV2_DEBUG_OPTIONS(20164,
+                                1,
+                                logv2::LogOptions{logv2::LogComponent::kTracking},
+                                "{trackingMetadata}",
+                                "trackingMetadata"_attr = TrackingMetadata::get(opCtx));
             TrackingMetadata::get(opCtx).setIsLogged(true);
         }
 
@@ -232,7 +234,10 @@ StatusWith<Shard::CommandResponse> ShardRemote::_runCommand(OperationContext* op
 
     if (!response.status.isOK()) {
         if (ErrorCodes::isExceededTimeLimitError(response.status.code())) {
-            LOG(0) << "Operation timed out with status " << redact(response.status);
+            LOGV2(22739,
+                  "Operation timed out {error}",
+                  "Operation timed out",
+                  "error"_attr = redact(response.status));
         }
         return response.status;
     }
@@ -281,8 +286,8 @@ StatusWith<Shard::QueryResponse> ShardRemote::_runExhaustiveCursorCommand(
 
         if (data.otherFields.metadata.hasField(rpc::kReplSetMetadataFieldName)) {
             // Sharding users of ReplSetMetadata do not require the wall clock time field to be set
-            auto replParseStatus = rpc::ReplSetMetadata::readFromMetadata(
-                data.otherFields.metadata, /*requireWallTime*/ false);
+            auto replParseStatus =
+                rpc::ReplSetMetadata::readFromMetadata(data.otherFields.metadata);
             if (!replParseStatus.isOK()) {
                 status = replParseStatus.getStatus();
                 response.docs.clear();
@@ -330,7 +335,8 @@ StatusWith<Shard::QueryResponse> ShardRemote::_runExhaustiveCursorCommand(
 
     if (!status.isOK()) {
         if (ErrorCodes::isExceededTimeLimitError(status.code())) {
-            LOG(0) << "Operation timed out " << causedBy(status);
+            LOGV2(
+                22740, "Operation timed out {error}", "Operation timed out", "error"_attr = status);
         }
         return status;
     }
