@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
@@ -44,7 +44,9 @@
 #include "mongo/db/stats/counters.h"
 #include "mongo/db/storage/duplicate_key_error_info.h"
 #include "mongo/executor/task_executor_pool.h"
+#include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/s/client/num_hosts_targeted_metrics.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/cluster_commands_helpers.h"
 #include "mongo/s/cluster_last_error_info.h"
@@ -59,7 +61,6 @@
 #include "mongo/s/write_ops/batched_command_response.h"
 #include "mongo/s/write_ops/chunk_manager_targeter.h"
 #include "mongo/s/write_ops/cluster_write.h"
-#include "mongo/util/log.h"
 #include "mongo/util/timer.h"
 
 namespace mongo {
@@ -77,7 +78,7 @@ void batchErrorToLastError(const BatchedCommandRequest& request,
 
     if (!response.getOk()) {
         // Command-level error, all writes failed
-        commandError = stdx::make_unique<WriteErrorDetail>();
+        commandError = std::make_unique<WriteErrorDetail>();
         commandError->setStatus(response.getTopLevelStatus());
         lastBatchError = commandError.get();
     } else if (response.isErrDetailsSet()) {
@@ -116,7 +117,7 @@ void batchErrorToLastError(const BatchedCommandRequest& request,
         }
 
         const int numUpserted = response.isUpsertDetailsSet() ? response.sizeUpsertDetails() : 0;
-        const int numMatched = response.getN() - numUpserted;
+        const auto numMatched = response.getN() - numUpserted;
         invariant(numMatched >= 0);
 
         // Wrap upserted id in "upserted" field
@@ -205,10 +206,9 @@ bool handleWouldChangeOwningShardError(OperationContext* opCtx,
     bool updatedShardKey = false;
     boost::optional<BSONObj> upsertedId;
     if (isRetryableWrite) {
-        if (MONGO_FAIL_POINT(hangAfterThrowWouldChangeOwningShardRetryableWrite)) {
-            log() << "Hit hangAfterThrowWouldChangeOwningShardRetryableWrite failpoint";
-            MONGO_FAIL_POINT_PAUSE_WHILE_SET_OR_INTERRUPTED(
-                opCtx, hangAfterThrowWouldChangeOwningShardRetryableWrite);
+        if (MONGO_unlikely(hangAfterThrowWouldChangeOwningShardRetryableWrite.shouldFail())) {
+            LOGV2(22759, "Hit hangAfterThrowWouldChangeOwningShardRetryableWrite failpoint");
+            hangAfterThrowWouldChangeOwningShardRetryableWrite.pauseWhileSet(opCtx);
         }
         RouterOperationContextSession routerSession(opCtx);
         try {
@@ -233,13 +233,9 @@ bool handleWouldChangeOwningShardError(OperationContext* opCtx,
             // If we do not get WouldChangeOwningShard when re-running the update, the document has
             // been modified or deleted concurrently and we do not need to delete it and insert a
             // new one.
-            updatedShardKey =
-                wouldChangeOwningShardErrorInfo &&
+            updatedShardKey = wouldChangeOwningShardErrorInfo &&
                 documentShardKeyUpdateUtil::updateShardKeyForDocument(
-                    opCtx,
-                    request->getNS(),
-                    wouldChangeOwningShardErrorInfo.get(),
-                    boost::get_optional_value_or(request->getWriteCommandBase().getStmtId(), 0));
+                                  opCtx, request->getNS(), wouldChangeOwningShardErrorInfo.get());
 
             // If the operation was an upsert, record the _id of the new document.
             if (updatedShardKey && wouldChangeOwningShardErrorInfo->getShouldUpsert()) {
@@ -264,7 +260,7 @@ bool handleWouldChangeOwningShardError(OperationContext* opCtx,
             }
 
             if (!response->isErrDetailsSet() || !response->getErrDetails().back()) {
-                auto error = stdx::make_unique<WriteErrorDetail>();
+                auto error = std::make_unique<WriteErrorDetail>();
                 error->setIndex(0);
                 response->addToErrDetails(error.release());
             }
@@ -283,10 +279,7 @@ bool handleWouldChangeOwningShardError(OperationContext* opCtx,
         try {
             // Delete the original document and insert the new one
             updatedShardKey = documentShardKeyUpdateUtil::updateShardKeyForDocument(
-                opCtx,
-                request->getNS(),
-                wouldChangeOwningShardErrorInfo.get(),
-                boost::get_optional_value_or(request->getWriteCommandBase().getStmtId(), 0));
+                opCtx, request->getNS(), wouldChangeOwningShardErrorInfo.get());
 
             // If the operation was an upsert, record the _id of the new document.
             if (updatedShardKey && wouldChangeOwningShardErrorInfo->getShouldUpsert()) {
@@ -307,7 +300,7 @@ bool handleWouldChangeOwningShardError(OperationContext* opCtx,
         response->setN(response->getN() + 1);
 
         if (upsertedId) {
-            auto upsertDetail = stdx::make_unique<BatchedUpsertDetail>();
+            auto upsertDetail = std::make_unique<BatchedUpsertDetail>();
             upsertDetail->setIndex(0);
             upsertDetail->setUpsertedID(upsertedId.get());
             response->addToUpsertDetails(upsertDetail.release());
@@ -319,6 +312,30 @@ bool handleWouldChangeOwningShardError(OperationContext* opCtx,
     return updatedShardKey;
 }
 
+void updateHostsTargetedMetrics(OperationContext* opCtx,
+                                BatchedCommandRequest::BatchType batchType,
+                                int nShardsOwningChunks,
+                                int nShardsTargeted) {
+    NumHostsTargetedMetrics::QueryType writeType;
+    switch (batchType) {
+        case BatchedCommandRequest::BatchType_Insert:
+            writeType = NumHostsTargetedMetrics::QueryType::kInsertCmd;
+            break;
+        case BatchedCommandRequest::BatchType_Update:
+            writeType = NumHostsTargetedMetrics::QueryType::kUpdateCmd;
+            break;
+        case BatchedCommandRequest::BatchType_Delete:
+            writeType = NumHostsTargetedMetrics::QueryType::kDeleteCmd;
+            break;
+
+            MONGO_UNREACHABLE;
+    }
+
+    auto targetType = NumHostsTargetedMetrics::get(opCtx).parseTargetType(
+        opCtx, nShardsTargeted, nShardsOwningChunks);
+    NumHostsTargetedMetrics::get(opCtx).addNumHostsTargeted(writeType, targetType);
+}
+
 /**
  * Base class for mongos write commands.
  */
@@ -375,7 +392,13 @@ private:
         // Assemble requests
         std::vector<AsyncRequestsSender::Request> requests;
         for (const auto& endpoint : swEndpoints.getValue()) {
-            requests.emplace_back(endpoint.shardName, command);
+            BSONObj cmdObjWithVersions = BSONObj(command);
+            if (endpoint.databaseVersion) {
+                cmdObjWithVersions =
+                    appendDbVersionIfPresent(cmdObjWithVersions, *endpoint.databaseVersion);
+            }
+            cmdObjWithVersions = appendShardVersion(cmdObjWithVersions, endpoint.shardVersion);
+            requests.emplace_back(endpoint.shardName, cmdObjWithVersions);
         }
 
         // Send the requests.
@@ -442,13 +465,32 @@ private:
                  const OpMsgRequest& request,
                  BatchedCommandRequest& batchedRequest,
                  BSONObjBuilder& result) const {
-        auto db = batchedRequest.getNS().db();
-        if (db != NamespaceString::kAdminDb && db != NamespaceString::kConfigDb) {
-            batchedRequest.setAllowImplicitCreate(false);
-        }
-
         BatchWriteExecStats stats;
         BatchedCommandResponse response;
+
+        // The batched request will only have WC if it was supplied by the client. Otherwise, the
+        // batched request should use the WC from the opCtx.
+        if (!batchedRequest.hasWriteConcern()) {
+            if (opCtx->getWriteConcern().usedDefault) {
+                // Pass writeConcern: {}, rather than {w: 1, wtimeout: 0}, so as to not override the
+                // configsvr w:majority upconvert.
+                batchedRequest.setWriteConcern(BSONObj());
+            } else {
+                batchedRequest.setWriteConcern(opCtx->getWriteConcern().toBSON());
+            }
+        }
+
+        // Write ops are never allowed to have writeConcern inside transactions. Normally
+        // disallowing WC on non-terminal commands in a transaction is handled earlier, during
+        // command dispatch. However, if this is a regular write operation being automatically
+        // retried inside a transaction (such as changing a document's shard key across shards),
+        // then batchedRequest will have a writeConcern (added by the if() above) from when it was
+        // initially run outside a transaction. Thus it's necessary to unconditionally clear the
+        // writeConcern when in a transaction.
+        if (TransactionRouter::get(opCtx)) {
+            batchedRequest.unsetWriteConcern();
+        }
+
         ClusterWriter::write(opCtx, batchedRequest, &stats, &response);
 
         bool updatedShardKey = false;
@@ -496,11 +538,10 @@ private:
                 debug.additiveMetrics.nModified = response.getNModified();
                 for (auto&& update : _batchedRequest.getUpdateRequest().getUpdates()) {
                     // If this was a pipeline style update, record which stages were being used.
-                    const auto& updateMod = update.getU();
+                    auto updateMod = update.getU();
                     if (updateMod.type() == write_ops::UpdateModification::Type::kPipeline) {
-                        auto request = AggregationRequest(_batchedRequest.getNS(),
-                                                          updateMod.getUpdatePipeline());
-                        auto pipeline = LiteParsedPipeline(request);
+                        auto pipeline = LiteParsedPipeline(_batchedRequest.getNS(),
+                                                           updateMod.getUpdatePipeline());
                         pipeline.tickGlobalStageCounters();
                     }
                 }
@@ -522,6 +563,13 @@ private:
         CurOp::get(opCtx)->debug().nShards =
             stats.getTargetedShards().size() + (updatedShardKey ? 1 : 0);
 
+        if (stats.getNumShardsOwningChunks().is_initialized())
+            updateHostsTargetedMetrics(opCtx,
+                                       _batchedRequest.getBatchType(),
+                                       stats.getNumShardsOwningChunks().get(),
+                                       stats.getTargetedShards().size() +
+                                           (updatedShardKey ? 1 : 0));
+
         if (auto txnRouter = TransactionRouter::get(opCtx)) {
             auto writeCmdStatus = response.toStatus();
             if (!writeCmdStatus.isOK()) {
@@ -573,8 +621,8 @@ private:
         return true;
     }
 
-    bool supportsReadConcern(repl::ReadConcernLevel level) const final {
-        return true;
+    ReadConcernSupportResult supportsReadConcern(repl::ReadConcernLevel level) const final {
+        return ReadConcernSupportResult::allSupportedAndDefaultPermitted();
     }
 
     void doCheckAuthorization(OperationContext* opCtx) const final {
@@ -613,7 +661,7 @@ private:
 
     std::unique_ptr<CommandInvocation> parse(OperationContext* opCtx,
                                              const OpMsgRequest& request) final {
-        return stdx::make_unique<Invocation>(
+        return std::make_unique<Invocation>(
             this,
             request,
             BatchedCommandRequest::cloneInsertWithIds(BatchedCommandRequest::parseInsert(request)));
@@ -651,7 +699,7 @@ private:
                 "Cannot specify runtime constants option to a mongos",
                 !parsedRequest.hasRuntimeConstants());
         parsedRequest.setRuntimeConstants(Variables::generateRuntimeConstants(opCtx));
-        return stdx::make_unique<Invocation>(this, request, std::move(parsedRequest));
+        return std::make_unique<Invocation>(this, request, std::move(parsedRequest));
     }
 
     std::string help() const override {
@@ -681,7 +729,7 @@ private:
 
     std::unique_ptr<CommandInvocation> parse(OperationContext* opCtx,
                                              const OpMsgRequest& request) final {
-        return stdx::make_unique<Invocation>(
+        return std::make_unique<Invocation>(
             this, request, BatchedCommandRequest::parseDelete(request));
     }
 
