@@ -92,7 +92,8 @@ BSONObj expectInsertsReturnStaleVersionErrorsBase(const NamespaceString& nss,
         WriteErrorDetail* error = new WriteErrorDetail;
         error->setStatus({ErrorCodes::StaleShardVersion, "mock stale error"});
         error->setErrInfo([&] {
-            StaleConfigInfo sci(nss, ChunkVersion(1, 0, epoch), ChunkVersion(2, 0, epoch));
+            StaleConfigInfo sci(
+                nss, ChunkVersion(1, 0, epoch), ChunkVersion(2, 0, epoch), ShardId(kShardName1));
             BSONObjBuilder builder;
             sci.serialize(&builder);
             return builder.obj();
@@ -106,6 +107,52 @@ BSONObj expectInsertsReturnStaleVersionErrorsBase(const NamespaceString& nss,
     return staleResponse.toBSON();
 }
 
+BSONObj expectInsertsReturnStaleDbVersionErrorsBase(const NamespaceString& nss,
+                                                    const std::vector<BSONObj>& expected,
+                                                    const executor::RemoteCommandRequest& request) {
+    ASSERT_EQUALS(nss.db(), request.dbname);
+
+    const auto opMsgRequest(OpMsgRequest::fromDBAndBody(request.dbname, request.cmdObj));
+    const auto actualBatchedInsert(BatchedCommandRequest::parseInsert(opMsgRequest));
+    ASSERT_EQUALS(nss.toString(), actualBatchedInsert.getNS().ns());
+
+    const auto& inserted = actualBatchedInsert.getInsertRequest().getDocuments();
+    ASSERT_EQUALS(expected.size(), inserted.size());
+
+    auto itInserted = inserted.begin();
+    auto itExpected = expected.begin();
+
+    for (; itInserted != inserted.end(); itInserted++, itExpected++) {
+        ASSERT_BSONOBJ_EQ(*itExpected, *itInserted);
+    }
+
+    BSONObjBuilder staleResponse;
+    staleResponse.append("ok", 1);
+    staleResponse.append("n", 0);
+
+    // Report a stale db version error for each write in the batch.
+    int i = 0;
+    std::vector<BSONObj> errors;
+    for (itInserted = inserted.begin(); itInserted != inserted.end(); ++itInserted) {
+        BSONObjBuilder errorBuilder;
+        errorBuilder.append("index", i);
+        errorBuilder.append("code", int(ErrorCodes::StaleDbVersion));
+
+        auto dbVersion = databaseVersion::makeNew();
+        errorBuilder.append("db", nss.db());
+        errorBuilder.append("vReceived", dbVersion.toBSON());
+        errorBuilder.append("vWanted", databaseVersion::makeIncremented(dbVersion).toBSON());
+
+        errorBuilder.append("errmsg", "mock stale db version");
+
+        errors.push_back(errorBuilder.obj());
+        ++i;
+    }
+    staleResponse.append("writeErrors", errors);
+
+    return staleResponse.obj();
+}
+
 /**
  * Mimics a single shard backend for a particular collection which can be initialized with a
  * set of write command results to return.
@@ -121,6 +168,7 @@ public:
 
         // Set up the RemoteCommandTargeter for the config shard
         configTargeter()->setFindHostReturnValue(kTestConfigShardHost);
+
         // Add a RemoteCommandTargeter for the data shard2
         targeterFactory()->addTargeterToReturn(ConnectionString(kTestShardHost1), [] {
             std::unique_ptr<RemoteCommandTargeterMock> targeter(
@@ -191,6 +239,12 @@ public:
         });
     }
 
+    void expectInsertsReturnStaleDbVersionErrors(const std::vector<BSONObj>& expected) {
+        onCommandForPoolExecutor([&](const executor::RemoteCommandRequest& request) {
+            return expectInsertsReturnStaleDbVersionErrorsBase(nss, expected, request);
+        });
+    }
+
     void expectInsertsReturnError(const std::vector<BSONObj>& expected,
                                   const BatchedCommandResponse& errResponse) {
         onCommandForPoolExecutor([&](const executor::RemoteCommandRequest& request) {
@@ -352,7 +406,7 @@ TEST_F(BatchWriteExecTest, StaleShardVersionReturnedFromBatchWithSingleMultiWrit
     }());
     request.setWriteConcern(BSONObj());
 
-    static const auto epoch = OID::gen();
+    const static auto epoch = OID::gen();
 
     class MultiShardTargeter : public MockNSTargeter {
     public:
@@ -407,7 +461,8 @@ TEST_F(BatchWriteExecTest, StaleShardVersionReturnedFromBatchWithSingleMultiWrit
             errDetail->setErrInfo([&] {
                 Status ssvStatus(StaleConfigInfo(nss,
                                                  ChunkVersion(101, 200, epoch),
-                                                 ChunkVersion(105, 200, epoch)),
+                                                 ChunkVersion(105, 200, epoch),
+                                                 ShardId(kShardName2)),
                                  "Stale shard version");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
@@ -450,7 +505,7 @@ TEST_F(BatchWriteExecTest,
     }());
     request.setWriteConcern(BSONObj());
 
-    static const auto epoch = OID::gen();
+    const static auto epoch = OID::gen();
 
     // This allows the batch to target each write operation
     // to a specific shard (kShardName2), to perform this test
@@ -507,7 +562,8 @@ TEST_F(BatchWriteExecTest,
             errDetail->setErrInfo([&] {
                 Status ssvStatus(StaleConfigInfo(nss,
                                                  ChunkVersion(101, 200, epoch),
-                                                 ChunkVersion(105, 200, epoch)),
+                                                 ChunkVersion(105, 200, epoch),
+                                                 ShardId(kShardName2)),
                                  "Stale shard version");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
@@ -522,7 +578,8 @@ TEST_F(BatchWriteExecTest,
             errDetail->setErrInfo([&] {
                 Status ssvStatus(StaleConfigInfo(nss,
                                                  ChunkVersion(101, 200, epoch),
-                                                 ChunkVersion(105, 200, epoch)),
+                                                 ChunkVersion(105, 200, epoch),
+                                                 ShardId(kShardName2)),
                                  "Stale shard version");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
@@ -564,7 +621,7 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromMultiWriteWithShard1Firs) {
     }());
     request.setWriteConcern(BSONObj());
 
-    static const auto epoch = OID::gen();
+    const static auto epoch = OID::gen();
 
     // This allows the batch to target each write operation
     // to a specific shard (kShardName2), to perform this test
@@ -611,7 +668,8 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromMultiWriteWithShard1Firs) {
             errDetail->setErrInfo([&] {
                 Status ssvStatus(StaleConfigInfo(nss,
                                                  ChunkVersion(101, 200, epoch),
-                                                 ChunkVersion(105, 200, epoch)),
+                                                 ChunkVersion(105, 200, epoch),
+                                                 ShardId(kShardName1)),
                                  "Stale shard version");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
@@ -636,7 +694,8 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromMultiWriteWithShard1Firs) {
             errDetail->setErrInfo([&] {
                 Status ssvStatus(StaleConfigInfo(nss,
                                                  ChunkVersion(101, 200, epoch),
-                                                 ChunkVersion(105, 200, epoch)),
+                                                 ChunkVersion(105, 200, epoch),
+                                                 ShardId(kShardName2)),
                                  "Stale shard version");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
@@ -689,7 +748,7 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromMultiWriteWithShard1FirstOK
     }());
     request.setWriteConcern(BSONObj());
 
-    static const auto epoch = OID::gen();
+    const static auto epoch = OID::gen();
 
     // This allows the batch to target each write operation
     // to a specific shard (kShardName2), to perform this test
@@ -736,7 +795,8 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromMultiWriteWithShard1FirstOK
             errDetail->setErrInfo([&] {
                 Status ssvStatus(StaleConfigInfo(nss,
                                                  ChunkVersion(101, 200, epoch),
-                                                 ChunkVersion(105, 200, epoch)),
+                                                 ChunkVersion(105, 200, epoch),
+                                                 ShardId(kShardName1)),
                                  "Stale shard version");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
@@ -761,7 +821,8 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromMultiWriteWithShard1FirstOK
             errDetail->setErrInfo([&] {
                 Status ssvStatus(StaleConfigInfo(nss,
                                                  ChunkVersion(101, 200, epoch),
-                                                 ChunkVersion(105, 200, epoch)),
+                                                 ChunkVersion(105, 200, epoch),
+                                                 ShardId(kShardName2)),
                                  "Stale shard version");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
@@ -813,7 +874,7 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromWriteWithShard1SSVShard2OK)
     }());
     request.setWriteConcern(BSONObj());
 
-    static const auto epoch = OID::gen();
+    const static auto epoch = OID::gen();
 
     // This allows the batch to target each write operation to perform this test
     class MultiShardTargeter : public MockNSTargeter {
@@ -867,7 +928,8 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromWriteWithShard1SSVShard2OK)
             errDetail->setErrInfo([&] {
                 Status ssvStatus(StaleConfigInfo(nss,
                                                  ChunkVersion(101, 200, epoch),
-                                                 ChunkVersion(105, 200, epoch)),
+                                                 ChunkVersion(105, 200, epoch),
+                                                 ShardId(kShardName1)),
                                  "Migration happened");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
@@ -903,7 +965,7 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromWriteWithShard1SSVShard2OK)
 // Test retryable errors
 //
 
-TEST_F(BatchWriteExecTest, StaleOp) {
+TEST_F(BatchWriteExecTest, StaleShardOp) {
     BatchedCommandRequest request([&] {
         write_ops::Insert insertOp(nss);
         insertOp.setWriteCommandBase([] {
@@ -924,7 +986,7 @@ TEST_F(BatchWriteExecTest, StaleOp) {
             operationContext(), singleShardNSTargeter, request, &response, &stats);
         ASSERT(response.getOk());
 
-        ASSERT_EQUALS(1, stats.numStaleBatches);
+        ASSERT_EQUALS(1, stats.numStaleShardBatches);
     });
 
     const std::vector<BSONObj> expected{BSON("x" << 1)};
@@ -935,7 +997,7 @@ TEST_F(BatchWriteExecTest, StaleOp) {
     future.default_timed_get();
 }
 
-TEST_F(BatchWriteExecTest, MultiStaleOp) {
+TEST_F(BatchWriteExecTest, MultiStaleShardOp) {
     BatchedCommandRequest request([&] {
         write_ops::Insert insertOp(nss);
         insertOp.setWriteCommandBase([] {
@@ -955,7 +1017,7 @@ TEST_F(BatchWriteExecTest, MultiStaleOp) {
             operationContext(), singleShardNSTargeter, request, &response, &stats);
         ASSERT(response.getOk());
 
-        ASSERT_EQUALS(3, stats.numStaleBatches);
+        ASSERT_EQUALS(3, stats.numStaleShardBatches);
     });
 
     const std::vector<BSONObj> expected{BSON("x" << 1)};
@@ -974,7 +1036,6 @@ TEST_F(BatchWriteExecTest, TooManyStaleShardOp) {
     // Retry op in exec too many times (without refresh) b/c of stale config (the mock
     // singleShardNSTargeter doesn't report progress on refresh). We should report a no progress
     // error for everything in the batch.
-
     BatchedCommandRequest request([&] {
         write_ops::Insert insertOp(nss);
         insertOp.setWriteCommandBase([] {
@@ -998,7 +1059,7 @@ TEST_F(BatchWriteExecTest, TooManyStaleShardOp) {
         ASSERT_EQUALS(response.getErrDetailsAt(0)->toStatus().code(), ErrorCodes::NoProgressMade);
         ASSERT_EQUALS(response.getErrDetailsAt(1)->toStatus().code(), ErrorCodes::NoProgressMade);
 
-        ASSERT_EQUALS(stats.numStaleBatches, (1 + kMaxRoundsWithoutProgress));
+        ASSERT_EQUALS(stats.numStaleShardBatches, (1 + kMaxRoundsWithoutProgress));
     });
 
     // Return multiple StaleShardVersion errors
@@ -1009,6 +1070,111 @@ TEST_F(BatchWriteExecTest, TooManyStaleShardOp) {
     future.default_timed_get();
 }
 
+TEST_F(BatchWriteExecTest, StaleDbOp) {
+    BatchedCommandRequest request([&] {
+        write_ops::Insert insertOp(nss);
+        insertOp.setWriteCommandBase([] {
+            write_ops::WriteCommandBase writeCommandBase;
+            writeCommandBase.setOrdered(false);
+            return writeCommandBase;
+        }());
+        insertOp.setDocuments({BSON("x" << 1)});
+        return insertOp;
+    }());
+    request.setWriteConcern(BSONObj());
+
+    // Execute request
+    auto future = launchAsync([&] {
+        BatchedCommandResponse response;
+        BatchWriteExecStats stats;
+        BatchWriteExec::executeBatch(
+            operationContext(), singleShardNSTargeter, request, &response, &stats);
+        ASSERT(response.getOk());
+
+        ASSERT_EQUALS(1, stats.numStaleDbBatches);
+    });
+
+    const std::vector<BSONObj> expected{BSON("x" << 1)};
+
+    expectInsertsReturnStaleDbVersionErrors(expected);
+    expectInsertsReturnSuccess(expected);
+
+    future.default_timed_get();
+}
+
+TEST_F(BatchWriteExecTest, MultiStaleDbOp) {
+    BatchedCommandRequest request([&] {
+        write_ops::Insert insertOp(nss);
+        insertOp.setWriteCommandBase([] {
+            write_ops::WriteCommandBase writeCommandBase;
+            writeCommandBase.setOrdered(false);
+            return writeCommandBase;
+        }());
+        insertOp.setDocuments({BSON("x" << 1)});
+        return insertOp;
+    }());
+    request.setWriteConcern(BSONObj());
+
+    auto future = launchAsync([&] {
+        BatchedCommandResponse response;
+        BatchWriteExecStats stats;
+        BatchWriteExec::executeBatch(
+            operationContext(), singleShardNSTargeter, request, &response, &stats);
+        ASSERT(response.getOk());
+
+        ASSERT_EQUALS(3, stats.numStaleDbBatches);
+    });
+
+    const std::vector<BSONObj> expected{BSON("x" << 1)};
+
+    // Return multiple StaleDbVersion errors, but less than the give-up number
+    for (int i = 0; i < 3; i++) {
+        expectInsertsReturnStaleDbVersionErrors(expected);
+    }
+
+    expectInsertsReturnSuccess(expected);
+
+    future.default_timed_get();
+}
+
+TEST_F(BatchWriteExecTest, TooManyStaleDbOp) {
+    // Retry op in exec too many times (without refresh) b/c of stale config (the mock
+    // singleShardNSTargeter doesn't report progress on refresh). We should report a no progress
+    // error for everything in the batch.
+    BatchedCommandRequest request([&] {
+        write_ops::Insert insertOp(nss);
+        insertOp.setWriteCommandBase([] {
+            write_ops::WriteCommandBase writeCommandBase;
+            writeCommandBase.setOrdered(false);
+            return writeCommandBase;
+        }());
+        insertOp.setDocuments({BSON("x" << 1), BSON("x" << 2)});
+        return insertOp;
+    }());
+    request.setWriteConcern(BSONObj());
+
+    auto future = launchAsync([&] {
+        BatchedCommandResponse response;
+        BatchWriteExecStats stats;
+        BatchWriteExec::executeBatch(
+            operationContext(), singleShardNSTargeter, request, &response, &stats);
+        ASSERT(response.getOk());
+        ASSERT_EQ(0, response.getN());
+        ASSERT(response.isErrDetailsSet());
+        ASSERT_EQUALS(response.getErrDetailsAt(0)->toStatus().code(), ErrorCodes::NoProgressMade);
+        ASSERT_EQUALS(response.getErrDetailsAt(1)->toStatus().code(), ErrorCodes::NoProgressMade);
+
+        ASSERT_EQUALS(stats.numStaleDbBatches, (1 + kMaxRoundsWithoutProgress));
+    });
+
+    // Return multiple StaleDbVersion errors
+    for (int i = 0; i < (1 + kMaxRoundsWithoutProgress); i++) {
+        expectInsertsReturnStaleDbVersionErrors({BSON("x" << 1), BSON("x" << 2)});
+    }
+
+    future.default_timed_get();
+}
+
 TEST_F(BatchWriteExecTest, RetryableWritesLargeBatch) {
     // A retryable error without a txnNumber is not retried.
 
@@ -1226,7 +1392,7 @@ public:
         repl::ReadConcernArgs::get(operationContext()) =
             repl::ReadConcernArgs(repl::ReadConcernLevel::kSnapshotReadConcern);
 
-        auto logicalClock = stdx::make_unique<LogicalClock>(getServiceContext());
+        auto logicalClock = std::make_unique<LogicalClock>(getServiceContext());
         logicalClock->setClusterTimeFromTrustedSource(kInMemoryLogicalTime);
         LogicalClock::set(getServiceContext(), std::move(logicalClock));
 
