@@ -558,7 +558,7 @@ DB.prototype.help = function() {
     print("\tdb.getLastErrorObj() - return full status object");
     print("\tdb.getLogComponents()");
     print("\tdb.getMongo() get the server connection object");
-    print("\tdb.getMongo().setSlaveOk() allow queries on a replication slave server");
+    print("\tdb.getMongo().setSecondaryOk() allow queries on a replication secondary server");
     print("\tdb.getName()");
     print("\tdb.getProfilingLevel() - deprecated");
     print("\tdb.getProfilingStatus() - returns if profiling is on and slow threshold");
@@ -575,7 +575,7 @@ DB.prototype.help = function() {
     print("\tdb.printCollectionStats()");
     print("\tdb.printReplicationInfo()");
     print("\tdb.printShardingStatus()");
-    print("\tdb.printSlaveReplicationInfo()");
+    print("\tdb.printSecondaryReplicationInfo()");
     print("\tdb.resetError()");
     print(
         "\tdb.runCommand(cmdObj) run a database command.  if cmdObj is a string, turns it into {cmdObj: 1}");
@@ -1033,8 +1033,8 @@ DB.prototype.printReplicationInfo = function() {
             print("cannot provide replication status from an arbiter.");
             return;
         } else if (!isMaster.ismaster) {
-            print("this is a slave, printing slave replication info.");
-            this.printSlaveReplicationInfo();
+            print("this is a secondary, printing secondary replication info.");
+            this.printSecondaryReplicationInfo();
             return;
         }
         print(tojson(result));
@@ -1048,6 +1048,12 @@ DB.prototype.printReplicationInfo = function() {
 };
 
 DB.prototype.printSlaveReplicationInfo = function() {
+    print(
+        "WARNING: printSlaveReplicationInfo is deprecated and may be removed in the next major release. Please use printSecondaryReplicationInfo instead.");
+    this.printSecondaryReplicationInfo();
+};
+
+DB.prototype.printSecondaryReplicationInfo = function() {
     var startOptimeDate = null;
     var primary = null;
 
@@ -1065,7 +1071,7 @@ DB.prototype.printSlaveReplicationInfo = function() {
         print("\t" + Math.round(ago) + " secs (" + hrs + " hrs) behind the " + suffix);
     }
 
-    function getMaster(members) {
+    function getPrimary(members) {
         for (i in members) {
             var row = members[i];
             if (row.state === 1) {
@@ -1077,7 +1083,7 @@ DB.prototype.printSlaveReplicationInfo = function() {
     }
 
     function g(x) {
-        assert(x, "how could this be null (printSlaveReplicationInfo gx)");
+        assert(x, "how could this be null (printSecondaryReplicationInfo gx)");
         print("source: " + x.host);
         if (x.syncedTo) {
             var st = new Date(DB.tsToSeconds(x.syncedTo) * 1000);
@@ -1088,7 +1094,7 @@ DB.prototype.printSlaveReplicationInfo = function() {
     }
 
     function r(x) {
-        assert(x, "how could this be null (printSlaveReplicationInfo rx)");
+        assert(x, "how could this be null (printSecondaryReplicationInfo rx)");
         if (x.state == 1 || x.state == 7) {  // ignore primaries (1) and arbiters (7)
             return;
         }
@@ -1105,7 +1111,7 @@ DB.prototype.printSlaveReplicationInfo = function() {
 
     if (L.system.replset.count() != 0) {
         var status = this.adminCommand({'replSetGetStatus': 1});
-        primary = getMaster(status.members);
+        primary = getPrimary(status.members);
         if (primary) {
             startOptimeDate = primary.optimeDate;
         }
@@ -1240,20 +1246,32 @@ DB.autocomplete = function(obj) {
 };
 
 DB.prototype.setSlaveOk = function(value) {
-    if (value == undefined)
-        value = true;
-    this._slaveOk = value;
+    print(
+        "WARNING: setSlaveOk() is deprecated and may be removed in the next major release. Please use setSecondaryOk() instead.");
+    this.setSecondaryOk(value);
 };
 
 DB.prototype.getSlaveOk = function() {
-    if (this._slaveOk != undefined)
-        return this._slaveOk;
-    return this._mongo.getSlaveOk();
+    print(
+        "WARNING: getSlaveOk() is deprecated and may be removed in the next major release. Please use getSecondaryOk() instead.");
+    return this.getSecondaryOk();
+};
+
+DB.prototype.setSecondaryOk = function(value) {
+    if (value == undefined)
+        value = true;
+    this._secondaryOk = value;
+};
+
+DB.prototype.getSecondaryOk = function() {
+    if (this._secondaryOk != undefined)
+        return this._secondaryOk;
+    return this._mongo.getSecondaryOk();
 };
 
 DB.prototype.getQueryOptions = function() {
     var options = 0;
-    if (this.getSlaveOk())
+    if (this.getSecondaryOk())
         options |= 4;
     return options;
 };
