@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kDefault
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/platform/basic.h"
 
@@ -35,6 +35,7 @@
 
 #include <algorithm>
 #include <boost/filesystem.hpp>
+#include <cctype>
 #include <memory>
 #include <set>
 #include <stdlib.h>
@@ -50,6 +51,7 @@
 #include "mongo/client/dbclient_base.h"
 #include "mongo/client/replica_set_monitor.h"
 #include "mongo/db/hasher.h"
+#include "mongo/logv2/log.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/platform/random.h"
 #include "mongo/scripting/engine.h"
@@ -57,8 +59,7 @@
 #include "mongo/shell/shell_options.h"
 #include "mongo/shell/shell_utils_extended.h"
 #include "mongo/shell/shell_utils_launcher.h"
-#include "mongo/util/fail_point_service.h"
-#include "mongo/util/log.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/processinfo.h"
 #include "mongo/util/quick_exit.h"
 #include "mongo/util/text.h"
@@ -122,24 +123,150 @@ extern const JSFile servers_misc;
 extern const JSFile replsettest;
 extern const JSFile data_consistency_checker;
 extern const JSFile bridge;
+extern const JSFile feature_compatibility_version;
 }  // namespace JSFiles
 
-MONGO_REGISTER_SHIM(BenchRunConfig::createConnectionImpl)
-(const BenchRunConfig& config)->std::unique_ptr<DBClientBase> {
-    const ConnectionString connectionString = uassertStatusOK(ConnectionString::parse(config.host));
+namespace {
 
+std::unique_ptr<DBClientBase> benchRunConfigCreateConnectionImplProvider(
+    const BenchRunConfig& config) {
+    const ConnectionString connectionString = uassertStatusOK(ConnectionString::parse(config.host));
     std::string errorMessage;
     std::unique_ptr<DBClientBase> connection(connectionString.connect("BenchRun", errorMessage));
     uassert(16158, errorMessage, connection);
-
     return connection;
 }
 
+auto benchRunConfigCreateConnectionImplRegistration = MONGO_WEAK_FUNCTION_REGISTRATION(
+    BenchRunConfig::createConnectionImpl, benchRunConfigCreateConnectionImplProvider);
+
+// helper functions for isBalanced
+bool isUseCmd(std::string code) {
+    size_t first_space = code.find(" ");
+    if (first_space)
+        code = code.substr(0, first_space);
+    return code == "use";
+}
+
+/**
+ * Skip over a quoted string, including quotes escaped with backslash
+ *
+ * @param code      String
+ * @param start     Starting position within string, always > 0
+ * @param quote     Quote character (single or double quote)
+ * @return          Position of ending quote, or code.size() if no quote found
+ */
+size_t skipOverString(const std::string& code, size_t start, char quote) {
+    size_t pos = start;
+    while (pos < code.size()) {
+        pos = code.find(quote, pos);
+        if (pos == std::string::npos) {
+            return code.size();
+        }
+        // We want to break if the quote we found is not escaped, but we need to make sure
+        // that the escaping backslash is not itself escaped.  Comparisons of start and pos
+        // are to keep us from reading beyond the beginning of the quoted string.
+        //
+        if (start == pos || code[pos - 1] != '\\' ||   // previous char was backslash
+            start == pos - 1 || code[pos - 2] == '\\'  // char before backslash was not another
+        ) {
+            break;  // The quote we found was not preceded by an unescaped backslash; it is real
+        }
+        ++pos;  // The quote we found was escaped with backslash, so it doesn't count
+    }
+    return pos;
+}
+
+bool isOpSymbol(char c) {
+    static std::string OpSymbols = "~!%^&*-+=|:,<>/?.";
+
+    for (size_t i = 0; i < OpSymbols.size(); i++)
+        if (OpSymbols[i] == c)
+            return true;
+    return false;
+}
+
+}  // namespace
+
 namespace shell_utils {
 
+
+bool isBalanced(const std::string& code) {
+    if (isUseCmd(code))
+        return true;  // don't balance "use <dbname>" in case dbname contains special chars
+    int curlyBrackets = 0;
+    int squareBrackets = 0;
+    int parens = 0;
+    bool danglingOp = false;
+
+    for (size_t i = 0; i < code.size(); i++) {
+        switch (code[i]) {
+            case '/':
+                if (i + 1 < code.size() && code[i + 1] == '/') {
+                    while (i < code.size() && code[i] != '\n')
+                        i++;
+                }
+                continue;
+            case '{':
+                curlyBrackets++;
+                break;
+            case '}':
+                if (curlyBrackets <= 0)
+                    return true;
+                curlyBrackets--;
+                break;
+            case '[':
+                squareBrackets++;
+                break;
+            case ']':
+                if (squareBrackets <= 0)
+                    return true;
+                squareBrackets--;
+                break;
+            case '(':
+                parens++;
+                break;
+            case ')':
+                if (parens <= 0)
+                    return true;
+                parens--;
+                break;
+            case '"':
+            case '\'':
+                i = skipOverString(code, i + 1, code[i]);
+                if (i >= code.size()) {
+                    return true;  // Do not let unterminated strings enter multi-line mode
+                }
+                break;
+            case '\\':
+                if (i + 1 < code.size() && code[i + 1] == '/')
+                    i++;
+                break;
+            case '+':
+            case '-':
+                if (i + 1 < code.size() && code[i + 1] == code[i]) {
+                    i++;
+                    continue;  // postfix op (++/--) can't be a dangling op
+                }
+                break;
+        }
+        if (i >= code.size()) {
+            danglingOp = false;
+            break;
+        }
+        if ("~!%^&*-+=|:,<>/?."_sd.find(code[i]) != std::string::npos)
+            danglingOp = true;
+        else if (!std::isspace(code[i]))
+            danglingOp = false;
+    }
+
+    return curlyBrackets == 0 && squareBrackets == 0 && parens == 0 && !danglingOp;
+}
+
+
 std::string dbConnect;
 
-static const char* argv0 = 0;
+static const char* argv0 = nullptr;
 EnterpriseShellCallback* enterpriseCallback = nullptr;
 
 void RecordMyLocation(const char* _argv0) {
@@ -176,37 +303,24 @@ BSONObj JSGetMemInfo(const BSONObj& args, void* data) {
     return b.obj();
 }
 
-#if !defined(_WIN32)
-thread_local unsigned int _randomSeed = 0;
-#endif
+thread_local auto _prng = PseudoRandom(0);
 
 BSONObj JSSrand(const BSONObj& a, void* data) {
-    unsigned int seed;
+    int64_t seed;
     // grab the least significant bits of either the supplied argument or
     // a random number from SecureRandom.
-    if (a.nFields() == 1 && a.firstElement().isNumber())
-        seed = static_cast<unsigned int>(a.firstElement().numberLong());
-    else {
-        std::unique_ptr<SecureRandom> rand(SecureRandom::create());
-        seed = static_cast<unsigned int>(rand->nextInt64());
+    if (a.nFields() == 1 && a.firstElement().isNumber()) {
+        seed = a.firstElement().safeNumberLong();
+    } else {
+        seed = SecureRandom().nextInt64();
     }
-#if !defined(_WIN32)
-    _randomSeed = seed;
-#else
-    srand(seed);
-#endif
+    _prng = PseudoRandom(seed);
     return BSON("" << static_cast<double>(seed));
 }
 
 BSONObj JSRand(const BSONObj& a, void* data) {
     uassert(12519, "rand accepts no arguments", a.nFields() == 0);
-    unsigned r;
-#if !defined(_WIN32)
-    r = rand_r(&_randomSeed);
-#else
-    r = rand();
-#endif
-    return BSON("" << double(r) / (double(RAND_MAX) + 1));
+    return BSON("" << _prng.nextCanonicalDouble());
 }
 
 BSONObj isWindows(const BSONObj& a, void* data) {
@@ -286,23 +400,10 @@ BSONObj computeSHA256Block(const BSONObj& a, void* data) {
  * > convertShardKeyToHashed("Whatever key")
  */
 BSONObj convertShardKeyToHashed(const BSONObj& a, void* data) {
-    const auto& objEl = a[0];
+    uassert(10151, "convertShardKeyToHashed accepts 1 argument", a.nFields() == 1);
+    const auto& objEl = a.firstElement();
 
-    uassert(10151,
-            "convertShardKeyToHashed accepts either 1 or 2 arguments",
-            a.nFields() >= 1 && a.nFields() <= 2);
-
-    // It looks like the seed is always default right now.
-    // But no reason not to allow for the future
-    auto seed = BSONElementHasher::DEFAULT_HASH_SEED;
-    if (a.nFields() > 1) {
-        auto seedEl = a[1];
-
-        uassert(10159, "convertShardKeyToHashed seed value should be a number", seedEl.isNumber());
-        seed = seedEl.numberInt();
-    }
-
-    auto key = BSONElementHasher::hash64(objEl, seed);
+    auto key = BSONElementHasher::hash64(objEl, BSONElementHasher::DEFAULT_HASH_SEED);
     return BSON("" << key);
 }
 
@@ -312,7 +413,7 @@ BSONObj replMonitorStats(const BSONObj& a, void* data) {
             a.nFields() == 1 && a.firstElement().type() == String);
 
     auto name = a.firstElement().valuestrsafe();
-    ReplicaSetMonitorPtr rsm = ReplicaSetMonitor::get(name);
+    auto rsm = ReplicaSetMonitor::get(name);
     if (!rsm) {
         return BSON(""
                     << "no ReplSetMonitor exists by that name");
@@ -406,6 +507,7 @@ void initScope(Scope& scope) {
     scope.execSetup(JSFiles::replsettest);
     scope.execSetup(JSFiles::data_consistency_checker);
     scope.execSetup(JSFiles::bridge);
+    scope.execSetup(JSFiles::feature_compatibility_version);
 
     initializeEnterpriseScope(scope);
 
@@ -439,11 +541,12 @@ bool Prompter::confirm() {
 
 ConnectionRegistry::ConnectionRegistry() = default;
 
-void ConnectionRegistry::registerConnection(DBClientBase& client, StringData uri) {
+void ConnectionRegistry::registerConnection(DBClientBase& client) {
     BSONObj info;
     if (client.runCommand("admin", BSON("whatsmyuri" << 1), info)) {
+        std::string connstr = client.getServerAddress();
         stdx::lock_guard<Latch> lk(_mutex);
-        _connectionUris[uri.toString()].insert(info["you"].str());
+        _connectionUris[connstr].insert(info["you"].str());
     }
 }
 
@@ -451,10 +554,15 @@ void ConnectionRegistry::killOperationsOnAllConnections(bool withPrompt) const {
     Prompter prompter("do you want to kill the current op(s) on the server?");
     stdx::lock_guard<Latch> lk(_mutex);
     for (auto& connection : _connectionUris) {
-        std::string errmsg;
+        auto status = ConnectionString::parse(connection.first);
+        if (!status.isOK()) {
+            continue;
+        }
 
-        auto uri = uassertStatusOK(MongoURI::parse(connection.first));
-        std::unique_ptr<DBClientBase> conn(uri.connect("MongoDB Shell", errmsg));
+        const ConnectionString cs(status.getValue());
+
+        std::string errmsg;
+        std::unique_ptr<DBClientBase> conn(cs.connect("MongoDB Shell", errmsg));
         if (!conn) {
             continue;
         }
@@ -474,20 +582,20 @@ void ConnectionRegistry::killOperationsOnAllConnections(bool withPrompt) const {
             if (auto elem = op["client"]) {
                 // mongod currentOp client
                 if (elem.type() != String) {
-                    warning() << "Ignoring operation " << op["opid"].toString(false)
+                    std::cout << "Ignoring operation " << op["opid"].toString(false)
                               << "; expected 'client' field in currentOp response to have type "
                                  "string, but found "
-                              << typeName(elem.type());
+                              << typeName(elem.type()) << std::endl;
                     continue;
                 }
                 client = elem.str();
             } else if (auto elem = op["client_s"]) {
                 // mongos currentOp client
                 if (elem.type() != String) {
-                    warning() << "Ignoring operation " << op["opid"].toString(false)
+                    std::cout << "Ignoring operation " << op["opid"].toString(false)
                               << "; expected 'client_s' field in currentOp response to have type "
                                  "string, but found "
-                              << typeName(elem.type());
+                              << typeName(elem.type()) << std::endl;
                     continue;
                 }
                 client = elem.str();
@@ -512,7 +620,7 @@ void ConnectionRegistry::killOperationsOnAllConnections(bool withPrompt) const {
 
 ConnectionRegistry connectionRegistry;
 
-void onConnect(DBClientBase& c, StringData uri) {
+void onConnect(DBClientBase& c) {
     if (shellGlobalParams.nokillop) {
         return;
     }
@@ -522,7 +630,7 @@ void onConnect(DBClientBase& c, StringData uri) {
         c.setClientRPCProtocols(*shellGlobalParams.rpcProtocols);
     }
 
-    connectionRegistry.registerConnection(c, uri);
+    connectionRegistry.registerConnection(c);
 }
 
 bool fileExists(const std::string& file) {
