@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kDefault
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 #include "mongo/config.h"
 #include "mongo/platform/basic.h"
@@ -172,6 +172,27 @@ TEST(AssertUtils, UassertStatusOKPreservesExtraInfo) {
     }
 }
 
+TEST(AssertUtils, UassertStatusOKWithContextPreservesExtraInfo) {
+    const auto status = Status(ErrorExtraInfoExample(123), "");
+
+    try {
+        uassertStatusOKWithContext(status, "foo");
+    } catch (const DBException& ex) {
+        ASSERT(ex.extraInfo());
+        ASSERT(ex.extraInfo<ErrorExtraInfoExample>());
+        ASSERT_EQ(ex.extraInfo<ErrorExtraInfoExample>()->data, 123);
+    }
+
+    try {
+        uassertStatusOKWithContext(status, "foo");
+    } catch (const ExceptionFor<ErrorCodes::ForTestingErrorExtraInfo>& ex) {
+        ASSERT(ex.extraInfo());
+        ASSERT(ex.extraInfo<ErrorExtraInfoExample>());
+        ASSERT_EQ(ex.extraInfo<ErrorExtraInfoExample>()->data, 123);
+        ASSERT_EQ(ex->data, 123);
+    }
+}
+
 TEST(AssertUtils, UassertTypedExtraInfoWorks) {
     try {
         uasserted(ErrorExtraInfoExample(123), "");
@@ -191,6 +212,45 @@ TEST(AssertUtils, UassertTypedExtraInfoWorks) {
     }
 }
 
+TEST(AssertUtils, UassertIncrementsUserAssertionCounter) {
+    auto userAssertions = assertionCount.user.load();
+    auto asserted = false;
+    try {
+        Status status = {ErrorCodes::BadValue, "Test"};
+        uassertStatusOK(status);
+    } catch (const DBException&) {
+        asserted = true;
+    }
+    ASSERT(asserted);
+    ASSERT_EQ(userAssertions + 1, assertionCount.user.load());
+}
+
+TEST(AssertUtils, InternalAssertWithStatus) {
+    auto userAssertions = assertionCount.user.load();
+    try {
+        Status status = {ErrorCodes::BadValue, "Test"};
+        internalAssert(status);
+    } catch (const DBException& ex) {
+        ASSERT_EQ(ex.code(), ErrorCodes::BadValue);
+        ASSERT_EQ(ex.reason(), "Test");
+    }
+    ASSERT_EQ(userAssertions, assertionCount.user.load());
+}
+
+TEST(AssertUtils, InternalAssertWithExpression) {
+    auto userAssertions = assertionCount.user.load();
+    try {
+        internalAssert(48922, "Test", false);
+    } catch (const DBException& ex) {
+        ASSERT_EQ(ex.code(), 48922);
+        ASSERT_EQ(ex.reason(), "Test");
+    }
+
+    internalAssert(48922, "Another test", true);
+
+    ASSERT_EQ(userAssertions, assertionCount.user.load());
+}
+
 TEST(AssertUtils, MassertTypedExtraInfoWorks) {
     try {
         msgasserted(ErrorExtraInfoExample(123), "");
@@ -210,24 +270,6 @@ TEST(AssertUtils, MassertTypedExtraInfoWorks) {
     }
 }
 
-// uassert and its friends
-DEATH_TEST(UassertionTerminationTest, uassert, "Terminating with uassert") {
-    uassert(40204, "Terminating with uassert", false);
-}
-
-DEATH_TEST(UassertionTerminationTest, uasserted, "Terminating with uasserted") {
-    uasserted(40205, "Terminating with uasserted");
-}
-
-DEATH_TEST(UassertionTerminationTest, uassertStatusOK, "Terminating with uassertStatusOK") {
-    uassertStatusOK(Status(ErrorCodes::InternalError, "Terminating with uassertStatusOK"));
-}
-
-DEATH_TEST(UassertionTerminationTest, uassertStatusOKOverload, "Terminating with uassertStatusOK") {
-    uassertStatusOK(
-        StatusWith<std::string>(ErrorCodes::InternalError, "Terminating with uassertStatusOK"));
-}
-
 // fassert and its friends
 DEATH_TEST(FassertionTerminationTest, fassert, "40206") {
     fassert(40206, false);
@@ -275,22 +317,8 @@ DEATH_TEST(FassertionTerminationTest,
         40213, {ErrorCodes::InternalError, "Terminating with fassertFailedWithStatusNoTrace"});
 }
 
-// massert and its friends
-DEATH_TEST(MassertionTerminationTest, massert, "Terminating with massert") {
-    massert(40214, "Terminating with massert", false);
-}
-
-
-DEATH_TEST(MassertionTerminationTest, massertStatusOK, "Terminating with massertStatusOK") {
-    massertStatusOK(Status(ErrorCodes::InternalError, "Terminating with massertStatusOK"));
-}
-
-DEATH_TEST(MassertionTerminationTest, msgasserted, "Terminating with msgasserted") {
-    msgasserted(40215, "Terminating with msgasserted");
-}
-
 // invariant and its friends
-DEATH_TEST(InvariantTerminationTest, invariant, "Invariant failure false " __FILE__) {
+DEATH_TEST_REGEX(InvariantTerminationTest, invariant, "Invariant failure.*false.*" __FILE__) {
     invariant(false);
 }
 
@@ -349,9 +377,37 @@ DEATH_TEST(InvariantTerminationTest,
               msg);
 }
 
+DEATH_TEST(InvariantTerminationTest, invariantStatusOK, "Terminating with invariantStatusOK") {
+    invariantStatusOK(Status(ErrorCodes::InternalError, "Terminating with invariantStatusOK"));
+}
+
+DEATH_TEST(InvariantTerminationTest,
+           invariantStatusOKOverload,
+           "Terminating with invariantStatusOK") {
+    invariantStatusOK(
+        StatusWith<std::string>(ErrorCodes::InternalError, "Terminating with invariantStatusOK"));
+}
+
+DEATH_TEST(InvariantTerminationTest,
+           invariantStatusOKWithContext,
+           "Terminating with invariantStatusOKWithContext") {
+    invariantStatusOKWithContext(
+        Status(ErrorCodes::InternalError, "Terminating with invariantStatusOKWithContext"),
+        "Terminating with invariantStatusOKWithContext");
+}
+
+DEATH_TEST(InvariantTerminationTest,
+           invariantStatusOKWithContextOverload,
+           "Terminating with invariantStatusOKWithContextOverload") {
+    invariantStatusOKWithContext(
+        StatusWith<std::string>(ErrorCodes::InternalError,
+                                "Terminating with invariantStatusOKWithContext"),
+        "Terminating with invariantStatusOKWithContextOverload");
+}
+
 #if defined(MONGO_CONFIG_DEBUG_BUILD)
 // dassert and its friends
-DEATH_TEST(DassertTerminationTest, invariant, "Invariant failure false " __FILE__) {
+DEATH_TEST_REGEX(DassertTerminationTest, invariant, "Invariant failure.*false.*" __FILE__) {
     dassert(false);
 }
 
