@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kControl
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kControl
 
 #include "mongo/platform/basic.h"
 
@@ -45,10 +45,13 @@
 #include "mongo/base/static_assert.h"
 #include "mongo/db/client.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/logv2/log.h"
 #include "mongo/platform/process_id.h"
 #include "mongo/util/concurrency/idle_thread_block.h"
+#include "mongo/util/errno_util.h"
+#include "mongo/util/exit.h"
+#include "mongo/util/exit_code.h"
 #include "mongo/util/hex.h"
-#include "mongo/util/log.h"
 #include "mongo/util/timer.h"
 
 
@@ -59,7 +62,7 @@ WatchdogPeriodicThread::WatchdogPeriodicThread(Milliseconds period, StringData t
 
 void WatchdogPeriodicThread::start() {
     {
-        stdx::lock_guard<stdx::mutex> lock(_mutex);
+        stdx::lock_guard<Latch> lock(_mutex);
 
         invariant(_state == State::kNotStarted);
         _state = State::kStarted;
@@ -74,7 +77,7 @@ void WatchdogPeriodicThread::shutdown() {
     stdx::thread thread;
 
     {
-        stdx::lock_guard<stdx::mutex> lock(_mutex);
+        stdx::lock_guard<Latch> lock(_mutex);
 
         bool started = (_state == State::kStarted);
 
@@ -99,7 +102,7 @@ void WatchdogPeriodicThread::shutdown() {
 }
 
 void WatchdogPeriodicThread::setPeriod(Milliseconds period) {
-    stdx::lock_guard<stdx::mutex> lock(_mutex);
+    stdx::lock_guard<Latch> lock(_mutex);
 
     bool wasEnabled = _enabled;
 
@@ -128,7 +131,7 @@ void WatchdogPeriodicThread::doLoop() {
     auto preciseClockSource = client->getServiceContext()->getPreciseClockSource();
 
     {
-        stdx::lock_guard<stdx::mutex> lock(_mutex);
+        stdx::lock_guard<Latch> lock(_mutex);
 
         // Ensure state is starting from a clean slate.
         resetState();
@@ -142,27 +145,32 @@ void WatchdogPeriodicThread::doLoop() {
         Date_t startTime = preciseClockSource->now();
 
         {
-            stdx::unique_lock<stdx::mutex> lock(_mutex);
+            stdx::unique_lock<Latch> lock(_mutex);
             MONGO_IDLE_THREAD_BLOCK;
 
 
             // Check if the period is different?
             // We are signalled on period changes at which point we may be done waiting or need to
             // wait longer.
-            while (startTime + _period > preciseClockSource->now() &&
-                   _state != State::kShutdownRequested) {
-                auto s = opCtx->waitForConditionOrInterruptNoAssertUntil(
-                    _condvar, lock, startTime + _period);
-
-                if (!s.isOK()) {
-                    // The only bad status is when we are in shutdown
-                    if (!opCtx->getServiceContext()->getKillAllOperations()) {
-                        error() << "Watchdog was interrupted, shuting down:, reason: "
-                                << s.getStatus();
-                    }
-
-                    return;
+            try {
+                opCtx->waitForConditionOrInterruptUntil(_condvar, lock, startTime + _period, [&] {
+                    return (startTime + _period) <= preciseClockSource->now() ||
+                        _state == State::kShutdownRequested;
+                });
+            } catch (const DBException& e) {
+                // The only bad status is when we are in shutdown
+                if (!opCtx->getServiceContext()->getKillAllOperations()) {
+                    LOGV2_FATAL_CONTINUE(
+                        23415,
+                        "Watchdog was interrupted, shutting down, reason: {e_toStatus}",
+                        "e_toStatus"_attr = e.toStatus());
+                    exitCleanly(ExitCode::EXIT_ABRUPT);
                 }
+
+                // This interruption ends the WatchdogPeriodicThread. This means it is possible to
+                // killOp this operation and stop it for the lifetime of the process.
+                LOGV2_DEBUG(23406, 1, "WatchdogPeriodicThread interrupted by: {e}", "e"_attr = e);
+                return;
             }
 
             // Are we done running?
@@ -198,8 +206,12 @@ void WatchdogCheckThread::run(OperationContext* opCtx) {
         check->run(opCtx);
         Microseconds micros = timer.elapsed();
 
-        LOG(1) << "Watchdog test '" << check->getDescriptionForLogging() << "' took "
-               << duration_cast<Milliseconds>(micros);
+        LOGV2_DEBUG(23407,
+                    1,
+                    "Watchdog test '{check_getDescriptionForLogging}' took "
+                    "{duration_cast_Milliseconds_micros}",
+                    "check_getDescriptionForLogging"_attr = check->getDescriptionForLogging(),
+                    "duration_cast_Milliseconds_micros"_attr = duration_cast<Milliseconds>(micros));
 
         // We completed a check, bump the generation counter.
         _checkGeneration.fetchAndAdd(1);
@@ -246,7 +258,7 @@ WatchdogMonitor::WatchdogMonitor(std::vector<std::unique_ptr<WatchdogCheck>> che
 }
 
 void WatchdogMonitor::start() {
-    log() << "Starting Watchdog Monitor";
+    LOGV2(23408, "Starting Watchdog Monitor");
 
     // Start the threads.
     _watchdogCheckThread.start();
@@ -254,7 +266,7 @@ void WatchdogMonitor::start() {
     _watchdogMonitorThread.start();
 
     {
-        stdx::lock_guard<stdx::mutex> lock(_mutex);
+        stdx::lock_guard<Latch> lock(_mutex);
 
         invariant(_state == State::kNotStarted);
         _state = State::kStarted;
@@ -263,7 +275,7 @@ void WatchdogMonitor::start() {
 
 void WatchdogMonitor::setPeriod(Milliseconds duration) {
     {
-        stdx::lock_guard<stdx::mutex> lock(_mutex);
+        stdx::lock_guard<Latch> lock(_mutex);
 
         if (duration > Milliseconds(0)) {
             dassert(duration >= Milliseconds(1));
@@ -275,19 +287,21 @@ void WatchdogMonitor::setPeriod(Milliseconds duration) {
             _watchdogCheckThread.setPeriod(_checkPeriod);
             _watchdogMonitorThread.setPeriod(duration);
 
-            log() << "WatchdogMonitor period changed to " << duration_cast<Seconds>(duration);
+            LOGV2(23409,
+                  "WatchdogMonitor period changed to {duration_cast_Seconds_duration}",
+                  "duration_cast_Seconds_duration"_attr = duration_cast<Seconds>(duration));
         } else {
             _watchdogMonitorThread.setPeriod(duration);
             _watchdogCheckThread.setPeriod(duration);
 
-            log() << "WatchdogMonitor disabled";
+            LOGV2(23410, "WatchdogMonitor disabled");
         }
     }
 }
 
 void WatchdogMonitor::shutdown() {
     {
-        stdx::lock_guard<stdx::mutex> lock(_mutex);
+        stdx::lock_guard<Latch> lock(_mutex);
 
         bool started = (_state == State::kStarted);
 
@@ -338,66 +352,98 @@ void checkFile(OperationContext* opCtx, const boost::filesystem::path& file) {
                                NULL);
     if (hFile == INVALID_HANDLE_VALUE) {
         std::uint32_t gle = ::GetLastError();
-        severe() << "CreateFile failed for '" << file.generic_string()
-                 << "' with error: " << errnoWithDescription(gle);
+        LOGV2_FATAL_CONTINUE(23416,
+                             "CreateFile failed for '{file_generic_string}' with error: "
+                             "{errnoWithDescription_gle}",
+                             "file_generic_string"_attr = file.generic_string(),
+                             "errnoWithDescription_gle"_attr = errnoWithDescription(gle));
         fassertNoTrace(4074, gle == 0);
     }
 
     DWORD bytesWrittenTotal;
     if (!WriteFile(hFile, nowStr.c_str(), nowStr.size(), &bytesWrittenTotal, NULL)) {
         std::uint32_t gle = ::GetLastError();
-        severe() << "WriteFile failed for '" << file.generic_string()
-                 << "' with error: " << errnoWithDescription(gle);
+        LOGV2_FATAL_CONTINUE(
+            23417,
+            "WriteFile failed for '{file_generic_string}' with error: {errnoWithDescription_gle}",
+            "file_generic_string"_attr = file.generic_string(),
+            "errnoWithDescription_gle"_attr = errnoWithDescription(gle));
         fassertNoTrace(4075, gle == 0);
     }
 
     if (bytesWrittenTotal != nowStr.size()) {
-        warning() << "partial write for '" << file.generic_string() << "' expected "
-                  << nowStr.size() << " bytes but wrote " << bytesWrittenTotal << " bytes";
+        LOGV2_WARNING(23411,
+                      "partial write for '{file_generic_string}' expected {nowStr_size} bytes but "
+                      "wrote {bytesWrittenTotal} bytes",
+                      "file_generic_string"_attr = file.generic_string(),
+                      "nowStr_size"_attr = nowStr.size(),
+                      "bytesWrittenTotal"_attr = bytesWrittenTotal);
     } else {
 
         if (!FlushFileBuffers(hFile)) {
             std::uint32_t gle = ::GetLastError();
-            severe() << "FlushFileBuffers failed for '" << file.generic_string()
-                     << "' with error: " << errnoWithDescription(gle);
+            LOGV2_FATAL_CONTINUE(23418,
+                                 "FlushFileBuffers failed for '{file_generic_string}' with error: "
+                                 "{errnoWithDescription_gle}",
+                                 "file_generic_string"_attr = file.generic_string(),
+                                 "errnoWithDescription_gle"_attr = errnoWithDescription(gle));
             fassertNoTrace(4076, gle == 0);
         }
 
         DWORD newOffset = SetFilePointer(hFile, 0, 0, FILE_BEGIN);
         if (newOffset != 0) {
             std::uint32_t gle = ::GetLastError();
-            severe() << "SetFilePointer failed for '" << file.generic_string()
-                     << "' with error: " << errnoWithDescription(gle);
+            LOGV2_FATAL_CONTINUE(23419,
+                                 "SetFilePointer failed for '{file_generic_string}' with error: "
+                                 "{errnoWithDescription_gle}",
+                                 "file_generic_string"_attr = file.generic_string(),
+                                 "errnoWithDescription_gle"_attr = errnoWithDescription(gle));
             fassertNoTrace(4077, gle == 0);
         }
 
         DWORD bytesRead;
-        auto readBuffer = stdx::make_unique<char[]>(nowStr.size());
+        auto readBuffer = std::make_unique<char[]>(nowStr.size());
         if (!ReadFile(hFile, readBuffer.get(), nowStr.size(), &bytesRead, NULL)) {
             std::uint32_t gle = ::GetLastError();
-            severe() << "ReadFile failed for '" << file.generic_string()
-                     << "' with error: " << errnoWithDescription(gle);
+            LOGV2_FATAL_CONTINUE(23420,
+                                 "ReadFile failed for '{file_generic_string}' with error: "
+                                 "{errnoWithDescription_gle}",
+                                 "file_generic_string"_attr = file.generic_string(),
+                                 "errnoWithDescription_gle"_attr = errnoWithDescription(gle));
             fassertNoTrace(4078, gle == 0);
         }
 
         if (bytesRead != bytesWrittenTotal) {
-            severe() << "Read wrong number of bytes for '" << file.generic_string() << "' expected "
-                     << bytesWrittenTotal << " bytes but read " << bytesRead << " bytes";
-            fassertNoTrace(50724, false);
+            LOGV2_FATAL_NOTRACE(50724,
+                                "Read wrong number of bytes for '{file_generic_string}' expected "
+                                "{bytesWrittenTotal} bytes but read {bytesRead} bytes",
+                                "file_generic_string"_attr = file.generic_string(),
+                                "bytesWrittenTotal"_attr = bytesWrittenTotal,
+                                "bytesRead"_attr = bytesRead);
         }
 
         if (memcmp(nowStr.c_str(), readBuffer.get(), nowStr.size()) != 0) {
-            severe() << "Read wrong string from file '" << file.generic_string() << nowStr.size()
-                     << " bytes (in hex) '" << toHexLower(nowStr.c_str(), nowStr.size())
-                     << "' but read bytes '" << toHexLower(readBuffer.get(), bytesRead) << "'";
-            fassertNoTrace(50717, false);
+            LOGV2_FATAL_NOTRACE(
+                50717,
+                "Read wrong string from file '{file_generic_string}{nowStr_size} bytes (in "
+                "hex) '{toHexLower_nowStr_c_str_nowStr_size}' but read bytes "
+                "'{toHexLower_readBuffer_get_bytesRead}'",
+                "file_generic_string"_attr = file.generic_string(),
+                "nowStr_size"_attr = nowStr.size(),
+                "toHexLower_nowStr_c_str_nowStr_size"_attr =
+                    toHexLower(nowStr.c_str(), nowStr.size()),
+                "toHexLower_readBuffer_get_bytesRead"_attr =
+                    toHexLower(readBuffer.get(), bytesRead));
         }
     }
 
     if (!CloseHandle(hFile)) {
         std::uint32_t gle = ::GetLastError();
-        severe() << "CloseHandle failed for '" << file.generic_string()
-                 << "' with error: " << errnoWithDescription(gle);
+        LOGV2_FATAL_CONTINUE(
+            23423,
+            "CloseHandle failed for '{file_generic_string}' with error: {errnoWithDescription_gle}",
+            "file_generic_string"_attr = file.generic_string(),
+            "errnoWithDescription_gle"_attr = errnoWithDescription(gle));
         fassertNoTrace(4079, gle == 0);
     }
 }
@@ -422,8 +468,11 @@ void checkFile(OperationContext* opCtx, const boost::filesystem::path& file) {
     int fd = open(file.generic_string().c_str(), O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
     if (fd == -1) {
         auto err = errno;
-        severe() << "open failed for '" << file.generic_string()
-                 << "' with error: " << errnoWithDescription(err);
+        LOGV2_FATAL_CONTINUE(
+            23424,
+            "open failed for '{file_generic_string}' with error: {errnoWithDescription_err}",
+            "file_generic_string"_attr = file.generic_string(),
+            "errnoWithDescription_err"_attr = errnoWithDescription(err));
         fassertNoTrace(4080, err == 0);
     }
 
@@ -437,15 +486,22 @@ void checkFile(OperationContext* opCtx, const boost::filesystem::path& file) {
                 continue;
             }
 
-            severe() << "write failed for '" << file.generic_string()
-                     << "' with error: " << errnoWithDescription(err);
+            LOGV2_FATAL_CONTINUE(
+                23425,
+                "write failed for '{file_generic_string}' with error: {errnoWithDescription_err}",
+                "file_generic_string"_attr = file.generic_string(),
+                "errnoWithDescription_err"_attr = errnoWithDescription(err));
             fassertNoTrace(4081, err == 0);
         }
 
         // Warn if the write was incomplete
         if (bytesWrittenTotal == 0 && static_cast<size_t>(bytesWrittenInWrite) != nowStr.size()) {
-            warning() << "parital write for '" << file.generic_string() << "' expected "
-                      << nowStr.size() << " bytes but wrote " << bytesWrittenInWrite << " bytes";
+            LOGV2_WARNING(23412,
+                          "parital write for '{file_generic_string}' expected {nowStr_size} bytes "
+                          "but wrote {bytesWrittenInWrite} bytes",
+                          "file_generic_string"_attr = file.generic_string(),
+                          "nowStr_size"_attr = nowStr.size(),
+                          "bytesWrittenInWrite"_attr = bytesWrittenInWrite);
         }
 
         bytesWrittenTotal += bytesWrittenInWrite;
@@ -453,12 +509,15 @@ void checkFile(OperationContext* opCtx, const boost::filesystem::path& file) {
 
     if (fsync(fd)) {
         auto err = errno;
-        severe() << "fsync failed for '" << file.generic_string()
-                 << "' with error: " << errnoWithDescription(err);
+        LOGV2_FATAL_CONTINUE(
+            23426,
+            "fsync failed for '{file_generic_string}' with error: {errnoWithDescription_err}",
+            "file_generic_string"_attr = file.generic_string(),
+            "errnoWithDescription_err"_attr = errnoWithDescription(err));
         fassertNoTrace(4082, err == 0);
     }
 
-    auto readBuffer = stdx::make_unique<char[]>(nowStr.size());
+    auto readBuffer = std::make_unique<char[]>(nowStr.size());
     size_t bytesReadTotal = 0;
     while (bytesReadTotal < nowStr.size()) {
         ssize_t bytesReadInRead = pread(
@@ -469,36 +528,52 @@ void checkFile(OperationContext* opCtx, const boost::filesystem::path& file) {
                 continue;
             }
 
-            severe() << "read failed for '" << file.generic_string()
-                     << "' with error: " << errnoWithDescription(err);
+            LOGV2_FATAL_CONTINUE(
+                23427,
+                "read failed for '{file_generic_string}' with error: {errnoWithDescription_err}",
+                "file_generic_string"_attr = file.generic_string(),
+                "errnoWithDescription_err"_attr = errnoWithDescription(err));
             fassertNoTrace(4083, err == 0);
         } else if (bytesReadInRead == 0) {
-            severe() << "read failed for '" << file.generic_string()
-                     << "' with unexpected end of file";
-            fassertNoTrace(50719, false);
+            LOGV2_FATAL_NOTRACE(
+                50719,
+                "read failed for '{file_generic_string}' with unexpected end of file",
+                "file_generic_string"_attr = file.generic_string());
         }
 
         // Warn if the read was incomplete
         if (bytesReadTotal == 0 && static_cast<size_t>(bytesReadInRead) != nowStr.size()) {
-            warning() << "partial read for '" << file.generic_string() << "' expected "
-                      << nowStr.size() << " bytes but read " << bytesReadInRead << " bytes";
+            LOGV2_WARNING(23413,
+                          "partial read for '{file_generic_string}' expected {nowStr_size} bytes "
+                          "but read {bytesReadInRead} bytes",
+                          "file_generic_string"_attr = file.generic_string(),
+                          "nowStr_size"_attr = nowStr.size(),
+                          "bytesReadInRead"_attr = bytesReadInRead);
         }
 
         bytesReadTotal += bytesReadInRead;
     }
 
     if (memcmp(nowStr.c_str(), readBuffer.get(), nowStr.size()) != 0) {
-        severe() << "Read wrong string from file '" << file.generic_string() << "' expected "
-                 << nowStr.size() << " bytes (in hex) '"
-                 << toHexLower(nowStr.c_str(), nowStr.size()) << "' but read bytes '"
-                 << toHexLower(readBuffer.get(), bytesReadTotal) << "'";
-        fassertNoTrace(50718, false);
+        LOGV2_FATAL_NOTRACE(
+            50718,
+            "Read wrong string from file '{file_generic_string}' expected {nowStr_size} "
+            "bytes (in hex) '{toHexLower_nowStr_c_str_nowStr_size}' but read bytes "
+            "'{toHexLower_readBuffer_get_bytesReadTotal}'",
+            "file_generic_string"_attr = file.generic_string(),
+            "nowStr_size"_attr = nowStr.size(),
+            "toHexLower_nowStr_c_str_nowStr_size"_attr = toHexLower(nowStr.c_str(), nowStr.size()),
+            "toHexLower_readBuffer_get_bytesReadTotal"_attr =
+                toHexLower(readBuffer.get(), bytesReadTotal));
     }
 
     if (close(fd)) {
         auto err = errno;
-        severe() << "close failed for '" << file.generic_string()
-                 << "' with error: " << errnoWithDescription(err);
+        LOGV2_FATAL_CONTINUE(
+            23430,
+            "close failed for '{file_generic_string}' with error: {errnoWithDescription_err}",
+            "file_generic_string"_attr = file.generic_string(),
+            "errnoWithDescription_err"_attr = errnoWithDescription(err));
         fassertNoTrace(4084, err == 0);
     }
 }
@@ -525,8 +600,10 @@ void DirectoryCheck::run(OperationContext* opCtx) {
     boost::system::error_code ec;
     boost::filesystem::remove(file, ec);
     if (ec) {
-        warning() << "Failed to delete file '" << file.generic_string()
-                  << "', error: " << ec.message();
+        LOGV2_WARNING(23414,
+                      "Failed to delete file '{file_generic_string}', error: {ec_message}",
+                      "file_generic_string"_attr = file.generic_string(),
+                      "ec_message"_attr = ec.message());
     }
 }
 
